PROGRAM SFIT


  !+
  !
  !  Program:
  !     sfit
  !
  !  Language:
  !     Fortran95
  !
  !  Purpose:
  !     Given an observed high-resolution spectrum and a grid of
  !     synthetic spectra on a parameter space { Teff, log g, nHe },
  !     find the best fit synthetic spectrum by varying parameters 
  !     Teff, log g, n_He and v sin i, with errors.
  !
  !     The observed high-resolution spectrum is currently assumed
  !     to be normalized to unity and shifted to laboratory wavelength.
  !
  !     !---------------------------------------!
  !     ! H                                  He !   
  !     !                        C  N  O     Ne !  
  !     ! Na Mg               Al Si P  S  Cl Ar !  
  !     !    Ca      Fe                         !
  !     !---------------------------------------!
  !
  !  Reference:
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory, 1999- )
  !     -- principal author
  !
  !     Mairead Skelly (MS) (Armagh Observatory: 2002, summer student)
  !     Glenn Hawe (GH) (Armagh Observatory: 2002, summer student)
  !     -- developed genetic algorithm components
  !
  !  History:
  !
  !     SFIT
  !     05-Oct-2000: CSJ
  !        adapted from SFIT_TGV to fit binary stars 
  !     24-Nov-2000: CSJ
  !        extension to include continuum fitting
  !     12-Oct-2000: CSJ
  !        new I/O interface
  !     ...........: CSJ
  !        various enhancements
  !
  !     SFIT_SYNTH
  !
  !     SFIT2
  !     05-Sep-2002: CSJ
  !        major revision to incorporate SFIT_SYNTH
  !     11-Sep-2002: GH & MS
  !        incorporated genetic algorithm method in solve
  !     15-Oct-2003 : CSJ
  !        Version 2.2 built: substantial corrections 
  !     07-Nov-2003 : CSJ
  !        Version 2.2a built: substantial corrections 
  !     04-Jun-2004 : CSJ
  !        Version 2.2c built: extended abundance arrays 
  !     28-Sep-2004 : CSJ
  !        Version 2.3 begun: more flexible language, improved methods 
  !     15-Oct-2004: CSJ
  !        major revision to incorporate better command parsing + macros
  !     09-Feb-2005: CSJ
  !        bug correction in sfit_normalize: CLISTA not initialized
  !     10-Feb-2005: CSJ
  !        added check on cr rejection to avoid removing entire spectrum
  !     22-Feb-2005: CSJ
  !        removed AMOD and ABIN from model data arrays
  !        introduced different dimensionality to model data arrays
  !        introduced CONT_REGS, CONT_REGD and CONT_REG_CLEAR
  !     22-Apr-2007: CWR
  !        Introduced new grid management/interpolation system based
  !        on Delaunay tetrahedralisation. See Winter (2006) for full
  !        details.
  !     20-SEP-2012: CSJ
  !        Version 3.1 built: compiles with gfortran, libraries realigned
  !        Delauney tetrahedralization unavailable ... tetpack unavailable in gfortran
  !     12-DEC-2013: CSJ
  !        Version 3.2: minor bug fixes
  !     01-MAY-2016; CSJ
  !        Version 3.3: Numerous bug fixes and array extensions
  !     19-DEC-2019: CSJ
  !        Version 3.4: Some code tidying. obsolete features removed. 
  !         
  !
  !-


  !  Global constants:
  USE XFIT_DIMS
  USE TAPM_PAR
  USE TAPP_PAR

  !  Local constants:
  !DOUBLE PRECISION, PARAMETER :: TAPM__PI = 3.141592653589793238462643383279503D+00
  !DOUBLE PRECISION, PARAMETER :: TAPP__C = 2.99792458D+08 


  !  Global variables:
  USE SFIT_GLOBAL

  USE SFIT_MOD_TGH
  USE SFIT_MOD_TGH_2
  USE SFIT_BIN_TGH
  USE SFIT_BIN_TGH_2
  USE SFIT_OBSDATA
  USE SFIT_SYNDATA
  USE SFIT_FITDATA

  !  Internal procedures
  USE SFIT_GENERAL
  !  USE SFIT_TOOLS
  !  USE SFIT_SOLVE
  !  USE SFIT_SYNTH
  !  USE SFIT_GA
  USE SFIT_MACROS

  USE mod_model_list
  USE mod_tetra_list


  !  No implicit typing
  IMPLICIT NONE


  !  External functions:
  EXTERNAL SPEC_SYNTH
  EXTERNAL SPEC_TGHV_BIN
  EXTERNAL SFIT_FPOLY

  !  Subroutine interfaces


  INTERFACE

     SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)

       CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).

     END SUBROUTINE KEY_GET

     SUBROUTINE SFIT_MAKEGRID (MODE, GRDFILE, NWMAX, NWMOD, WMOD)
       IMPLICIT NONE

       INTEGER,INTENT(IN)            :: MODE
       CHARACTER,INTENT(IN)          :: GRDFILE*(*)
       INTEGER,INTENT(IN)            :: NWMAX
       INTEGER,INTENT(OUT)           :: NWMOD
       REAL,DIMENSION(NWMAX),INTENT(OUT) :: WMOD

     END SUBROUTINE SFIT_MAKEGRID

  END INTERFACE


  !  Local variables:

  ! program identifier
  CHARACTER(LEN=4), PARAMETER :: PROG__ID = "sfit"
  CHARACTER(LEN=5), PARAMETER :: PROG__VERS = "3.4  "
  CHARACTER(LEN=11), PARAMETER :: PROG__DATE = "19-DEC-2019"
  CHARACTER(LEN=80), PARAMETER :: PROG__URL = "http://star.arm.ac.uk/~csj/software_store/spectrum_guide/sfit.html"

  ! spare integers
  INTEGER   :: I,J,K,M,N

  ! spare reals
  REAL      :: A,B,C, X,Y,Z

  ! integer status
  INTEGER :: IOSTAT

  ! logfile information
  INTEGER :: LOGFILE = 11
  INTEGER :: SUMFILE = 12


  !  Command line control
  INTEGER      :: NREAD  = 0    ! unit number for input channel
  INTEGER      :: NWRITE = 0    ! unit number for output channel
  INTEGER      :: NECHO  = 1    ! echo 
  INTEGER      :: NARGS  = 0    ! number of arguments 

  CHARACTER(LEN=1024):: COMMAND = ' ' ! holder for current command line
  CHARACTER(LEN=128) :: KEYWRD     ! holder for current command or keyword
  CHARACTER(LEN=2)   :: KEYSEP = ';,'
  CHARACTER(LEN=16)  :: PARCOM     ! command for parameter entry 
  CHARACTER(LEN=16)  :: WAVCOM     ! command for wavelength definition 
  CHARACTER(LEN=16)  :: MTHCOM     ! command for method definition 
  CHARACTER(LEN=16)  :: DATCOM     ! command for data definition 
  CHARACTER(LEN=16)  :: CNTCOM     ! command for continuum definition 
  CHARACTER(LEN=16)  :: SPCCOM     ! command for spectrum parameter
  CHARACTER(LEN=128) :: DATFIL     ! file for data definition 
  CHARACTER(LEN=25)  :: GAPAR      ! name of genetic algorithm parameter

  CHARACTER(LEN=128) :: FITFIL     ! file for output of final fit 
  CHARACTER(LEN=128) :: NRMFIL     ! file for output of normalized data 
  CHARACTER(LEN=128) :: PARFIL     ! file for output of final solution 

  CHARACTER(LEN=16)  :: MACRO_N     ! name for macro

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance
  INTEGER   :: NC                ! loop counter for wavebands
  INTEGER   :: NM                ! loop counter for wavelength masks

  !  Model grid
  INTEGER           :: MODMODE   ! format for models
  CHARACTER(LEN=128) :: GRDFILE   ! Holder for binary model grid
  REAL              :: MOD_DW = 0 ! model resolution (delta w) 
  REAL              :: OBS_DW = 0 ! data resolution (delta w) 


  !  Fit method
  CHARACTER(LEN=10) :: METHOD = 'formal'

  !  Fit coefficients
  INTEGER, PARAMETER :: NPAR = 13  ! Maxumum number of parameters in fit   
  INTEGER            :: NP = NPAR   ! Number of parameters in chi-squared fit
  REAL, DIMENSION(NPAR) :: PARS  = 0. ! Parameters passed to chi-squared solver
  REAL, DIMENSION(NPAR) :: PARS0 = 0. ! Starting values saved...
  REAL, DIMENSION(NPAR) :: PARS1 = 0. ! Last values saved...
  REAL, DIMENSION(NPAR) :: DPARS = 0. ! Relative differences
  REAL, DIMENSION(NPAR) :: DERIVS = 0. ! Derivatives of flux wrt PARS
  REAL, DIMENSION(NPAR) :: DELTA_P = 0. !  variational amounts
  REAL      :: DPSUM             ! Sum ( DPARS )
  CHARACTER(LEN=25) :: CPARS(NPAR)    ! Descriptors for free parameters

  ! holders for various fluxes from fit
  REAL :: TMP__F  ! temporary holder for total flux
  REAL :: TMP__C  ! total continuum
  REAL :: TMP__C1 ! star 1 continuum
  REAL :: TMP__C2 ! star 2 continuum

  !  Chi-squared minimisation variables.
  INTEGER, PARAMETER :: NCHM = NPAR ! Maximum number of parameters in fit 
  REAL      :: ALPHA(NCHM,NCHM) = 0.  ! curvature array
  REAL :: COVAR(NCHM,NCHM) = 0.  ! Array of covariances
  REAL :: SCOVAR(NAMAX,NAMAX) ! Array of covariances
  REAL      :: CHISQ, CHISQ_0    ! Chi-squared residual (and saved value)
  REAL      :: ALAMDA            ! chi-squared search radius
  REAL      :: DPAR              ! sum of changes to fit parameters
  REAL      :: DCONV = 1.0E-03   ! Convergence criterion


  !  Chi-squared surface search
  REAL, DIMENSION(NTMAX,NGMAX,NHMAX) :: CHSQ_TEST

  INTEGER, PARAMETER      :: NTGRID = 51
  INTEGER, PARAMETER      :: NEGRID = 11
  REAL, DIMENSION(NTGRID) :: TEFF_TEST
  REAL, DIMENSION(NEGRID) :: EBV_TEST


  !  Amoeba control variables
  INTEGER, PARAMETER  :: NAMBD = NPAR + 1 ! Logical dimension of AMOEBA
  INTEGER   :: MP = NPAR + 1        ! number of AMOEBA corners (= NP + 1)
  REAL      :: AMB__DEF = 1.0E-03   ! goodness of fit tolerance (default)
  REAL      :: AMB__TOL = 1.0E-03   ! goodness of fit tolerance
  REAL, ALLOCATABLE :: AMB__P(:,:)  ! function parameters at AMOEBA corners (mp,np)
  REAL, ALLOCATABLE :: AMB__X(:)    ! function parameters passed to FUNK (np)
  REAL, ALLOCATABLE :: AMB__Y(:)    ! function values at vertices (np)
  INTEGER   :: AMB__ITER            ! number of AMOEBA iterations

  !  Levenburg-Marqaurdt control variables
  REAL      :: LMQ__DEF = 1.0E-03   ! goodness of fit tolerance (default)
  REAL      :: LMQ__TOL = 1.0E-03   ! goodness of fit tolerance

  !  GA control variables
  INTEGER   :: GA__MAXGEN = 10    ! maximum number of generation iterations
  INTEGER   :: GA__POPS = 50      ! size of population
  INTEGER   :: GA__STRING = 10    ! length of parent string
  INTEGER   :: GA__NPVAR = 0      ! number of parameters being varied
  REAL      :: GA__CROSS = 0.8    ! probability of crossover
  REAL      :: GA__MUT = 0.02     ! probability of mutation 
  REAL      :: GA__TOL = 1.0      ! accuracy of chi-squared test
  INTEGER   :: GA__CON = 10       ! number of contestents in tournament selection
  INTEGER   :: GA__SUBSTRING = 16 ! length of substring encoding one parameter
  INTEGER   :: GA__ALPHA = 3      ! size of alphabet used
  INTEGER   :: GA__KEPT = 3       ! the best GA__KEPT individuals are copied next generation
  CHARACTER(LEN=25) :: GA__SELECT = 'rank' ! method of selection
  CHARACTER(LEN=3)  :: GA__ELITE = 'no'    ! option of keeping copies of the best 
  ! individuals from one generation to the next

  INTEGER, ALLOCATABLE :: GA__PARENT(:,:) ! array of parents for single generation
  REAL, ALLOCATABLE :: GA__SUIT(:)     ! suitability values for single generation
  INTEGER, DIMENSION(NPAR):: GA__NPLOC ! location of variable parameters   
  INTEGER, DIMENSION(1) :: SEED
  REAL, ALLOCATABLE :: BITVAL(:) ! scaling factor between 0 and 1 determing parameter valueneration



  !  Wavelength region definition
  REAL      :: WOBS1 = 0.
  REAL      :: WOBS2 = 0.        ! wavelength range to fit
  INTEGER   :: NOBS1, NOBS2      ! pointers to above
  REAL      :: FWHM = 0.0        ! FWHM for instrumental profile
  REAL      :: SSOBS = 0.01      ! Standard deviation of data


  !  Wavelength mask regions
  INTEGER   :: NMASK = 0         ! number of mask regions
  REAL      :: WMASK(100,3)      ! start and end of region to mask and weight
  REAL      :: WLAMBDA           ! line to include
  REAL      :: WDL               ! half-width of line region to include


  !  Continuum fit variables
  !  Chi-squared minimisation control variables.
  INTEGER   :: CNT__LISTA(NPAR)     ! Regression control array
  INTEGER   :: CNT__MFIT            ! Number of coefficients in fit
  INTEGER   :: CMODE = 0            ! selects continuum fit
  REAL      :: CPARAM = 0.0         ! mode-dependent fit parameter
  INTEGER   :: CPOLY = 0            ! ordor of polynomial
  REAL      :: CPARAM2              ! 2nd parameter for special fits
  INTEGER   :: NCCF = NPAR          ! maximum order of continuum fit
  REAL, DIMENSION(NPAR) :: CCOEF    ! Continuum polynomial coefficients
  REAL, DIMENSION(NPAR) :: CCERR    ! Continuum coefficient errors
  REAL :: CCOVAR(NPAR,NPAR)
  REAL      :: CCH2                 ! chi^2 for continuum fit


  ! Cosmic ray rejection threshold
  REAL      :: CRTHSH = 1.E30    ! default cosmic ray rejection threshold


  ! Continuum mask variables
  INTEGER   :: NCREG=0          ! number of continuum mask regions
  REAL      :: CREGX(512,2)     ! start and end of each continuum mask region
  INTEGER   :: CREG(NOMAX)=1    ! wavelength regions for cont poly fit
  INTEGER   :: CMSK(NOMAX)=0    ! wavelength to exclude from continuum filter



  ! Spectrum synthesis variables 
  INTEGER   :: SYN__LSF = 0  ! length of Synfile
  ! INTEGER   :: SYN__NVAB = 0 ! Number of variable abundances  
  !REAL,    DIMENSION(NAMAX) :: VABUN ! Abundances to solve (log n + c)
  !REAL,    DIMENSION(NAMAX) :: VABU0 !
  !INTEGER, DIMENSION(NAMAX) :: VELID ! Chemical identifiers


  ! Work arrays for fluxes
  REAL :: FWRK1(NOMAX)
  REAL :: FWRK2(NOMAX)  
  REAL :: FWRK3(NOMAX)  
  REAL :: CSIG(NOMAX)   ! smmothed spectrum errors  





  !------------------------------------------------------------------------------


  CALL SFIT_ANNOUNCE ( PROG__ID, PROG__VERS, PROG__DATE, PROG__URL  )

  CALL SFIT_STARTUP



  GLOBAL: DO 

     !  Get the next command
     CALL KEY_GET ( KEYWRD, COMMAND, 'sfit>', ';', NREAD, NECHO, IOSTAT )
     CALL STR_LCASE ( KEYWRD )


     !  Exit condition
     GROUP: IF ( KEYWRD(1:3) == 'end' ) THEN ; EXIT
     ELSEIF ( KEYWRD(1:4)=='exit' ) THEN ; EXIT
     ELSEIF ( KEYWRD(1:4)=='quit' ) THEN ; EXIT


     ! ** Read the model spectra **

     !  Read Model spectrum grid - model  by model
     ELSEIF ( KEYWRD(1:6)=='models' ) THEN 

        !  decode grid type 
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PI ( KEYWRD, MODMODE )
        !  grid title
        READ  (*,'(A)') MODTITLE
        WRITE (*,*) 'Grid 1:', modmode, modtitle
        !  read grid
        CALL SFIT_RDGRID (MODMODE, &
             NWMAX,NTMAX,NGMAX,NHMAX, &
             NWMOD,NTMOD,NGMOD,NHMOD, &
             WMOD, TMOD, GMOD, HMOD, &
             FMOD, CMOD )

        MOD_DW = (WMOD(NWMOD) - WMOD(1)) / FLOAT(NWMOD - 1)

     !  Save model grids
     ELSEIF (KEYWRD(1:9)=='save_grid') THEN 

        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PC  ( KEYWRD, GRDFILE )
        CALL SFIT_WRBIN ( GRDFILE, & 
             NWMAX,NTMAX,NGMAX,NHMAX, &
             NWMOD,NTMOD,NGMOD,NHMOD, &
             WMOD, TMOD, GMOD, HMOD, &
             FMOD, CMOD )

     !  Read model grids
     ELSEIF (KEYWRD(1:9)=='read_grid') THEN 
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PC  ( KEYWRD, GRDFILE )
        CALL SFIT_RDBIN ( GRDFILE, & 
             NWMAX,NTMAX,NGMAX,NHMAX, &
             NWMOD,NTMOD,NGMOD,NHMOD, &
             WMOD, TMOD, GMOD, HMOD, &
             FMOD, CMOD )
        MOD_DW = (WMOD(NWMOD) - WMOD(1)) / FLOAT(NWMOD - 1)


     ! Make the model grid in tetrahedralized form
     ELSEIF ( KEYWRD(1:9)=='make_grid' ) THEN
        !  decode grid type
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PI ( KEYWRD, MODMODE )
        !  grid title
        READ  (*,'(A)') GRDFILE
        READ  (*,'(A)') MODTITLE
        WRITE (*,*) 'Grid 1:', modmode, modtitle, grdfile
        !  read grid
        print *, "Calling sfit_makegrid... ", GRDFILE
        CALL SFIT_MAKEGRID (MODMODE, GRDFILE, NWMAX, NWMOD, WMOD)


     !  Load model grid in tetrahedralized form
     ELSEIF (KEYWRD(1:9)=='load_grid') THEN
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PC  ( KEYWRD, GRDFILE )
        print *, "Loading grid: ", GRDFILE
        CALL SFIT_LOADGRID ( GRDFILE, &
             NWMAX,NTMAX,NGMAX,NHMAX, &
             NWMOD,NTMOD,NGMOD,NHMOD, &
             WMOD, TMOD, GMOD, HMOD, &
             FMOD, CMOD )
        MOD_DW = (WMOD(NWMOD) - WMOD(1)) / FLOAT(NWMOD - 1)
        write (*,*) grdfile

        ! Lets mod_model_read know what to read
        FBIN (1,1,1,1) = 2.0
        CBIN (1,1,1,1) = 3.0


     !  **  Second Model spectrum grid  **

     ! Build grid from individual files
     ELSEIF (KEYWRD(1:6)=='model2') THEN 
        !  decode grid type 
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PI  ( KEYWRD, MODMODE )
        !  grid title
        READ  (*,'(A)') MODTITLE
        WRITE (*,*) 'Grid 2:', modmode, modtitle
        !  read grid
        CALL SFIT_RDGRID (MODMODE, &
             NWMAX2,NTMAX2,NGMAX2,NHMAX2, &
             NWMOD2,NTMOD2,NGMOD2,NHMOD2, &
             WMOD2, TMOD2, GMOD2, HMOD2, &
             FMOD2, CMOD2 )

     !  Save model grid 2 (bin for binary companion ))
     ELSEIF (KEYWRD(1:9)=='save_grd2') THEN 
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PC  ( KEYWRD, GRDFILE )
        CALL SFIT_WRBIN ( GRDFILE, & 
             NWMAX2, NTMAX2, NGMAX2, NHMAX2,  &
             NWMOD2,NTMOD2,NGMOD2,NHMOD2, &
             WMOD2, TMOD2, GMOD2, HMOD2, &
             FMOD2, CMOD2 )

     !  Read model grid 2
     ELSEIF (KEYWRD(1:9)=='read_grd2') THEN 
        CALL KEY_GET ( KEYWRD, COMMAND, 'model>', ';', NREAD, NECHO,IOSTAT )
        CALL KEY_PC  ( KEYWRD, GRDFILE )
        CALL SFIT_RDBIN ( GRDFILE, & 
             NWMAX2, NTMAX2, NGMAX2, NHMAX2,  &
             NWMOD2,NTMOD2,NGMOD2,NHMOD2, &
             WMOD2, TMOD2, GMOD2, HMOD2, &
             FMOD2, CMOD2 )

     !  Convert helium fraction to log abundances scale
     ELSEIF (KEYWRD(1:11)=='grid_log_he') THEN 
        WHERE ( HMOD /= 0.) 
           HMOD = LOG10 ( HMOD / ( 1. - HMOD ) )
        END WHERE
     ELSEIF (KEYWRD(1:12)=='grid_log_he2') THEN 
        WHERE ( HMOD2 /= 0.) 
           HMOD2 = LOG10 ( HMOD2 / ( 1. - HMOD2 ) )
        END WHERE


     ! ** Define Sterne model atmosphere (for SYNTH) **

     ELSEIF (KEYWRD(1:6)=='sterne') THEN
        CALL KEY_PC  ( COMMAND, SYN__FILE )
        CALL STN_VERIFY ( SYN__FILE )

     ! ** Define atomic linelist (for SYNTH) **

     ELSEIF (KEYWRD(1:8)=='linelist') THEN
        CALL KEY_PC  ( COMMAND, SYN__LINES )
        CALL STN_VERIFY ( SYN__LINES )


     !  ** Define some options **

     !   solution method
     ELSEIF (KEYWRD(1:13)=='method_formal') THEN ; METHOD = 'formal'
     ELSEIF (KEYWRD(1:12)=='method_chisq') THEN ; METHOD = 'chi-squared'
     ELSEIF (KEYWRD(1:12)=='method_leven') THEN ; METHOD = 'levenburg'
     ELSEIF (KEYWRD(1:13)=='method_amoeba') THEN ; METHOD = 'amoeba'
     ELSEIF (KEYWRD(1:14)=='method_genetic') THEN ; METHOD = 'genetic'

     !  wavelength range for solution
     ELSEIF (KEYWRD(1:12)=='method_range') THEN
        CALL KEY_PR ( COMMAND, WOBS1 )
        CALL KEY_PR ( COMMAND, WOBS2 )

     !  tolerance
     ELSEIF (KEYWRD(1:10)=='method_tol') THEN
        CALL KEY_PR ( COMMAND, AMB__TOL )
        IF (AMB__TOL<1.E-7) AMB__TOL = AMB__DEF
        LMQ__TOL = AMB__TOL
        IF (LMQ__TOL<1.E-7) LMQ__TOL = LMQ__DEF

     !  mask intervals
     ELSEIF (KEYWRD(1:10)=='data_masks' .OR. KEYWRD(1:13)=='data_mask_set' .OR. KEYWRD(1:3)=='dms') THEN
        NMASK = NMASK + 1
        CALL KEY_PR  ( COMMAND, WMASK(NMASK,1) )
        CALL KEY_PR  ( COMMAND, WMASK(NMASK,2) )
        CALL KEY_PR  ( COMMAND, WMASK(NMASK,3) )
        IF ( ABS(WMASK(NMASK,1)+WMASK(NMASK,2)) < 1 ) THEN
           NMASK = 0
        ENDIF

     ! line masks 
     ELSEIF (KEYWRD(1:10)=='data_lines' .OR. KEYWRD(1:13)=='data_line_set' .OR. KEYWRD(1:3)=='dls') THEN
        NMASK = NMASK + 1
        CALL KEY_PR  ( COMMAND, WLAMBDA )
        CALL KEY_PR  ( COMMAND, WDL )
        WMASK(NMASK,1) = WLAMBDA-WDL
        WMASK(NMASK,2) = WLAMBDA+WDL
        WMASK(NMASK,3) = SSOBS / 100.D0
        IF ( WLAMBDA < 1 ) THEN
           NMASK = 0
        ENDIF

     ! display continuum regions
     ELSEIF (KEYWRD(1:10)=='data_maskd' .OR. KEYWRD(1:14)=='data_mask_disp' .OR. KEYWRD(1:3)=='dmd') THEN
        WRITE (*,*) ' Mask regions: '
        WRITE (*,'(I3,3F8.2)') (NM, WMASK(NM, 1:3),NM=1,NMASK)

     ! clear continuum regions
     ELSEIF (KEYWRD(1:15)=='data_mask_clear' .OR. KEYWRD(1:3)=='dmc') THEN
        NMASK = 0
        WRITE (*,*) ' Mask regions cleared '


     !  continuum fitting method
     ! Polynomial fitting (get order)
     ELSEIF (KEYWRD(1:9)=='cont_poly') THEN
        CMODE = 2
        CALL KEY_PI (COMMAND, CPOLY)
        CPOLY = CPOLY + 1

     ! Gaussian smoothing (get FWHM)
     ELSEIF (KEYWRD(1:9)=='cont_filt') THEN
        CMODE = 1
        CALL KEY_PR (COMMAND, CPARAM)

     ! Double pass filter (get FWHM for low and high-pass filters ))
     ELSEIF (KEYWRD(1:9)=='cont_spec') THEN
        CMODE = 3
        CALL KEY_PR (COMMAND, CPARAM)
        CALL KEY_PR (COMMAND, CPARAM2)
        IF (NINT(CPARAM2) == 0) CMODE = 1

     ! define individual continuum region
     ELSEIF (KEYWRD(1:9)=='cont_regs' .OR. KEYWRD(1:12)=='cont_reg_set' .OR. KEYWRD(1:3)=='crs') THEN
        NCREG=NCREG+1
        CALL KEY_PR (COMMAND, CREGX(NCREG,1) )
        CALL KEY_PR (COMMAND, CREGX(NCREG,2) )
        IF ( ABS(CREGX(NCREG,1)+CREGX(NCREG,2)) < 1 ) THEN
           NCREG = 0
        ENDIF

     ! display continuum regions
     ELSEIF (KEYWRD(1:9)=='cont_regd' .OR. KEYWRD(1:13)=='cont_reg_disp' .OR. KEYWRD(1:3)=='crd') THEN
        WRITE (*,*) ' Continuum regions: '
        WRITE (*,'(I3,2F8.2)') (NM, CREGX(NM, 1:2),NM=1,NCREG)

     ! display continuum regions
     ELSEIF (KEYWRD(1:14)=='cont_reg_clear' .OR. KEYWRD(1:3)=='crc') THEN
        NCREG = 0
        WRITE (*,*) ' Continuum regions cleared '

     ! ** parameters defining the solution for SOLVE **

     !  Initialize parameters 
     ELSEIF (KEYWRD(1:9)=='slv_teff1') THEN ; CALL SFIT_SLV_GET ( 1 ) 
     ELSEIF (KEYWRD(1:9)=='slv_logg1') THEN ; CALL SFIT_SLV_GET ( 2 ) 
     ELSEIF (KEYWRD(1:8)=='slv_ahe1') THEN ; CALL SFIT_SLV_GET ( 3 ) 
     ELSEIF (KEYWRD(1:10)=='slv_vsini1') THEN ; CALL SFIT_SLV_GET ( 4 ) 
     ELSEIF (KEYWRD(1:8)=='slv_rad1') THEN ; CALL SFIT_SLV_GET ( 5 ) 
     ELSEIF (KEYWRD(1:9)=='slv_vrad1') THEN ; CALL SFIT_SLV_GET ( 6 ) 

     ELSEIF (KEYWRD(1:9)=='slv_teff2') THEN ; CALL SFIT_SLV_GET (  7 ) 
     ELSEIF (KEYWRD(1:9)=='slv_logg2') THEN ; CALL SFIT_SLV_GET (  8 ) 
     ELSEIF (KEYWRD(1:8)=='slv_ahe2') THEN ; CALL SFIT_SLV_GET (  9 ) 
     ELSEIF (KEYWRD(1:10)=='slv_vsini2') THEN ; CALL SFIT_SLV_GET ( 10 ) 
     ELSEIF (KEYWRD(1:8)=='slv_rad2') THEN ; CALL SFIT_SLV_GET ( 11 ) 
     ELSEIF (KEYWRD(1:9)=='slv_vrad2') THEN ; CALL SFIT_SLV_GET ( 12 ) 

     ELSEIF (KEYWRD(1:7)=='slv_ebv') THEN ; CALL SFIT_SLV_GET ( 13 ) 

     !  Fix parameters 
     ELSEIF (KEYWRD(1:9)=='fix_teff1') THEN ; CALL SFIT_SLV_FIX ( 1 ) 
     ELSEIF (KEYWRD(1:9)=='fix_logg1') THEN ; CALL SFIT_SLV_FIX ( 2 ) 
     ELSEIF (KEYWRD(1:8)=='fix_ahe1') THEN ; CALL SFIT_SLV_FIX ( 3 ) 
     ELSEIF (KEYWRD(1:10)=='fix_vsini1') THEN ; CALL SFIT_SLV_FIX ( 4 ) 
     ELSEIF (KEYWRD(1:8)=='fix_rad1') THEN ; CALL SFIT_SLV_FIX ( 5 ) 
     ELSEIF (KEYWRD(1:9)=='fix_vrad1') THEN ; CALL SFIT_SLV_FIX ( 6 ) 

     ELSEIF (KEYWRD(1:9)=='fix_teff2') THEN ; CALL SFIT_SLV_FIX (  7 ) 
     ELSEIF (KEYWRD(1:9)=='fix_logg2') THEN ; CALL SFIT_SLV_FIX (  8 ) 
     ELSEIF (KEYWRD(1:8)=='fix_ahe2') THEN ; CALL SFIT_SLV_FIX (  9 ) 
     ELSEIF (KEYWRD(1:10)=='fix_vsini2') THEN ; CALL SFIT_SLV_FIX ( 10 ) 
     ELSEIF (KEYWRD(1:8)=='fix_rad2') THEN ; CALL SFIT_SLV_FIX ( 11 ) 
     ELSEIF (KEYWRD(1:9)=='fix_vrad2') THEN ; CALL SFIT_SLV_FIX ( 12 ) 

     ELSEIF (KEYWRD(1:7)=='fix_ebv') THEN ; CALL SFIT_SLV_FIX ( 13 ) 

     !  ** variables requied in the solutiuon for SYNTH **
     
     ELSEIF (KEYWRD(1:9)=='syn_vturb') THEN
        CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )
        IF ( NARGS == 0 ) CALL STR_ERR ( 'syn_vturb',' missing argument ' )
        IF ( NARGS > 0  ) CALL KEY_PR ( COMMAND, SYN__VTURB )
        SYN__ABUN(SYN__NPAR-2) = SYN__VTURB 
        IF ( NARGS > 1  ) CALL KEY_PR ( COMMAND, SYN__DABU(SYN__NPAR-2) )

     ELSEIF (KEYWRD(1:9)=='syn_vsini') THEN
        CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )
        IF ( NARGS == 0 ) CALL STR_ERR ( 'syn_vsini',' missing argument ' )
        IF ( NARGS > 0  ) CALL KEY_PR ( COMMAND, SYN__VSINI )
        SYN__ABUN(SYN__NPAR-1) = SYN__VSINI 
        IF ( NARGS > 1  ) CALL KEY_PR ( COMMAND, SYN__DABU(SYN__NPAR-1) )

     ELSEIF (KEYWRD(1:8)=='syn_vrad') THEN
        CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )
        IF ( NARGS == 0 ) CALL STR_ERR ( 'syn_vrad',' missing argument ' )
        IF ( NARGS > 0  ) CALL KEY_PR ( COMMAND, SYN__VRAD )
        SYN__ABUN(SYN__NPAR) = SYN__VRAD 
        IF ( NARGS > 1  ) CALL KEY_PR ( COMMAND, SYN__DABU(SYN__NPAR) )

     ELSEIF (KEYWRD(1:8)=='syn_beta') THEN
        CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )
        IF ( NARGS == 0 ) CALL STR_ERR ( 'syn_beta',' missing argument ' )
        IF ( NARGS > 0  ) CALL KEY_PR ( COMMAND, SYN__BETA )

     ! abundances 
     ELSEIF (KEYWRD(1:9)=='abundance') THEN
        CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )

        ! check syntax
        IF ( NARGS < 3 ) THEN
           CALL STR_ERR  ( 'abundance',' missing argument(s) ' )
           CALL STR_TEXT ( ' Use: abundance <z> <ab> <d_ab>  ' )

           ! translate arguments
        ELSE
           CALL KEY_PI ( COMMAND, I )
           CALL KEY_PR ( COMMAND, X )
           CALL KEY_PR ( COMMAND, Y )
           ! locate array index corresponding to element identifier
           J = NAMAX   !  dummy value if rubbish entered
           DO K = 1,SYN__NPAR-1
              IF ( I == SYN__ELID(K) ) J = K
           ENDDO
        ENDIF
        IF ( X > 0.001 ) THEN
           SYN__ABUN(J) = X
           SYN__DABU(J) = Y
        ELSE 
           ! do not change currrent abundance
           SYN__DABU(J) = Y
        ENDIF

        WRITE (LOGFILE,*) 'SYNTH: Number of abundances to fit: ', SYN__NPAR
        WRITE (LOGFILE,*) 'Initial values for abundances and vt'
        WRITE (LOGFILE,*) (SYN__ELID(N),N=1,SYN__NPAR)
        WRITE (LOGFILE,*) (SYN__ABUN(N),N=1,SYN__NPAR)
        WRITE (LOGFILE,*) (SYN__DABU(N),N=1,SYN__NPAR)


     ! **  Data defining observations  **

     ! Read in spectrum -- spectrum2 format
     ELSEIF (KEYWRD(1:13)=='data_spectrum') THEN
        CALL KEY_GET ( DATFIL, COMMAND, 'spectrum>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PC ( DATFIL, OBSFILE )
        CALL SFIT_RDOBS (2)
        ! default units
        WOBS_UNITS = 'Angstroms'
        FOBS_UNITS = 'normalized'
        FOBS_SCALE = 1.0
        FOBS_NORM = .TRUE.

     ! Read in spectrum -- pamela format
     ELSEIF (KEYWRD(1:11)=='data_pamela') THEN
        CALL KEY_GET ( DATFIL, COMMAND, 'pamela>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PC ( DATFIL, OBSFILE )
        CALL SFIT_RDOBS (1)
        ! default units
        WOBS_UNITS = 'Angstroms'
        FOBS_UNITS = 'normalized'
        FOBS_SCALE = 1.0
        FOBS_NORM = .TRUE.

     ! Read instrumental profile
     ELSEIF (KEYWRD(1:9)=='data_inst') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_inst>',  ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PR ( DATCOM, FWHM )
        SINST = FWHM / 2.354

     ! Read cosmic ray threshold 
     ELSEIF (KEYWRD(1:11)=='data_cosmic') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_cosmic>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PR ( DATCOM, CRTHRSH )
        ! reset default value
        IF ( CRTHRSH < 1.E-20 ) CRTHRSH = 1.E30

     ! Read velocity broadening for this spectrum
     ELSEIF  (KEYWRD(1:10)=='data_drift') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_drift>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PR ( DATCOM, DVSYSKM )
        IF (NWOBS > 1) DVSYS = DVSYSKM * (WOBS(1)+WOBS(NWOBS)) * 1.E3 / 2. / TAPP__C

     ! Read average standard deviation  for this spectrum
     ELSEIF (KEYWRD(1:10)=='data_sigma') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_sigma>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PR ( DATCOM, SSOBS )
        IF (NWOBS > 0) SOBS(1:NWOBS) = SSOBS

     ! Read data units for this spectrum (default is 'normalized')
     ELSEIF (KEYWRD(1:15)=='data_flux_units') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_flux_units>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PC ( DATCOM, FOBS_UNITS )
        IF ( FOBS_UNITS(1:4)/='norm' ) FOBS_NORM=.FALSE.

     ! Read scaling factor for oberved spectrum (default is 1.0)
     ELSEIF (KEYWRD(1:15)=='data_flux_scale') THEN
        CALL KEY_GET ( DATCOM, COMMAND, 'data_flux_scale>', ';', NREAD, NECHO, IOSTAT )
        CALL KEY_PR ( DATCOM, FOBS_SCALE )


        !  ** Define options for SPECTRUM **

     ELSEIF (KEYWRD(1:7)=='spc_par') THEN
        WRITE (*,*) ' SPC_PAR obsolete -- use spc_nmode, spc_ntheta, etc... '
        EXIT

     ELSEIF (KEYWRD(1: 9)=='spc_nmode')    THEN ; CALL KEY_PI (COMMAND, SPC__NMODE)
     ELSEIF (KEYWRD(1:10)=='spc_ntheta')   THEN ; CALL KEY_PI (COMMAND, SPC__NTHETA)
     ELSEIF (KEYWRD(1:10)=='spc_noptyp')   THEN ; CALL KEY_PI (COMMAND, SPC__NOPAC)
     ELSEIF (KEYWRD(1:11)=='spc_nsource')  THEN ; CALL KEY_PI (COMMAND, SPC__NSOURCE)
     ELSEIF (KEYWRD(1:10)=='spc_nhedat')   THEN ; CALL KEY_PI (COMMAND, SPC__NHEDAT)
     ELSEIF (KEYWRD(1:12)=='spc_nhestark') THEN ; CALL KEY_PI (COMMAND, SPC__NHESTARK)


        ! Read in GA parameters

     ELSEIF  (KEYWRD(1:7)=='gen_par') THEN

        GAPAR(1:3) = '   '
        DO WHILE ( GAPAR(1:3)/='end' .AND. GAPAR(1:1)/='}')

           CALL KEY_GET (GAPAR, COMMAND, 'gen>', ';', NREAD, NECHO, IOSTAT )
           CALL STR_LCASE (GAPAR)    

           WRITE (*,*) GAPAR, COMMAND(1:30) 

           IF (GAPAR(1:11)=='generations') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)         
              CALL KEY_PI (GAPAR, GA__MAXGEN)

           ELSEIF (GAPAR(1:10)=='population') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)         
              CALL KEY_PI (GAPAR, GA__POPS) 

           ELSEIF (GAPAR(1:10)=='prob_cross') THEN
              CALL KEY_GET (GAPAR, COMMAND, '>',  ';', NREAD, 0, IOSTAT)
              CALL KEY_PR (GAPAR, GA__CROSS)

           ELSEIF (GAPAR(1:10)=='prob_mut') THEN
              CALL KEY_GET (GAPAR, COMMAND, '>',  ';', NREAD, 0, IOSTAT)
              CALL KEY_PR (GAPAR, GA__MUT)

           ELSEIF (GAPAR(1:9)=='tolerance') THEN
              CALL KEY_GET (GAPAR, COMMAND, '>',  ';', NREAD, 0, IOSTAT)
              CALL KEY_PR (GAPAR, GA__TOL)

           ELSEIF (GAPAR(1:11)=='contestants') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PI (GAPAR, GA__CON)

           ELSEIF (GAPAR(1:16)=='substring_length') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PI (GAPAR, GA__SUBSTRING)

           ELSEIF (GAPAR(1:13)=='alphabet_size') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PI (GAPAR, GA__ALPHA) 

           ELSEIF (GAPAR(1:16)=='selection_method') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PC (GAPAR, GA__SELECT) 

           ELSEIF (GAPAR(1:7)=='elitism') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PC (GAPAR, GA__ELITE)   

           ELSEIF (GAPAR(1:4)=='kept') THEN
              CALL KEY_GET (GAPAR, COMMAND,  '>', ';', NREAD, 0, IOSTAT)
              CALL KEY_PI (GAPAR, GA__KEPT) 

           ENDIF
        ENDDO




        !  Normalise the data to starting model before starting solution

     ELSEIF (KEYWRD(1:6)=='normal') THEN

        ! Prepare data for solution
        CALL SFIT_PREPARE

        !      Evaluate fit for current parameters
        WRITE (*,*) 'Computing formal solution'
        WRITE (*,'(A)') '  Teff_1  lg_g1  nHe_1 vsini1  lg_r1  dv_1 &
             &  Teff_2  lg_g2  nHe_2 vsini2  lg_r2  dv_2  E(B-V)'
        WRITE (*,'(2(1X,0P,2F7.2,F7.3,2F7.2,F7.1),F8.2)')   PARS
        DO NW = 1,NWOBS
           CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
        END DO

        !      Normalize the data to the continuum
        WRITE (*,*) 'Normalising data using method ', CMODE
        CALL SFIT_NORMALIZE

        FOBS_UNITS = 'normalized'
        FOBS_SCALE = 1.0
        FOBS_NORM = .TRUE.



      ! Find velocity shift between observation and model

     ELSEIF (KEYWRD(1:5)=='vcorr') THEN

        ! Prepare data for solution
        CALL SFIT_PREPARE

        !      Evaluate fit for current parameters, except for velocities
        VRAD1 = PARS(6); PARS(6) = 0.
        DO NW = 1,NWOBS
           CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
        END DO
        PARS(6) = VRAD1

        CALL SFIT_VCORR


     !  Compute standard deviation from current fit and continuum regions 

     ELSEIF (KEYWRD(1:10)=='cont_sigma') THEN

        !   Check continuum regions defined
        IF ( NCREG > 0 .AND. NTMOD > 0 ) THEN

           !  Prepare data for solution
           CALL SFIT_PREPARE

           !  Evaluate fit for current parameters
           DO NW = 1,NWOBS
              CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
           END DO

           !  Normalize the continuum
           CALL SFIT_C_SIGMA

        ELSEIF ( NTMOD <= 0 ) THEN

           !  Assume data is correctly normalized
           FFIT(1:NW) = 1.0

           !  Normalize the continuum
           CALL SFIT_C_SIGMA

        ELSE

           !  Error -- no continuum defined
           WRITE (*,*) 'calc_sigma: no continuum regions defined'

        END IF


        !  ** Seek best  solution in model grid (SOLVE) **

     ELSEIF (KEYWRD(1:5)=='solve') THEN

        ! Prepare data for solution
        CALL SFIT_PREPARE

        !  Scale data
        FOBS = FOBS * FOBS_SCALE
        SOBS = SOBS * FOBS_SCALE

        ! Solve using multidimensional downhill simplex method (AMOEBA)
        SOLVE_METHOD: IF (METHOD(1:6)=='formal') THEN
           WRITE (*,*) 'You are just computing a formal solution (default)'
           CALL SFIT_FORMAL

           ! Solve using multidimensional downhill simplex method (AMOEBA)
        ELSEIF (METHOD(1:6)=='chi-sq') THEN
           WRITE (*,*) 'You are constructing the chi-squared surface'
           CALL SFIT_CHISQ

           ! Solve using multidimensional downhill simplex method (AMOEBA)
        ELSEIF (METHOD(1:6)=='amoeba') THEN
           WRITE (*,*) 'You are using amoeba!'
           WRITE (*,*) '** warning ** amoeba forces initial values to be inside model grid'
           WRITE (*,*) '* 2013.2.28 * but does NOT prevent extrapolation outside grid boundaries.'
           WRITE (*,*) '              YOU are responsible for checking your results are plausible.'
           CALL SFIT_AMOEBA

           ! Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)
        ELSEIF (METHOD(1:9)=='levenburg') THEN
           WRITE (*,*) 'You are using the Levenburg-Marquardt procedure'
           CALL SFIT_LEVENBURG

           ! Solve using GA
        ELSEIF (METHOD(1:7)=='genetic') THEN
           WRITE (*,*) 'You are using the genetic algorithm'
           CALL SFIT_GENETIC


        ENDIF SOLVE_METHOD

        !  Compute the formal solutions
        ! this section of code replace below, saved in case of failure.
        DERIVS(1) = -99.  ! swithc off derivatives
        FITDATA__SAVE_FIT = .TRUE. ! save the fit components
        DO NW = 1,NWOBS
           CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
        END DO
        FITDATA__SAVE_FIT = .FALSE. ! and do not ovwerwrite them
        R1 = PARS(5)
        R2 = PARS(11)
        PARS(5)  =   0.
        PARS(11) = -99.
        DO NW = 1,NWOBS
           CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FIT1(NW),DERIVS,NP)
        END DO
        PARS(5) = -99.
        PARS(11) =  0.
        DO NW = 1,NWOBS
           CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FIT2(NW),DERIVS,NP)
        END DO
        PARS(5)  = R1
        PARS(11) = R2

        !  compute formal solutions directly
        !      DO NW = 1,NWOBS
        !        CALL SPEC_TGHV_FLX (WOBS(NW),PARS,FFIT(NW),TMP__F,TMP__C,FIT1(NW),TMP__C1,FIT2(NW),TMP__C2,NP)
        !        IF (FOBS_NORM) THEN
        !          FIT1(NW) = FIT1(NW) / TMP__C
        !          FIT2(NW) = FIT2(NW) / TMP__C
        !        ENDIF
        !      END DO



        !  Rescale data and outputs
        FOBS = FOBS / FOBS_SCALE
        SOBS = SOBS / FOBS_SCALE
        FFIT = FFIT / FOBS_SCALE
        FIT1 = FIT1 / FOBS_SCALE
        FIT2 = FIT2 / FOBS_SCALE

        !  Write result and fits to file

        FITFIL = TRIM(OBSFILE)//'.fit'
        OPEN  (1,FILE=FITFIL,STATUS='UNKNOWN')
        WRITE (1,'(A)') TRIM(OBSTITLE)//': SFIT2 model fit (solve)'
        WRITE (1,'(2A)') '  W_obs      F_obs       s_obs         A_tot         A_1         A_2', &
          '     C_nu,tot    C_nu,1     C_nu,2     Creg  '  
        WRITE (1,*) NWOBS
        WRITE (1,'(0P,F10.3,1P,2E12.3,2x,3E12.3,2x,3E12.3,2x,i3)') &
             ( WOBS(NW),FOBS(NW),SOBS(NW),FFIT(NW),FIT1(NW),FIT2(NW), & 
             FITDATA__CTOT(NW),FITDATA__CNU1(NW),FITDATA__CNU2(NW),CREG(NW),NW=1,NWOBS )
        CLOSE (1)
        WRITE (*,'(2A)') 'Renormalized spectrum and fits written to ',FITFIL


        !      Write results to reporting file
        PARFIL = TRIM(OBSFILE)//'.slv_pars'
        OPEN (1, FILE=PARFIL,STATUS='UNKNOWN')
        WRITE (1,'(A)') TRIM(OBSTITLE)//': SFIT2 fit parameters (solve)'
        WRITE (1,*)
        DO I = 1,NP
           WRITE (1,'(A25,'': '',1P,E11.3,'' +/- '',E11.3)') CPARS(I),PARS(I),SQRT(COVAR(I,I))
        ENDDO
        WRITE (1,'(A25,'': '',F8.3,'' +/- '',F8.3)') 'chi^2 ', CHISQ/(NWOBS-NP)
        CLOSE (1)
        WRITE (*,'(2A)') 'Best fit parameters written to ',PARFIL




        !  ** Carry out solution for real-time synthesis (SYNTH) **)

     ELSEIF (KEYWRD(1:5)=='synth' .AND. .NOT. FOBS_NORM ) THEN

        ! Check that spectrum is normalized
        WRITE (*,*) ' SYNTH ** currently operates on a normalised spectrum only '
        WRITE (*,*)

     ELSEIF (KEYWRD(1:5)=='synth' .AND. FOBS_NORM ) THEN

        ! Prepare the data for solution
        CALL SFIT_PREPOBS

        ! Solve using multidimensional downhill simplex method (AMOEBA)
        SYNTH_METHOD: IF (METHOD(1:6)=='formal') THEN
           WRITE (*,*) 'You are computing a formal solution only (default)'
           CALL SYNTH_FORMAL

        ELSEIF (METHOD(1:6)=='amoeba') THEN
           WRITE (*,*) 'You are using amoeba!'
           CALL SYNTH_AMOEBA

           ! Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)
        ELSEIF (METHOD(1:9)=='levenburg') THEN
           WRITE (*,*) 'You are using the Levenburg-Marquardt procedure'
           CALL SYNTH_LEVENBURG

        ENDIF SYNTH_METHOD


        !     Print out the fitting function
        CALL SFIT_SVFIT 

        !      Write results to reporting file
        PARFIL = TRIM(OBSFILE)//'.syn_pars'
        OPEN (1, FILE=PARFIL,STATUS='UNKNOWN')
        WRITE (1,'(A)') TRIM(OBSTITLE)//': SFIT2 fit parameters (synth)'
        WRITE (1,*)
        DO I = 1,SYN__NPAR
           WRITE (1,'(I25,'': '',F8.3,'' +/- '',F8.3)') SYN__ELID(I),SYN__ABUN(I),SQRT(SCOVAR(I,I))
        ENDDO
        WRITE (1,'(A25,'': '',F8.3)') 'chi^2 ', CHISQ/(NWOBS-SYN__NPAR)
        CLOSE (1)
        WRITE (*,'(2A)') 'Best fit parameters written to ',PARFIL


        ! save current data

     ELSEIF (KEYWRD(1:4)=='save') THEN

        FITFIL = TRIM(OBSFILE)//'_fit.sp2'
        CALL SP2WR ( WOBS, FFIT, NWOBS,  TRIM(OBSTITLE)//': model fit', 41, FITFIL)

        NRMFIL = TRIM(OBSFILE)//'_dat.sp2'
        CALL SP2WR ( WOBS, FOBS, NWOBS,  TRIM(OBSTITLE)//': renormalized', 41, NRMFIL)



     !~~
     !
     !   The following commands deal with how the program is run, 
     !   allowing swithces to be set, macros defined and so on
     !
     !~~

     ! brief help; eventually give help on individual commands
     ELSEIF (KEYWRD(1:4)=='help') THEN ; CALL SFIT_HELP (' ')

     ! up to date information in valid modes of SFIT and date last tested
     ELSEIF (KEYWRD(1:5)=='valid') THEN ; CALL SFIT_VALID

     ! echo commands to the terminal
     ELSEIF (KEYWRD(1:4)=='echo') THEN

        CALL KEY_PC (COMMAND, PARCOM)
        IF (PARCOM(1:1) == "2" )                         NECHO = 2
        IF (PARCOM(1:2) == "on"  .OR. PARCOM(1:1)=="1" ) NECHO = 1
        IF (PARCOM(1:3) == "off" .OR. PARCOM(1:1)=="0" ) NECHO = 0

     ! set an SFIT  parameter
     ELSEIF (KEYWRD(1:3)=='set') THEN ; CALL SFIT_SET

     ! show an SFIT  parameter
     ELSEIF (KEYWRD(1:4)=='show') THEN ; CALL SFIT_SHOW

     ! record a macro
     ELSEIF (KEYWRD(1:6)=='define') THEN
        ! get the macro name and define it
        CALL KEY_PC (COMMAND, MACRO_N)
        CALL SFIT_DEF_MACRO ( MACRO_N, COMMAND, NREAD, NECHO ) 

     ! play a macro
     ELSEIF (KEYWRD(1:3)=='run') THEN
        ! get the macro name and recover it to the COMMAND line
        CALL KEY_PC  ( COMMAND, MACRO_N )
        CALL SFIT_RUN_MACRO ( MACRO_N, COMMAND, NECHO ) 

     ! unrecognised command
     ELSEIF (KEYWRD(1:1) /= ' ' .AND. KEYWRD(1:1) /= ';' &
          .AND. KEYWRD(1:1) /= '}' &
          .AND. KEYWRD(1:1) /= '{' ) THEN
        CALL STR_OUTC ( 'command unknown', KEYWRD(1:16) )

        !  No more commands
     ENDIF GROUP

  ENDDO GLOBAL




  ! Define Internal Procedures
CONTAINS



  SUBROUTINE SFIT_STARTUP

    DOUBLE PRECISION, EXTERNAL :: TAP_ABSOL

    !  Open a log file
    OPEN (LOGFILE,FILE="SFIT.LOG",STATUS="UNKNOWN")

    OPEN (SUMFILE,FILE="SFIT.SUM",STATUS="UNKNOWN")
    WRITE (SUMFILE,'(A)') & 
         'Spectrum                 &
         & Teff_1    +/-  lg g1    +/-  nHe_1   +/-  &
         & vsini1    +/-  lg r1    +/-   dv_1   +/-  &
         & chi^2 '


    ! Set up default i/o channels
    CALL STR_DEF (NREAD, NWRITE)
    CALL STR_NULL ( COMMAND ) 
    CALL STR_NULL ( KEYWRD ) 

    ! Other defaults
    OBSFOLDER(1:1) = '.'          ! default folder for data
    NWOBS   = 0
    NWMOD   = 0
    NWMOD2  = 0                   ! set 2nd grid to 0

    ! Initialize abundances to solar values
    SYN__NPAR = 50
    SYN__ELID(1:SYN__NPAR) = [ &
          1, 2, 3, 5, 6,  7, 8, 9,10,11, 12,13,14, &
         15,16,17,18,20, 21,22,23,24,25, 26, &
         27,28,29,30,31, 32,38,39,40,42, 54, 56, &
         57,58,59,60,62, 63,64,65,66,70, &
         82,    &
         0,0,0 ]
    DO I = 1,SYN__NPAR - 3
       SYN__ABUN(I) = TAP_ABSOL ( SYN__ELID(I) )
       SYN__DABU(I) = 0.0
    END DO

    ! Initialize microturbulence
    SYN__ELID(SYN__NPAR - 2) = 0
    SYN__ABUN(SYN__NPAR - 2) = 5.0
    SYN__DABU(SYN__NPAR - 2) = 0.0

    ! Dummy values for remainder of array
    SYN__ELID (SYN__NPAR-1:NAMAX) = -99
    SYN__ABUN (SYN__NPAR-1:NAMAX) = 0.0
    SYN__DABU (SYN__NPAR-1:NAMAX) = 0.0

    ! Default values for SLV parameters
    PARS  = [ 20., 3., 0.5, 0., 0., 0., 20., 3., 0.5, 0., -99., 0., 0. ]
    PARS0 = PARS
    DELTA_P = 0.
    CPARS = [CHARACTER(LEN=25) :: 'Teff 1', 'log g 1', 'nHe 1', 'vsini 1', 'log R 1', 'vrad 1', &
         'Teff 2', 'log g 2', 'nHe 2', 'vsini 2', 'log R 2', 'vrad 2', 'E(B-V)' ]

  END SUBROUTINE SFIT_STARTUP



  SUBROUTINE STN_VERIFY (STN__FILE)

    ! Open Sterne model atmosphere and report parameters

    CHARACTER(LEN=*), INTENT(IN) :: STN__FILE

    ! Parameters read from Sterne model
    REAL :: STN__TEFF  ! Teff 
    REAL :: STN__GRAV  ! Gravity 
    REAL :: STN__AHE  ! Helium abundance
    REAL :: STN__AH  ! Hydrogen abundance
    REAL :: STN__ACN  ! C+N abundance


    OPEN  (1,FILE=STN__FILE,STATUS='OLD')
    READ  (1,'(5E12.5)') STN__TEFF,STN__GRAV,STN__AHE,STN__AH,STN__ACN 
    CLOSE (1)

    WRITE (*,*) 'stn__verify:'
    WRITE (*,'(A,''  Teff='',F7.0,'' log g='',F6.2)') &
         TRIM(SYN__FILE), STN__TEFF, ALOG10(STN__GRAV)
    WRITE (*,*)

    WRITE (LOGFILE,'(''Model '',A,''  Teff='',F7.0,'' log g='',F6.2)') &
         TRIM(SYN__FILE), STN__TEFF, ALOG10(STN__GRAV)

  END SUBROUTINE STN_VERIFY



  SUBROUTINE SFIT_SLV_GET ( NPAR )

    !  Subroutine to read value and Delta_value for a 
    !  fit parameter specified by NPAR

    INTEGER :: NPAR
    INTEGER :: NARG

    CALL KEY_NARG ( COMMAND , KEYSEP , NARG, IOSTAT )

    IF ( NARG == 0 ) DELTA_P(NPAR) = ABS( PARS(NPAR) / 20.) + 0.1 
    IF ( NARG  > 0 ) CALL KEY_PR ( COMMAND, PARS( NPAR ) )
    IF ( NARG  > 1 ) CALL KEY_PR ( COMMAND, DELTA_P( NPAR ) )
    IF ( NARG == 1 .AND. ABS(DELTA_P(NPAR)) < 0.001 ) &
         DELTA_P(NPAR) = ABS( PARS(NPAR) / 20.) + 0.1 

    PARS0( NPAR ) = PARS( NPAR )

  END SUBROUTINE SFIT_SLV_GET




  SUBROUTINE SFIT_SLV_FIX ( NPAR )

    !  Subroutine to fix start value for parameter to 
    !  a) specified value
    !  b) last solution if no value give 


    INTEGER :: NPAR, NARGS

    ! determine number of arguments
    CALL KEY_NARG ( COMMAND , KEYSEP , NARGS, IOSTAT )

    ! set new value and fix it
    IF ( NARGS > 0 ) THEN
       CALL KEY_PR ( COMMAND, PARS( NPAR ) )
       DELTA_P( NPAR )  = 0.0
       PARS0( NPAR ) = PARS( NPAR )

       ! use last solution and fix it
    ELSE
       DELTA_P( NPAR )  = 0.0
       PARS0( NPAR ) = PARS( NPAR )
    END IF

  END SUBROUTINE SFIT_SLV_FIX




  !  SUBROUTINE SFIT_SLV_LAST ( NPAR )
  !
  !    !  Subroutine to fix start value for parameter to last solution 
  !    !  fit parameter specified by NPAR
  !
  !    INTEGER :: NPAR
  !
  !    CALL KEY_PR ( COMMAND, PARS( NPAR ) )
  !    DELTA_P( NPAR )  = 0.0
  !
  !    PARS0( NPAR ) = PARS( NPAR )
  !
  !  END SUBROUTINE SFIT_SLV_LAST




  SUBROUTINE SFIT_SET

    !    Subroutine to set a specific global SFIT variable
    !    Use SFIT_SHOW to show individual or all SFIT variables.

    !    28/09/04
    !    Subroutine curently superceded by commands to set specific variables

    !    Example: 
    !
    !      SET  SINST  0.5

    CHARACTER(LEN=20) :: SFIT__VAR
    REAL :: TEST

    !    

    !    Extract variable to be defined from command string
    CALL KEY_GET ( SFIT__VAR, COMMAND, 'set>', ';', NREAD, NECHO, IOSTAT )
    CALL STR_LCASE ( SFIT__VAR )


    !    Identify variable, extract value from command string and return value

    SET_VAR: IF ( SFIT__VAR(1:4)=='test' ) THEN 
       CALL KEY_PR ( COMMAND, TEST )


       !    Indicate if variable was not identified
    ELSE
       CALL STR_OUTC (' SFIT variable not known', SFIT__VAR(1:10) ) 

    END IF SET_VAR

  END SUBROUTINE SFIT_SET




  SUBROUTINE SFIT_SHOW

    !    Subroutine to show one or all global SFIT variables


    !    Example: 
    !
    !      SHOW SINST 
    !      SHOW ALL

    CHARACTER(LEN=20) :: SFIT__VAR
    REAL :: TEST

    !.    

    CALL KEY_GET ( SFIT__VAR, COMMAND, 'show>', ';', NREAD, NECHO, IOSTAT )
    CALL STR_LCASE ( SFIT__VAR )

    !    Identify variable to print and return
    SHOW_VAR: IF ( SFIT__VAR(1:4)=='test' ) THEN 
       CALL STR_OUTR ( prog__id//':  test ', TEST)

    ELSEIF ( SFIT__VAR(1:4)=='meth' ) THEN 
       CALL SFIT_SHOW_METHOD
    ELSEIF ( SFIT__VAR(1:4)=='cont' ) THEN 
       CALL SFIT_SHOW_CONTINUUM
    ELSEIF ( SFIT__VAR(1:4)=='data' ) THEN 
       CALL SFIT_SHOW_DATA
    ELSEIF ( SFIT__VAR(1:6)=='slv' ) THEN 
       CALL SFIT_SHOW_SOLVE
    ELSEIF ( SFIT__VAR(1:6)=='syn' ) THEN 
       CALL SFIT_SHOW_SYNTH
    ELSEIF ( SFIT__VAR(1:3)=='gen' ) THEN 
       CALL SFIT_SHOW_GA
    ELSEIF ( SFIT__VAR(1:5)=='macro' ) THEN 
       CALL SFIT_SHOW_MACROS

    ELSEIF ( SFIT__VAR(1:3)=='all' ) THEN 
       CALL SFIT_SHOW_METHOD
       CALL SFIT_SHOW_GA
       CALL SFIT_SHOW_DATA
       CALL SFIT_SHOW_CONTINUUM
       CALL SFIT_SHOW_SOLVE
       CALL SFIT_SHOW_SYNTH
       CALL SFIT_SHOW_MACROS
       CALL STR_LF ( 1 )

    ELSE
       CALL STR_OUTC ( ' SFIT variable not known', SFIT__VAR(1:10) ) 

       !    Indicate if variable was not identified
    END IF SHOW_VAR

  END SUBROUTINE SFIT_SHOW

  SUBROUTINE SFIT_SHOW_METHOD
    CALL STR_LF ( 1 )
    CALL STR_OUTC ( prog__id//':  method', METHOD  )
    CALL STR_OUTR ( prog__id//':  method_tolerance', AMB__TOL )
    CALL STR_OUTR ( prog__id//':  method_range (1)', WOBS1 )
    CALL STR_OUTR ( prog__id//':  method_range (2)', WOBS2 )
  END SUBROUTINE SFIT_SHOW_METHOD

  SUBROUTINE SFIT_SHOW_CONTINUUM
    CALL STR_LF ( 1 )
    CALL STR_OUTI ( prog__id//':  continuum', CMODE )
    CALL STR_OUTI ( prog__id//':  cont_poly', CPOLY-1 )
    CALL STR_OUTR ( prog__id//':  cont_window1', CPARAM )
    CALL STR_OUTR ( prog__id//':  cont_window2', CPARAM2 )
  END SUBROUTINE SFIT_SHOW_CONTINUUM

  SUBROUTINE SFIT_SHOW_DATA
    CALL STR_LF ( 1 )
    CALL STR_OUTC ( prog__id//':  data_spectrum', DATFIL )
    CALL STR_OUTR ( prog__id//':  data_inst ', SINST )
    CALL STR_OUTR ( prog__id//':  data_cosmic', CRTHRSH )
    CALL STR_OUTR ( prog__id//':  data_drift', DVSYS )
    CALL STR_OUTR ( prog__id//':  data_sigma', SSOBS )
  END SUBROUTINE SFIT_SHOW_DATA

  SUBROUTINE SFIT_SHOW_MACROS
    CALL STR_LF ( 1 )
    CALL STR_TEXT ( prog__id//':  macros' )
    DO IMAC = 1,MAC__NMAC
       CALL STR_LEN  ( MAC__STO(IMAC), MAC__LEN )
       CALL STR_TEXT ( MAC__NAM(IMAC)//' : '//MAC__STO(IMAC)(1:MAC__LEN) )
       WRITE (*,*)
    END DO
  END SUBROUTINE SFIT_SHOW_MACROS

  SUBROUTINE SFIT_SHOW_SOLVE
    !! doesnt show DPARS or CPARS ...
    CALL STR_LF ( 1 )
    CALL STR_OUTR ( prog__id//':  slv_teff1 ', PARS(1) )
    CALL STR_OUTR ( prog__id//':  slv_logg1 ', PARS(2) )
    CALL STR_OUTR ( prog__id//':  slv_ahe1  ', PARS(3) )
    CALL STR_OUTR ( prog__id//':  slv_vsini1', PARS(4) )
    CALL STR_OUTR ( prog__id//':  slv_logr1 ', PARS(5) )
    CALL STR_OUTR ( prog__id//':  slv_vrad1 ', PARS(6) )
    CALL STR_OUTR ( prog__id//':  slv_teff2 ', PARS(7) )
    CALL STR_OUTR ( prog__id//':  slv_logg2 ', PARS(8) )
    CALL STR_OUTR ( prog__id//':  slv_ahe2  ', PARS(9) )
    CALL STR_OUTR ( prog__id//':  slv_vsini2', PARS(10) )
    CALL STR_OUTR ( prog__id//':  slv_logr2 ', PARS(11) )
    CALL STR_OUTR ( prog__id//':  slv_vrad2 ', PARS(12) )
    CALL STR_OUTR ( prog__id//':  slv_ebv   ', PARS(13) )
  END SUBROUTINE SFIT_SHOW_SOLVE

  SUBROUTINE SFIT_SHOW_SYNTH
    CALL STR_LF ( 1 )
    CALL STR_OUTR ( prog__id//':  syn_vsini ', SYN__VSINI )
    CALL STR_OUTR ( prog__id//':  syn_beta ',  SYN__BETA )
    CALL STR_OUTR ( prog__id//':  syn_vrad ',  SYN__VRAD )
  END SUBROUTINE SFIT_SHOW_SYNTH

  SUBROUTINE SFIT_SHOW_GA
  END SUBROUTINE SFIT_SHOW_GA




  SUBROUTINE SFIT_PREPARE
    !     Prepare observed spectrum for fitting

    INTEGER :: NT, NG, NH       ! local grid loop indices

    !     Trim the spectrum and set masks
    CALL SFIT_PREPOBS

    !     Resample the model grid onto observed wavelength grid.
    WBIN(1:NWOBS) = WOBS(1:NWOBS); NWBIN = NWOBS 

    DO NT = 1,NTMOD
       DO NG = 1,NGMOD
          DO NH = 1,NHMOD
             CALL REMAP (WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, &
                  WBIN,FBIN(1:NWBIN,NT,NG,NH),NWBIN, 1.)
             CALL REMAP (WMOD,CMOD(1:NWMOD,NT,NG,NH),NWMOD, &
                  WBIN,CBIN(1:NWBIN,NT,NG,NH),NWBIN, 1.)
          ENDDO
       ENDDO
    ENDDO


    DO NT = 1,NTMOD2
       DO NG = 1,NGMOD2
          DO NH = 1,NHMOD2
             CALL REMAP (WMOD2,FMOD2(1:NWMOD2,NT,NG,NH),NWMOD2, &
                  WBIN,FBIN2(1:NWBIN,NT,NG,NH),NWBIN, 1.)
             CALL REMAP (WMOD2,CMOD2(1:NWMOD2,NT,NG,NH),NWMOD2, &
                  WBIN,CBIN2(1:NWBIN,NT,NG,NH),NWBIN, 1.)
          ENDDO
       ENDDO
    ENDDO


  END SUBROUTINE SFIT_PREPARE




  SUBROUTINE SFIT_PREPOBS
    !     Prepare observed spectrum for fitting

    INTEGER :: NOBS1, NOBS2    ! local array locators
    INTEGER :: NM              ! local mask loop index
    INTEGER :: NREJ            ! cosmic ray rejection count


    !     Check observed spectrum exists. 
    !     In the cazse of a request for a formal solution it need not. 
    !     In this case, create a dummy spectrum.
    IF ( .NOT. NWOBS > 0 ) THEN 
       WOBS(1:NWMOD) = WMOD(1:NWMOD)
       FOBS(1:NWMOD) = 1.0
       SOBS(1:NWMOD) = 1.E-5
       NWOBS = NWMOD
    ENDIF

    !     Check wavelength range has been defined
    IF ( WOBS1 > 1. .AND. WOBS2 > 1.) THEN
       
       !   Restrict WOBS to wavelength range give by WOBS1, WOBS2
       CALL LOCATE ( WOBS, NWOBS, WOBS1, NOBS1 )
       CALL LOCATE ( WOBS, NWOBS, WOBS2, NOBS2 )

       !  Extract prescribed wavelength range.  
       NWOBS = NOBS2-NOBS1+1
       WOBS(1:NWOBS) = WOBS(NOBS1:NOBS2)
       FOBS(1:NWOBS) = FOBS(NOBS1:NOBS2)
       SOBS(1:NWOBS) = SOBS(NOBS1:NOBS2)

    ELSE
        !  If not, set equal observed range (error trap added 2020.xi.30)
       WRITE(*,'(A,2F8.1,A)') 'method-range ',WOBS1,WOBS2, ': using observed range (default)' 
       NOBS1 = 1    
       NOBS2 = NWOBS
    ENDIF

    !     Remove cosmic rays from normalized data
    !            warn if too many points rejected..
    NREJ = COUNT ( FOBS(1:NWOBS) > CRTHRSH )
    IF ( FOBS_NORM .AND. NREJ > FLOAT (NWOBS)/20. ) THEN
       WRITE(*,'(I6,A,F6.2,A)') NREJ, &
            ' points above cosmic threshold, ', CRTHRSH, &
            '. Cut not applied.'
    ELSE
       WHERE ( FOBS(1:NWOBS) > CRTHRSH ) SOBS(1:NWOBS) = 1.E30
    ENDIF


    !     Apply mask to spectrum for use in fit
    CMSK=1
    IF (NMASK .GT. 0) THEN
       DO NM = 1,NMASK
          !! this could be replaced by a "WHERE" block
          CALL LOCATE ( WOBS, NWOBS, WMASK(NM,1), NOBS1 )
          CALL LOCATE ( WOBS, NWOBS, WMASK(NM,2), NOBS2 )
          SOBS(NOBS1:NOBS2) = WMASK(NM,3)
          IF ( NOBS1<NWOBS .AND. NOBS2>1 ) CMSK(NOBS1:NOBS2)=0
       ENDDO
    ENDIF

    ! setup the continuum mask
    CREG=0
    IF (NCREG>0) THEN
       DO NM=1,NCREG
          !! this could be replaced by a "WHERE" block
          CALL LOCATE ( WOBS, NWOBS, CREGX(NM,1), NOBS1 )
          CALL LOCATE ( WOBS, NWOBS, CREGX(NM,2), NOBS2 )
          IF ( NOBS1<NWOBS .AND. NOBS2>1 ) CREG(NOBS1:NOBS2)=1
       ENDDO
    ENDIF

  END SUBROUTINE SFIT_PREPOBS




  SUBROUTINE SFIT_RDOBS ( ISP )

    INTEGER :: ISP  ! file format identifier
    INTEGER :: PAMEND ! end of file flag
    INTEGER :: NWOBS2 ! temporary wavelength counter

    INTEGER :: NWLIM ! max number of wavelengths allowed 
    REAL                            :: DW
    REAL, ALLOCATABLE, DIMENSION(:) :: TEMPW
    REAL, ALLOCATABLE, DIMENSION(:) :: TEMPF


    ! Read data using "PAMELA format"

    IF (ISP == 1) THEN
       OBSTITLE = OBSFILE(1:40)
       OPEN  (1,FILE=OBSFILE,STATUS='OLD')
       PAMEND = 0
       NW = 0
       DO WHILE (PAMEND == 0) 
          NW = NW + 1
          IF (NW > NOMAX) THEN
             WRITE (*,'(A,I6,A,I6)') 'SFIT_RDOBS: nw =',NW,' exceeds maximum ',NOMAX
             WRITE (*,'(A)')         '            spectrum truncated '
             EXIT
          ENDIF
          READ  (1,*,IOSTAT=PAMEND) WOBS(NW),FOBS(NW),SOBS(NW) 
       ENDDO
       NWOBS = NW - 1
       CLOSE (1)


       ! Read data using "SPECTRUM format 2"
    ELSEIF (ISP == 2) THEN

       OPEN  (1,FILE=OBSFILE,STATUS='OLD')
       READ  (1,'(A)') OBSTITLE
       READ  (1,*)
       READ  (1,*) NWOBS
       ALLOCATE ( TEMPW (NWOBS) )
       ALLOCATE ( TEMPF (NWOBS) )
       READ  (1,*) (TEMPW(NW),TEMPF(NW),NW=1,NWOBS)
       CLOSE (1)

       OBS_DW = (TEMPW(NWOBS) - TEMPW(1)) / FLOAT(NWOBS - 1)

       NWLIM = MIN ( NOMAX, NWMAX ) 

       ! check whether observed spectral resolution is greater than the model resolution
       ! and, if so, degrade the observed resolution. 
       IF (MOD_DW > OBS_DW) THEN

          WRITE (*,'(A,F6.3,A,F6.3)') 'SFIT_RDOBS: dw =',OBS_DW,' << model dw ',MOD_DW

          NWOBS2 = 1 + (TEMPW(NWOBS)-TEMPW(1)) / MOD_DW
          WRITE (*,'(A,F12.6)')   '            spectrum degraded: DW = ',MOD_DW

          WOBS(1:NWOBS2) = TEMPW(1) + [(I,I=0,NWOBS2-1)]*MOD_DW
          CALL REMAP ( TEMPW, TEMPF, NWOBS, WOBS, FOBS, NWOBS2, 0.0 )
          NWOBS = NWOBS2

          ! check whether the observed spectrum has more data points than allowed
          ! in either NOMAX or in the modelarrays NWMAX, NWMAX2. 
       ELSEIF (NWOBS > NWLIM) THEN

          WRITE (*,'(A,I6,A,I6)') 'SFIT_RDOBS: nw =',NWOBS,' exceeds maximum ',NWLIM

          DW = (TEMPW(NWOBS)-TEMPW(1)) / (NWLIM-1)
          WRITE (*,'(A,F12.6)')   '            spectrum degraded: DW = ',DW

          WOBS(1:NWLIM) = TEMPW(1) + [(I,I=0,NWLIM-1)]*DW
          CALL REMAP ( TEMPW, TEMPF, NWOBS, WOBS, FOBS, NOMAX, 0.0 )
          NWOBS = NWLIM

       ELSE
          WOBS(1:NWOBS) = TEMPW(1:NWOBS)
          FOBS(1:NWOBS) = TEMPF(1:NWOBS)
       ENDIF

       DEALLOCATE ( TEMPW )
       DEALLOCATE ( TEMPF )

       !     Insert error array here
       SOBS(1:NWOBS) = SSOBS   ! default value 

    ENDIF

    ! update drift velocity to new sepctrum
    DVSYS = DVSYSKM * (WOBS(1)+WOBS(NWOBS)) * 1.E3 / 2. / TAPP__C

    ! Report to screen and log file
    WRITE (*,'(/2A)') ' Stellar spectrum: ',OBSFILE(1:60)
    WRITE (*,'(A,I6,2F6.0)')    '  NWOBS, WRANGE :',NWOBS,WOBS(1),WOBS(NWOBS)
    WRITE (*,*)
    WRITE (LOGFILE,'(/2A)') ' Stellar spectrum: ',OBSFILE(1:60)
    WRITE (LOGFILE,'(A,I6,2F6.0)')    '  NWOBS, WRANGE :',NWOBS,WOBS(1),WOBS(NWOBS)
    WRITE (LOGFILE,*)


    !     Trap zero values in fluxes
    WHERE (FOBS < 1.E-20) FOBS = 1.E-20

  END SUBROUTINE SFIT_RDOBS



  SUBROUTINE SFIT_SVFIT

    FITFIL = TRIM(OBSFILE)//'.fit'
    OPEN  (1,FILE=FITFIL,STATUS='UNKNOWN')
    WRITE (1,'(A)') TRIM(OBSTITLE)//': SFIT2 model fit (synth)'
    WRITE (1,*)
    WRITE (1,*) NWOBS
    WRITE (1,'(0P,F10.3,1P,3E12.3)') (WOBS(NW),FOBS(NW),SOBS(NW),FFIT(NW),NW=1,NWOBS)
    CLOSE (1)
    WRITE (*,'(2A)') 'Spectrum and fits written to ',FITFIL

  END SUBROUTINE SFIT_SVFIT





  SUBROUTINE SFIT_NORMALIZE

    !  Approximately normalize the spectrum to an approximate fit

    REAL :: WZERO  ! 1st wavelength in range
    REAL :: AV     ! mean value in vector
    REAL :: SD     ! standard deviation about mean in vector

    !      Find continuum fitting function with low-pass filter
    IF (CMODE == 1) THEN
       FFIT(1:NWOBS) = FFIT(1:NWOBS) / FOBS(1:NWOBS)
       FSIG = SOBS
       WHERE (CREG==0) 
          FSIG=MIN(SOBS*1.E10,1.E10)
       ENDWHERE
       CALL QSMOOTHW (WOBS,FFIT,FSIG,CFIT,CSIG/2.534,NWOBS,CPARAM)
       ! write(21,'(6F10.3,I5)')(WOBS(J),FOBS(J),FFIT(J),FSIG(J),CFIT(J),CSIG(J),CREG(J),J=1,NWOBS)


       !      Find continuum fitting function as low-order polynomial
    ELSEIF (CMODE == 2) THEN
       FFIT(1:NWOBS) = FFIT(1:NWOBS) / FOBS(1:NWOBS)
       FSIG = SOBS
       WHERE (CREG==0) 
          FSIG=MIN(SOBS*1.E10,1.E10)
       ENDWHERE
       WZERO = WOBS(1)
       WOBS = WOBS - WZERO
       DO N = 1,NP
          CNT__LISTA(N) = N
       ENDDO
       CALL NR_LFIT (WOBS,FFIT,FSIG,NWOBS, &
            CCOEF,CPOLY, CNT__LISTA,CPOLY, &
            CCOVAR,NCCF, CCH2, SFIT_FPOLY)
       DO NW = 1,NWOBS
          CALL PNEVAL (CPOLY, CCOEF, WOBS(NW), CFIT(NW))
       ENDDO
       WOBS = WOBS + WZERO


       !      Eliminate points >3sigma from low-pass filter fit
    ELSEIF (CMODE == 3) THEN

       ! compute first approx to ratio
       FWRK1(1:NWOBS) = FFIT(1:NWOBS)/FOBS(1:NWOBS)

       ! reduce impact of non-continuum regions in first smoothing
       CSIG = SOBS
       WHERE ( CREG==0 )
          CSIG = 1.E3*CSIG
       ENDWHERE

       ! compute smoothed ratio
       CALL QSMOOTHW (WOBS,FWRK1,CSIG,FWRK2,CSIG,NWOBS,CPARAM2)

       ! compute sd about ratio
       FWRK1(1:NWOBS)=FWRK1(1:NWOBS)/FWRK2(1:NWOBS)
       CALL STATS (FWRK1(1:NWOBS),1,NWOBS,AV,SD)

       ! find renormalisation function, excluding lines
       WHERE ( ABS (1.-FWRK1(1:NWOBS)) < 3*SD .AND. CREG(1:NWOBS)/=0 )
          FWRK3(1:NWOBS) = FFIT(1:NWOBS)/FOBS(1:NWOBS)
       ELSEWHERE
          FWRK3(1:NWOBS) = FWRK2(1:NWOBS)
       ENDWHERE

       ! apply low-pass filter to find continuum fitting function
       CALL QSMOOTHW (WOBS,FWRK3,SOBS,CFIT,CSIG,NWOBS,CPARAM)

       !! write(22,'(7f10.3)') (wobs(j),fobs(j),ffit(j),fwrk1(j),fwrk2(j),fwrk3(j),cfit(j),j=1,nwobs)

    ENDIF


    !      Renormalize observed fluxes and repeat
    FOBS = FOBS * CFIT
    SOBS = SOBS * CFIT

  END SUBROUTINE SFIT_NORMALIZE



  SUBROUTINE SFIT_C_SIGMA

    !  Compute sigma from current fit in "continuum" regions

    REAL :: WZERO  ! 1st wavelength in range
    REAL :: AV     ! mean value in vector
    REAL :: SD     ! standard deviation about mean in vector
    REAL, ALLOCATABLE, DIMENSION(:) :: FCONT
    INTEGER, DIMENSION(1) :: FSHAP

    !  Normalise the spectrum
    FWRK1(1:NWOBS) = FFIT(1:NWOBS) / FOBS(1:NWOBS)

    !  Extract fluxes from continuum regions
    FSHAP = SHAPE ( PACK ( FWRK1, MASK = (CREG == 1) ) ) 
    IF ( .NOT. ALLOCATED (FCONT) ) ALLOCATE ( FCONT(FSHAP(1)) )
    FCONT = PACK ( FWRK1, MASK = (CREG == 1) )

    ! Find average and standard deviation
    CALL STATS (FCONT,1,FSHAP(1),AV,SD)
    DEALLOCATE ( FCONT )

    !  Report sigma and how it was computed
    WRITE (*,*) ' sigma = ',SD/AV
    WRITE (*,*) ' computed from ',FSHAP(1),' continuum points, with average value',AV

    !  Set sigma on all data points to computed sigma 
    SOBS = SD / AV

    !  Reapply current mask to spectrum for use in fit
    IF (NMASK .GT. 0) THEN
       DO NM = 1,NMASK
          CALL LOCATE ( WOBS, NWOBS, WMASK(NM,1), NOBS1 )
          CALL LOCATE ( WOBS, NWOBS, WMASK(NM,2), NOBS2 )
          SOBS(NOBS1:NOBS2) = WMASK(NM,3)
       ENDDO
    ENDIF



  END SUBROUTINE SFIT_C_SIGMA





  SUBROUTINE SFIT_VCORR

    !    Find velosity shift between observation and model
    !    from cross-correlation 

    INTEGER, PARAMETER :: FFT__DIM = 2**16         ! must be power of 2
    REAL, DIMENSION(:), ALLOCATABLE :: FFT__F1, FFT__F2
    REAL, DIMENSION(:), ALLOCATABLE :: FFT

    REAL, DIMENSION(:), ALLOCATABLE :: WLOG   ! log wavelength 
    REAL, DIMENSION(:), ALLOCATABLE :: WLLG   ! log wavelength on linear intervals
    REAL, DIMENSION(:), ALLOCATABLE :: FOBL   ! observed data on log wavelength scalce
    REAL, DIMENSION(:), ALLOCATABLE :: FITL   ! model data on log wavelength scale
    REAL :: WLOG_0 , WLOG_D                   ! desrciprots for log lambda grid


    REAL :: DWLOG  ! the wavelength shift
    REAL :: DVRAD  ! the velcosity shift

    WRITE (*,*) ' SFIT_VCORR: ** code incomplete - - no result ** '

    !    check array sizes
    IF ( NWOBS > FFT__DIM ) THEN
       WRITE (*,*) ' SFIT_VCORR: array sizes too large '
       WRITE (*,*) ' NWOBS = ',NWOBS,'  FFT__DIM = ',FFT__DIM
       WRITE (*,*) ' Velocity not changed '
       RETURN
    END IF

    !    Allocate space for temporary arrays
    IF (.NOT. ALLOCATED(WLOG) ) ALLOCATE ( WLOG (NWOBS) ) 
    IF (.NOT. ALLOCATED(WLLG) ) ALLOCATE ( WLLG (NWOBS) ) 
    IF (.NOT. ALLOCATED(FOBL) ) ALLOCATE ( FOBL (NWOBS) ) 
    IF (.NOT. ALLOCATED(FITL) ) ALLOCATE ( FITL (NWOBS) ) 
    IF (.NOT. ALLOCATED(FFT__F1) ) ALLOCATE ( FFT__F1 (FFT__DIM) )
    IF (.NOT. ALLOCATED(FFT__F2) ) ALLOCATE ( FFT__F2 (FFT__DIM) )
    IF (.NOT. ALLOCATED(FFT)  ) ALLOCATE ( FFT (2*FFT__DIM) )

    !    Map wavelength data onto logarithmic wavelength space with
    !    equal intervals in log lambda
    WLOG = LOG10(WOBS(1:NWOBS))
    WLOG_0 = INT (WLOG(1) )
    WLOG = WLOG - WLOG_0
    WLOG_D = (WLOG(NWOBS)-WLOG(1)) / FLOAT(NWOBS - 1)
    DO N = 1, NWOBS
       WLLG(N) = WLOG(1)+(N-1)*WLOG_D
    ENDDO

    !    Map flux data onto resampled wavelength data
    CALL REMAP ( WLOG, FOBS, NWOBS, WLLG, FOBL, NWOBS, 1. )
    CALL REMAP ( WLOG, FFIT, NWOBS, WLLG, FITL, NWOBS, 1. )

    !    Apply high band-pass filter and truncate

    !    Prepare data arrays for CCF -- pad with zeros
    FFT__F1(1:NWOBS) = FOBL; FFT__F1(NWOBS+1:FFT__DIM) = 1.
    FFT__F2(1:NWOBS) = FITL; FFT__F2(NWOBS+1:FFT__DIM) = 1.

    !    Assume data is normalized and subtract continuum
    FFT__F1 = FFT__F1 - 1.; FFT__F2 = FFT__F2 - 1.

    !    Compute the cross-correlation function using a FFT
    !     CALL NR_CORREL ( FFT__F1, FFT__F2, FFT__DIM, FFT )

    !    Locate maximum in CCF


    !    Convert to velocity, assuming data in Angstroms
    DVRAD = TAPP__C * ( 10.**(-DWLOG) - 1. ) 

    !    Replace PARS(6): velocity of star 1 with measured shift
    PARS(6) = DVRAD


  END SUBROUTINE SFIT_VCORR




  SUBROUTINE SFIT_FORMAL

    ! Compute formal solution for specified parameters


    !  Initalize parameters
    PARS = PARS0

    !  Compute formal solution
    DERIVS(1) = -99.
    DO NW = 1,NWOBS
       CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
    END DO

    !  Reevaluate final solution (to get covariances)
    COVAR(1,1) = 0.
    CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS,  PARS,NP, &
         CHISQ,COVAR,NCHM, SPEC_TGHV_BIN)

    !  Invert covariance matrix to find standard errors
    !   first - set any diagonal elements not used to a large number 
    !   to avoid singular matrices....
    !
    DO I=1,NP
       IF (DELTA_P(I)<1.E-5) COVAR(I,I)=1.E30
    ENDDO
    AMB__X(1:NP) = 1. 
    CALL NR_GAUSSJ (COVAR,NCHM,NP,AMB__X,1,1)

    !      Report results
    CALL SFIT_QK_REPORT (NP)

  END SUBROUTINE SFIT_FORMAL




  SUBROUTINE SFIT_CHISQ

    ! Compute chi^2 on T,g,nH surface,

    INTEGER :: IMIN, JMIN, KMIN   ! locators withing grid
    INTEGER :: NSK = 1            ! Grid skip parameter

    DOUBLE PRECISION, DIMENSION(3) :: PARAMS

!!! Decide whether or not we're using CWR's extensions
    IF ( ASSOCIATED(MODSLIST) .AND. ASSOCIATED(TETRALIST) ) THEN

       PARAMS = PARS(1:3)
       PRINT *, "Calling MODLIST_CHISQ"
       CALL MODLIST_CHISQ(MODSLIST, 3, PARS0, CHISQ, PARAMS)
       PARS(1:3) = PARAMS
    ELSE
       !  Initalize parameters
       PARS = PARS0

       WRITE (*,*) " CHISQ surface "
       WRITE (*,*)
       WRITE (*,*) " Teff1   log g1    ahe1    chi^2"

       WRITE (11,*) " CHISQ surface "
       WRITE (11,*)
       WRITE (11,*) " Teff1   log g1    ahe1    chi^2"

       !     Teff loop
       DO NT = 1,NTMOD,NSK
          PARS(1) = TMOD(NT)

          !       log g loop
          DO NG = 1,NGMOD,NSK
             PARS(2) = GMOD(NG)

             DO NH = 1,NHMOD,NSK
                PARS(3) = HMOD(NH)

                !         chi-squared
                CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS, PARS,NP, &
                     CHISQ,COVAR,NCHM, SPEC_TGHV_BIN)
                CHSQ_TEST(NT,NG,NH) = CHISQ
                WRITE (11,'(3F8.3,1PE13.3)') PARS(1:3), CHISQ/(NWOBS-NP)
                WRITE (* ,'(3F8.3,1PE13.3)') PARS(1:3), CHISQ/(NWOBS-NP)

             END DO
          END DO
       END DO


       !    Locate minimum
       IMIN = 1; JMIN = 1; KMIN = 1; CHISQ = 1.E30
       DO NT = 1,NTMOD,NSK
          DO NG = 1,NGMOD,NSK
             DO NH = 1,NHMOD,NSK
                IF (CHSQ_TEST(NT,NG,NH)<CHISQ) THEN
                   IMIN = NT; JMIN = NG; KMIN = NH; CHISQ = CHSQ_TEST(NT,NG,NH)
                ENDIF
             END DO
          END DO
       ENDDO

       !    Recompute chi-squared for best position in grid
       PARS(1) = TMOD(IMIN)
       PARS(2) = GMOD(JMIN)
       PARS(3) = HMOD(KMIN)

    ENDIF

    DERIVS(1) = -99.
    DO NW = 1,NWOBS
       CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
    END DO

    !      chi-squared
    CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS, PARS,NP, &
         CHISQ,COVAR,NCHM, SPEC_TGHV_BIN)

    !      invert covariance matrix to find standard errors
    DO I=1,NP
       IF (DELTA_P(I)<1.E-5) COVAR(I,I)=1.E30
    ENDDO
    ALLOCATE(AMB__X(NP))
    AMB__X(1:NP) = 1. 
    CALL NR_GAUSSJ (COVAR,NCHM,NP,AMB__X,1,1)

    !      Return results to main program
    PARS0 = PARS

    !      Report results
    DO I=1,6 
       COVAR(I,I) = SQRT(COVAR(I,I)) ; COVAR(I,I) = MIN (COVAR(I,I),999.0) 
    ENDDO


    WRITE (*,*)
    WRITE (*,'(1X,A)') OBSFILE
    WRITE (*,'(A)') '  Teff_1  lg g1  nHe_1 vsini1   q_1   dv_1 &
         &  Teff_2  lg g2  nHe_2 vsini2   q_2   dv_2 '
    WRITE (*,'(2(1X,0P,2F7.2,F7.3,2F7.2,F7.1)/, &
         &     2(1X,0P,2F7.2,F7.1,2F7.2,F7.1)/,1P,5E10.2)')  &
         PARS, (COVAR(I,I),I=1,NP), &
         CHISQ/(NWOBS-NP)
    WRITE(*,*)

    WRITE (LOGFILE,*)
    WRITE (LOGFILE,'(1X,A)') OBSFILE
    WRITE (LOGFILE,'(A)') '  Teff_1  lg g1  nHe_1 vsini1   q_1   dv_1 &
         &  Teff_2  lg g2  nHe_2 vsini2   q_2   dv_2 '
    WRITE (*,'(2(1X,0P,2F7.2,F7.3,2F7.2,F7.1)/, &
         &     2(1X,0P,2F7.2,F7.3,2F7.2,F7.1)/,1P,5E10.2)')  &
         PARS, (COVAR(I,I),I=1,NP), &
         CHISQ/(NWOBS-NP)
    WRITE(LOGFILE,*)

    WRITE (SUMFILE,'(A25, (1X,0P,6F7.3,4F7.2,2F7.1), 1P,E10.2)')  &
         OBSFILE, (PARS(I),COVAR(I,I),I=1,6), &
         CHISQ/(NWOBS-NP)

    DEALLOCATE (AMB__X)
    
  END SUBROUTINE SFIT_CHISQ




  SUBROUTINE SFIT_AMOEBA

    ! Solve using multidimensional downhill simplex method (AMOEBA)

    REAL :: SSSS, SSS2    ! integrals from trapezoidal quadrature

    !      Set up initial guesses
    PARS = PARS0

    IF (.NOT. ASSOCIATED(MODSLIST) .AND. .NOT. ASSOCIATED(TETRALIST) ) THEN
       !  Prevent extrapolation
       PARS(1)  = MAX ( MIN ( PARS(1), MAXVAL ( TMOD(1:NTMOD) ) ), MINVAL ( TMOD(1:NTMOD ) ) )
       PARS(2)  = MAX ( MIN ( PARS(2), MAXVAL ( GMOD(1:NGMOD) ) ), MINVAL ( GMOD(1:NGMOD ) ) )
       PARS(3)  = MAX ( MIN ( PARS(3), MAXVAL ( HMOD(1:NHMOD) ) ), MINVAL ( HMOD(1:NHMOD ) ) )
       PARS(4)  = MAX ( PARS(4), 0. )
       !      IF ( ABS(DELTA_P(5)) >0. ) PARS(5)  = MAX ( PARS(5), 0. )
       IF ( NTMOD2 == 0 ) THEN
          PARS(7)  = MAX ( MIN ( PARS(7), MAXVAL ( TMOD(1:NTMOD) ) ), MINVAL ( TMOD(1:NTMOD ) ) )
          PARS(8)  = MAX ( MIN ( PARS(8), MAXVAL ( GMOD(1:NGMOD) ) ), MINVAL ( GMOD(1:NGMOD ) ) )
          PARS(9)  = MAX ( MIN ( PARS(9), MAXVAL ( HMOD(1:NHMOD) ) ), MINVAL ( HMOD(1:NHMOD ) ) )
       ELSE
          PARS(7)  = MAX ( MIN ( PARS(7), MAXVAL ( TMOD2(1:NTMOD2) ) ), MINVAL ( TMOD2(1:NTMOD2 ) ) )
          PARS(8)  = MAX ( MIN ( PARS(8), MAXVAL ( GMOD2(1:NGMOD2) ) ), MINVAL ( GMOD2(1:NGMOD2 ) ) )
          PARS(9)  = MAX ( MIN ( PARS(9), MAXVAL ( HMOD2(1:NHMOD2) ) ), MINVAL ( HMOD2(1:NHMOD2 ) ) )
       ENDIF
       PARS(10) = MAX ( PARS(10), 0. )
       !      IF ( ABS(DELTA_P(11))>0. ) PARS(11) = MAX ( PARS(11), 0. )
       PARS(13) = MAX ( PARS(13), 0. )
    ENDIF

    WRITE (*,*)
    WRITE (*,*) ' Initial parameters '
    WRITE (*,'(A)') '  Teff_1  lg g1  nHe_1 vsini1  lg q_1 dv_1 &
         &  Teff_2  lg g2  nHe_2 vsini2  lg q_2 dv_2  E(B-V)'
    WRITE (*,'(2(1X,0P,2F7.2,F7.3,2F7.2,F7.1),F8.2)')   PARS
    WRITE (*,'(2(1X,0P,2F7.2,F7.3,2F7.2,F7.1),F8.2)')   DELTA_P
    WRITE (*,*)

    ALLOCATE (AMB__P(NPAR+1,NPAR))
    ALLOCATE (AMB__X(NPAR))
    ALLOCATE (AMB__Y(NPAR+1))

    !      Set simplex vertices to starting guesses
    DO I=1,NCHM
       AMB__P(1:MP,I) = PARS(I) + DELTA_P(I)
       AMB__P(I+1,I)  = PARS(I)
    END DO
    PARS1 = PARS

    !      Evaluate function at corners
    DO I = 1,MP
       AMB__X(1:NP) = AMB__P(I,1:NP)  ! pick values for next corner
       COVAR(1,1) = -99.              ! do not evaluate covariance matrix
       CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS,  AMB__X,NP, &
            CHISQ,COVAR,NCHM, SPEC_TGHV_BIN)
       AMB__Y(I) = CHISQ
    END DO

    !      Find solution and save it
    CALL NR_AMBCHSQ (AMB__P, AMB__Y, MP, NP, NCHM, AMB__TOL, AMB__ITER, &
         SPEC_TGHV_BIN, WOBS, FOBS, SOBS, NWOBS)
    DO I=1,NCHM
       PARS(I) = AMB__P(I+1,I)
    END DO
    PARS0 = PARS

    !      Reevaluate final solution (to get covariances)
    COVAR(1,1) = 0.
    CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS,  PARS,NP, &
         CHISQ,COVAR,NCHM, SPEC_TGHV_BIN)

    !      Invert covariance matrix to find standard errors
    !      first - set any diagonal elements not used to a large number 
    !      to avoid singular matrices....
    !
    DO I=1,NP
       IF (DELTA_P(I)<1.E-5) COVAR(I,I)=1.E30
    ENDDO
    AMB__X(1:NP) = 1. 
    CALL NR_GAUSSJ (COVAR,NCHM,NP,AMB__X,1,1)

    !      Report results
    CALL SFIT_QK_REPORT (NP)

    DEALLOCATE (AMB__P)
    DEALLOCATE (AMB__X)
    DEALLOCATE (AMB__Y)

  END SUBROUTINE SFIT_AMOEBA




  SUBROUTINE SFIT_LEVENBURG

    !  Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)

    ! Chi-squared minimisation control variables.
    INTEGER, DIMENSION(NPAR) :: LMQ__LISTA = 0    ! Regression control array
    INTEGER   :: LMQ__MFIT = 0           ! Number of coefficients in fit
    REAL      :: LMQ__ALLAST
    REAL      :: LMQ__CHSQ0
    REAL      :: LMQ__DPAR, LMQ__DPAR0   ! Measures of convergence
    INTEGER, PARAMETER :: LMQ__MAXITER = 50      ! Maximum iterations
    REAL, PARAMETER :: LMQ__DCONV  =  1.E-4      ! Convergence criterion

    !                            Initialize

    !      Determin e which parameters are to be used in solution
    LMQ__MFIT = 0
    DO N = 1,NP
       IF ( DELTA_P(N) > 1.E-9 ) THEN
          LMQ__MFIT = LMQ__MFIT + 1
          LMQ__LISTA(LMQ__MFIT) = N
       ELSE
          LMQ__LISTA(NP+1+LMQ__MFIT-N) = N
       ENDIF
    ENDDO

    !      Compute initial estimate for theta (using trial Teff)
    !      not required by SFIT
    PARS = PARS0

    WRITE (*,*)
    WRITE (*,*) ' Initial parameters '
    WRITE (*,'(A)') '  Teff_1  lg g1  nHe_1  vsini1   q_1   dv_1 &
         &  Teff_2  lg g2  nHe_2  vsini2   q_2   dv_2  E(B-V)'
    WRITE (*,'(2(1X,0P,2F7.2,F8.4,2F7.2,F7.1),F8.2)')   PARS
    WRITE (*,*)
    WRITE (*,'(A, 12I3)') ' Solving for: ',LMQ__LISTA(1:LMQ__MFIT)
    WRITE (*,'(A, 12I3)') ' Fixed   for: ',LMQ__LISTA(NP:LMQ__MFIT+1:-1)
    WRITE (*,*)

    DERIVS(1) = -99.
    DO NW = 1,NWOBS
       CALL SPEC_TGHV_BIN (WOBS(NW),PARS,FFIT(NW),DERIVS,NP)
    ENDDO

    !      Initial call to chi-squared minimisation engine.
    ALAMDA = -0.01
    LMQ__ALLAST = ALAMDA
    CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, PARS,NP, &
         LMQ__LISTA,LMQ__MFIT, &
         COVAR,ALPHA,NCHM, CHISQ, SPEC_TGHV_BIN, ALAMDA)

    !      Report results.
    LMQ__DPAR = 0.; LMQ__DPAR0 = 999.
    WRITE (*,'(/,2(1X,0P,3F7.3,3F7.2)/,1P,5E10.2)')  &
         PARS, CHISQ/(NWOBS-NP), LMQ__DPAR, LMQ__DCONV, LMQ__ALLAST

    ! Avoid iterations if only a formal soution is required
        IF ( LMQ__MFIT > 0 ) THEN

       !                             Iterate

       ALAMDA = 0.2
       SOLVE_LMQ: DO I = 1,LMQ__MAXITER

          !        Next chi-squared minimization step
          LMQ__ALLAST = ALAMDA
          LMQ__CHSQ0 = CHISQ
          CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, PARS,NP, &
               LMQ__LISTA,LMQ__MFIT, &
               COVAR,ALPHA,NCHM, CHISQ, SPEC_TGHV_BIN, ALAMDA)

          !        Test for convergence
          LMQ__DPAR = 0.
          DO N = 1,LMQ__MFIT
             M = LMQ__LISTA(N)
             LMQ__DPAR = LMQ__DPAR + ABS(PARS0(M) - PARS(M))**2
          ENDDO
          LMQ__DPAR = SQRT(LMQ__DPAR)
          PARS0 = PARS

          !        Report results.
          WRITE (*,'(/,2(1X,0P,3F7.3,3F7.2)/,1P,5E10.2)')  &
               PARS, CHISQ/(NWOBS-NP), LMQ__DPAR, LMQ__DCONV, LMQ__ALLAST

          IF ( (LMQ__DPAR < LMQ__TOL .AND. LMQ__DPAR > 0.) &
               .OR. ( CHISQ > 1.02*LMQ__CHSQ0 .AND. LMQ__DPAR > 0. .AND. I > 1 ) &
               .OR. ( CHISQ > LMQ__CHSQ0 .AND. LMQ__DPAR > LMQ__DPAR0 .AND. I > 1 ) &
               .OR. ( LMQ__DPAR == 0. ) ) EXIT 
          LMQ__DPAR0 = LMQ__DPAR

       ENDDO SOLVE_LMQ
    ENDIF
    !                             Complete

    !      Final calls to find errors and save results
    CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, PARS,NP, &
         LMQ__LISTA,LMQ__MFIT, & 
         COVAR,ALPHA,NCHM, CHISQ, SPEC_TGHV_BIN, 0.0)


    !  Report results
    CALL SFIT_QK_REPORT (LMQ__MFIT)
    
  END SUBROUTINE SFIT_LEVENBURG



  SUBROUTINE SFIT_QK_REPORT (NP) 
    !=   Report summary results to terminal, logfile and summary file
    !    called by sfit_amoeba amd sfit_levenburg
    INTEGER NP

    DO I=1,NP
       COVAR(I,I) = SQRT(COVAR(I,I)) ; COVAR(I,I) = MIN (COVAR(I,I),999.0) 
    ENDDO

    WRITE (*,*)
    WRITE (*,'(1X,A)') OBSFILE
    WRITE (*,'(A)') '  Teff_1  lg g1   nHe1  vsini1  lg q1    dv1 &
         & Teff_2  lg g2   nHe2  vsini2  lg q2    dv2 '
    WRITE (*,'(2(1X,0P,2F7.2,F8.4,2F7.2,F7.1),F8.2/, &
         &     2(1X,0P,2F7.2,F8.4,2F7.2,F7.1),F8.2/,1P,5E10.2)')  &
         PARS, (COVAR(I,I),I=1,NP), &
         CHISQ/(NWOBS-NP)
    WRITE(*,*)

    WRITE (LOGFILE,*)
    WRITE (LOGFILE,'(1X,A)') OBSFILE
    WRITE (LOGFILE,'(A)') '  Teff_1  lg g1   nHe1  vsini1  lg q1    dv1 &
         & Teff_2  lg g2   nHe2  vsini2  lg q2    dv2 '
    WRITE (LOGFILE,'(2(1X,0P,2F7.2,F8.4,2F7.2,F7.1),F8.2/, &
         & 2(1X,0P,2F7.2,F8.4,2F7.2,F7.1),F8.2/,1P,5E10.2)')  &
         PARS, (COVAR(I,I),I=1,NP), &
         CHISQ/(NWOBS-NP)
    WRITE(LOGFILE,*)

    WRITE (SUMFILE,'(A25, (1X,0P,6F7.3,4F7.2,2F7.1), 1P,E10.2)')  &
         OBSFILE, (PARS(I),COVAR(I,I),I=1,NP), CHISQ/(NWOBS-NP)

  END SUBROUTINE SFIT_QK_REPORT



  SUBROUTINE SFIT_GENETIC
    ! Solve using genetic algorithm

    REAL :: x                   ! Random number
    INTEGER,DIMENSION(1) :: II  ! Location of best indivual in the current generation
    INTEGER :: MAX__ATTEMPTS=3  ! Maximum number of attempts after converging to local minima
    INTEGER :: COUNT              ! Counter for repeat starts
    REAL,ALLOCATABLE :: GA__BEST_CHI(:)         ! Best chi-squared values
    REAL,ALLOCATABLE :: GA__BEST_PARS(:,:)   ! Best parameter values



    GA__NPVAR=0
    DO I=1,NPAR                  
       IF (DELTA_P(I)/=0) THEN
          GA__NPVAR=GA__NPVAR+1   ! finds how many parameters are being varied        
          GA__NPLOC(GA__NPVAR)=I  ! stores the location of each variable parameter
       END IF
    END DO


    GA__STRING=GA__SUBSTRING*GA__NPVAR   ! sets the stringlength 

    ALLOCATE(GA__PARENT(GA__POPS,GA__STRING))
    ALLOCATE(GA__SUIT(GA__POPS)) 
    ALLOCATE(BITVAL(GA__SUBSTRING))
    ALLOCATE(GA__BEST_CHI(MAX__ATTEMPTS))
    ALLOCATE(GA__BEST_PARS(MAX__ATTEMPTS,NPAR))

    GA__BEST_CHI=1.E30
    GA__SUIT=1.E30            ! initializes suitability array

    ! Prevent GA__KEPT from being set too high
    IF(GA__KEPT>=MIN(10,GA__POPS)) THEN
       GA__KEPT=4
    END IF

    DO N=1,GA__SUBSTRING
       BITVAL(N)= REAL(GA__ALPHA**(N-1.0))/((GA__ALPHA**GA__SUBSTRING)-1.0)               
    END DO

    WRITE(*,'(A35,I4)')'Population size = ',GA__POPS 
    WRITE(*,'(A35,I4)')'Maximum number of generations = ',GA__MAXGEN
    WRITE(*,'(A35,I4)')'Substring length = ',GA__SUBSTRING
    WRITE(*,'(A35,I4)')'Alphabet size = ',GA__ALPHA
    WRITE(*,'(A35,A25)')'Selection method = ',GA__SELECT
    WRITE(*,'(A35,A3)')'Elitism : ',GA__ELITE
    IF(GA__ELITE(1:3)=='yes') THEN
       WRITE(*,'(A35,I4)')'Kept = ',GA__KEPT
    END IF

    COUNT=1
10  WRITE(*,'(A14,I3)')'Attempt number ',COUNT
    WRITE(*,'(A6,3x,A18,3x,A18,6(4x,A10))')'Gen','Best red chi-sq','Ave red chi-sq',&
         &(CPARS(GA__NPLOC(I)),I=1,GA__NPVAR)   


    DO I=1,GA__POPS   
       DO J=1,GA__STRING     
          CALL RANDOM_NUMBER(x)     
          GA__PARENT(I,J)=INT(x*(REAL(GA__ALPHA)))  ! creates a random string of digits which
          ! depends on the alphabet being used
       ENDDO
    ENDDO

    M=0
    DO WHILE (MINVAL(GA__SUIT)>GA__TOL)

       M=M+1       ! generation counter   

       DO I=1,GA__POPS    

          CALL GA_CALCPARAM 
          covar(1,1)=-99
          CALL NR_CHISQ(WOBS,FOBS,SOBS,NWOBS,PARS1,NP, & 
               CHISQ,COVAR,NCHM,SPEC_TGHV_BIN)


          ! Suitability is determined by how far the reduced chi squared value is from 0.
          GA__SUIT(I)=CHISQ/(NWOBS-NP)        

          ! Uncomment the following line to view each individual in each generation
          ! WRITE(*,'(I2,3x,F16.7,5x,2(F9.6,5x),F10.3)')I,GA__SUIT(I),(PARS1(GA__NPLOC(J)),J=1,GA__NPVAR)                                      
       END DO

       II=MINLOC(GA__SUIT)      ! Find location of reduced chi-sq value 
       I=II(1)                 ! which is closest to 0

       ! Calculate the parameter values of the most suitable individual again,
       ! so that they can be displayed on screen
       CALL GA_CALCPARAM        


       WRITE(*,'(1x,I3,6x,F11.6,6x,F17.6,4x,6(2x,F12.7))')M,GA__SUIT(I),&
            &(SUM(GA__SUIT))/GA__POPS,(PARS1(GA__NPLOC(I)),I=1,GA__NPVAR)

       IF(M+1>GA__MAXGEN) THEN
          GA__BEST_CHI(COUNT)=GA__SUIT(I)
          GA__BEST_PARS(COUNT,:)=PARS1(:)
          EXIT   
       END IF


       ! If the maximum and minimum values of reduced chi-squared are within a certain distance,
       ! but the reduced chi-squared value has not yet converged to the required tolerance from 
       ! zero, then the values have converged to a local minimum, rather than the global minimum.
       ! If so, then we start the algorithm again with a new random starting population.

       IF(MAXVAL(GA__SUIT)-MINVAL(GA__SUIT)<GA__TOL .AND. GA__SUIT(I)>GA__TOL) THEN      
          GA__BEST_CHI(COUNT)=GA__SUIT(I)
          GA__BEST_PARS(COUNT,:)=PARS1(:)
          COUNT=COUNT+1
          IF(COUNT>MAX__ATTEMPTS) THEN 
             PRINT*,' '
             PRINT*,' Converged to local minimum : no more repeats! '
             EXIT
          END IF
          PRINT*,' '
          PRINT*,' Converged to local minimum : starting again... '
          GOTO 10
       END IF

       CALL GA_BREED          

    END DO

    ! Display on screen the best values obtained overall.

    II=MINLOC(GA__BEST_CHI)
    I=II(1)

    IF(COUNT>1) THEN
       PRINT*,' '
       PRINT*,'Best values :'
    END IF

    PRINT*,' '
    WRITE (*,'(A13,F11.3)') 'chisq', GA__BEST_CHI(I)*(NWOBS-NP)   
    WRITE (*,'(A13,F11.5)') 'reduced chisq', GA__BEST_CHI(I)
    WRITE (*,'(6A14)')  ((CPARS(J)),J=1,6)
    WRITE (*,'(6(F9.3,6x))')((GA__BEST_PARS(I,K)),K=1,6)
    ! WRITE (*,'(6(E9.4,5x))')((DELTA_P(K)-PARS0(K))/(GA__ALPHA**GA__SUBSTRING),K=1,6)  
    WRITE (*,'(6A14)')  ((CPARS(J)),J=7,12)
    WRITE (*,'(6(F9.3,6x))')((GA__BEST_PARS(I,K)),K=7,12) 

  END SUBROUTINE SFIT_GENETIC


  !  Decodes one parent string to determine its parameters

  SUBROUTINE GA_CALCPARAM

    INTEGER :: START   ! starting position for each substring
    REAL :: Y  ! number between 0 and 1 determining the value of the parameter                   

    PARS=PARS0  ! sets the variable parameters to lower range values and 
    ! sets the rest of the parameters to their constant values 

    DO J=1,GA__NPVAR
       START=GA__SUBSTRING*(J-1)    ! starting position for decoding each substring               
       Y=0.0
       DO N=1,GA__SUBSTRING
          Y=Y+REAL(GA__PARENT(I,START+N))*BITVAL(N) ! Y is a real number between 0 and 1                                               
       END DO
       K=GA__NPLOC(J)             
       PARS(K)=PARS0(K)+Y*(DELTA_P(K)-PARS0(K)) ! Evaluates the value of the parameter         
    END DO

    PARS1=PARS

  END SUBROUTINE GA_CALCPARAM




  SUBROUTINE GA_BREED
    ! Creates a new generation using tournament, fitness proportionate or rank selection


    INTEGER :: GA__SON(GA__POPS,GA__STRING) ! next generation
    INTEGER :: GA__POT__DAD(GA__CON)   ! potential parents
    INTEGER :: GA__DAD(2,GA__STRING)   ! array for 2 chosen parents
    INTEGER :: GA__CROSSPOS            ! position of crossover
    INTEGER :: GA__WINNER              ! suitability of potential dad
    REAL :: GA__FIT(0:GA__POPS)     ! an array which gives the cumulative                                    	              
    ! fitness of each individual 
    REAL :: X                       ! random number
    REAL :: GA__RANK(GA__POPS)      ! array giving the rank of each individual 
    ! according to suitability.
    REAL :: MAXVALUE                ! Furthest distance from 1 for all the 
    ! reduced chisq in current generation
    REAL :: SUMFIT                  ! Scaling factor for random number  
    REAL :: MINN                    ! Rank equation parameter
    REAL :: MAXX                    ! Rank equation parameter
    REAL :: TOT                     ! Scaling factor for random number 
    REAL :: GA__PROB(0:GA__POPS) ! array giving cumulative probability of 
    ! an individual being chosen based on rank                                					      



    IF (GA__ELITE(1:2)=='no') THEN   ! Keep no individuals deliberately if 
       GA__KEPT=0                    ! elitism is switched off.
    END IF


    ! This do loop ranks the current population.  The individual with the best
    ! suitability is given the highest rank (=GA__POPS) while the individual with
    ! the lowest suitability is given the lowest rank (=1).	
    ! Method : For individual 'x' in the population, 1 is added to its rank each 
    ! time an individual is found with a suitability greater than or equal than it.  
    ! (i.e. each time an individual is found with a reduced chi-squared value 
    ! not closer to 1 than x's reduced chi-squared value)     

    IF(GA__ELITE(1:3)=='yes' .OR. GA__SELECT(1:4)=='rank') THEN     
       GA__RANK=0                                              
       DO J=1,GA__POPS                         
          DO K=1,GA__POPS                         
             IF(GA__SUIT(K)>=GA__SUIT(J)) THEN
                GA__RANK(J)=GA__RANK(J)+1
             ENDIF
          ENDDO !(J)
       ENDDO !(K)
    END IF

    IF(GA__ELITE(1:3)=='yes') THEN         

       K=-1
       I=1

       ! Locate the individual(s) with the highest rank and make them
       ! a son.  If more are needed, reduce the rank until the next 
       ! the individual(s) with the next best rank are found.

       main : DO 
          K=K+1                        
          one : DO J=1,GA__POPS
             IF(GA__RANK(J)==MAXVAL(GA__RANK)-K) THEN  
                GA__SON(I,:)=GA__PARENT(J,:)
                I=I+1 
                IF(I>GA__KEPT) EXIT main
             END IF
          END DO one
          CYCLE    
       END DO main

    END IF

    ! If selection method if fitness-prop, then a fitness array is needed.
    ! The raw fitness for individual I is defined as 
    ! raw fitness = maximum value of reduced chisq in the generation
    !                 - reduced chisq for individual I
    ! Thus the greater the raw fitness, the better the individual.
    ! The GA__FIT array is actually an array of cumulative fitness, so that when
    ! a random number is chosen and scaled, it corresponds to a particular 
    ! individual.

    IF(GA__SELECT(1:7)=='fitness') THEN
       MAXVALUE=MAXVAL(GA__SUIT)
       GA__FIT(0)=0.0
       DO I=1,GA__POPS
          GA__FIT(I)=GA__FIT(I-1)+(MAXVALUE-GA__SUIT(I))  
       END DO
       SUMFIT=GA__FIT(GA__POPS)
    END IF

    ! Similar to above, but a cumulative probability array is used, based on 
    ! the linear ranking method proposed by Baker (1985).   

    IF(GA__SELECT(1:4)=='rank') THEN
       MAXX=2.0                      ! Must be between 1 and 2
       MINN=2-MAXX
       GA__PROB(0)=0.0 
       DO J=1,GA__POPS
          GA__PROB(J)=GA__PROB(J-1)+(MINN+(MAXX-MINN)*(GA__RANK(J)-1)/(GA__POPS-1))/GA__POPS 
       END DO
       TOT=0   
       DO J=1,GA__POPS 
          TOT=TOT+J
       END DO
    END IF

    DO I=1+GA__KEPT,GA__POPS          ! each mating results in only 1 offspring

       IF (GA__SELECT(1:10)=='tournament') THEN       ! tournament selection

          ! Choose two parents 
          DO J=1,2
             DO K=1,GA__CON                ! randomly select contestants
                CALL RANDOM_NUMBER(x)
                GA__POT__DAD(K)=NINT(x*(GA__POPS-1))+1
             ENDDO  !(K)

             GA__DAD(J,:)=GA__PARENT(GA__POT__DAD(1),:)  ! first assumes the throne
             GA__WINNER=GA__SUIT(GA__POT__DAD(1))

             DO K=2,GA__CON                        ! tournament begins
                IF (GA__SUIT(GA__POT__DAD(K))<GA__WINNER) THEN
                   GA__DAD(J,:)=GA__PARENT(GA__POT__DAD(K),:)
                   GA__WINNER=GA__SUIT(GA__POT__DAD(K))
                ENDIF
             ENDDO !(K)

          ENDDO    !(J)                           ! two parents chosen


       ELSEIF(GA__SELECT(1:7)=='fitness') THEN      
          ! fitness proportionate selection
          ! Choose two parents 
          DO J=1,2                          
             CALL RANDOM_NUMBER(x)      
             ! Scale random number so that it corresponds to an individual in the fitness array    
             x=x*SUMFIT                         
             DO K=1,GA__POPS
                IF(GA__FIT(K-1)<x .AND. GA__FIT(K)>=x) THEN
                   GA__DAD(J,:)=GA__PARENT(K,:)
                   EXIT 
                ENDIF
             ENDDO

          ENDDO  !(J)


       ELSEIF(GA__SELECT(1:4)=='rank') THEN    ! rank selection

          ! Choose two parents            
          DO J=1,2

             CALL RANDOM_NUMBER(x)

             ! Scale random number so that it corresponds to an individual in the
             ! probability array. 
             ! (If there are no joint ranks, then SUM(GA__RANK)/TOT = 1) 
             x=x*(SUM(GA__RANK))/TOT  

             DO K=1,GA__POPS
                IF(GA__PROB(K-1)<x .AND. GA__PROB(K)>=x) THEN    
                   GA__DAD(J,:)=GA__PARENT(K,:)
                   EXIT  
                ENDIF
             ENDDO

          ENDDO !(J)

       ENDIF

       !! create an offspring !!

       CALL RANDOM_NUMBER(x)
       IF (x<GA__CROSS) THEN                   ! probability of crossover
          CALL RANDOM_NUMBER(x)
          GA__CROSSPOS=NINT(x*(GA__STRING-2))+1
          GA__SON(I,1:GA__CROSSPOS)=GA__DAD(1,1:GA__CROSSPOS)
          GA__SON(I,GA__CROSSPOS+1:GA__STRING)=GA__DAD(2,GA__CROSSPOS+1:GA__STRING)
       ELSE
          GA__SON(I,:)=GA__DAD(1,:)  ! one parent replicates into next gen       
       ENDIF

       !! mutate the offspring !!
       DO N=1,GA__STRING
          CALL RANDOM_NUMBER(x)
          IF (x<GA__MUT) THEN         ! probability of mutation
             GA__SON(I,N)=NINT(x*GA__ALPHA)        ! mutate
          ENDIF
       ENDDO !(N)

    ENDDO !(I)

    GA__PARENT=GA__SON        ! the offspring become the next generation

  END SUBROUTINE GA_BREED




  SUBROUTINE SYNTH_FORMAL
    !  Compute formal solution for specified parameters

    REAL    :: SSSS, SSS2    ! integrals from trapezoidal quadrature
    INTEGER :: SNP           ! Synonym for SYN__NPAR
    REAL,    DIMENSION(NAMAX) :: SYN__PARS ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__PAR0 ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__DPAR ! variable amount 
    REAL,    DIMENSION(NAMAX) :: SYN__DRVS ! derivatives 

    !! there appears to be a bug in this routine
    !! it did not work during my analysis of V2076 - hst data
    write (*,*) "SYNTH_FORMAL ** Warning ** probable bug in this routine "

    !  Collect parameters
    SNP = SYN__NPAR
    SYN__PARS = SYN__ABUN
    SYN__DPAR = SYN__DABU

    !  Clear out saved values in SPEC_SYNTH using NPAR = 0
    SYN__DRVS = -99.
    CALL SPEC_SYNTH (WOBS(1),SYN__PARS,FFIT(1),SYN__DRVS,0)

    !  Report initial parameters and which are variable
    !
    WRITE (*,'(/,1X,A)') 'SYNTH_FORMAL'  
    WRITE (*,*) ' Initial parameters '
    WRITE (*,'(0P,20I7)')   (SYN__ELID(N),N=1,SNP)
    WRITE (*,'(0P,20F7.3)') (SYN__PARS(N),N=1,SNP)
    WRITE (*,*) 

    !  Compute formal solution
    SYN__DRVS = -99.
    DO NW = 1,NWOBS
       CALL SPEC_SYNTH (WOBS(NW),SYN__PARS,FFIT(NW),SYN__DRVS,SNP)
    END DO

    !  Compute mean variance on fluxes from mean scatter around fit
    DFIT(1:NW) = (FOBS(1:NW) - FFIT(1:NW))**2
    CALL QD_TRAP_R (WOBS,DFIT,SOBS,1,NWOBS, SSS2, SSSS)
    SSS2 = SQRT (SSS2 / (WOBS(NWOBS)-WOBS(1)) )
    ! SOBS = SSS2 -- line deleted 2018.x.08 -- did not understand what it did.

    !  Report result to sdtout
    WRITE (*,'(//,25X,10(I7))') (SYN__ELID(I),I=1,SNP-1)
    WRITE (*,'(A25)') OBSFILE
    WRITE (*,'(0P13F7.2)') (SYN__PARS(I),I=1,SNP), SSS2

    !  Report result to logfile
    WRITE (LOGFILE,'(//,25X,10(I7))') (SYN__ELID(I),I=1,SNP-1)
    WRITE (LOGFILE,'(A25)') OBSFILE
    WRITE (LOGFILE,'(0P13F7.2)') (SYN__PARS(I),I=1,SNP), SSS2

  END SUBROUTINE SYNTH_FORMAL




  SUBROUTINE SYNTH_AMOEBA
    !  Multidimensional downhill simplex method (AMOEBA)

    !  ** I do not trust what this is doing in terms of free parameters 
    !  ** it seems to pass the whole composition vector into AMP__P, 
    !  ** whereas this should only contain variable components of SYN__ELID
    !  >> major logic reorganisation needed <<

    !  3.v.19 *** failing with chi^2 run time error (nr_chisq)

    REAL :: SSSS, SSS2    ! integrals from trapezoidal quadrature

    INTEGER :: SNP ! Synonym for SYN__NPAR
    INTEGER :: AMB__MFIT ! Counter for truly free paramters
    REAL,    DIMENSION(NAMAX) :: SYN__PARS ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__PAR0 ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__DPAR ! variable amount 
    REAL,    DIMENSION(NAMAX) :: SYN__DRVS ! derivatives 



    !      Set up initial guesses
    SNP = SYN__NPAR
    SYN__PARS = SYN__ABUN
    SYN__DPAR = SYN__DABU

    !      Clear out saved values in SPEC_SYNTH using NPAR = 0
    SYN__DRVS = -99.
    CALL SPEC_SYNTH (WOBS(1),SYN__PARS,FFIT(1),SYN__DRVS,0)

    !      Determine which parameters are to be used in solution
    AMB__MFIT = 0
    DO N = 1,SNP
       IF ( SYN__DPAR(N).GT.1.E-9 ) AMB__MFIT = AMB__MFIT + 1
    ENDDO

    !  Report initial parameters and which are variable
    !
    WRITE (*,'(/,1X,A)') 'SYNTH_AMOEBA'  
    WRITE (*,*) ' Initial parameters '
    WRITE (*,'(0P,20I7)')   (SYN__ELID(N),N=1,SNP)
    WRITE (*,'(0P,20F7.3)') (SYN__PARS(N),N=1,SNP)
    WRITE (*,'(0P,20F7.3)') (SYN__DABU(N),N=1,SNP)
    WRITE (*,'(A, I4, A)') ' Solving for ', AMB__MFIT, 'unknowns'
    WRITE (*,*)

    ALLOCATE (AMB__P(SNP+1,SNP))
    ALLOCATE (AMB__X(SNP))
    ALLOCATE (AMB__Y(SNP+1))

    !      Set simplex vertices to starting guesses
    DO I=1,SNP
       AMB__P(1:SNP+1,I) = SYN__PARS(I) * (1+SYN__DPAR(I)) 
       AMB__P(I+1,I)  = SYN__PARS(I) 
    END DO

    !      Evaluate function at corners
    DO I = 1,SNP+1
       AMB__X(1:SNP) = AMB__P(I,1:SNP)
       SCOVAR = -99.
       CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS,  AMB__X,NPAR, CHISQ,SCOVAR,NAMAX, SPEC_SYNTH)
       AMB__Y(I) = CHISQ
    END DO

    !      Find solution
    CALL NR_AMBCHSQ (AMB__P(1:SNP+1,1:SNP), AMB__Y(1:SNP+1), &
         SNP+1, SNP, SNP, AMB__TOL, AMB__ITER, &
         SPEC_SYNTH, WOBS, FOBS, SOBS, NWOBS)
    DO I=1,SNP 
       SYN__PARS(I) = AMB__P(I+1,I)
    END DO

    !      Compute mean variance on fluxes from mean scatter around fit
    SYN__DRVS = -99.
    DO NW = 1,NWOBS
       CALL SPEC_SYNTH (WOBS(NW),SYN__PARS,FFIT(NW),SYN__DRVS,SNP)
    END DO
    DFIT(1:NW) = (FOBS(1:NW) - FFIT(1:NW))**2
    CALL QD_TRAP_R (WOBS,DFIT,SOBS,1,NWOBS, SSS2, SSSS)
    SSS2 = sqrt (SSS2 / (WOBS(NWOBS)-WOBS(1)) )
    SOBS = SSS2   !! what does this do ?? 2018.x.08 ??

    !      Reevaluate final solution (to get covariances)  
    !!     Use of derivative weighted sigma may be inappopriate
    SCOVAR = 0.
    CALL NR_CHISQ (WOBS,FOBS,SOBS,NWOBS,  SYN__PARS,SNP, CHISQ,SCOVAR,NAMAX, SPEC_SYNTH)
    !      Invert covariance matrix to find standard errors
    AMB__X(1:SNP) = 1. 
    CALL NR_GAUSSJ (SCOVAR(1:SNP,1:SNP),SNP,SNP,AMB__X(1:SNP),1,1)

    !     Write header line
    WRITE (*,'(//,25X,10(I7,A7))') (SYN__ELID(I),'  +/-  ',I=1,SNP-1),0,'   +/- ',0,' chi^2 '
    WRITE (*,'(A25,0P13F7.2)') &
         OBSFILE, (SYN__PARS(I),SQRT(SCOVAR(I,I)),I=1,SNP), CHISQ/(NWOBS-SNP)

    WRITE (LOGFILE,'(//,25X,10(I7,A7))') (SYN__ELID(I),'  +/-  ',I=1,SNP-1),0,'   +/- ',0,' chi^2 '
    WRITE (LOGFILE,'(A25,0P13F7.2)') &
         OBSFILE, (SYN__PARS(I),SQRT(SCOVAR(I,I)),I=1,SNP), CHISQ/(NWOBS-SNP)

    DEALLOCATE (AMB__P)
    DEALLOCATE (AMB__X)
    DEALLOCATE (AMB__Y)


  END SUBROUTINE SYNTH_AMOEBA




  SUBROUTINE SYNTH_LEVENBURG

    !  Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)

    INTEGER :: SNP ! Synonym for SYN__NPAR
    REAL,    DIMENSION(NAMAX) :: SYN__PARS ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__PAR0 ! synth parameters
    REAL,    DIMENSION(NAMAX) :: SYN__DPAR ! variable amount 
    REAL,    DIMENSION(NAMAX) :: SYN__DRVS ! derivatives 


    ! Chi-squared minimisation control variables.
    INTEGER, DIMENSION(NAMAX) :: LMQ__LISTA = 0    ! Regression control array
    REAL, DIMENSION(NAMAX,NAMAX) :: LMQ__COVAR = 0.0   ! Regression control array
    REAL, DIMENSION(NAMAX,NAMAX) :: LMQ__ALPHA = 0.0   ! Regression control array
    INTEGER   :: LMQ__MFIT = 0           ! Number of coefficients in fit
    REAL      :: LMQ__ALLAST
    REAL      :: LMQ__ALAMDA
    REAL      :: LMQ__DPAR, LMQ__DPAR0   ! Measures of convergence
    REAL      :: LMQ__CHSQ0 = 1.0E38    
    INTEGER, PARAMETER :: LMQ__MAXITER = 10      ! Maximum iterations
    !!    REAL, PARAMETER :: LMQ__DCONV  =  1.E-4      ! Convergence criterion

    !                            Initialize

    !  Local values for SYN variables
    SNP       = SYN__NPAR
    SYN__PARS = SYN__ABUN
    SYN__DPAR = SYN__DABU

    !      Determine which parameters are to be used in solution
    LMQ__MFIT = 0
    DO N = 1,SNP
       IF ( SYN__DPAR(N).GT.1.E-9 ) THEN
          LMQ__MFIT = LMQ__MFIT + 1
          LMQ__LISTA(LMQ__MFIT) = N
       ELSE
          LMQ__LISTA(SNP+1+LMQ__MFIT-N) = N
       ENDIF
    ENDDO

    !  Report initial parameters and which are variable
    !
    WRITE (*,'(/,1X,A)') 'SYNTH_LEVENBURG:  0 '  
    WRITE (*,*) ' Initial parameters '
    WRITE (*,'(0P,20I8)')   (SYN__ELID(N),N=1,SNP)
    WRITE (*,'(0P,20F8.3)') (SYN__PARS(N),N=1,SNP)
    WRITE (*,'(0P,20F8.3)') (SYN__DABU(N),N=1,SNP)
    WRITE (*,'(A, 40I3)') ' Solving for: ',LMQ__LISTA(1:LMQ__MFIT)
    WRITE (*,'(A, 40I3)') ' representing Z= ',SYN__ELID(LMQ__LISTA(1:LMQ__MFIT))
    WRITE (*,'(A, 40I3)') ' Fixed   for: ',LMQ__LISTA(SNP:LMQ__MFIT+1:-1)
    WRITE (*,*)



    ! Initialize to establish weights
    SYN__DRVS = -99.
    DO NW = 1,NWOBS
       CALL SPEC_SYNTH (WOBS(NW),SYN__PARS,FFIT(NW),SYN__DRVS,SNP)
    ENDDO

    ! 10.vi.2021 I don't see how derivatives were used to affect the weights.
    ! Would expect to multiply 1./SOBS by max(SYN_DRVS). Thats not how it
    ! works ... coefficients in nr_mrqcof are wt*dyda

    !      Initial call to chi-squared minimisation engine.
    WRITE (*,'(/,1X,A)') 'SYNTH_LEVENBURG:  1'
    WRITE (*,'(1X,A)')   'Solving for elemental abundances and microturbulent, rotational and radial velocities'  
    WRITE (*,'(0P,20I8)')   (SYN__ELID(N),N=1,SNP)
    WRITE (*,'(0P,20F8.3)') (SYN__PARS(N),N=1,SNP)
    
    SYN__DRVS = 0.
    SYN__PAR0 = SYN__PARS
    LMQ__DPAR = 0.; LMQ__DPAR0 = 999.
    LMQ__ALAMDA = -0.01
    LMQ__ALLAST = LMQ__ALAMDA
    CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, SYN__PARS,SNP, &
         LMQ__LISTA,LMQ__MFIT, &
         LMQ__COVAR,LMQ__ALPHA,NAMAX, CHISQ, SPEC_SYNTH, LMQ__ALAMDA)


    !      Announce calculations
    WRITE (*,'(/,1X,A)') 'SYNTH_LEVENBURG:  1'  
    WRITE (*,'(A,I2,4(A,1P,1E10.3))') ' Iter:',1,': chisq =',CHISQ

    !                             Iterate

    LMQ__ALAMDA = 0.2
    LMQ__CHSQ0 = CHISQ
    SYNTH_LMQ: DO I = 2,LMQ__MAXITER

       WRITE (*,'(/,1X,A,I2)') 'SYNTH_LEVENBURG: ', I

       !        Next chi-squared minimization step
       LMQ__ALLAST = LMQ__ALAMDA
       CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, SYN__PARS,SNP, &
            LMQ__LISTA,LMQ__MFIT, &
            LMQ__COVAR,LMQ__ALPHA,NAMAX, CHISQ, SPEC_SYNTH, LMQ__ALAMDA)

       !        Test for convergence
       LMQ__DPAR = 0.
       DO N = 1,LMQ__MFIT
          M = LMQ__LISTA(N)
          LMQ__DPAR = LMQ__DPAR + (SYN__PAR0(M) - SYN__PARS(M))**2
       ENDDO
       LMQ__DPAR = SQRT(LMQ__DPAR)
       SYN__PAR0 = SYN__PARS

       !        Report results.
       WRITE (*,'(0P,20F8.3)') (SYN__PARS(N),N=1,SNP)
       WRITE (*,'(A,I2,4(A,1P,1E10.3))') ' Iter:',I,' lmq__dpar =',LMQ__DPAR,& 
            ': lmq__tol =',LMQ__TOL,': chisq =',CHISQ,': lmq__chsq0 =',LMQ__CHSQ0

       ! First condition fails if LMQ method gives exact solution.
       ! I presume it was meant as an error trap, but seems to be preventing completion
       ! moinitor -- csj 24.4.16
       !! IF ( (LMQ__DPAR < LMQ__TOL .AND. LMQ__DPAR>0.) &
       !!      & .OR. CHISQ > LMQ__CHSQ0 - LMQ__TOL  ) EXIT 

       IF ( LMQ__DPAR < LMQ__TOL ) WRITE(*,'(A)') ' dpar < tol : converged '
       IF ( CHISQ > LMQ__CHSQ0 + LMQ__TOL  ) WRITE(*,'(A)') ' chi^2 > chi0^2 - tol : diverging '
       IF ( LMQ__DPAR < LMQ__TOL ) EXIT
       ! IF ( CHISQ > LMQ__CHSQ0 + LMQ__TOL  ) EXIT  !! emergency exit 
       LMQ__DPAR0 = LMQ__DPAR
       LMQ__CHSQ0 = CHISQ

    END DO  SYNTH_LMQ

    !                             Complete

    !      Final calls to find errors and save results

    ! require derivatives for all parameters in fit to avoid singular covariance matrix
    !    SYN__DABU(1:NP) = 0.1 

    ! compute covariance matrix
    CALL NR_MRQMIN (WOBS,FOBS,SOBS,NWOBS, SYN__PARS,SNP, &
         LMQ__LISTA,LMQ__MFIT, & 
         LMQ__COVAR,LMQ__ALPHA,NAMAX, CHISQ, SPEC_SYNTH, 0.0)

    ! now fetch the complete formal solution
    DO NW = 1,NWOBS
       CALL SPEC_SYNTH (WOBS(NW),SYN__PARS,FFIT(NW),SYN__DRVS,SNP)
    ENDDO


    !  Return results
    SYN__ABUN = SYN__PARS

    !  Errors -- not returned
    !  SYN__DABU(1:SNP) = [ (SQRT(LMQ__COVAR(N,N)), N=1,SNP) ]

    !      Report results
    WRITE (*,*)
    WRITE (*,'(0P,20I8)')   (SYN__ELID(N),N=1,SNP)
    WRITE (*,'(0P,20F8.3)') (SYN__PARS(N),N=1,SNP)
    WRITE (*,'(0P,20F8.3)') (SQRT(LMQ__COVAR(N,N)),N=1,SNP)
    WRITE (*,'(A25,0P13F7.2)') &
         OBSFILE, CHISQ/(NWOBS-SNP)
    WRITE (*,*)

    WRITE (LOGFILE,'(2(1X,0P,10I7))')   (SYN__ELID(N),N=1,SNP)
    WRITE (LOGFILE,'(2(1X,0P,10F7.3))') (SYN__PARS(N),N=1,SNP)
    WRITE (LOGFILE,'(2(1X,0P,10F7.3))') (SQRT(LMQ__COVAR(N,N)),N=1,SNP)
    WRITE (LOGFILE,'(A25,0P13F7.2)') &
         OBSFILE, CHISQ/(NWOBS-SNP)

    ! copy error matrix for reporting
    SCOVAR(1:SNP,1:SNP) = LMQ__COVAR(1:SNP,1:SNP)

  END SUBROUTINE SYNTH_LEVENBURG



END PROGRAM SFIT



