#!/bin/sh
#
# Spectrum - Shell script front-end to Spectrum
#
# Authors:
#    C. Simon Jeffery (csj@arm.ac.uk) - Original csh version
#    Chris Winter (cwr@arm.ac.uk)     - Bourne sh version
#


###
# Clean up links and other cruft
###
clean_up()
{
	rm -f BALMER EPSILON HEI* PROF_* VCS* WAVE
	rm -f PEACH_TABLES LINES MODEL SPECTRUM.TMP OPDATA OPELEVELS FEDATA
        rm -f FLAMBDA.TMP FCONTIN.TMP
}

###
# Clean up links and other cruft then exit
###
clean_up_exit()
{
	clean_up
	exit 1
}


###
# How to use this script
###
print_usage()
{
        echo ""
        echo "Spectrum- Usage Information"
        echo "=========================="
        echo ""
        echo "Spectrum [models [linelist [control [label] ] ] ]"
        echo ""
	echo "  models   - File containing one or more models"
	echo "  linelist - File containing atomic and line data"
	echo "  control  - File containing program control data"
	echo "  label    - Label for output data files"
        echo ""
}

###
# Check if Sterne is actually installed, and look for
# the global data folder
###
sanity_check()
{
	# Try looking in LTECODES environment variable
        SPECTRUM=$LTECODES/bin/spectrum

        if [ ! -x "$SPECTRUM" ]
        then
                # Does whereis know where spectrum is? (i.e. is spectrum
                # installed in /usr/local/bin/spectrum)
	        WHEREIS=`whereis spectrum | awk '{print $2}'`

		if [ -x "$WHEREIS" ]
		then
		        SPECTRUM="$WHEREIS"		
		else
		        echo "Can't find Spectrum. Please set the LTECODES environment"
			echo "variable as described in the installation instructions."
			echo "Exiting..."
			exit 1
		fi
	fi


        # Now to look for the data directory

        # Set to null just incase
        GLOBAL_DATA=""

        # First, check in the default locations
        #
	# $LTECODES/share/osdata        - Environment variable
        # /usr/local/share/osdata       - The default installer location
        # /usr/local/osdata             - Possible secondary location
        # /usr/osdata                   - Possible secondary location
        # /opt/osdata                   - Possible secondary location
        # ~/osdata                      - Check user's home directory
        # ./osdata                      - Check current directory
        # ../osdata                     - Getting desperate
        # ../../osdata                  - Very desperate
        # ./data                        - Last chance effort - check if we're
        #                                 actually *in* osdata
        for i in \
		 $LTECODES/share/osdata \
                 /usr/local/share/osdata \
                 /usr/local/osdata \
                 /usr/osdata \
                 /opt/osdata \
                 ~/osdata \
                 ./osdata \
                 ../osdata \
                 ../../osdata \
                 ~/data \
                 ./data
        do
                # Is this it?
                if [ -d "$i" ]
                then
                        GLOBAL_DATA="$i"

                        # Ok, the last case in for loop is special
                        # so check for it
                        if [ "$i" = "./data" ]
                        then
                                GLOBAL_DATA=./
                        fi

			# Found the directory, so exit loop
			break
                fi
        done

        # Was our search successful?
        if [ ! -d "$GLOBAL_DATA"  ]
        then
                echo "Can't find the global data directory. Please set the LTECODES"
                echo "environment variable as described in the installation"
                echo "instructions. Exiting..."
                exit 1
        fi
}


###
# Ask user for Spectrum model file(s)
###
get_model()
{
   echo -n 'File containing model(s): '
   read MODEL

}


###
# Check that model file exists.
###
check_model()
{
        # Set to zero just in case
        MODEL_ERROR=0

        # While file $MODEL doesn't exist and $MODEL_ERROR < 5
        while [ -z "$MODEL" -a ! -r "$MODEL" -a ! -r "$GLOBAL_DATA/models/$MODEL" -a "$MODEL_ERROR" -lt 5 ]
        do
                echo -n "Can't find $MODEL. Please try again: "

                read MODEL
                MODEL_ERROR=`expr $MODEL_ERROR + 1`
        done

        if [ "$MODEL_ERROR" -ge 5 ]
        then
                # Something is wrong. Bail out
                echo "Number of attempts exceeded. Exiting..."
                clean_up
                exit 1
	elif [ ! -r "$MODEL" ]
	then
		# Just checking that $MODEL points to the right location!
		# MODEL="$GLOBAL_DATA/qub_models/$MODEL"
		MODEL="$GLOBAL_DATA/models/$MODEL"
        fi
}


###
# Ask user for line list
###
get_lines()
{
   echo -n 'File containing linelist (<CR> for default): '
   read LINES

}


###
# Check that line list exists.
###
check_lines()
{
        if [[ $LINES = "-"] || [$LINES = "" ]]
	then
		LINES="$GLOBAL_DATA/spectrum_data/lines_def.d"
	fi

	if [ ! -z "$LINES" ]
	then
        	# Set to zero just in case
        	LINES_ERROR=0

			

        	# While file $LINES doesn't exist and $LINES_ERROR < 5
        	while [ ! -r "$LINES" -a "$LINES_ERROR" -lt 5 ]
        	do
                	echo -n "Can't find $LINES. Please try again: "

                	read LINES
                	LINES_ERROR=`expr $LINES_ERROR + 1`
        	done

        	if [ "$LINES_ERROR" -ge 5 ]
        	then
                	# Something is wrong. Bail out
                	echo "Number of attempts exceeded. Exiting..."
                	clean_up
                	exit 1
        	fi
	else
		# Ok, we're using the default line list.
		# Check that it actually exists before going any futher.
		LINES="$GLOBAL_DATA/spectrum_data/lines_def.d"

		if [ ! -r "$LINES" ]
		then
			echo "Error! Can't find default line list. Exiting ..."
			clean_up
			exit 1
		fi
	fi
}


###
# Ask user for control file
###
get_control()
{
   echo -n 'File containing Spectrum control input (<CR> for terminal): '
   read CONTROL

}


###
# Check that control file exists.
###
check_control()
{
	if [ ! -z "$CONTROL" ]
	then
        	# Set to zero just in case
        	CONTROL_ERROR=0

        	# While file $LINES doesn't exist and $LINES_ERROR < 5
        	while [ -z "$CONTROL" -a ! -r "$CONTROL" -a "$CONTROL_ERROR" -lt 5 ]
        	do
                	echo -n "Can't find $CONTROL. Please try again: "

                	read CONTROL
                	CONTROL_ERROR=`expr $CONTROL_ERROR + 1`
        	done

        	if [ "$CONTROL_ERROR" -ge 5 ]
        	then
                	# Something is wrong. Bail out
                	echo "Number of attempts exceeded. Exiting..."
                	clean_up
                	exit 1
        	fi
	fi
}


###
# Ask user for output label
###
get_label()
{
   echo -n 'Save output files as (<CR> for default): '
   read LABEL

}


###
# Ask user to input paramters
###
get_params()
{
        get_model
        check_model
        get_lines
        check_lines
        get_control
        check_control
	get_label
}


###
# Validate the user's parameters
###
check_params()
{
        case "$#" in
                "0")
                        get_params
                        ;;
                "1")
                        MODEL=$1
		        check_model
			get_lines
			check_lines
			get_control
        		check_control
			get_label
                        ;;
                "2")
                        MODEL=$1
        		check_model
                        LINES=$2
        		check_lines
			get_control
        		check_control
			get_label
                        ;;
                "3")
                        MODEL=$1
        		check_model
                        LINES=$2
        		check_lines
                        CONTROL=$3
        		check_control
			get_label
                        ;;
                "4")
                        MODEL=$1
        		check_model
                        LINES=$2
        		check_lines
                        CONTROL=$3
        		check_control
                        LABEL=$4
                        ;;
                *)
                        print_usage
                        exit 1
                        ;;
        esac
}


###
# Tidy files from previous run
###
tidy_up()
{
	# Rename spectrum.000
	if [ -e "spectrum.000" ]
	then
		echo ""
		echo "Saving spectrum.000 as spectrum.000%"
		mv spectrum.000 spectrum.000%
	fi

	# Rename dipso.001
	if [ -e "dipso.001" ]
	then
		echo "Saving dipso.001 as dipso.001%"
		mv dipso.001 dipso.001%
	fi
}


###
# Setup all the links before running Spectrum
###
setup_links()
{
	ln -s $MODEL MODEL

	ln -s $LINES LINES

	ln -s $GLOBAL_DATA/spectrum_data/balmer.d BALMER
	ln -s $GLOBAL_DATA/spectrum_data/epsilon.d EPSILON
	ln -s $GLOBAL_DATA/spectrum_data/he1_lines.d HEI_NIST
	ln -s $GLOBAL_DATA/spectrum_data/he1_stark.d HEI_STARK
	ln -s $GLOBAL_DATA/spectrum_data/he1_beauchamp.d HEI_BEAUCHAMP
	ln -s $GLOBAL_DATA/spectrum_data/prof_4026.d PROF_4026
	ln -s $GLOBAL_DATA/spectrum_data/prof_4388.d PROF_4388
	ln -s $GLOBAL_DATA/spectrum_data/prof_4471.d PROF_4471
	ln -s $GLOBAL_DATA/spectrum_data/prof_4921.d PROF_4921
	ln -s $GLOBAL_DATA/spectrum_data/heii_red.d HEII_RED
	ln -s $GLOBAL_DATA/spectrum_data/heii.d HEII
	ln -s $GLOBAL_DATA/spectrum_data/peach_tables PEACH_TABLES
#	ln -s $GLOBAL_DATA/spectrum_data/opwlgrid.d WAVE
	ln -s $GLOBAL_DATA/spectrum_data/grid05-500-1-900-25-5000.d WAVE
	ln -s $GLOBAL_DATA/spectrum_data/op_all.e OPELEVELS

	ln -s $GLOBAL_DATA/spectrum_data/vcslyman VCSLYMAN
	ln -s $GLOBAL_DATA/spectrum_data/vcsbalmer VCSBALMER
	ln -s $GLOBAL_DATA/spectrum_data/vcspaschen VCSPASCHEN

	ln -s $GLOBAL_DATA/opacity/op_mono OPDATA
	ln -s $GLOBAL_DATA/opacity/fe_op FEDATA

	ln -s spectrum.000 SPECTRUM.TMP
	ln -s spectrum.flux FLAMBDA.TMP
	ln -s spectrum.fcon FCONTIN.TMP
}


###
# Run Spectrum
###
run_program()
{
	if [ -z $CONTROL ]
	then
		# Terminal Input
		$SPECTRUM
	else
		# Control file input
		$SPECTRUM <$CONTROL
	fi
}


###
# Rename the output files with the user's chosen label
###
rename_output()
{
	# If $LABEL is empty, don't do 
	if [ ! -z "$LABEL" ]
	then
		if [ -e "spectrum.000" ]
		then 
			mv spectrum.000 $LABEL.000
		fi

		if [ -e "spectrum.flux" ]
		then 
			mv spectrum.flux $LABEL.flux
		fi

		if [ -e "spectrum.fcon" ]
		then 
			mv spectrum.fcon $LABEL.fcon
		fi

                for i in `ls dipso.*`
                do
                        EXT=${i#dipso.} # Grabs extension
                        mv $i $LABEL.$EXT
                done

		if [ -e "ATSCAN" ] 
		then
			mv ATSCAN $LABEL.grid
		fi

		if [ -e "fort.25" ]
		then
			mv fort.25 $LABEL.summary
		fi
	fi
}


###
# Main Program
###

# Catch any Ctl^C interrupts
trap clean_up_exit 2

MODEL=""
MODEL_ERROR=0

LINES=""
LINES_ERROR=0

CONTROL=""
CONTROL_ERROR=0

LABEL=""

SPECTRUM=""
SPEC_ERROR=0

GLOBAL_DATA=""
GD_ERROR=0

sanity_check

check_params $@
tidy_up
setup_links
run_program
rename_output
clean_up

exit 0
