PROGRAM SPECTRUM

  !+

  !  Program:
  !     SPECTRUM
 
  !  Version:
  !     5.2.1
 
  !  Language:
  !    Approaching Fortran95

  !  Purpose:
  !     Calculates LTE equivalent widths, line profiles
  !     and synthetic spectra for hydrogen, helium and
  !     metal lines and blends for hot star atmoospheres.
       
  !  History:

  !     5.2.1     C.S.Jeffery, Armagh       12-Aug-2020
  !     Accepts errors on equivalenth widtsh for computing error on line abundances
  !     EWABUND converted to f90 (but not GOTO cleaned)

  !     5.2.0     C.S.Jeffery, Armagh       18-Jan-2019
  !     Includes code for artificially inserting chemical stratification 
  !     Implements structures sterne output

  !     5.1.0     C.S.Jeffery, Armagh       13-Nov-2012
  !     Inclusion of code for reading in output from Bridger pulsation code
  !     and for computing dynamic spectra

  !     4.1.0     C.S.Jeffery, Armagh       29-Aug-2012
  !     Removal of obsolete QUB input features
  !     Upgrade to opacity library calls (for compatability with SterneOS)
  !     Other rationalisation towards f90 compliance

  !     4.0.3     C.S.Jeffery, Armagh       11-May-2007
  !     ...Very long...


  !  Authors:
  !     P.L. Dufton, Queens University Belfast
  !     D.J. Lennon, Queens University Belfast
  !     E.S. Conlon, Queens University Belfast
  !     C.S. Jeffery, Armagh Observatory
  !     N.T. Behara, Armagh Observatory + ...
  !
  !  Principal References:
  !
  !  Version 3
  !    C.S.Jeffery, V.M.Woolf, D.L.Pollacco, 2001. A&A 376, 497
  !    Time-resolved spectral analysis of the pulsating helium star V652 Her
     
  !     -


  !     data structure for sterne model atmosphere
  USE LTE_STRUCTURES
  USE LTE_WLGRID, ONLY : REFWL

  !     Global mathematical constants
  USE TAPM_PAR

  !     SPECTRUM global variables
  USE SPC_GLOBAL
  USE SPC_CONTROL
  USE SPC_PULSE_PUB, ONLY : PULS__OK, PULS__NSTART, PULS__NEND, PULS__NSKIP

  !     Double precision version        
  IMPLICIT DOUBLE PRECISION (A-H,O-Z)

  !     Global physical constants
  !  INCLUDE 'qub_constants.f'

  !     Global constants and variables
  !  INCLUDE 'common_all.f'

  !  Explicit Subroutine interfaces
  INTERFACE

     SUBROUTINE KEY_COM (PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_COM

     SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_GET

     FUNCTION NR_LOCATE (XX,X)
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XX
       DOUBLE PRECISION, INTENT(IN) :: X
       INTEGER :: NR_LOCATE
     END FUNCTION NR_LOCATE

     SUBROUTINE NR_POLINT (XA,YA,X,Y,DY)
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XA
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: YA
       DOUBLE PRECISION, INTENT(IN)               :: X
       DOUBLE PRECISION, INTENT(OUT) :: Y
       DOUBLE PRECISION, INTENT(OUT) :: DY
     END SUBROUTINE NR_POLINT

  END INTERFACE



  !     Version information
  CHARACTER (6)   :: SPC__VERSION = '5.2.2' 
  CHARACTER (10)  :: SPC__DATE = '2020.11.20' 
  CHARACTER (40)  :: SPC__AUTHOR = 'C.S.Jeffery (Armagh Observatory)' 
  CHARACTER (80)  :: SPC__WWW  = 'http://193.63.77.2:2805/~SJeffery/' // &
       'software_store/guide/spectrum.html'
  CHARACTER (80)  :: SPC__HELP = 'http://193.63.77.2:2805/~SJeffery/' // &
       'software_store/guide/spectrum.html'

  !     Command line control
  CHARACTER (140) :: COMMAND ! holder for current command line
  CHARACTER (16)  :: KEYWRD  ! holder for current command or keyword
  CHARACTER (LEN=16) :: PARCOM  ! command for parameter entry
  REAL            :: WAVM
  CHARACTER (LEN=2) :: CDBG  ! debugger keyword


  !     Filenames
  CHARACTER (120) :: F__LINES = 'LINES' !  Linelist
  CHARACTER (120) :: F__MODEL = 'MODEL' !  Model atmosphere
  CHARACTER (120) :: F__STATIC  !  Name of file containing static model envelope
  CHARACTER (120) :: F__MASSZ   !  Name of file containing dynamic mass zoning
  CHARACTER (120) :: F__HYDRO = 'HYDRO' !  Scaling for hydrodynamic atmosphere



  !     Local holder for current model atmosphere
  TYPE (MODEL_ATMOS) STN__MODEL

!!!!**** new modification (2017 - 5.2.0)
  !     Local holder for stratified abundances
  INTEGER :: XABU  ! Element to be stratified
  INTEGER :: NXABU ! number of depth points
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__TAUXABU !  optical depths for stratified abundances
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__STABU  ! depth-dependant abundances
!!!!****

  !     Local holders for microturbulent and radial velocities
  INTEGER                         :: NVRAD
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__VR_TAU ! optical depths for radial velocities 
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__VRAD ! depth-dependent radial velocities 
  INTEGER                         :: NVTRB = 1
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__VT_TAU ! optical depths for turbulence velocities 
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__VTRB ! depth-dependent for turbulence velocities 
  INTEGER                         :: NDTEMP = 0
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__DT_TAU ! optical depths for temperature perturbation scaling factor
  DOUBLE PRECISION, DIMENSION(KD) :: SPC__DTEMP ! depth-dependent temperature perturbation 


  !     Local holders for revised optical depth scales
  INTEGER :: SPC__TAUX      ! expansion factor for tau_scale      
  INTEGER :: SPC__NTAU      ! number of depths in new tau_scale      
  DOUBLE PRECISION, DIMENSION (KD) :: SPC__TAUN !  new tau scale
  DOUBLE PRECISION :: SPC__TAUW !  reference wavelength for new tau scale (A)


  !     Identifiers and fixed values for abundances
  INTEGER, PARAMETER        :: NAMAX = 90 ! max number of abundances
  INTEGER                   :: SPC__NABU        ! Number of abundances
  INTEGER, DIMENSION(NAMAX) :: SPC__ELID        ! element identifiers
  REAL,    DIMENSION(NAMAX) :: SPC__ABUN = -99. ! synth abundances
  REAL                      :: SPC__WTOL = 1.0  ! Tolerance for line identification

  !     .


  !     Define Debug output file if appropriate
  !     These values should be set by a "debug" coommand
  !     and this section should appear whenever the debug command is
  !     issued.
!  IF (BUGMN.OR.BUGWH.OR.BUGIN.OR.BUGLN.OR.BUGOP.OR.BUGAT.OR.&
!       BUGMD.OR.BUGOU.OR.BUGOT    ) THEN
     OPEN(UNIT=NDEBUG,FILE='SPECTRUM.DBG',STATUS='UNKNOWN')
     WRITE(NDEBUG,900)BUGMN,BUGWH,BUGIN,BUGLN,BUGOP,BUGAT,BUGMD,BUGOU,BUGOT
900  FORMAT(//5X,'BUGMN',5X,'BUGWH',5X,'BUGIN',5X,'BUGLN',5X,&
          'BUGOP',5X,'BUGAT',5X,'BUGMD',5X,'BUGOU',5X,'BUGOT'//9L10)      
!  ENDIF



  !     IOMODE = 0 :  command line data entry
  CALL SPC_STARTUP
  DO

     !     get command line
     CALL KEY_GET ( KEYWRD, COMMAND, 'spc>', ';', NREAD, I4 )
     CALL STR_LCASE ( KEYWRD )

     !  act on exit condition immediately
     IF (KEYWRD(1:3) == 'end' .OR. KEYWRD(1:4)  == 'exit' &
          .OR. KEYWRD(1:1)  == 'q')   THEN
        EXIT

        ! echo commands to the terminal
     ELSEIF (KEYWRD(1:4)=='echo') THEN

        CALL KEY_PC (COMMAND, PARCOM)
        IF (PARCOM(1:1) == "2" )                         I4 = 2
        IF (PARCOM(1:2) == "on"  .OR. PARCOM(1:1)=="1" ) I4 = 1
        IF (PARCOM(1:3) == "off" .OR. PARCOM(1:1)=="0" ) I4 = 0

        !  help
     ELSEIF (KEYWRD(1:4) == 'help') THEN 
        WRITE (*,*) 'See: '//SPC__HELP

        !     assign various flags and switches
     ELSEIF (KEYWRD(1:4) == 'nmod') THEN
        CALL KEY_PI (COMMAND, NMOD)

     ELSEIF (KEYWRD(1:6) == 'nsourc') THEN 
        CALL KEY_PI (COMMAND, NSOURCE)
        NSOURCE = MIN(NSOURCE,3)

     ELSEIF (KEYWRD(1:6) == 'wavmax') THEN
        CALL KEY_PR (COMMAND, WAVM)
        WAVMAX = WAVM

     ELSEIF (KEYWRD(1:4) == 'wtol') THEN
        CALL KEY_PD (COMMAND, WTOL)
        SPC__WTOL = WTOL

        !     line_threshold
     ELSEIF (KEYWRD(1:7) == 'line_th') THEN
        CALL KEY_PD (COMMAND, GFN_THRESHOLD)

        !     HeI Stark broadening data
     ELSEIF (KEYWRD(1:6) == 'nhedat') THEN
        NHEDEF = NHEDAT
        CALL KEY_PI (COMMAND, NHEDAT)
        IF ( NHEDAT /= NHEDEF ) CALL DP_ATHEI

        !     HeI Stark broadening -- impact approximation
     ELSEIF (KEYWRD(1:6) == 'nhesta') THEN
        CALL KEY_PI (COMMAND, NHESTARK)

        !     Opacity data
     ELSEIF (KEYWRD(1:6) == 'noptyp') THEN
        CALL KEY_PI (COMMAND, NOPTYP)

        IF (NOPTYP == 0) THEN
           WRITE (*,*) "NOPTYP = 0: using QUB opacities"
        ELSEIF (NOPTYP == 1) THEN
           WRITE (*,*) "NOPTYP = 1: using KIEL opacities"
        ELSEIF (NOPTYP == 2) THEN
           WRITE (*,*) "NOPTYP = 2: using OP opacities"
        ELSE
           WRITE (*,*) "Illegal value for NOPTYP:",NOPTYP
           WRITE (*,*) "0 = QUB opacities"
           WRITE (*,*) "1 = KIEL opacities (assumed)"
           WRITE (*,*) "2 = OP opacities"
           NOPTYP = 1
        ENDIF
        WRITE (*,*) "NOPTYP overriden by 'read_model'"

        !     Debugger
     ELSEIF (KEYWRD(1:5) == 'debug') THEN
        CALL KEY_PC (COMMAND, CDBG)
        IF ( CDBG == 'mn' ) BUGMN = .TRUE. 
        IF ( CDBG == 'wh' ) BUGWH = .TRUE. 
        IF ( CDBG == 'in' ) BUGIN = .TRUE. 
        IF ( CDBG == 'ln' ) BUGLN = .TRUE. 
        IF ( CDBG == 'op' ) BUGOP = .TRUE. 
        IF ( CDBG == 'at' ) BUGAT = .TRUE. 
        IF ( CDBG == 'md' ) BUGMD = .TRUE. 
        IF ( CDBG == 'ou' ) BUGOU = .TRUE. 
        IF ( CDBG == 'ot' ) BUGOT = .TRUE. 
        WRITE(NDEBUG,900)BUGMN,BUGWH,BUGIN,BUGLN,BUGOP,BUGAT,BUGMD,BUGOU,BUGOT



        !     ************************************************
        !     *  l i n e l i s t  ...                        *
        !     *                                              *
        !     *  specify linelist, default LINES             *
        !     ************************************************

     ELSEIF (KEYWRD(1:8) == 'linelist') THEN
        CALL KEY_PC (COMMAND, F__LINES)



        !     ************************************************
        !     *  m o d e l  ...                              *
        !     *                                              *
        !     *  specify model atmosphere, default MODEL     *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'model') THEN
        CALL KEY_PC (COMMAND, F__MODEL)



        !     ************************************************
        !     * p u l s   ...                                *
        !     *                                              *
        !     *  specify and read hydro pulsation model,     *
        !     *  requires 3 filenames (static, masszones and dynamic)  
        !     *  NO defaults, files must be NLPULSE outputs  *
        !     ************************************************

     ELSEIF (KEYWRD(1:11) == 'puls_static') THEN
        CALL KEY_PC (COMMAND, F__STATIC)
     ELSEIF (KEYWRD(1:10) == 'puls_massz') THEN
        CALL KEY_PC (COMMAND, F__MASSZ)
     ELSEIF (KEYWRD(1:10) == 'puls_hydro') THEN
        CALL KEY_PC (COMMAND, F__HYDRO)

     ELSEIF (KEYWRD(1:9) == 'puls_lims') THEN
        CALL KEY_PI (COMMAND, PULS__NSTART)
        CALL KEY_PI (COMMAND, PULS__NEND)
        CALL KEY_PI (COMMAND, PULS__NSKIP)
        write(*,*) puls__nstart,puls__nend,puls__nskip
        
     ELSEIF (KEYWRD(1:9) == 'read_puls') THEN
        write (*,*) f__static
        write (*,*) f__massz
        write (*,*) f__hydro
        CALL SPC_READPULS ( F__STATIC, F__MASSZ, F__HYDRO )
        PULS__NEND = PULS__NT


        !     ************************************************
        !     *  i n t e n s  ...                            *
        !     *                                              *
        !     *  data for specific intensities,              *
        !     *  computed if ntheta > 0                      *
        !     ************************************************

     ELSEIF (KEYWRD(1:6) == 'ntheta') THEN
        CALL KEY_PI (COMMAND, NTHETA)
        NTHETA = MIN(NTHETA,KTH)
        IF ( NTHETA > 0 .AND. NSOURCE /= 3 ) THEN
           WRITE (*,*) ' * Warning * '
           WRITE (*,*) ' nsource, ntheta = ',NSOURCE, NTHETA
           WRITE (*,*) ' Setting nsource = 3 for intensities '
           NSOURCE = 3 
        ENDIF

     ELSEIF (KEYWRD(1:7) == 'costmin')THEN
        CALL KEY_PD (COMMAND, COSTMIN)

     ELSEIF (KEYWRD(1:8) == 'costheta') THEN
        IF (NTHETA  <=  0) THEN 
           WRITE (*,*) ' Illegal value for NTHETA', NTHETA
           STOP
        ELSE 
           DO I=1,NTHETA
              CALL KEY_PD ( COMMAND, COSTHETA(I) )
              COSTHETA(I)=MAX(COSTHETA(I),COSTMIN)
           ENDDO
           WRITE (*,'(A,20F6.2)') 'COS(THETA) = ',COSTHETA(1:NTHETA) 
        ENDIF



        !     ************************************************
        !     *  v t u r b ...                               *
        !     *                                              *
        !     *  data for microturbulent velocities          *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'vturb') THEN 
        CALL KEY_PD (COMMAND, VT)
        NVTRB = 1
        !         Disperse microturbulent velocity
        QUB__VTRB = VT

     ELSEIF (KEYWRD(1:6) == 'nvturb') THEN 
        CALL KEY_PI (COMMAND, NVTRB)

     ELSEIF (KEYWRD(1:8) == 'vtrb_tau') THEN
        !     read in pairs of tau and vturb 
        !     have to map these onto actual tau in model atmosphere later
        IF (NVTRB <= 1) NVTRB = KD ! if NVTRB not defined, then use number given. 
        VTRB_READ: DO I=1,NVTRB
           CALL KEY_COM (COMMAND, 'tau, v_trb :', ';', NREAD, I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__VT_TAU(I))
           CALL KEY_PD (COMMAND, SPC__VTRB(I))
           IF (SPC__VT_TAU(I) <= 0.) THEN
              NVTRB = I - 1
              EXIT VTRB_READ 
           ENDIF
        ENDDO VTRB_READ
        VT = SPC__VTRB(1)
        !       Disperse initial microturbulent velocity to avoid empty arrays
        QUB__VTRB = VT




        !     ************************************************
        !     *  v r a d ...                                 *
        !     *                                              *
        !     *  data for radial velocities                  *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'vrad ') THEN 
        CALL KEY_PD (COMMAND, VRAD)
        NVRAD = 1

     ELSEIF (KEYWRD(1:5) == 'nvrad') THEN 
        CALL KEY_PI (COMMAND, NVRAD)

     ELSEIF (KEYWRD(1:8) == 'vrad_tau') THEN
        !     read in pairs of tau and vrad 
        !     have to map these onto actual tau in model atmosphere later
        IF (NVRAD <= 1) NVRAD = KD ! if NVRAD not defined, then use number given
        VRAD_READ: DO I=1,NVRAD
           CALL KEY_COM (COMMAND, 'tau, v_rad :', ';', NREAD, I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__VR_TAU(I))
           CALL KEY_PD (COMMAND, SPC__VRAD(I))
           IF (SPC__VR_TAU(I) <= 0.) THEN
              NVRAD = I - 1
              EXIT VRAD_READ 
           ENDIF
        ENDDO VRAD_READ
        VRAD = SPC__VRAD(1)


!!!!**** new modification (2017: 5.2.0)
        !     ************************************************
        !     *  n x a b u ...                               *
        !     *                                              *
        !     *  data for stratified abundances              *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'nxabu ') THEN 
        CALL KEY_PI (COMMAND, XABU)
        WRITE (*,*) 'Stratified element :', XABU

     ELSEIF (KEYWRD(1:8) == 'tau_xabu') THEN
        NXABU = 1
        !     read in pairs of tau and abundance 
        IF (NXABU <= 1) NXABU = KD ! if NXABU not defined, then use number given
        XABU_READ: DO I=1,NXABU
           CALL KEY_COM (COMMAND, 'tau, abu :', ';', NREAD, I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__TAUXABU(I))
           CALL KEY_PD (COMMAND, SPC__STABU(I))
           IF (SPC__TAUXABU(I) <= 0.) THEN
              NXABU = I - 1
              EXIT XABU_READ 
           ENDIF
        ENDDO XABU_READ
        WRITE(*,*) 'Number of optical depth points for stratification definition: ', NXABU

        CALL SPC_STRATMODEL(XABU,NXABU,SPC__TAUXABU,SPC__STABU)
!!!!****end modification(2017)

        !     ************************************************
        !     *  d t _ t a u                                 *
        !     *                                              *
        !     *  data for adjusting local temperature        *
        !     ************************************************

     ELSEIF (KEYWRD(1:8) == 'dt_tau') THEN
        !     read in pairs of tau and delta t (fractional) 
        !     have to map these onto actual tau in model atmosphere later
        IF (NDTEMP <= 1) NDTEMP = KD ! if NVRAD not defined, then use number given
        DT_READ: DO I=1,NDTEMP
           CALL KEY_COM (COMMAND, 'tau, dT :', ';', NREAD, I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__DT_TAU(I))
           CALL KEY_PD (COMMAND, SPC__DTEMP(I))
           IF (SPC__DT_TAU(I) <= 0.) THEN
              NDTEMP = I - 1
              EXIT DT_READ 
           ENDIF
        ENDDO DT_READ



        !     **************************************************
        !     *  t a u _ e x p                                 *
        !     *                                                *
        !     *  expand optical depth scale by integer factor  *
        !     **************************************************

     ELSEIF (KEYWRD(1:7) == 'tau_exp') THEN 
        CALL KEY_PI (COMMAND, SPC__TAUX)

        X = SPC__TAUX
        SPC__NTAU = SPC__TAUX * (NDEPTH-1) + 1
        TAU(1:NDEPTH) = DLOG10 ( TAU(1:NDEPTH) )
        SPC__TAUN(SPC__NTAU) = TAU(NDEPTH)
        DO I = 1,NDEPTH-1
           II = SPC__TAUX * (I-1) + 1
           SPC__TAUN(II) = TAU (I)
           DO J = 1, SPC__TAUX-1
              SPC__TAUN(II+J) = TAU(I) + FLOAT(J)/X * (TAU(I+1)-TAU(I))
           ENDDO
        ENDDO
        TAU(1:NDEPTH) = 10.**TAU(1:NDEPTH)
        SPC__TAUN(1:SPC__NTAU) = 10.** SPC__TAUN(1:SPC__NTAU)
        SPC__TAUW = TAU_WL           

        CALL SPC_TAUSCALE ( SPC__NTAU, SPC__TAUN, SPC__TAUW )


        !     ************************************************
        !     *  t a u s c a l e                             *
        !     *                                              *
        !     *  read new optical depth scale                *
        !     ************************************************

     ELSEIF (KEYWRD(1:8) == 'tauscale') THEN 
        CALL KEY_PI (COMMAND, SPC__NTAU)

        DO I=1,SPC__NTAU
           CALL KEY_COM (COMMAND, 'tau :',';', NREAD,  I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__TAUN(I))
        ENDDO
        SPC__TAUW = TAU_WL

        CALL SPC_TAUSCALE ( SPC__NTAU, SPC__TAUN, SPC__TAUW )



        !     ************************************************
        !     *  t a u l o g s c                             *
        !     *                                              *
        !     *  read new optical depth scale (as logs)      *
        !     ************************************************

     ELSEIF (KEYWRD(1:8) == 'taulogsc') THEN 
        CALL KEY_PI (COMMAND, SPC__NTAU)

        DO I=1,SPC__NTAU
           CALL KEY_COM (COMMAND, 'log tau :',';', NREAD,  I4, IOSTAT)
           CALL KEY_PD (COMMAND, SPC__TAUN(I))
        ENDDO
        SPC__TAUN = 10.** SPC__TAUN
        SPC__TAUW = TAU_WL

        CALL SPC_TAUSCALE ( SPC__NTAU, SPC__TAUN, SPC__TAUW )



        !     ************************************************
        !     *  r e a d _ m o d e l                         *
        !     *                                              *
        !     *   Read in model atmosphere                   *
        !     ************************************************

     ELSEIF (KEYWRD(1:10) == 'read_model' .OR. &
             KEYWRD(1:8)  == 'read_qub'  ) THEN

        !     Read file containing models
        LF = LEN_TRIM(F__MODEL)
        CALL SPC_READQUB ( F__MODEL(1:LF) )
        CALL SPC_ABMODEL ( SPC__NABU, SPC__ELID, SPC__ABUN )



        !     ************************************************
        !     *  r e a d _ s t e r n e                       *
        !     *                                              *
        !     *   Read in sterne model: 2006 format          *
        !     ************************************************

     ELSEIF (KEYWRD(1:11) == 'read_sterne') THEN

        !     Read file containing models
        !       this option under test
        LF = LEN_TRIM(F__MODEL)
        CALL SPC_READMODEL ( F__MODEL(1:LF), STN__MODEL, I4, IOSTAT )
        CALL SPC_COPYMODEL ( STN__MODEL )
        CALL SPC_ABMODEL ( SPC__NABU, SPC__ELID, SPC__ABUN )
        !! is equivalent of spc_abmodel carried out ? 


        !     ************************************************
        !     *  r e a d _ l i n e s                       *
        !     *                                              *
        !     *  Read in atomic data                         *
        !     * This should be done _after_ reading in model *
        !     * atmosphere in order to apply line selection  *
        !     * criteria                                     *
        !     ************************************************

     ELSEIF (KEYWRD(1:11) == 'read_lines') THEN

        !     Initialize NLINE to zero
        NLINE=0

        !     Read in atomic data for individual metal lines
        LF = LEN_TRIM(F__LINES)
        CALL SPC_LINES ( F__LINES(1:LF), 0., 0. )



        !     ************************************************
        !     *  a b u n d                                   *
        !     *                                              *
        !     *  Set abundances for synthesis                *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'abund') THEN         
        !   Read element identifiers and abundance 
        CALL KEY_PI (COMMAND, I)
        CALL KEY_PD (COMMAND, X)
        ! locate array index corresponding to element identifier
        J = NAMAX 
        DO K = 1,SPC__NABU-1
           IF ( I == SPC__ELID(K) ) J = K
        ENDDO
        ! units required: number fraction
        IF ( X > 0.001 ) SPC__ABUN(J) = X


        !     ************************************************
        !     *  a b m o d e l                               *
        !     *                                              *
        !     *  Set abundances for synthesis from model     *
        !     ************************************************

     ELSEIF (KEYWRD(1:7) == 'abmodel') THEN         
        CALL SPC_ABMODEL ( SPC__NABU, SPC__ELID, SPC__ABUN )


        !     ************************************************
        !     *  a b f r a c                                 *
        !     *                                              *
        !     *  Set abundances for synthesis : Nel/Ntot     *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'abfrac') THEN         
        !   Read element identifiers and abundance 
        CALL KEY_PI (COMMAND, I)
        CALL KEY_PD (COMMAND, X)
        ! locate array index corresponding to element identifier
        J = NAMAX 
        DO K = 1,SPC__NABU-1
           IF ( I == SPC__ELID(K) ) J = K
        ENDDO
        IF ( 0. <= X .AND. X <= 1.0 ) SPC__ABUN(J) = LOG10(X) + ANORM        


        !     ************************************************
        !     *  a b l o g                                   *
        !     *                                              *
        !     *  Set abundances for synthesis : log Nel/Ntot *
        !     ************************************************

     ELSEIF (KEYWRD(1:5) == 'ablog') THEN         
        !   Read element identifiers and abundance 
        CALL KEY_PI (COMMAND, I)
        CALL KEY_PD (COMMAND, X)
        ! locate array index corresponding to element identifier
        J = NAMAX 
        DO K = 1,SPC__NABU-1
           IF ( I == SPC__ELID(K) ) J = K
        ENDDO
        IF ( X <= 0. ) SPC__ABUN(J) = X + ANORM



        !     ************************************************
        !     *  c a l c _ p r o f                           *
        !     *                                              *
        !     *  Calculate line profile for                  *
        !     *  Individual hydrogen, helium or metal lines  *
        !     *  or close blends of lines of same element    *
        !     ************************************************

     ELSEIF (KEYWRD(1:9) == 'calc_prof') THEN


        NMODE = 0

        !     *** Read from command line
        !     NSP        is element number 
        !     -ve for high tolerance identification
        !     RLAMB      is wavelength in Angstroms
        !     ABUND      is logarithmic abundance with [H] = 12
        !     or natural abundance with [H] = 1
        !     VT         is microturbulent velocity in km/s

        CALL KEY_PI (COMMAND, NSP)
        CALL KEY_PD (COMMAND, RLAMB)
        CALL KEY_PD (COMMAND, ABUND)
        CALL KEY_PD (COMMAND, VT)

        !     Set tolerance for line identification
        IF (NSP > 0) THEN
           WTOL=MIN(10.0, SPC__WTOL)
        ELSE
           NSP=-NSP
           WTOL=MIN(0.1, SPC__WTOL)
        ENDIF

        !     Disperse microturbulent velocity
        QUB__VTRB(1:NDEPTH) = (/ (VT, J=1,NDEPTH) /)

        !     Check for type of helium line
        IF (NSP == IHE) CALL HETYPE

        !     Check logarithmic abundance
        IF (ABUND > 1.0) ABUND=MIN(10.**(ABUND-ANORM),1.D0)

        !     Debug output
        IF (BUGMN) WRITE(NDEBUG,951)NSP,RLAMB,ABUND,VT,ANORM
951     FORMAT(//' calc_prof'/' NSP=',I5,5X,'WAVE=',0PF10.2, &
             5X,'ABUND=',1PE12.5,5X,'VT=',0PF10.2,'ANORM=',0PF10.2)

        IF ( PULS__OK ) THEN
           CALL SPC_WH_PULS
        ELSEIF ( NVRAD > 1 .OR. VRAD /= 0. ) THEN
           CALL SPC_WH_MOVE
        ELSE
           CALL SPC_WH_STAT
        ENDIF



        !     ************************************************
        !     *  c a l c _ a b u n d                         *
        !     *                                              *
        !     *  Calculate abundance and line profile for    *
        !     *  Individual hydrogen, helium or metal lines  *
        !     *  or close blends of lines of same element    *
        !     ************************************************

     ELSEIF (KEYWRD(1:10) == 'calc_abund') THEN


        NMODE = 0

        !     *** Read from terminal
        !     NSP        is element number 
        !     -ve for high tolerance identification
        !     RLAMB      is wavelength in Angstroms
        !     EWIDTH     is equivalent width in mA.
        !     DELTEW     is erroro on equivalent width in mA.
        !     VT         is microturbulent velocity in km/s

        CALL KEY_PI (COMMAND, NSP)
        CALL KEY_PD (COMMAND, RLAMB)
        CALL KEY_PD (COMMAND, EWIDTH)
        EWIDTH = EWIDTH/1000.   ! convert from mA to A.
        CALL KEY_PD (COMMAND, DELTEW)
        DELTEW = DELTEW/1000.
        CALL KEY_PD (COMMAND, VT)  ! default is 0.0  if missing

        !     Set tolerance for line identification
        IF (NSP > 0) THEN
           WTOL=MIN(10.0, SPC__WTOL)
        ELSE
           NSP=-NSP
           WTOL=MIN(0.1, SPC__WTOL)
        ENDIF

        !     Disperse microturbulent velocity
        QUB__VTRB(1:NDEPTH) = (/ (VT, J=1,NDEPTH) /)

        !     Check for type of helium line
        IF (NSP == IHE) CALL HETYPE

        !     Debug output
        IF (BUGMN) WRITE(NDEBUG,952)NSP,RLAMB,EWIDTH*1000,VT
952     FORMAT(//' calc_abund'/' NSP=',I5,5X,'WAVE=',0PF10.2, &
             5X,'EWIDTH=',1PE12.5,5X,'VT=',0PF10.2)

        !     Calculate abundance and line profile, given an equivalent width 
        CALL SPC_EWABUND (EWIDTH, DELTEW)



        !     ***************************************************************
        !     *  c a l c _ s y n t h                                        *
        !     *                                                             *
        !     *  Spectrum synthesis                                         *
        !     ***************************************************************

     ELSEIF (KEYWRD(1:10) == 'calc_synth') THEN
        NMODE = 11

        CALL KEY_PD (COMMAND, XMIN)
        CALL KEY_PD (COMMAND, XMAX)

        !     Read in atomic data for synthesised spectrum
        CALL SPC_SCAN (XMIN, XMAX)

        !     Establish abundances for this synthesis.
        CALL SPC_ABUND (SPC__NABU, SPC__ELID, SPC__ABUN)

        !     Compute for only microturbulence and abundance set
        IVTSY=1; VTSY(1)=VT
        IABSY=1

        DO L=1,NELSY
           ABUSY(L)=MIN(10.0**(ABUSY(L)-ANORM),1.D0)
        ENDDO
        ! write(*,*) abusy(1:nelsy)

        !     Call main subroutine
        IF ( PULS__OK ) THEN
           CALL SPC_WH_PULS
        ELSEIF ( NVRAD > 1 .OR. VRAD /= 0. ) THEN
           CALL SPC_WH_MOVE
        ELSE
           CALL SPC_WH_STAT
        ENDIF


        !     ***************************************************************
        !     *  c a l c _ l i n e s                                        *
        !     *                                                             *
        !     *  Calculate equivalent widths for all lines in list          *
        !     ***************************************************************

     ELSEIF (KEYWRD(1:10) == 'calc_lines') THEN

        ! mode for cycling over lines, etc.
        NMODE = 20

        CALL KEY_PD (COMMAND, XMIN)
        CALL KEY_PD (COMMAND, XMAX)

        !     Read in atomic data for synthesised spectrum
        CALL SPC_SCAN (XMIN, XMAX)

        !     Establish abundances for this synthesis.
        CALL SPC_ABUND (SPC__NABU, SPC__ELID, SPC__ABUN)

        !     Compute for only microturbulence and abundance set
        IVTSY=1; VTSY(1)=VT
        IABSY=1

        !     Set tolerance for line identification
        !     (can be very small because wavelengths are exact - from the linelist)
        WTOL=MIN(0.001, SPC__WTOL)

        !     Check abundances are in correct form
        DO L=1,NELSY
           ABUSY(L)=MIN(10.0**(ABUSY(L)-ANORM),1.D0)
        ENDDO


        !     Loop over all lines in linelist
        DO L=1,NELSY

           ! Check wavelength within limits
           RLAMB = WAVSY(L)
           IF ( XMIN<RLAMB .AND. RLAMB<XMAX ) THEN 

              ! Basic line data
              NSP   = NSPSY(L)
              ABUND = ABUSY(L)
              VT    = VTSY(1)

              ! Disperse microturbulent velocity
              QUB__VTRB(1:NDEPTH) = (/ (VT, J=1,NDEPTH) /)
              ! Check for type of helium line
              IF (NSP == IHE) CALL HETYPE

              ! compute the line
              CALL SPC_WH_STAT

           ENDIF
        ENDDO



        !     ***************************************************************
        !     *  c h e c k                                                  *
        !     *                                                             *
        !     *  Confirm options curently selected                          *
        !     ***************************************************************

     ELSEIF (KEYWRD(1:5) == 'check') THEN

        !     Radiative transfer options
        CALL CHECK_S
        !     HeI options
        CALL CHECK_HE1



     ENDIF
  ENDDO



CONTAINS



  SUBROUTINE SPC_STARTUP
    !     +
    !     
    !     S P C _ S T A R T U P
    !     
    !     This subroutine sets default values for flags, switches and variables
    !     and initialises the i/o streams for the command interpreter
    !     
    !     -

    WRITE (*,*)
    WRITE (*,*) '  SSS  PPP   EEEE   CCC  TTTTT  RRR   U   U  M   M '
    WRITE (*,*) ' S     P  P  E     C       T    R  R  U   U  MM MM '
    WRITE (*,*) '  SS   PPP   EEE   C       T    RRR   U   U  M M M '
    WRITE (*,*) '    S  P     E     C       T    R R   U   U  M   M '
    WRITE (*,*) ' SSS   P     EEEE   CCC    T    R  R   UUU   M   M '
    WRITE (*,*)
    WRITE (*,*) ' Version ', SPC__VERSION
    WRITE (*,*) ' Date    ', SPC__DATE
    WRITE (*,*) ' Author  ', SPC__AUTHOR
    WRITE (*,*) ' Help    ', SPC__HELP
    WRITE (*,*)
    WRITE (*,*) ' Please wait while atomic data is loaded '
    WRITE (*,*)



    !     Set up default i/o channels
    CALL STR_DEF (NREAD, NWRITE)
    COMMAND(1:20) = '                    '
    COMMAND(21:40) = '                    '
    COMMAND(41:60) = '                    '
    COMMAND(61:80) = '                    '
    COMMAND(1:11) = 'spc_startup'
    KEYWRD(1:3)   = '   '

    !     Set default program options
    NMOD    = 1
    ! NMSK    = 0  ! redundant
    WAVMAX  = 0.
    NSOURCE = 3               ! Snu from Feautrier
    NHEDAT  = 1
    NHESTARK = 2
    NMODE   = 0

    NDEPTH  = 0

    NTHETA  = 0
    COSTMIN = 0.005

    NVTRB   = 1
    VT      = 0.0

    NVRAD   = 1
    VRAD    = 0.0        

    TAU_WL  = 4000.
    REFWL   = TAU_WL

    GFN_THRESHOLD = 1.E-2     ! exclude lines weaker than this


    !     Read in general atomic data for atoms and ions
    CALL DP_ATDATA

    !     Read in atomic data for individual metal lines
    !     CALL DP_LINES ('LINES')

    !     Read in diffuse helium line atomic data
    CALL DP_ATHEI

    !     Read in ionized helium line atomic data
    CALL DP_ATHEII

    !     Initialize atomic numbers etc
    CALL OP_ALP

    !     Read in the WL grid on which to compute the opacities
    CALL SPC_READWL

    !     Read in OP cross sections and energy levels
    CALL OP_BFIN 

    !     Read in Iron Project cross sections
    !     CALL OP_FEIN  (obsolete 26.xi.2020)

    !     Read in Peach tables
    CALL OP_PEACHIN

    !     Initialize abundances to solar values (or meteoritic)
    SPC__NABU = NAMAX-1
    DO I = 1,SPC__NABU
       SPC__ELID(I) = I
       SPC__ABUN(I) = TAP_ABSOL ( SPC__ELID(I) )
       IF ( SPC__ABUN(I) < 0 ) SPC__ABUN(I) = TAP_ABMET ( SPC__ELID(I) )
    END DO
    FRACT = 10.**(SPC__ABUN(2)-SPC__ABUN(1))
    SPC__ABUN(1) = 1.0 / (1.0 + FRACT)
    SPC__ABUN(2:NAMAX) = 1.0E-12*(10.0**COSMIC(2:NAMAX)) / (1.0 + FRACT)
    ALPHA(1:NAMAX) = SPC__ABUN(1:NAMAX)

    !     Dummy values for remainder of array
    SPC__ELID (SPC__NABU+1:NAMAX) = -99
    SPC__ABUN (SPC__NABU+1:NAMAX) = 0.0

    !     Initialize stratified abundances
    QUB__LSTRAT = .FALSE.
    QUB__QSTRAT = .FALSE.
    DO  I = 1,NAMAX
       QUB__ABSTRAT (I,1:KD) =  SPC__ABUN (I)
    END DO
    CALL OP_ALPSET (50, ALPHA)
    CALL OP_ALPMU  (50)


  END SUBROUTINE SPC_STARTUP





  SUBROUTINE SPC_WH_STAT

    !+
    !     Main subroutine for a stationary atmosphere
    !
    !     Assigns velocities if necessary
    !     Sets up wavelength scale (WAVEG)
    !     Calculates line opacity (LINOP)
    !     Calculate optical depths and fluxes (INTEG)
    !     Calculates normalised fluxes and equivalent width (EQUIV)
    !     Outputs equivalent widths and line profiles
    !               to terminal and LP (OUTFOR)
    !               in DIPSO format (OUTDIP)
    !
    !-


    IF (BUGWH) THEN
       WRITE (*,*) 'SPC_WH_STAT ', nmode,nsource,nhedat,nhestark,ntheta,nvtrb,nvrad
    ENDIF
    NMODX = NMODE


    !     Disperse depthe-dependent velocity
    QUB__VRAD(1:NDEPTH) = (/ (0.0, J=1,NDEPTH) /)

    !     Disperse depth-dependent microturbulence
    IF (NVTRB > 1) THEN
       CALL SPC_VDATA ( 'VTRB', NVTRB, SPC__VT_TAU, SPC__VTRB )
    ELSEIF ( NVTRB <= 1 .AND. VT /= 0. ) THEN
       QUB__VTRB = VT
    ELSE 
       QUB__VTRB = 0.0
    ENDIF


    !     Disperse depth-dependent temperature correction
    !     only apply it once for each time it is defined
    IF (NDTEMP > 1) THEN

       CALL SPC_VDATA ( 'TTAU', NDTEMP, SPC__DT_TAU, SPC__DTEMP )

       !     .. recalculate the ionisation fractiona and opacity constants
       CALL OP_IONFRN (NDEPTH)
       ! CALL DP_RD_IONFRN
       ! CALL OPK_MODEL (TEFF,GRAV,NDEPTH,TAU,TEMP,PG,PE,OP)
       ! CALL OP_IONFILL ( F_ION, U_ION, 92, 6, NDEPTH )
       CALL OP_ATFILL (NDEPTH)
       CALL SPC_VDWC (AH, AHE)

    ENDIF




    !     Generate wavelength scale

    !     Spectrum synthesis calculation
    IF (NMODE == 10.OR.NMODE == 11) THEN
       NW=NWSY
       SYMM=.FALSE.
       DO  I=1,NW
          W(I)=WSY(I)
       ENDDO
       !     OTHER METAL CALCULATIONS
    ELSE
       CALL WAVEG
    ENDIF

    !     Debug output
    IF(BUGWH) THEN
       WRITE(NDEBUG,901)NW,WAVMAX,(W(I),I=1,NW)
    ENDIF


    !     CALCULATES LINE OPACITY FOR METALS
    IF (NMODE == 10.OR.NMODE ==11) WRITE (*,*) 'CALCULATE LINE OPACITIES'
    CALL LIN_OP


    !     Debug output
    IF(BUGWH) THEN
       WRITE(NDEBUG,900) NW,NDEPTH,((RLMU(I,J),I=1,NW),J=1,NDEPTH)
    ENDIF


    !     Checks if line shifts are required and apply
    CALL DP_VSHIFT ( 1.D0 )

    !     Calculate optical depth scale and fluxes
    IF (NMODE == 11) WRITE (*,*) 'CALCULATE RADIATIVE TRANSFER'
    IF (NMODX >= 20) NMODE = 0
    CALL INTEG
    NMODE = NMODX

    !     Calculate equivalent width and normalised fluxes
    IF (NMODE == 11) WRITE (*,*) 'NORMALIZE AND OUTPUT'
    CALL EQUIV


    !     Remove temperature perturbation
    IF (NDTEMP > 1) THEN
       CALL SPC_VDATA ( 'TINV', NDTEMP, SPC__DT_TAU, SPC__DTEMP )

       !     .. recalculate the ionisation fractiona and opacity constants
       CALL OP_IONFRN (NDEPTH)
       ! CALL DP_RD_IONFRN
       ! CALL OPK_MODEL (TEFF,GRAV,NDEPTH,TAU,TEMP,PG,PE,OP)
       ! CALL OP_IONFILL ( F_ION, U_ION, 92, 6, NDEPTH )
       CALL OP_ATFILL (NDEPTH)
       CALL SPC_VDWC (AH, AHE)

    ENDIF


    !     Iteration to correct abundance
    IF(NMODE == 1) THEN
       RETURN

       !     Standard output
    ELSE

       !     Output to terminal
       CALL OUTTER

       !     Output to disc
       CALL OUTDIS

       !     Output to DIPSO
       CALL OUTDIP

    ENDIF


    !  Debug Format Statements
900 FORMAT(//' ** SPC_WH_STAT **',5X,'NW=',I5,5X,'NDEPTH=',I5//&
         16X,'LINE OPACITY'/1500(1P,10E12.5/))
901 FORMAT(//' ** SPC_WH_STAT **',5X,'NW=',I5,5X,'WAVMAX=',F10.3//&
         16X,'WAVELENGTHS'/20(1P,10E12.5))

  END SUBROUTINE SPC_WH_STAT




  SUBROUTINE SPC_WH_MOVE

    !+
    !     Main subroutine to treat a moving atmopshere
    !
    !     Assigns velocities if necessary
    !     Sets up wavelength scale (WAVEG)
    !     Calculates line opacity (LINOP)
    !     Calculate optical depths and fluxes (INTEG)
    !     Calculates normalised fluxes and equivalent width (EQUIV)
    !     Outputs equivalent widths and line profiles
    !               to terminal and LP (OUTFOR)
    !               in DIPSO format (OUTDIP)
    !
    !-

    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: RL_TEMP  ! temporary store for line opacity
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: IN_TEMP  ! temporary store for line intensity
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: IC_TEMP  ! temporary store for continuum intensity

    !.

    IF (BUGWH) THEN
       WRITE (*,*) 'SPC_WH_MOVE ', nmode,nsource,wavm,nhedat,nhestark,&
            ntheta,nvtrb,nvrad
    ENDIF


    !     Disperse depthe-dependent velocity
    IF (NVRAD > 1) THEN
       CALL SPC_VDATA ( 'VRAD', NVRAD, SPC__VR_TAU, SPC__VRAD )
    ELSEIF ( NVRAD <= 1 .AND. VRAD /= 0. ) THEN
       QUB__VRAD = VRAD
    ELSE 
       QUB__VRAD = 0.0
    ENDIF

    !     Disperse depth-dependent microturbulence
    IF (NVTRB > 1) THEN
       CALL SPC_VDATA ( 'VTRB', NVTRB, SPC__VT_TAU, SPC__VTRB )
    ELSEIF ( NVTRB <= 1 .AND. VT /= 0. ) THEN
       QUB__VTRB = VT
    ELSE 
       QUB__VTRB = 0.0
    ENDIF

    !     Disperse depth-dependent temperature correction
    !     only apply it once for each time it is defined
    IF (NDTEMP > 1) THEN
       CALL SPC_VDATA ( 'TTAU', NDTEMP, SPC__DT_TAU, SPC__DTEMP )

       !     .. recalculate the ionisation fractiona and opacity constants
       CALL OP_IONFRN (NDEPTH)
       ! CALL DP_RD_IONFRN
       ! CALL OPK_MODEL (TEFF,GRAV,NDEPTH,TAU,TEMP,PG,PE,OP)
       ! CALL OP_IONFILL ( F_ION, U_ION, 92, 6, NDEPTH )
       CALL OP_ATFILL (NDEPTH)
       CALL SPC_VDWC (AH, AHE)

    ENDIF



    !     Generate wavelength scale

    !     Spectrum synthesis calculation
    IF(NMODE == 10.OR.NMODE == 11) THEN
       NW=NWSY
       SYMM=.FALSE.
       DO  I=1,NW
          W(I)=WSY(I)
       ENDDO
       !     OTHER METAL CALCULATIONS
    ELSE
       CALL WAVEG
    ENDIF

    !     Debug output
    IF(BUGWH) THEN
       WRITE(NDEBUG,901)NW,WAVMAX,(W(I),I=1,NW)
    ENDIF




    !     CALCULATES LINE OPACITY FOR METALS
    IF (NMODE == 11) WRITE (*,*) 'CALCULATE LINE OPACITIES'
    CALL LIN_OP


    !     Debug output
    IF(BUGWH) THEN
       WRITE(NDEBUG,900) NW,NDEPTH,((RLMU(I,J),I=1,NW),J=1,NDEPTH)
    ENDIF

    !     Store in temporary array
    ALLOCATE ( RL_TEMP(NW,NDEPTH) )
    RL_TEMP = RLMU(1:NW,1:NDEPTH)


    !     Execute loop over angles; these may have been defined on input 
    !     or they may have to be selected automatically. 
    IF (NTHETA == 0) THEN
       NTHETA = 5
       COSTHETA(1:6) = (/ 1.0, 0.8, 0.6, 0.4, 0.2, 0.0/)
    ENDIF

    !     Set up temporary arrays to store intensities
    ALLOCATE ( IN_TEMP(NW,NTHETA) )
    ALLOCATE ( IC_TEMP(NW,NTHETA) )


    !     Solve radiative transfer for each angle
    DO NTH = 1,NTHETA

       RLMU(1:NW,1:NDEPTH) = RL_TEMP


       !        Checks if line shifts are required and apply
       CALL DP_VSHIFT ( COSTHETA(NTH) )


       !        Calculate optical depth scale and fluxes
       IF (NMODE == 11) WRITE (*,*) 'CALCULATE RADIATIVE TRANSFER'
       CALL INTEG


       !        Store intensities for appropriate cosine angle
       IN_TEMP(1:NW,NTH) = INTENS(1:NW,NTH)
       IC_TEMP(1:NW,NTH) = INTENC(1:NW,NTH)

    ENDDO


    !     Copy intensities back to common blocks.
    DO NTH = 1,NTHETA
       INTENS(1:NW,NTH) = IN_TEMP(1:NW,NTH)
       INTENC(1:NW,NTH) = IC_TEMP(1:NW,NTH)
       IF (NMODE == 0) INTENC(1:NW,NTH) = IC_TEMP(1,NTH)
    ENDDO


    !     Trapezoidal integration over mu to obtain fluxes 
    NTH = 1
    A = 0.5 * COSTHETA(NTH) * ABS (COSTHETA(NTH+1)-COSTHETA(NTH))
    FLUX (1:NW) = A * INTENS(1:NW,NTH)
    FLUXC(1:NW) = A * INTENC(1:NW,NTH)
    DO NTH = 2,NTHETA
       A = 0.5 * COSTHETA(NTH) * ABS (COSTHETA(NTH+1)-COSTHETA(NTH-1))
       FLUX (1:NW) = FLUX (1:NW) + A * INTENS(1:NW,NTH)
       FLUXC(1:NW) = FLUXC(1:NW) + A * INTENC(1:NW,NTH)
    ENDDO

    !     Convert to astrophysical fluxes and normlise
    FLUX (1:NW) = 2 * TAPM__PI * FLUX (1:NW)
    FLUXC(1:NW) = 2 * TAPM__PI * FLUXC(1:NW)
    !     FLUXN(1:NW) = FLUX(1:NW)/FLUXC(1:NW)


    !     Calculate equivalent width and normalised fluxes
    IF (NMODE == 11) WRITE (*,*) 'NORMALIZE AND OUTPUT'
    CALL EQUIV


    !     Remove temperature perturbation
    IF (NDTEMP > 1) THEN
       CALL SPC_VDATA ( 'TINV', NDTEMP, SPC__DT_TAU, SPC__DTEMP )

       !     .. recalculate the ionisation fractiona and opacity constants
       CALL OP_IONFRN (NDEPTH)
       ! CALL DP_RD_IONFRN
       ! CALL OPK_MODEL (TEFF,GRAV,NDEPTH,TAU,TEMP,PG,PE,OP)
       ! CALL OP_IONFILL ( F_ION, U_ION, 92, 6, NDEPTH )
       CALL OP_ATFILL (NDEPTH)
       CALL SPC_VDWC (AH, AHE)

    ENDIF


    !     Iteration to correct abundance
    IF(NMODE == 1) THEN
       RETURN

       !     Standard output
    ELSE

       !     Output to terminal
       CALL OUTTER

       !     Output to disc
       CALL OUTDIS

       !     Output to DIPSO
       CALL OUTDIP

    ENDIF


    !  Debug Format Statements
900 FORMAT(//' ** SPC_WH_MOVE **',5X,'NW=',I5,5X,'NDEPTH=',I5//&
         16X,'LINE OPACITY'/1500(1P,10E12.5/))
901 FORMAT(//' ** SPC_WH_MOVE **',5X,'NW=',I5,5X,'WAVMAX=',F10.3//&
         16X,'WAVELENGTHS'/50(1P,E12.5/))

  END SUBROUTINE SPC_WH_MOVE




  SUBROUTINE SPC_WH_PULS

    !+
    !     Main subroutine to treat a moving atmosphere
    !     as derived from the Bridger pulsation code 
    !     and read in with command "hydro" 
    !
    !     Temperature, pressure and radius perturbations are
    !     supplied in normalised units, velocities are supplied 
    !     in standard units (*check which*), all on a mass scale.
    ! 
    !     1. The pulsation model must be transposed onto the optical
    !     depth scale for the given hydrostatic LTE model atmosphere,
    !     which is assumed to be the standard/reference model. 
    !
    !     2. The perturbations are then applied to the model 
    !     atmosphere, any equation of state and opacity calculations
    !     should be checked, and then the required radiative transfer can 
    !     be carried out. 
    !     
    !     Sets up wavelength scale (WAVEG)
    !     Calculates line opacity (LINOP)
    !     Calculate optical depths and fluxes (INTEG)
    !     Calculates normalised fluxes and equivalent width (EQUIV)
    !     Outputs equivalent widths and line profiles
    !               to terminal and LP (OUTFOR)
    !               in DIPSO format (OUTDIP)
    !
    !-

    ! Require physical constant
    USE TAPP_PAR, ONLY : TAPP__R ! gas constant

    ! Require public pulsation variables
    USE SPC_PULSE_PUB
    USE STN_PARAMETERS, ONLY : STN__TEFF => TEFF
    USE LTE_MODEL, ONLY : LTE__TEMP => TEMP
    USE LTE_MODEL, ONLY : LTE__PG => P_TOTAL
    USE LTE_MODEL, ONLY : LTE__PE => P_E
    USE OP_GLOBAL, ONLY : OP__ALPMU, OP_ALPNN, OP_ALPAM 

    ! Temporary arrays for radiative transfer solution (dimensions are wavelength and depth)
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: RL_TMP  ! temporary store for line opacity
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: IN_TMP  ! temporary store for line intensity
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: IC_TMP  ! temporary store for continuum intensity

    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: ATMOS__Z   ! depth variable
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: ATMOS__M   ! mass
    DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: ATMOS__D   ! density

    !   structure for holding scaled hydro model on atmosphere zoning 
    TYPE(PULS_HYDRO), ALLOCATABLE, DIMENSION(:,:) :: ATMOS__PULS

    !   arrays for remembering original PG and TEMP
    DOUBLE PRECISION, DIMENSION(KD) :: TEMP_0
    DOUBLE PRECISION, DIMENSION(KD) :: PTOT_0
    DOUBLE PRECISION, DIMENSION(KD) :: PEL_0
    !   interpolation pointers 
    INTEGER, DIMENSION(KD) :: NNN

    !   real holders for stn_press
    REAL :: TT4B,PG4B,PE4B


    !.

    !  Modes disallowed:
    IF (NMODE == 1) RETURN

    !  Save Teff for Eq of State call
    STN__TEFF = TEFF

    !  Debug 
    IF (BUGWH) THEN
       WRITE (*,*) 'SPC_WH_PULS ', nmode,nsource,wavm,nhedat,nhestark,ntheta
    ENDIF

    !     Compute mass and distance from surface for current model (plane parallel?)
    !     dz = dtau / rho /kappa  
    !     dm = rho dz = dtau / kappa       

    ALLOCATE (ATMOS__M(NDEPTH))
    ALLOCATE (ATMOS__D(NDEPTH))
    ALLOCATE (ATMOS__Z(NDEPTH))
    ATMOS__D = PG * OP__ALPMU / TEMP / (TAPP__R * 1.D7)
    ATMOS__M(1) = TAU(1) / ( OP(1) * 0.5 )
    ATMOS__Z(1) = TAU(1) / ( ATMOS__D(1) * OP(1) * 0.5 )
    DO J = 2,NDEPTH
       ATMOS__M(J) = ATMOS__M(J-1) + (TAU(J)-TAU(J-1)) * 2. / ( OP(J-1) + OP(J) )
       ATMOS__Z(J) = ATMOS__Z(J-1) + (TAU(J)-TAU(J-1)) * 2. / &
            ( ATMOS__D(J-1)*OP(J-1) + ATMOS__D(J)*OP(J) )
    ENDDO

    !    Report model atmosphere

    WRITE (*,*)
    WRITE (*,*) '#SPC_WH_PULS: Model atmosphere'  
    WRITE (*,*) '#NDEPTH: ',NDEPTH
    WRITE (*,*) '#J   log tau     log P     log T    log ro    log kp     log z     log m' 
    DO J = 1,NDEPTH,1
       WRITE (*,'(I3,7F10.5)') J,DLOG10(TAU(J)),DLOG10(PG(J)),DLOG10(TEMP(J)),DLOG10(ATMOS__D(J)), & 
            DLOG10(OP(J)),DLOG10(ATMOS__Z(J)),DLOG10(ATMOS__M(J))
    ENDDO

    WRITE (*,*)
    WRITE (*,*) 'SPC_READPULS: CHECK Model 1'  
    WRITE (*,'(A5,5(/1P,20E9.2))') 'M   ', PULS__ZONEM(1:PULS__ND)
    WRITE (*,'(A5,5(/20F9.2))') 'R/Ro', PULS__MODEL(1:PULS__ND,1)%R
    WRITE (*,'(A5,5(/20F9.2))') 'P/Po', PULS__MODEL(1:PULS__ND,1)%P
    WRITE (*,'(A5,5(/20F9.2))') 'T/To', PULS__MODEL(1:PULS__ND,1)%T
    WRITE (*,'(A5,5(/20F9.2))') 'V   ', PULS__MODEL(1:PULS__ND,1)%V

    !  Interpolate pulsation model onto model photosphere at these masses

    ALLOCATE (ATMOS__PULS(NDEPTH,PULS__NT))
    DO J = 1,NDEPTH
       NNN(J) = NR_LOCATE (PULS__ZONEM,ATMOS__M(J))
       NNN(J) = MAX(1,NNN(J)-1)
       NNN(J) = MIN(NNN(J),PULS__ND-3)
    END DO
    DO K = 1,PULS__NT
       DO J = 1,NDEPTH
          N = NNN(J)
          !CALL NR_POLINT (PULS__ZONEM,PULS__MODEL(1:PULS__ND,K)%R,PULS__ND,ATMOS__M(J),ATMOS__PULS(J,K)%R,ERR)
          CALL NR_POLINT (PULS__ZONEM(N:N+3),PULS__MODEL(N:N+3,K)%P,ATMOS__M(J),ATMOS__PULS(J,K)%P,ERR)
          CALL NR_POLINT (PULS__ZONEM(N:N+3),PULS__MODEL(N:N+3,K)%T,ATMOS__M(J),ATMOS__PULS(J,K)%T,ERR)
          CALL NR_POLINT (PULS__ZONEM(N:N+3),PULS__MODEL(N:N+3,K)%V,ATMOS__M(J),ATMOS__PULS(J,K)%V,ERR)
       ENDDO
    ENDDO
    WRITE (*,*)
    WRITE (*,*) 'SPC_WH_PULS: Pulsation model interpolated in atmosphere mass distribution'  
    WRITE (*,'(A5,5(/1P,20E9.2))') 'M   ', ATMOS__M(1:NDEPTH)
    WRITE (*,'(A5,5(/1P,20I9))') 'N   ', NNN(1:NDEPTH)
    !WRITE (*,'(A5,5(/20F9.2))') 'R/Ro', ATMOS__PULS(1:NDEPTH,1)%R
    WRITE (*,'(A5,5(/20F9.2))') 'P/Po', ATMOS__PULS(1:NDEPTH,1)%P
    WRITE (*,'(A5,5(/20F9.2))') 'T/To', ATMOS__PULS(1:NDEPTH,1)%T
    WRITE (*,'(A5,5(/20F9.2))') 'V   ', ATMOS__PULS(1:NDEPTH,1)%V

    ! Remember original model atmosphere
    TEMP_0 = TEMP
    PTOT_0 = PG
    PEL_0  = PE

    !  Execute major loop over phases
    PULS_PHASES: DO K = PULS__NSTART,PULS__NEND,PULS__NSKIP    

       !     Apply local corrections to temperature, pressure and velocity 
       !     (on first iteration compute unperturbed spectrum)
       !     Recalculate equation of state for new pressure, density structure
       QUB__VRAD = 0.0
       IF ( K > 0 ) THEN
          TEMP(1:NDEPTH) = TEMP_0(1:NDEPTH) * ATMOS__PULS(1:NDEPTH,K)%T
          PG(1:NDEPTH)   = PTOT_0(1:NDEPTH) * ATMOS__PULS(1:NDEPTH,K)%P
          QUB__VRAD(1:NDEPTH) = ATMOS__PULS(1:NDEPTH,K)%V
          DO J = 1,NDEPTH
             TT4B=TEMP(J)
             PG4B=PG(J)
             CALL STN_PRESS (J,TT4B,PG4B,PE4B)
             PE(J)=PE4B
          ENDDO
       ENDIF

       IF ( K < 2 ) THEN
          WRITE (*,*)
          WRITE (*,*) 'SPC_WH_PULS: Model atmosphere to be used'  
          WRITE (*,'(A5,5(/1P,20E9.2))') 'Tau   ',  TAU(1:NDEPTH)
          WRITE (*,'(A5,5(/1P,20E9.2))') 'T   ', TEMP(1:NDEPTH)
          WRITE (*,'(A5,5(/1P,20E9.2))') 'PG  ', PG(1:NDEPTH)
          WRITE (*,'(A5,5(/1P,20E9.2))') 'PE  ', PE(1:NDEPTH)
       ENDIF

       !   Identify key phases where optical depth scale has to be improved
       IF ( QUB__VRAD(1)*QUB__VRAD(NDEPTH) < 0 ) THEN

       ENDIF

       ! Recalculate opacity constants etc...
       ! ** check that only need T, PG for this -- did we need to scale PEL as well ? 
       LTE__TEMP(1:NDEPTH) = TEMP(1:NDEPTH)
       LTE__PG(1:NDEPTH)   = PG(1:NDEPTH)
       LTE__PE(1:NDEPTH)   = PE(1:NDEPTH)
       CALL OP_ALPMU  (NDEPTH)
       CALL OP_IONFRN (NDEPTH)
       CALL OP_ATFILL (NDEPTH)
       CALL SPC_VDWC (AH, AHE)

       !     Generate wavelength scale

       !     Spectrum synthesis calculation
       IF (NMODE == 10.OR.NMODE == 11) THEN
          NW=NWSY
          SYMM=.FALSE.
          W(1:NW)=WSY(1:NW)
          !     OTHER METAL CALCULATIONS
       ELSE
          CALL WAVEG
       ENDIF

       !     Debug output
       IF (BUGWH) &
            WRITE(NDEBUG,901)NW,WAVMAX,(W(I),I=1,NW)


       !     CALCULATES LINE OPACITY FOR METALS
       IF (NMODE == 11) WRITE (*,*) 'Calculating line opacities'
       CALL LIN_OP

       !     Debug output
       IF (BUGWH) &
            WRITE (NDEBUG,900) NW,NDEPTH,((RLMU(I,J),I=1,NW),J=1,NDEPTH)


       !     Store in temporary array
       ALLOCATE ( RL_TMP(NW,NDEPTH) )
       RL_TMP = RLMU(1:NW,1:NDEPTH)


       !     Execute loop over angles; these may have been defined on input 
       !     or they may have to be selected automatically. 
       IF (NTHETA == 0) THEN
          NTHETA = 5
          COSTHETA(1:6) = (/ 1.0, 0.8, 0.6, 0.4, 0.2, 0.0/)
       ENDIF

       !     Set up temporary arrays to store intensities
       ALLOCATE ( IN_TMP(NW,NTHETA) )
       ALLOCATE ( IC_TMP(NW,NTHETA) )

       ! write(*,*) 'SPC_WH_PULS  --  ABUSY:',abusy(1:nelsy)

       !     Solve radiative transfer for each angle
       DO NTH = 1,NTHETA

          RLMU(1:NW,1:NDEPTH) = RL_TMP

          !        Checks if line shifts are required and apply
          CALL DP_VSHIFT ( COSTHETA(NTH) )

          !        Calculate optical depth scale and fluxes
          IF (NMODE == 11) WRITE (*,*) 'Calculating radiative transfer: mu=',COSTHETA(NTH)
          CALL INTEG

          !        Store intensities for appropriate cosine angle
          IN_TMP(1:NW,NTH) = INTENS(1:NW,NTH)
          IC_TMP(1:NW,NTH) = INTENC(1:NW,NTH)

       ENDDO

       !     Copy intensities back to common blocks.
       DO NTH = 1,NTHETA
          INTENS(1:NW,NTH) = IN_TMP(1:NW,NTH)
          INTENC(1:NW,NTH) = IC_TMP(1:NW,NTH)
          IF (NMODE == 0) INTENC(1:NW,NTH) = IC_TMP(1,NTH)
       ENDDO


       !     Trapezoidal integration over mu to obtain fluxes 
       NTH = 1
       A = 0.5 * COSTHETA(NTH) * ABS (COSTHETA(NTH+1)-COSTHETA(NTH))
       FLUX (1:NW) = A * INTENS(1:NW,NTH)
       FLUXC(1:NW) = A * INTENC(1:NW,NTH)
       DO NTH = 2,NTHETA
          A = 0.5 * COSTHETA(NTH) * ABS (COSTHETA(NTH+1)-COSTHETA(NTH-1))
          FLUX (1:NW) = FLUX (1:NW) + A * INTENS(1:NW,NTH)
          FLUXC(1:NW) = FLUXC(1:NW) + A * INTENC(1:NW,NTH)
       ENDDO

       !     Convert to astrophysical fluxes and normalise
       FLUX (1:NW) = 2 * TAPM__PI * FLUX (1:NW)
       FLUXC(1:NW) = 2 * TAPM__PI * FLUXC(1:NW)
       !     FLUXN(1:NW) = FLUX(1:NW)/FLUXC(1:NW)


       !     Calculate equivalent width and normalised fluxes
       IF (NMODE == 11) WRITE (*,*) 'NORMALIZE AND OUTPUT'
       CALL EQUIV

       !     Nonstandard outputs required
       WRITE (*,*) 'SPC_WH_PULS: Model ', K  
       CALL OUTTER 
       !  CALL OUTDIS
       CALL OUTDIP

       DEALLOCATE (RL_TMP)
       DEALLOCATE (IN_TMP)
       DEALLOCATE (IC_TMP)

    ENDDO PULS_PHASES

    ! Tidy up
    TEMP = TEMP_0
    PG   = PTOT_0
    PE   = PEL_0
    QUB__VRAD = 0.

    !     .. recalculate the ionisation fractiona and opacity constants
    CALL OP_IONFRN (NDEPTH)
    CALL OP_ATFILL (NDEPTH)
    CALL SPC_VDWC (AH, AHE)

    DEALLOCATE (ATMOS__PULS)
    DEALLOCATE (ATMOS__M)



    !  Debug Format Statements
900 FORMAT(//' ** SPC_WH_PULS **',5X,'NW=',I5,5X,'NDEPTH=',I5//&
         16X,'LINE OPACITY'/1500(1P,10E12.5/))
901 FORMAT(//' ** SPC_WH_PULS **',5X,'NW=',I5,5X,'WAVMAX=',F10.3//&
         16X,'WAVELENGTHS'/50(1P,E12.5/))

  END SUBROUTINE SPC_WH_PULS


!!!***2017: 5.2.0
  SUBROUTINE SPC_STRATMODEL (ZSTRAT,NTAU,TAU_XABU,ST_ABU)

    !+
    !
    !  Purpose:     
    !     Input abundance stratification to SPECTRUM arrays
    !
    !  History:
    !     28-MAR-2017
    !         Original version
    !
    !  Author:
    !     N.T.Behara
    !
    !  Use:
    !     CALL SPC_STRATMODEL (Z,N,tau_xabu)
    !     
    !  Subroutine Arguments (Given):
    !     ZSTRAT   : TYPE(integer)   : Element to be stratified
    !     NTAU     : TYPE(integer)   : Number of tau points
    !     TAU_XABU : TYPE(vector) : vector of tau values describing element stratification
    !     ST_ABU   : TYPE(vector) : vector of abundance values describing element stratification
    !-

    !  LTE Structures
    !USE LTE_STRUCTURES

    !  SPC Globals
    USE SPC_GLOBAL

    !  Safe definitions
    IMPLICIT NONE

    !  Subroutine Arguments :
    INTEGER :: ZSTRAT         ! Element to be stratified
    INTEGER :: NTAU           ! Number of tau points used to define stratification
    DOUBLE PRECISION, DIMENSION (NTAU) :: TAU_XABU ! tau values describing element stratification
    DOUBLE PRECISION, DIMENSION (NTAU) :: ST_ABU ! Abundance values describing element stratification
    DOUBLE PRECISION, DIMENSION (NTAU) :: ST_ABUx ! Abundance values describing element stratification, number fractions

    !     Other local variables
    INTEGER :: I,J  ! loop index
    INTEGER :: ND   ! number of depth points
    INTEGER :: JD   ! depth point index
    INTEGER :: IZ   ! atomic number identifier

    ! Variables for tau calculation
    INTEGER :: NP   
    DOUBLE PRECISION :: TJ
    DOUBLE PRECISION, DIMENSION (NDEPTH) :: ABU_TMP ! interpolated abundances
    DOUBLE PRECISION, DIMENSION (NDEPTH) :: ERR_ABU ! error, interpolated abundances

    REAL, DIMENSION (NDEPTH) :: STRAT

    !     Local variables for composition normalisation
    DOUBLE PRECISION :: C__SOL, C__STAR, AH_SOL, AMU_SOL, AMU_STAR

    ! From spc_global

    ND = NDEPTH

    !-----------------------------------------------------------------------

    !  get some composition dependent variables

    AMU =  AH * 1.00  +  AHE* 4.00  +  AC *12.01  +  AN *14.01 &
         +  AO *16.00  +  ANE*20.18  +  AMG*24.31  +  AAL*26.98 &
         +  ASI*28.09  +  AS *32.06  +  ACA*40.08  +  AFE*55.85

    !     find abundance normalisation constant such that
    !     : log (sum(mu(i).n(i))) = solar
    !     

    AH_SOL   = 1. - SUM( 10.**(COSMIC(2:26)-12.) )
    C__SOL   = 12. - LOG10(AH_SOL)
    ! write (*,*) 'ah, c',ah_sol, c__sol
    AMU_SOL  = LOG10( SUM( ATW(1:26)*10.**(COSMIC(1:26)-12.) ) )
    AMU_STAR = LOG10( SUM( ATW(1:26)*ALPHA(1:26) ) )
    C__STAR  = C__SOL + AMU_SOL - AMU_STAR
    ! write (*,*) 'c*, mu*, c, mu', c__star, amu_star, c__sol, amu_sol 
    ANORM = C__STAR

    !-----------------------------------------------------------------------
    ! Compute vertical abundance profile from input parameters (tau_xabu, Ntau)
    ! Assume tau is increasing? Not in log format

    ! convert abundances to number fractions
    ST_ABUx = 10.**(ST_ABU-ANORM) 

    ! Only one tau/abu set provided, so homogeneous composition
    ! More than one tau/abu pair, so compute distribution
    ! For all depths less than TAU_XABU(1), abundance is set to ST_ABU(1)
    ! For all depths greater than TAU_XABU(Ntau), abundance is set to ST_ABU(Ntau)
    ! Between these values, ST_ABU(TAU) is obtained by quadratic interpolatiion in TAU_XABU. 
    IF ( NTAU == 1 ) THEN
       ABU_TMP(1:ND) = ST_ABUx(1)
    ELSE
       !NP   = MIN(NTAU-1,2)
       DO J = 1,ND
          TJ = TAU(J) !TAU from spc_global
          IF ( TJ <= TAU_XABU(1) ) THEN
             ABU_TMP(J) = ST_ABUx(1)
          ELSEIF ( TJ >= TAU_XABU(NTAU) ) THEN
             ABU_TMP(J) = ST_ABUx(NTAU)
          ELSE
             !CALL MTH_D_TABINT ( NTAU,TAU_XABU,ST_ABUx,NP,TJ,ABU_TMP(J) )
             CALL MTH_LININT(TAU_XABU, ST_ABUx, NTAU, TJ, ABU_TMP(J), ERR_ABU(J))
          ENDIF
       ENDDO

       !write(*,*) 'err', ERR_ABU

    ENDIF

    ! set depth-dependant abundances
    STRAT = ABU_TMP

    !-----------------------------------------------------------------------

    !  if first call to subroutine, distribute 
    !  uniform composition through structure
    IF ( QUB__LSTRAT .EQV. .FALSE.) THEN
       DO IZ = 1, KI !KI from spc_global, number of elements
          QUB__ABSTRAT(IZ,:) = ALPHA(IZ)
       ENDDO
    ENDIF

    ! Set Strat label for element (Zstrat) & model
    QUB__QSTRAT(ZSTRAT) = .TRUE.
    QUB__LSTRAT = .TRUE.

    ! copy in the stratified elements
    QUB__ABSTRAT(ZSTRAT,1:ND) = STRAT(1:ND)

    ! compute the depth-dependent mean atomic weight
    DO JD = 1,ND
       QUB__AMU(JD) = 0.0
       DO IZ=1,KI
          QUB__AMU(JD) = QUB__AMU(JD) + QUB__ABSTRAT(IZ,JD)*ATW(IZ)
       ENDDO
    ENDDO

    !-----------------------------------------------------------------------
    !-------------------------------------------------------------------------

    !     Load up common blocks for OP: stratified layers
    IF (ZSTRAT .LE. 40) THEN
       CALL OP_ALPSET_Z ( ND, ZSTRAT, QUB__ABSTRAT(ZSTRAT,1:ND))  

       CALL OP_ALPMU (NDEPTH)

       !     Calculate ionisation fractions and atomic data
       CALL OP_IONFRN (NDEPTH)
       CALL OP_ATFILL (NDEPTH)
    ENDIF

    !-------------------------------------------------------------------------

    !     Report model input

    WRITE (NWRITE,*)
    WRITE (NWRITE,'(2X,A)') 'SPC_STRATMODEL: '
    IF ( QUB__LSTRAT ) THEN 
       WRITE (NWRITE,'(A,L)') 'Model is stratified ',QUB__LSTRAT
       WRITE (NWRITE,'(A,I2)') 'Element : ', ZSTRAT
       WRITE (NWRITE,'(a,9999(es12.5,a))') 'tau: ',(TAU(i), ",",i=1,ND)
       WRITE (NWRITE,'(a,9999(es12.5,a))') 'ab : ',(QUB__ABSTRAT(ZSTRAT,i), ",",i=1,ND)
       !     WRITE (NWRITE,*) 'amu_j: '
       !     WRITE (NWRITE,*) QUB__AMU(1:ND)
    ENDIF


  END SUBROUTINE SPC_STRATMODEL
!!!!**2017



END PROGRAM SPECTRUM
