PROGRAM STERNE_OS

  !+
  !
  !  Program:
  !     STERNE_OS
  
  !  Version:
  !     3.0.1

  !  Language:
  !     Approaching Fortran95

  !  Purpose:
  !     Calculate LTE models with line-blanketing for homogeneous
  !     and inhomogeneous stellar atmospheres.

  !  Description:
  !     This is a model atmosphere program originally written
  !     to model RCrB and EHe star atmospheres. It will treat any early-type
  !     star providing gravity is not too high (the equation of state breaks 
  !     for white dwarf atmospheres), the effective temperature is not too 
  !     high (we don't treat non-LTE) or low (neither do we consider molecules).
  !     At high L/M ratios, one should be careful about winds, non-LTE and 
  !     speherical geometry -- but this code does a pretty good job for low-mass
  !     stars. Provision for stratified atmopsheres is present, but the
  !     self-consistent calculation of diffusively stratified models is 
  !     done by a later program.  
  
  !  History:
  !     1.0       B.Wolf & D.Schoenberner, Berlin 1975
  !     2.0       U.Heber & R.Moeller, Kiel       1990
  !     2.1-2.3   C.S Jeffery, St. Andrews/Armagh 1990-1998
  !     2.4       P.M.Harrison, Armagh            20-May-1998
  !     2.4b      C.S.Jeffery, Armagh             24-Feb-1999
  !     2.5       C.S.Jeffery, Armagh             04-Mar-1999
  !     2.5b      C.S.Jeffery, Armagh             18-May-1999
  !     2.5c      C.S.Jeffery, Armagh             20-Oct-2000
  !     2.6       C.S.Jeffery, Armagh             13-Nov-2000
  !     2.6a      C.S.Jeffery, Armagh             18-Feb-2002
  !     2.6d      C.S.Jeffery, Armagh             26-May-2004
  !     3.0       N.T.Behara, Armagh              21-Nov-2005
  !     3.0.1     C.S.Jeffery, Armagh             18-Jul-2007
  !     3.4       C.S.Jeffery, Armagh             10-Mar-2019  
  !     3.5       C.S.Jeffery, Armagh             16-Jul-2020  
  !     3.5.1     C.S.Jeffery, Armagh             26-Nov-2020  
  
  !  Version 1
  !     A simple LTE  code written by Wolf & Schoenberner (Berlin) which
  !     accounted for continuous opacity sources only, but which did include
  !     extra data for carbon and nitrogen.
  !     It was given to CCP7 and implemented on the Cray XMP and VAX/VMS
  !     in about 1987.
  
  !  Version 2
  !     Subsequently Heber (Kiel) adapted the code to include He-line blocking
  !     and some UV line-blanketing, and to allow some composition stratification.
  !     Moeller (Kiel) computed opacity distribution
  !     functions for He-rich mixtures, and these were incorporated in Version 2 of the code.
  !     This was given to CCP7 in 1990, and implemented on the Cray XMP at RAL.
  
  !  Version 2.1
  !     This version represents a port to a Unix Sun SPARCstation. The
  !     Cray code did not port easily, with very old Fortran giving problems.
  !     Since the opacity library had already been ported as part of the QUB code SPECTRUM
  !     the opacity sections were stripped out, and a very slightly modified form of the
  !     the QUB library substituted - the library now known as OPK. A lot of work was left
  !     to do integrating this library with the remaining STERNE code, and cleaning out other
  !     dead code. The philosophy at this stage was to change as little as possible,
  !     and only to fix code that was giving wrong answers. Hence a lot of improvements are
  !     still possible.

  !  Version 2.2
  !     On the port to SOLARIS, version 2.1 broke down completely. It was not clear what
  !     had happened - whether it was a compiler/arithmetic problem or some violation of
  !     memory. Substantial sections of code were cleaned up looking for the bug, although
  !     much remains to be done. The problem was traced to subroutine ALKOEF which was
  !     giving bad values to array C when scattering dominated.
  !     ALKOEF code was replace by code from PARABEL in the SPECTRUM package - I had already
  !     solved the problem of poorly behaved transfer solutions in scattering layers.
  
  !  Version 2.3
  !     The stability of the Avrett-Loeser treatment of radiative transfer
  !     was in some doubt, so it has been replaced by a Feautrier scheme. This
  !     major change coincided with a port to OSF/1 and Linux. FLXODF has been
  !     redically rewritten and now calls RTF_FEAUTRIER and RTF_PHI, where
  !     the RTF library provides a number of radiative transfer utilities.
  
  !  Version 2.4
  !     A number of bug fixes have been incorporated to allow convergence over
  !     a wide region of logg-Teff space. These have also been incorporated
  !     into v2.3.
  !     Most importantly however, the whole continuous opacity treatment
  !     has been changed. In this version all b-f opacities are treated using
  !     data for the Opacity Project.
  !     Therefore version 2.4 does not yet replace version 2.3. Instead there
  !     are now 2 codes with completely independent treatments of continuous
  !     opacity.

  !  Version 2.4b
  !     In version 2.4b the OP library has been isolated from the remainder
  !     of the program, communication is entirely by subroutine calls.
  !     Also, OPK is consistent between STERNE and SPECTRUM, removing a
  !     conflict between the use of common blocks. Further work to isolate
  !     subroutine libraries is probably needed in OPK, or else some
  !     routines need to migrate from OPK to OP. Some bugs removed.
  !
  !  Main features:
  !     Opacities:
  !        Free-Free            -  Power Law approximations
  !        Bound-free           -  Individual transition treatment using
  !                                Opacity Project data
  !        bound-bound          -  ATLAS6-type opacity distribution functions
  !                                (from Kurucz & Peytremann linelists)
  !     Convection              -  NOT TREATED
  !     Radiative transfer      -  Avrett-Loeser (originally)
  !                             -  Feautrier (since v2.3)
  !     Temperature Correction  -  Lucy-Unsold
  !
  !  Version 2.5
  !     We attempt to introduce "Opacity Sampling" in order to account for
  !     the line opacity. Linelists are read in once the wavelength grid
  !     is established, selecting lines within DNU of the prescribed
  !     frequencies. This frequency grid is now completely flexible, both
  !     the OP data and BB data are established at initialization.
  !     The line opacity itself is required by a new subroutine FLXOS
  !     which replaces FLXODF. The type of b-b opacity to use is selected
  !     via the character variable 'KEY__OPBB' which may take one of the
  !     following values.
  !
  !          'CONTINUUM': continuum only using subroutine FLXODF
  !          'ODF': opacity distribution fucntion using FLXODF
  !          'SAMPLING': opacity sampling using FLXOS

  !  Version 2.5b
  !     We also find that the OP b-f opacities may not yet be trustworthy
  !     so it is necessary to be able to switch between 'Kiel' and 'OP'
  !     libraries. This will be done by a separate opacity keyword.
  !
  !         'KEY__OPBB' see above  will become       'KEY__OPBB'
  !         continuous opacities will be given by 'KEY__OPBF'
  !     allowed values are
  !         'KIEL'      -  old ATLAS type opacities
  !         'OP'        -  OP opacities
  !     To be done:
  !     Opacities:
  !        EITHER Kurucz ATLAS9 - type odfs to be introduced
  !        OR     b-b opacities from Opacity Project to be incorporated
  !               possibly using an opacity sampling technique.
  !     Code structure:
  !        A complete rewrite is needed (with proper design methods) in order
  !        to obtain a code which can be properly extended.
  
  !  Version 2.5c
  !     Minor changes to 'inmod', 'outmod' to allow free-format reading of
  !     datafiles, to 'inalpha' to preserve metal abundances from previous models
  !     Removed 'band' since it only applied to restricted spectral synthesis
  !     Hence parameter 'ifi' no longer required.
 
  !     The line opacity given to FLXODF by LINOP was found to become
  !     excessively high (10^23!!) for some low gravity models.
  !     The fix is a cap on CAPODF of, currently, 10^6.  (21/9/00).
  !     Some minor adjustments to the convergence criterion (TEMKOR) were also made.
  
  !  Version 2.6a -- 2.6d
  !     Changes introduced to allow the use of ATLAS9 type ODFS using KEY__OPBB
  !     with a value of OD9 (instead of ODF which refers to atlas6 type odf's).
  !     Conversion for Linux  + Intel Fortran Compiler completed.
  !     Improved handling of convergence criteria. Moved this within
  !     subroutine TEMKOR.
  
  !  Version 3.0
  !     A full new version incorporating b-f x-sections from the Opacity 
  !     Project and b-b opacity using opacity sampling.
  
  !  Version 3.1
  !     Moved to use the gfortran compiler  to allow more cross-platform
  !     flexibity. This followed a decade of using the Intel fortran compiler 
  !     on redhat linux boxes, including the 50-node cluster in Armagh.
  !     A consequence of the compiler change was a that large number of 
  !     issues were thrown up, which had to be resolved. At the same time, 
  !     some code rationalization was implemented. There remain opportunities 
  !     for more code cleaning. 
  !                                                     csj: 16.8.12
  !
  !  Version 3.2
  !     Various minor code upgrades 2012 - 2018  
  
  !  Version 3.4
  !     Skipped 3.3 and migrated to 3.4.0 in readiness to merge the diffusion
  !     version (SterneD) from Natalie. That was initially developed as strand
  !     3.4.2 by Conor Byrne, unsuccessfully.  I rolled back to 3.4.0 to start
  !     incrementally adding diffusion elements in due course. Meanwhile, we
  !     have added bf opacity for Sr, Y, Zr and more flexibility on input of
  !     abundances ( previously only allowed C, N, Mg, Al, Si, P, S, Ca and Fe).
  !                                                   csj:  01.04.20
  
  !  Version 3.5
  !     Completely overhauled the input dialogue .. this will mean major changes to
  !     a) scripts ... sterneos.input
  !     b) documentation  ... printed and online
  !     At this stage -- diffusion code is not yet implemented.   
  !                                                     csj: 16:07.20

  !  Version 3.5.1
  !     The OP routines were storing photo-ionisation x-sections
  !     very inefficiently - 4d arrays were sparsely filled in most cases.
  !     Redesigning the x-section packing (quite easy) we had to merge the
  !     input routines OP_BFIN and OP_FEIN in order to slot the iron x-sections
  !     in correctly. This highlighted a) poor code structure in OP_BFIN,
  !     b) an error assigning excitation energies to the Fe x-sections,
  !     and c) an absence of iron-group opacities other than iron. Finding
  !     additional opacities was not easy; only TiI, CrI and NiII were located.
  !     Moreover, the file structures and conventions took some untangling.
  !                                                      csj: 26.11.20
  

  !  Principal References:
  !
  !  Version 2.6
  !    C.S.Jeffery, V.M.Woolf, D.L.Pollacco, 2001. A&A 376, 497
  !    Time-resolved spectral analysis of the pulsating helium star V652 Her
  !  Version 3.0
  !    Behara, N. T. , Jeffery, C. S., 2006. A&A, 451, 643-650
  !    LTE model atmospheres with new opacities: I. Methods and General Properties
  !
  !-


  ! Global variables and constants
  USE stn_parameters
  USE stn_switches
  USE stn_composition
  USE stn_intgra
  USE stn_convergence

  USE lte_wlgrid
  USE lte_model
  USE op_peach
  USE op_global

  USE TAP_PAR, ONLY : TAP__RNULL


  ! Switch off global typing
  IMPLICIT NONE    

  
  !  Explicit Subroutine interfaces
  INTERFACE

     SUBROUTINE KEY_COM (PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_COM

     SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_GET

     LOGICAL FUNCTION KEY_MATCH (STRING,VALUE,OPTIONS)       
       CHARACTER*(*), INTENT(IN)           ::  STRING
       CHARACTER*(*), INTENT(IN)           ::  VALUE
       CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS
     END FUNCTION KEY_MATCH
     
     LOGICAL FUNCTION KEY_CONTAINS (STRING,VALUE,OPTIONS)       
       CHARACTER*(*), INTENT(IN)           ::  STRING
       CHARACTER*(*), INTENT(IN)           ::  VALUE
       CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS
     END FUNCTION KEY_CONTAINS
     
  END INTERFACE

  
!     Version information
  CHARACTER (6)   :: STN__VERSION_INIT = '3.5.1' 
  CHARACTER (10)  :: STN__DATE = '2020.11.26'
  CHARACTER (40)  :: STN__AUTHOR = 'C.S.Jeffery'
  CHARACTER (80)  :: STN__HELP = 'http://193.63.77.2:2805/~SJeffery/' // &
                                 'software_store/guide/sterne.html'

  !     Command line control
  CHARACTER (100) :: COMMAND   ! holder for current command line
  CHARACTER (12)  :: KEYWRD    ! holder for current command or keyword
  CHARACTER (16)  :: PARCOM    ! command for parameter entry
  INTEGER         :: I4        ! echo level for reporting
  INTEGER         :: KSTATUS   ! error / eof flag
  INTEGER :: NREAD, NWRITE

  !     Filenames and units
  CHARACTER (100) :: F__LINES = 'LINES'  !  Linelist
  CHARACTER (100) :: F__MODEL = 'MODEL'  !  Model atmosphere
  CHARACTER (100) :: F__ABUND = 'ABUNDS' !  Additional abundances
  
  INTEGER :: N__LINES          !  Unit for linelist
  INTEGER :: N__MODEL          !  Unit for model
  INTEGER :: N__ABUND = 20     !  Unit for abundances
  INTEGER :: F__ABUND_OK       !  Status of files containing abundances

  INTEGER :: F__STATUS         !  status flag for file read/write


  !  Simple counters etc.
  INTEGER :: I, J, COUNT

  !  CPU time monitoring
  REAL :: CPU0, CPU1, CPU2 ! counter


  !  Convergence checking (see stn_convergence)
  REAL :: ERR             ! Maximum error in flux
  REAL :: RMS             ! RMS temperature correction
  REAL :: EPS             ! Convergence criteria

  !  as above
  REAL, ALLOCATABLE, DIMENSION (:) :: T_NEW   ! New temperature (after correction) 
  REAL, ALLOCATABLE, DIMENSION (:) :: DT_0    ! for Temkor subroutine, saved correction
  REAL, ALLOCATABLE, DIMENSION (:) :: DELT    ! Last temperature correction
  REAL, ALLOCATABLE, DIMENSION (:) :: DELF    ! Eddington flux error (fractional)

 
  !  opacity switches (move to stn_switches ?)
  INTEGER :: newtemp_it = 0  ! tells subroutine linop that it has a new temperature
  CHARACTER(len=12) :: KEY_OPBF        ! Selects b-f opacity treatment: 'KIEL' or 'OP'
  CHARACTER(len=12) :: KEY_OPBB        ! Selects b-b opacity treatment: 'CONT', 'ODF'
  ! or 'SAMPLING'
  INTEGER :: LINE_COUNT


  ! Local composition and depth-dependent arrays (stn_composition, optical_depth?)
  INTEGER :: IZ  ! Local pointer for composition
  REAL    :: AB  ! Temporary value hoder for abundance
  REAL    :: ALPNN(40), XX, YY
  REAL, ALLOCATABLE, DIMENSION (:) :: PPE, PP, CAP4, GRAD_MODEL, T_MODEL



!.

  STN__VERSION = STN__VERSION_INIT
  
  CALL CPU_TIME (CPU0)
  CALL STN_STARTUP

!  Input.
!   Read model parameters and switches

  CALL STN_INPUT_CLASSIC
  IF ( TEFF > 1 ) &
       CALL STN_INALPHA
  
  IF ( TEFF < 1 ) &  
       CALL STN_INPUT

  CALL STN_ANNOUNCE
  FLUSH (6)

  CALL PRINT_ALPHA
  
  !! initialise values:
  DELT = 0.0
  T_NEW = 0.0

  IF (IGREY /= 0) THEN
     WRITE(*,*) 'Using previous model to start iteration'
     CALL INMOD (F__MODEL,ND, XX,YY,ALPNN,OP_MAXNA,T_MODEL,GRAD_MODEL,CAP4,PP,PPE)

     ! Use Teff and/or log g from previous model
     ! This is only for restarting previous model
     ! we've lost the mechanism for using as approx for new model
     !TEFF = XX
     !GLG = YY

     ! Use composition from previous model
     DO I=1,OP_MAXNA
        OP_ALPNN(I,1:ND) = ALPNN(I)
     END DO

     ! import pressure, temperature and radiation pressure from previous model      
     P_E     = 10.**PPE             
     P_TOTAL = 10.**PP             
     KAPPA   = 10.**CAP4
     GRAD    = GRAD_MODEL
     TEMP    = T_MODEL

     ! Reconcile temperature structure with required Teff
     IF ( ABS ( (TEFF - XX) / TEFF ) > 0.01 ) TEMP = TEMP * TEFF / XX 

     ! Reconcile pressure structure with required log g
     IF ( ABS ( (GLG - YY) ) > 0.01 ) THEN
        P_E     = P_E     * 10.**(GLG-YY) 
        P_TOTAL = P_TOTAL * 10.**(GLG-YY) 
     ENDIF

     WRITE (*,*) 'Model read'

     !****************
  END IF

  !! read in wavelength grid, wl, and allocate and initialise emergent fluxes 
  !! and line blocking factors to be used when calculating the emergent flux 

  CALL READ_WLGRID
  WRITE(*,*) 'WAVELENGTH GRID READ'

  CALL STN_STAND
  WRITE(*,*) 'OPTICAL DEPTH STRATIFICATION DEFINED'

  CALL OP_ALP

  CALL STN_STRATI
  WRITE(*,*) 'CHEMICAL STRATIFICATION DEFINED (IF ANY)'

  !! read in OP continuous opacities depending 
  !! on the choice of KEY_OPBF in the input file 
  !! read in OP continuous opacities depending 
  !! on the choice of KEY_OPBF in the input file 

  IF (KEY_OPBF(1:2) == 'OP') THEN
     CALL OP_BFIN 
     WRITE(*,*) 'OP_BFIN: OPACITIES READ' 
     WRITE(*,*)
     FLUSH(6)
   
  ELSE
     CALL OP_PEACHIN
     WRITE(*,*) 'OP_PEACHIN: OPACITY TABLES READ'
     WRITE(*,*)
     FLUSH(6)
     
  END IF

  !! read in linelist for opacity sampling depending
  !! on the choice of KEY_OPBB in the input file

  IF (KEY_OPBB(1:3) == 'SAM') THEN
     CALL BB_IN (BB_DVC) 
  END IF

  CALL CPU_TIME(CPU1)
  WRITE(*,*) 'DATA PREP COMPLETE:',CPU1-CPU0,' s'
  WRITE(*,*)
  FLUSH(6)
  

!!!!   All relevant data has been read   !!!!
!!!!     Start computing                 !!!!

  !! Start temperature iteration    

  WRITE(*,*) 'START TEMPERATURE ITERATION'


  DO COUNT=1,MAX_ITER

     NEWTEMP_IT = NEWTEMP_IT + 1    ! iteration number for subroutine linop

     CALL OP_ATFILL (ND)
     WRITE (*,*) 'atfill' ; flush(6)

     
     !! Hydrostatic equilibrium 
     CALL STN_HYDRO (KEY_OPBF) 
          WRITE (*,*) 'hydro' ; flush(6)

     !! Compute ionization fractions
     CALL OP_IONFRN (ND)

     !! set line count to 0:
     LINE_COUNT = 0    

     !! Calculate emergent flux.
     WRITE(*,*) 'flx ',KEY_OPBB(1:3)
     IF (KEY_OPBB(1:3) == 'ODF') THEN
        CALL FLXODF (NEWTEMP_IT,KEY_OPBF)
     ELSE IF (KEY_OPBB(1:3) == 'SAM') THEN
        CALL FLXOS (VMIC,KEY_OPBF, LINE_COUNT)
    ELSE IF (KEY_OPBB(1:3) == 'CON') THEN
        CALL FLXCON (NEWTEMP_IT,KEY_OPBF)
     END IF

     WRITE(*,*) 'BB OPACITY: NO. LINES:', line_count

     !! Temperature correction.
     CALL STN_TEMKOR (T_NEW,DT_0,DELT,DELF,ERR,RMS)
 
     !! Test convergence
!!! also need to test for abundances...

      WRITE(*,*) 'Test convergence', COUNT, ERR

      IF (ERR <= EPS) THEN
         WRITE (*,*) 'Flux converged'
         EXIT
      END IF

      IF (RMS <= EPS/100.0) THEN
         WRITE (*,*) 'Temperature correction converged'
         EXIT
      END IF

!      IF (LM == IFO) THEN
!         WRITE(*,*) 'Maximum number of iterations reached'
!         EXIT
!      END IF

     !! Output current model

      CALL OUTPUT1     (KEY_OPBF,DELT,1,T_NEW)
      CALL OUTMOD     ! 

      ! Formatted with labels
      CALL STN_OUTMODEL (KEY_OPBF,KEY_OPBB)

     !! apply temperature correction

      DO I = 1,ND
         TEMP(I) = T_NEW(i)
      END DO

     LM = LM+1

     CALL CPU_TIME(CPU2)
     WRITE(*,'(/,x,a,i4,a,i6,a/)') &
          'LAST ITERATION TOOK: ',INT(CPU2-CPU1),' S: TOTAL ELAPSED: ',INT(CPU2-CPU0),' S'
     CPU1=CPU2

  END DO
  !! end temperature iteration

  !! Output final model after convergence or final iteration
  !! including files for DIPSO and SPECTRUM

   CALL OUTPUT1      (KEY_OPBF,DELT,0,T_NEW)
   CALL OUTMOD
   CALL STN_OUTMODEL (KEY_OPBF,KEY_OPBB)
   CALL STERNE_QUB



CONTAINS

  SUBROUTINE STN_STARTUP

    !+
    !     S T N _ S T A R T U P
    !     sets default values for flags, switches and variables
    !     and initialises the i/o streams for the command interpreter
    !-

    INTEGER, DIMENSION(8) :: DATE_VALS

    ! external functions
    DOUBLE PRECISION :: TAP_ABSOL
    DOUBLE PRECISION :: TAP_EMASS

    !.

    WRITE (*,*)
    WRITE (*,*)
    WRITE (*,*) '  SSS  TTTTT  EEEEE  RRR   N   N  EEEEE          OOO    SSS   '
    WRITE (*,*) ' S       T    E      R  R  NN  N  E             O   O  S      '
    WRITE (*,*) '  SS     T    EEE    RRR   N N N  EEE           O   O   SS    '
    WRITE (*,*) '    S    T    E      R R   N  NN  E             O   O     S   '
    WRITE (*,*) ' SSS     T    EEEEE  R  R  N   N  EEEEE  _____   OOO   SSS    '
    WRITE (*,*)
    WRITE (*,*) ' Version ', STN__VERSION
    WRITE (*,*) ' Date    ', STN__DATE
    WRITE (*,*) ' Authors ', STN__AUTHOR
    WRITE (*,*) ' Help    ', STN__HELP
    WRITE (*,*)

    CALL DATE_AND_TIME ( VALUES = DATE_VALS )
    WRITE (*,'(A,I5,2I3)') '  Run Date ', DATE_VALS(1:3)
    WRITE (*,'(A,2(I2,'':''),I2)') '  Run Time  ', DATE_VALS(5:7)
    WRITE (*,*)
    FLUSH (6)

    
    !     Set up default i/o channels
    CALL STR_DEF (NREAD, NWRITE)
    COMMAND(1:20) = '                    '
    COMMAND(21:40) = '                    '
    COMMAND(41:60) = '                    '
    COMMAND(61:80) = '                    '
    COMMAND(1:11) = 'stn_startup'
    KEYWRD(1:3)   = '   '


    
    !! initialise counters:
    NIT = 0

    !  Convergence criterion for max(df/f)           
    EPS = 0.001


    ! Collect solar abundances
    DO IZ = 1,NZ
       CALL TAP_ENAME ( IZ, ELEM(IZ), F__STATUS )   !! check status=0 before use 
       EPSSOL(IZ) = TAP_ABSOL ( IZ )
       ATW(IZ)    = TAP_EMASS ( IZ )
    ENDDO
    WHERE (EPSSOL .EQ. TAP__RNULL) EPSSOL = 0.00D00
    ALPSOL = 10.**(EPSSOL-12.00)
    WRITE (*,*) 'Solar'
    WRITE (*,'(2f5.2,1p,8e9.2,0p,f9.6)') &
         alpsol(1:2), alpsol(6:8), alpsol(10:16:2), alpsol(26), sum(alpsol)
    CALL NORMALISE (ALPSOL)
    BETSOL = ALPSOL*ATW
    CALL NORMALISE (BETSOL)

    WRITE (*,'(2f5.2,1p,8e9.2,0p,f9.6)') &
         alpsol(1:2), alpsol(6:8), alpsol(10:16:2), alpsol(26), sum(alpsol)

    ! Zero the default abundance array
    ALPMET = 0.
    FLUSH (6)

  END SUBROUTINE STN_STARTUP


  
  SUBROUTINE STN_ALLOCATE

    !+
    !     S T N _ A L L O C A T E
    !     allocates arrays dependent on number of depth points
    !-

    ! composition arrays
    ALLOCATE (ALPHA(NZ,ND))

    ! structure arrays
    ALLOCATE (TAU(ND))
    ALLOCATE (ALQ(ND))
    ALLOCATE (TEMP(ND))
    ALLOCATE (T_NEW(ND))
    ALLOCATE (T_MODEL(ND))
    ALLOCATE (P_TOTAL(ND))
    ALLOCATE (PP(ND))
    ALLOCATE (P_E(ND))
    ALLOCATE (PPE(ND))
    ALLOCATE (CAP4(ND))
    ALLOCATE (GRAD_MODEL(ND))
    ALLOCATE (DELT(ND))
    ALLOCATE (DELF(ND))

    ! opacity arrays
    ALLOCATE (KAPPA(ND))
    ALLOCATE (KAPPA_NU(ND))
    ALLOCATE (KAPPA_BB(ND))
    ALLOCATE (SIG(ND))
    ALLOCATE (GRAD(ND))
    ALLOCATE (DENS(ND))
    ALLOCATE (DIST(ND))
    ALLOCATE (XINT(ND))
    ALLOCATE (DT_0(ND))
    DT_0 = 0.0

    ! flux arays
    ALLOCATE (FLX(ND))
    ALLOCATE (TNS(ND))
    ALLOCATE (KAPP(ND))
    ALLOCATE (KAPJ(ND))
    ALLOCATE (KAPF(ND))
    ALLOCATE (B(ND))      

  END SUBROUTINE STN_ALLOCATE


  
  SUBROUTINE STN_DEALLOCATE

    !+
    !     S T N _ D E A L L O C A T E
    !     deallocates arrays dependent on number of depth points
    !-

    DEALLOCATE ( ALPHA )
    DEALLOCATE ( TAU, ALQ, TEMP, T_NEW, DELT, DELF ) 
    DEALLOCATE ( PPE, PP, CAP4, GRAD_MODEL, T_MODEL)
    DEALLOCATE ( P_TOTAL, P_E , DENS, DIST, XINT, DT_0 ) 
    DEALLOCATE ( KAPPA, KAPPA_NU, KAPPA_BB, SIG, GRAD )
    DEALLOCATE ( FLX, TNS, KAPP, KAPJ, KAPF, B )
   
  END SUBROUTINE STN_DEALLOCATE


  
  SUBROUTINE STN_ANNOUNCE

    !+
    !     S T N _ A N N O U N C E
    !     reports the startup options
    !-
    
    WRITE(*,*)
    WRITE(*,*) 'MODEL PARAMETERS'
    WRITE(*,*)
    WRITE(*,*) 'EFFECTIVE TEMPERATURE  :  TEFF ',TEFF
    WRITE(*,*) 'SURFACE GRAVITY        : LOG G ',GLG
    WRITE(*,*) 'MICROTURBULENT VELOCITY:  VMIC ',VMIC
    WRITE(*,*)
    WRITE(*,*) 'CONTINUOUS (BF) OPACITY:       ',KEY_OPBF
    WRITE(*,*) 'LINE (BB) OPACITY      :       ',KEY_OPBB
    WRITE(*,*) 'NUMBER OF DEPTH POINTS :       ',ND
    WRITE(*,*) 'NUMBER OF FREQUENCIES  : (???) ',N_OPWL
    IF (IGREY ==0) & 
         WRITE(*,*) 'STARTING MODEL         :       GREY '
    WRITE(*,*)
    
  END SUBROUTINE STN_ANNOUNCE

  

  SUBROUTINE STN_INPUT

    !+
    !     S T N _ I N P U T
    !     Basic control parameters for STERNE
    !     Introduced 2020 July
    !-
    

    !     Abundance inputs
    INTEGER            :: EL__Z          ! Atomic number for current element 
    REAL(8)            :: EL__A          ! Abundance (number fraction) for element
    REAL(8)            :: EL__B          ! Abundance (mass fraction) for element
    REAL(8)            :: EL__SC         ! Logarithmic scaling factor for abundances
    INTEGER            :: EL__J1, EL__J2 ! Pointer to tau range for abundance
    ! REAL(8)            :: EL__SUM        ! Abundance summation (working) 
    REAL(8)            :: EL__FAC        ! Abundance adjustment factor (working)
    
    REAL(8), EXTERNAL  :: TAP_ABSOL
    
    !.


    ! Defaults

    I4       = 0

    ND       = 50       ! Number of depth points
    CALL STN_ALLOCATE   ! Set up arrays with ND depth points by default
    
    IGREY    = 0        ! Start from grey model
    LM       = 1        ! Initialize model counter = 0
    MAX_ITER = 300      ! Maximum number of iterations
    
    KEY_OPBF = 'OP'     ! OP continuous opacities
    KEY_OPBB = 'SAM'    ! Opacity sampling
    BB_DVC   = 0.001    ! delta nu / nu for wavelength grid
    VMIC     = 0.       ! microtrubulent velocity (cm/s)
    REFWL    = 4000.    ! Reference wavelength for optical depth scale
    EP_0     = 0.       ! opacity threshold; never used; obsolete

    F__MODEL = 'MODIN'  ! Default name for input model defined externally
    
    N_OPWL = 3
    ALLOCATE(OPWL(N_OPWL))
    OPWL = (/ 5000., 2400., 2000. /)
    
    ! Loop over input commands
    DO

       !     get command line
       CALL KEY_GET ( KEYWRD, COMMAND, 'stn>', ';', NREAD, I4, KSTATUS )
       FLUSH (6)

       !     trap EOF 
       IF ( KSTATUS /= 0 ) THEN
          WRITE (*,*) '** STN_INPUT ** : Error ',KSTATUS,' reading commands'
          STOP
       ENDIF
       
       !     ensure case compliance
       CALL STR_LCASE ( KEYWRD )

       !  act on exit condition immediately
       IF (KEYWRD(1:3) == 'stop' .OR. KEYWRD(1:4)  == 'exit' &
            .OR. KEYWRD(1:1)  == 'q')   STOP
       
       !    run the program
       IF ( KEY_MATCH ( KEYWRD, 'run' ) ) THEN
          ! Read number of iterations to execute 
          CALL KEY_PI ( COMMAND, MAX_ITER ) 

          ! Check composition is OK
          DO J = 1,ND
             CALL NORMALISE ( ALPHA(1:NZ,J) )
          ENDDO
          ALPMET   = ALPHA(1:NZ,ND)

          ! Copy composition to opacity arrays
          OP_ALPNN(1:OP_MAXNA,1:ND) = ALPHA(1:OP_MAXNA,1:ND)
          
          EXIT
          
       !    model name
       ELSEIF ( KEY_MATCH ( KEYWRD, 'label' ) )  THEN
          CALL KEY_PC ( COMMAND, STN__LABEL )

       !    effective temperature
       ELSEIF ( KEY_MATCH ( KEYWRD, 'teff' ) .OR.  KEY_MATCH ( KEYWRD, 'temp' ) )  THEN
          CALL KEY_PR ( COMMAND, TEFF )

       !    surface gravity
       ELSEIF ( KEY_MATCH ( KEYWRD, 'grav' ) .OR.  KEY_MATCH ( KEYWRD, 'logg' ) )  THEN
          CALL KEY_PR ( COMMAND, GLG )
             
       !    redefine number of depth points
       !    this will work if ndepth done before abund, but not after
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ndepth' ) ) THEN
          CALL KEY_PI ( COMMAND, ND )
          CALL STN_DEALLOCATE
          CALL STN_ALLOCATE

          
       ! ab_metal
       !   
       !    metal abundances scaled to solar
       !    defines abundances for all elememnts > helium at all optical depths   
       !    ok for solar-type hydrogen, but not mass-conservative for helium-rich    
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ab_metal' ) ) THEN
          CALL KEY_PD ( COMMAND, EL__SC )
          DO EL__Z = 3,NZ
             EL__B =  10.**EL__SC * BETSOL(EL__Z)
             BETMET(EL__Z)      = EL__B 
             ALPMET(EL__Z)      = EL__B / ATW(EL__Z)
          ENDDO

          ! Ensure H+H add up to 1 - Sum(metals)  in mass fraction
          IF ( BETMET(1)+BETMET(2) > 0. ) THEN
             EL__B = 10.**EL__SC * SUM ( BETSOL(3:NZ) )
             EL__FAC = (1. - EL__B) / (BETMET(1)+BETMET(2))
             BETMET(1) = BETMET(1) * EL__FAC
             BETMET(2) = BETMET(2) * EL__FAC
          ENDIF
          CALL NORMALISE ( BETMET )

          
          ! Then convert to number fraction
          ALPMET = BETMET / ATW
          CALL NORMALISE ( ALPMET )

          
          ! Populate all zones          
          CALL FILL_ALPHA          

          WRITE (*,*)
          WRITE (*,*) 'STN_INPUT: ab_metal check: alpha, beta (1,2,6,7,8,10,12,14,16,26, Sum)  '
          WRITE (*,'(2f5.2,1p,8e9.2,0p,f9.6)') &
               alpmet(1:2), alpmet(6:8), alpmet(10:16:2), alpmet(26), sum(alpmet)
          WRITE (*,'(2f5.2,1p,8e9.2,0p,f9.6)') &
               betmet(1:2), betmet(6:8), betmet(10:16:2), betmet(26), sum(betmet)

          
       !    ab_scale
       !   
       !    scale abundance for single element at all optical depths
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ab_scale' ) ) THEN
          CALL KEY_PI (COMMAND, EL__Z)
          CALL KEY_PD (COMMAND, EL__SC)
          EL__A              = EL__SC * ALPSOL(EL__Z)
          ALPMET(EL__Z)      = EL__A
          BETMET(EL__Z)      = EL__A * ATW(EL__Z)
          CALL CONSERVE
          CALL FILL_ALPHA          


       !    ab_elem
       !
       !    defines abundance for single element at all optical depths
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ab_elem' ) ) THEN
          CALL KEY_PI (COMMAND, EL__Z)
          CALL KEY_PD (COMMAND, EL__A)
          ALPMET(EL__Z)      = EL__A
          BETMET(EL__Z)      = EL__A * ATW(EL__Z)

          IF (EL__Z > 2) CALL CONSERVE
          CALL FILL_ALPHA          


       !    ab_norm
       !
       !    normalize abundances for all elements at all optical depths
       !    should be redundant -- if this works then checked at 'run' time.   
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ab_norm' ) ) THEN
          DO J = 1,ND
             CALL NORMALISE ( ALPHA(1:NZ,J) )
          ENDDO

          !! we have to work out what order logic happens for
          !! defining abundances, assumig defaults, distributing over layers,

       ! ab_strat
       !         
       !    abundance stratification
       !    defines abundance for single element over range of optical depths   
       ELSEIF ( KEY_MATCH ( KEYWRD, 'ab_strat' ) ) THEN
          CALL KEY_PI (COMMAND, EL__Z)
          CALL KEY_PI (COMMAND, EL__J1)
          CALL KEY_PI (COMMAND, EL__J2)
          CALL KEY_PD (COMMAND, EL__A)
          EL__J1 = MAX (1,EL__J1)
          EL__J2 = MIN (EL__J2,ND)
          ALPHA(EL__Z,EL__J1:EL__J2) = EL__A
                 
       !    initial model grey
       ELSEIF ( KEY_MATCH ( KEYWRD, 'grey' ) ) THEN
          IGREY = 0   ! logic wrong

       !    start from converged or partially converge model
       ELSEIF ( KEY_MATCH ( KEYWRD, 'non-grey' ) ) THEN
          IGREY = 1   ! logic wrong

       !    name of file containing starting model
       ELSEIF ( KEY_MATCH ( KEYWRD, 'model' ) ) THEN
          CALL KEY_PC ( COMMAND, F__MODEL )
          WRITE (*,*) 'Starting model: ',F__MODEL

       !    iteration counter if restarting model
       ELSEIF ( KEY_MATCH ( KEYWRD, 'iter' ) ) THEN
          CALL KEY_PI ( COMMAND, LM ) 
       
       !    maximum number of iterations
       ELSEIF ( KEY_MATCH ( KEYWRD, 'maxit' ) ) THEN
          CALL KEY_PI ( COMMAND, MAX_ITER ) 

       !    continuous opacity switch
       ELSEIF ( KEY_MATCH ( KEYWRD, 'opacity' ) ) THEN
          CALL KEY_PC ( COMMAND, KEY_OPBF )
          IF ( KEY_OPBF(1:2) /= 'op' .AND. KEY_OPBF(1:2) /= 'ki' ) THEN
             WRITE (*,*) '** STN_INPUT ** bad KEY_OPBF, assuming default: OP'
             KEY_OPBF = 'OP'
          ENDIF

       !    line opacity switch   
       ELSEIF ( KEY_MATCH ( KEYWRD, 'lines' ) ) THEN
          CALL KEY_PC ( COMMAND, KEY_OPBB )
          IF ( KEY_OPBB(1:3) /= 'sam' .AND. KEY_OPBB(1:3) /= 'odf' &
               .AND. KEY_OPBB(1:3) /= 'con' ) THEN
             WRITE (*,*) '** STN_INPUT ** bad KEY_OPBB, assuming default: SAM'
             KEY_OPBF = 'SAM'
          ENDIF
       
       !    opacity sampling interval 
       ELSEIF ( KEY_MATCH ( KEYWRD, 'lambda_int' ) ) THEN
          CALL KEY_PR ( COMMAND, BB_DVC )
       
       !    reference wavelength for optical depth scale
       ELSEIF ( KEY_MATCH ( KEYWRD, 'lambda_ref' ) ) THEN
          CALL KEY_PR ( COMMAND, REFWL  ) 
       
       !    number and values of reference wavelengths
       ELSEIF ( KEY_MATCH ( KEYWRD, 'lambda_n' ) ) THEN
          CALL KEY_PI ( COMMAND, N_OPWL )
          DEALLOCATE (OPWL)
          ALLOCATE(OPWL(N_OPWL))
          READ(*,*) OPWL(1:N_OPWL)
       
       !    microturbulent velocity (cm/s)
       ELSEIF ( KEY_MATCH ( KEYWRD, 'vmicro' ) ) THEN
          CALL KEY_PR ( COMMAND, VMIC )
          VMIC = VMIC * 1.E5 ! convert to cm/s    

       ! echo commands to the terminal
       ELSEIF (KEYWRD(1:4)=='echo') THEN

          CALL KEY_PC (COMMAND, PARCOM)
          IF (PARCOM(1:1) == "2" )                         I4 = 2
          IF (PARCOM(1:2) == "on"  .OR. PARCOM(1:1)=="1" ) I4 = 1
          IF (PARCOM(1:3) == "off" .OR. PARCOM(1:1)=="0" ) I4 = 0
          
       !  help
       ELSEIF (KEYWRD(1:4) == 'help') THEN 
          WRITE (*,*) 'See: '//STN__HELP

          
       ENDIF

    ENDDO

  END SUBROUTINE STN_INPUT

  
  SUBROUTINE STN_INPUT_CLASSIC

    !+
    !     S T N _ I N P U T _ C L A S S I C
    !     takes classic form of terminal input
    !     use is deprecated as of 2020 December.
    !-

    INTEGER :: IOERROR ! status flag to test for classic input
    
    !    effective temparature and gravity
    READ(*,*,IOSTAT=IOERROR) TEFF, GLG

    !    test for classic input .. flagged by fail on first line
    IF ( IOERROR /= 0 ) THEN
       TEFF = 0.
       RETURN
    ENDIF

    !    number of depth points
    READ(*,*) ND
    CALL STN_ALLOCATE
    
    !    composition: H, He, C, N          ! (Li - B redundant)
    ALPHA = 0.
    READ(*,*) ALPMET(1:2),ALPMET(6:7)
    !    composition: O, Mg, Al, Si, S, Ca, Fe
    READ(*,*) ALPMET(8), ALPMET(12), ALPMET(13), ALPMET(14), ALPMET(16), ALPMET(20), ALPMET(26)
    ALPHA(1:56,ND) = ALPMET(1:56)
    
    READ(*,*) LM, MAX_ITER  !! , IFO, KIMAX - last two variable thought redundant
    !    grey model ?
    READ(*,*) IGREY
    !    stratification parameters
    READ(*,*) IEL, TAUX0, ALPHAX0
    IF ( IEL>0 .AND. IEL<8) ALPHA(IEL,1) = ALPHAX0
    
    !    opacity type switches
    READ(*,*) KEY_OPBF
    READ(*,*) KEY_OPBB
    !    opacity sampling interval and microturbulence (BB_DVC, VMIC)
    READ(*,*) BB_DVC, VMIC, EP_0
    !    number of opacity wavelengths and ref wavelentgh
    READ(*,*) N_OPWL
    ALLOCATE(OPWL(N_OPWL))
    READ(*,*) (OPWL(i), i=1,N_OPWL)
    READ(*,*) REFWL
    
    !! Extra abundances -- currently uniform composition
    
    !! Check for existence of additional input file
    !! This is a temporary solution to add more opacities without disrupting the existng I/O
    !! So ABUNDS must use the same units as the main input file -- which is not what Spectrum uses.
    !! This should be improved in the next version
    OPEN (UNIT=N__ABUND,FILE=F__ABUND,IOSTAT=F__ABUND_OK,STATUS='OLD')
    IF (F__ABUND_OK == 0) THEN
       WRITE(*,*) 'ADDITIONAL ABUNDANCES :     ',F__ABUND
       DO WHILE (F__ABUND_OK == 0)
          READ (N__ABUND,*,IOSTAT=F__ABUND_OK) IZ, AB
          IF (F__ABUND_OK == 0) THEN
             ALPMET(IZ)   = AB
             ALPHA(IZ,ND) = AB
          ENDIF
       ENDDO
    ENDIF
    
  END SUBROUTINE STN_INPUT_CLASSIC


  
  SUBROUTINE NORMALISE ( ARRAY ) 

    REAL, DIMENSION(:) :: ARRAY
    REAL               :: SUMA

    SUMA = SUM ( ARRAY )
    ARRAY = ARRAY / SUMA

  END SUBROUTINE NORMALISE


  SUBROUTINE CONSERVE 

    REAL(8) :: EL__FAC
    
    ! adjust H+He for metals to conserve numbers
    IF ( ALPMET(1)+ALPMET(2) > 0 .AND. SUM(ALPMET) /= 1.0 ) THEN
       EL__FAC = 1./(ALPMET(1)+ALPMET(2))
       ALPMET(1) = (1. - SUM(ALPMET(3:NZ))) * ALPMET(1) * EL__FAC
       ALPMET(2) = (1. - SUM(ALPMET(3:NZ))) * ALPMET(2) * EL__FAC
    ENDIF

  END SUBROUTINE CONSERVE


  SUBROUTINE FILL_ALPHA

    INTEGER :: EL__Z
    
    ! distribute over all zones
    DO EL__Z = 1,NZ
       ALPHA (EL__Z,1:ND) = ALPMET(EL__Z)
    ENDDO

  END SUBROUTINE FILL_ALPHA


  SUBROUTINE PRINT_ALPHA

    REAL :: MUSOL, MUMET  !! mean atomic weights
    REAL :: ANSOL !! normalisation constant for the Sun
    REAL :: ANMET !! normalisation constant for the star
    REAL :: ANORM !! normalisation constant ANSOL - ANMET
    
    ! Compute and check mass fractions
    BETMET = ALPMET * ATW
    CALL NORMALISE (BETMET)
    
    ! Compute logarithmic abudances
    ANSOL  = ALOG10( SUM (ATW*ALPSOL) )
    ANMET  = ALOG10( SUM (ATW*ALPMET) )
    ANORM  = 12.0 + ANSOL - ANMET
    
    ! compute epsilons for star  !! possibly not correct at present
    EPSMET = ALOG10(ALPMET) + ANORM

    ! check mean atomic weights
    MUSOL = SUM ( ATW*BETSOL ) / SUM ( BETSOL )
    MUMET = SUM ( ATW*BETMET ) / SUM ( BETMET )
    
    ! report adopted composition clearly to user
    WRITE(*,*) 'ELEMENTAL ABUNDANCES'
    WRITE(*,*)
    WRITE(*,'(3(A8,F7.3))') ' ANSOL ',ANSOL,' ANMET ',ANMET,' ANORM ',ANORM
    WRITE(*,'(3(A8,F7.3))') ' MUSOL ',MUSOL,' MUMET ',MUMET
    WRITE(*,*) 
    WRITE(*,*) '    ELEMENT        Z  alpha     beta      eps      alpha_sun beta_sun  eps_sun   beta/beta_sun'
    WRITE(*,'(A18,I3,1X,1P,2E10.3,0P,F7.3,2x,1P,2E10.3,0P,F7.3,2x,F7.3)') &
         & (ELEM(I),I,ALPMET(I),BETMET(I),EPSMET(I),ALPSOL(I),BETSOL(I),EPSSOL(I),BETMET(I)/BETSOL(I),I=1,40)
    WRITE(*,*)

  END SUBROUTINE PRINT_ALPHA

  
END PROGRAM STERNE_OS

!!#######################################################################


