      PROGRAM STERNE_FORMAT
*+
* 
* Name:
*     STERNE_FORMAT
*
* Purpose:
*     Reformat a STERNE model atmosphere for input to SPECTRUM
*     and for input to DIPSO (ie the flux distribution)
*
* Language:
*     Fortran 77
*
* Description:
*
*     This code operates on ouput from the line-blanketed version 
*
* Authors:
*     CSJ: C.S.Jeffery (CCP7)
*
* History:
*     21-JAN-1993 (CSJ): original
*     21-MAR-1996 (CSJ): adapted to ensure composition data always present
*
* Bugs:
*
*-

*  Double precision

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)


*  i/o file names nad headers

      CHARACTER*20   FROOT
      CHARACTER*20   FSTERNE
      CHARACTER*20   FSPECTRUM
      CHARACTER*20   FDIPSO
      CHARACTER SPHEAD(2)*79


*  Model description

      PARAMETER      (KD=50)     ! Maximum onumber of zones in model atmosphere

      CHARACTER*10 CDATE         ! Computation date
      CHARACTER*10 CTIME         ! Computation time

      DOUBLE PRECISION TEFF      ! Effective temperature
      DOUBLE PRECISION GRAV      ! Surface gravity 

      INTEGER NDEPTH             ! Number of layers in model atmosphere
      DOUBLE PRECISION TAU(KD)   ! Optical depth grid for model atmospheres
      DOUBLE PRECISION TT(KD)    ! Temperature as function of optical depth
      DOUBLE PRECISION GRAD(KD)  ! dPr / d tau ....
      DOUBLE PRECISION PG(KD)    ! Gas pressure ....
      DOUBLE PRECISION PE(KD)    ! Electron pressure ....
      DOUBLE PRECISION OP(KD)    ! Continuous opacity ....


      DOUBLE PRECISION ALPHA(7)  ! Atmospheric composition (homogeneous)
      DOUBLE PRECISION ALPMET(7) !

      DOUBLE PRECISION AB(7,KD)  ! Depth dependent composition

      INTEGER NW                 ! Number of wavelength points in spectrum
      DOUBLE PRECISION W(1000)   ! Wavelengths
      DOUBLE PRECISION F(1000)   ! Astrophysical fluxes
      DOUBLE PRECISION B(1000)   ! Blocking factor

*.


* Identify the file containing a converged STERNE model

11    WRITE (6,*) 'Enter filename of STERNE model to be reformatted'
      WRITE (6,*) 'In current directory, extension ".MODEL" assumed'
12    READ  (*,'(A)') FROOT
      IF (FROOT(1:3).EQ.'   ') GOTO 99

*   Construct legal filenames
      DO J = 20,1,-1
         IF (FROOT(J:J).EQ.' ') JL = J-1
      ENDDO
      FSTERNE = FROOT(1:JL)//'.MODEL'
      FSPECTRUM = FROOT(1:JL)//'.q'
      FDIPSO    = FROOT(1:JL)//'.s'


* Read in the converged model structure and flux distribution

*   Open the input model
      OPEN  (1,FILE=FSTERNE,STATUS='OLD',ERR=98)

*   Read the model structure
      NDEPTH = KD
*     READ  (1,'(2A10)') CDATE,CTIME
      READ  (1,'(F6.0,F5.0)') TEFF, GRAV !, ALC, ALN, ALPMET
      READ  (1,*) ALPHA
      READ  (1,'(10F8.0)') (TT(J),J=1,NDEPTH)
      READ  (1,'(10E10.0)') (GRAD(J), J=1,NDEPTH)  
      READ  (1,'(10F8.0)') (OP(J),J=1,NDEPTH)
      READ  (1,'(10F8.0)') (PG(J),J=1,NDEPTH)
      READ  (1,'(10F8.0)') (PE(J),J=1,NDEPTH)
      READ  (1,'(5E15.0)') (TAU(J),J=1,NDEPTH)

*   Read the depth dependent composition
      READ (1,*) IDUMMY
      READ (1,104) ((AB(I,J),I=1,7),J=1,50)
104   FORMAT (F6.0,5X,F6.0,4X,5E11.0)

*   Read the flux distribution
      READ (1,*) NW
      READ (1,105) (W(K),F(K),B(K),K=1,NW)
105   FORMAT (F8.0,E12.0,F6.0)

*   Close the input
      CLOSE (1)

* Process the model structure

*   Take antilogs of the model
      GRAV = 10.**GRAV
      IF (OP(1).LT.0.) THEN
         DO J = 1,NDEPTH
           OP(J) = 10.**OP(J)
         ENDDO
      ENDIF
      DO J = 1,NDEPTH
         PG(J) = 10.**(PG(J))
         PE(J) = 10.**(PE(J))
      ENDDO

*   Sort out the composition data
      IF (ALPHA(1)+ALPHA(2).GT.2.0) THEN
         DO I = 1,7
            ALPHA(I) = ALPHA(I) / 10.
         ENDDO
      ENDIF

*   all abundances are fractional
*   defaults: ACN=0 means C and N take log abundances
      AH  = ALPHA(1)
      AHE = ALPHA(2)
      ACN = ALPHA(6) + ALPHA(7)

*   check C and N abundances, 
*   IF zero, adopt solar values * metallicity factor
*   ratios for sum over metals = 1. from Anders and Grevesse (1988)
                        AC  = ALPHA(6)
      IF (AC.LT.0.0001) AC  = 10.**(ALC-3.440)
                        AN  = ALPHA(7)
      IF (AN.LT.0.0001) AN  = 10.**(ALN-3.950)

*   then scale remaining metals to solar ratios
      AO  = 10.**(ALPMET(1)-3.07)
      ANE = 10.**(ALPMET(2)-3.91) ! use Mg as reference
      AMG = 10.**(ALPMET(2)-4.42)
      AAL = 10.**(ALPMET(3)-5.53)
      ASI = 10.**(ALPMET(4)-4.45)
      AS  = 10.**(ALPMET(5)-4.79)
      ACA = 10.**(ALPMET(6)-5.64)
      AFE = 10.**(ALPMET(7)-4.33)

*   Conservation of numbers
      SUM = AH  + AHE + AC  + AN  + AO  + ANE + AMG
     1                + AAL + ASI + AS  + ACA + AFE
      OVR = SUM - 1.0
      IF (AH.GT.AHE) AH  = AH  - OVR
      IF (AHE.GT.AH) AHE = AHE - OVR


* Report the model 

      WRITE (6,*)
      WRITE (6,'(1X,A,F8.1)') 'Teff  = ', TEFF
      WRITE (6,'(1X,A,F8.1)') 'log g = ', LOG10(GRAV)
      WRITE (6,'(1X,A,F8.5)') 'nH    = ', AH
      WRITE (6,'(1X,A,F8.5)') 'nHe   = ', AHE
      WRITE (6,'(1X,A,F8.5)') 'nC+nN = ', ACN
      WRITE (6,*)


* Write out the model for SPECTRUM

      OPEN  (2,FILE=FSPECTRUM,STATUS='NEW')
      WRITE (2,'(F12.5,1PE12.5,0P3F12.5)') TEFF, GRAV, AHE, AH, ACN
*     IF (ACN.GT.0.0001) THEN
         WRITE (2,'(1P5E12.5)') AC ,AN ,AO ,ANE,AMG
         WRITE (2,'(1P5E12.5)') AAL,ASI,AS ,ACA,AFE
*     ENDIF

      DO J = 1,NDEPTH
         WRITE (2,'(1P5E12.5)') TAU(J),TT(J),PG(J),PE(J),OP(J)
      ENDDO
      CLOSE (2)


* Write out the spectrum for DIPSO

*   Create some header lines
      WRITE (SPHEAD(1),'(''STERNE: Teff='',F6.0,'' log g='',F3.1,
     :       '' H='',F4.2,'' He='',F4.2,'' CN='',F7.5)')
     :       TEFF, DLOG10(GRAV), AH, AHE, ACN
      SPHEAD(2) = CDATE//CTIME 

*   Write the file - SPECTRUM format 2
      OPEN  (2,FILE=FDIPSO,STATUS='NEW')
      WRITE (2,'(A)') SPHEAD(1)
      WRITE (2,'(A)') SPHEAD(2)
      WRITE (2,'(I10)') NW
      WRITE (2,'(F10.1,E12.5)') (W(K),F(K),K=1,NW)
      CLOSE (2)

*   Go and get the next model
      FROOT = '   '
      GOTO 12

*.
98    WRITE (*,*) 'Error opening file, try again'
      GOTO 12

99    END
