  SUBROUTINE BB_KAP (JD,INU,TT,VMIC,KBB,OPA, ep_0, line_count)

!+
!
!  Calculate line opacity due to lines within DNU of 
!  frequency NU. Uses opacity sampling technique:
!  (Sneden et al., 1976, ApJ 204, 281)
! 
!
!  Language:
!     Fortran 90
!
!  Input arguments:
!     JD                 !  Depth pointer
!     INU                !  Frequency pointer
!     TT                 !  Temperature             (K)
!     VMIC               !  Local microturbulent velocity (cm/s) 
!     OPA                !  continuous opacity at depth point JD
!     ep_0               !  threshold of opacity
!
!  Output argument:
!     KBB                !  Line opacity k(nu)
!
!  Authors:
!     CSJ: C. S. Jeffery (Armagh Observatory)
!
!  History:
!     03-MAR-1999 (CSJ): New program
!     17-MAY-2004 (NTB): modified
!-
     
      USE LTE_PRECISIONS
      USE BB_GLOBAL
      USE TAPP_PAR
      USE TAPM_PAR
      USE OP_GLOBAL
      USE LTE_WLGRID

      IMPLICIT NONE

!  Subroutine Arguments (Given).
      INTEGER :: JD              !  Depth pointer
      INTEGER :: INU             !  Frequency pointer
      REAL :: TT    !  Temperature (K)
      REAL :: VMIC  !  Microturbulent velocity (cm/s)
      REAL :: OPA   !  continuous opacity
      REAL :: EP_0  !  threshold of opacity
      INTEGER :: LINE_COUNT  ! counter for lines selected

!  Subroutine Arguments (Returned).
      REAL(KIND=QQ) :: KBB   !  Line opacity per gram at frequency nu

!  External functions called.
      REAL, EXTERNAL ::  BB_VOIGT        !  Voigt function for this package

!  Local variables.
      INTEGER :: IK              !  Atomic species (dummy pointer for OP)
      INTEGER :: KKK             !  Atomic species ( = NZ )
      INTEGER :: JJ              !  Ion pointer    ( = NZ - NE + 1 )
      INTEGER :: I

      REAL :: ABB    ! Line opacity per atom at frequency nu
      REAL :: CSTIM  ! global and stimulated emission multiplier
      REAL :: VMIC2  ! square of microturbulent velocity
      REAL :: GAMMA  ! local value of adiabatic index -- assumed const = 4./3.
      REAL :: CS2    ! square local sound speed (cm/s)
      REAL :: BKT    ! kT in same units as excitation energies

      REAL :: BBDOP(BB_MAXBA)    ! Doppler width
      REAL :: BBDNU(BB_MAXBA)    ! Scaled Doppler width      
      REAL :: BBLDP(BB_MAXBA)    ! log Doppler width      
      REAL(KIND=QQ) :: ABBX            ! integrand - no damping
      REAL(KIND=QQ) :: ABBD            ! integrand - Doppler approximation
      REAL(KIND=QQ) :: ABBV            ! integrand - natural damping

      REAL :: NG(BB_MAXNI,BB_MAXBA) !  n(k,j)/g(k,j)

      REAL :: LLNU   !  local frequency ( nu )
      REAL :: LLNC   !  scaled frequency ( nu/c )

      REAL :: AM
      REAL :: DOPPNU
      REAL :: XXI
      REAL :: SUMBB
      INTEGER :: NP, NQ
 
!!$      real :: ATEST           !  test 'a' value for line broadening 
!!$      DATA    ATEST / 0.001 /

!  Local constants.

!  sqrt(2k/mH)/c 
      REAL, PARAMETER :: DOPPLER = 2 * TAPP__K*1.D7 * TAPP__NA 

!  sqrt(pi)e^2/mc                    ! e (cgs) =  e * 10c (SI) ???
      REAL, PARAMETER :: CONST1 = TAPM__SQTPI * (TAPP__E*TAPP__C*1.D1)**2 & 
                         & / (TAPP__ME*1.D3) / (TAPP__C*1.D2)   

!  -h/k
      REAL, PARAMETER :: CONST2 = - TAPP__H / TAPP__K 

!  1/k
      REAL, PARAMETER :: BKINV = 1. / (TAPP__K*1.D7)

!! for line id checking:
      REAL, PARAMETER :: CMERG = TAPP__H*1.D7 * TAPP__C*1.D2  ! Conversion of energy (cm^-1) to (erg)

!.

!  Check whether lines to include
      IF ( BB_NUS(INU) .LT. BB_NUF(INU) ) THEN

!   Scaled frequency
       LLNU   = OP_NU(INU)
       LLNC   = LLNU / TAPP__C / 1.D2

!   Scaled microturbulence squared (v_t/c)^2
       VMIC2  = VMIC*VMIC

!   Boltzmann scale (1 / kT)
       BKT = BKINV / TT

!   ** Insert limit to ensure VMIC <= CS the local sound speed
       GAMMA = 4. / 3.   !! not always true but approx not critical.       
       CS2   = GAMMA / BKT / OP__ALPMU(JD) 
       VMIC2 = MIN (VMIC2, CS2) 

!   Global multiplier including stimulated emission
       CSTIM = CONST1 * (1. - EXP(CONST2*LLNU/TT))

!   Number of particles per gram
       AM  = TAPP__NA / OP__ALPMU(JD)

!   Ion abundances / partition functions

!     reset to zero
        DO KKK = 1,BB_MAXBA
          DO JJ = 1,BB_MAXNI
            NG(JJ,KKK) = -85.
          ENDDO
        ENDDO

!     collect abundances and partition functions from COMMON
        DO IK = 1,OP_MAXNA
          IF (OP_ALPNN(IK,JD).GT.0. .AND. OP_ALPIMIN(IK).GT.0 ) THEN
            KKK        = OP_ALPZ(IK)

!         Doppler half-width
            DOPPNU    = LLNC * SQRT(DOPPLER*TT/OP_ALPAM(IK) + VMIC2)
            BBDOP(KKK) = DOPPNU
            BBLDP(KKK) = LOG10 (DOPPNU)
            BBDNU(KKK) = 1.E0 / (DOPPNU*DOPPNU)

!         convolve abundances, partition functions and Doppler width
            DO JJ = OP_ALPIMIN(IK),OP_ALPIMAX(IK)
              NG(JJ,KKK) = OP_ALPNN(IK,JD) * OP_XION(JJ,IK,JD) &
                              &  / (OP_XUU(JJ,IK,JD) * BBDOP(KKK))
              NG(JJ,KKK) = MAX (  1.E-37, NG(JJ,KKK) )
              NG(JJ,KKK) = LOG (  NG(JJ,KKK)  )
            ENDDO
          ENDIF

       ENDDO


!   Particle density (log n_perturbers)
       XXI = 0.0          ! radiative damping

!   Loop over lines in frequency interval, integrating line contributions
       SUMBB = 0.
       DO I = BB_NUS(INU),BB_NUF(INU)
!        if (BB_KK(I) == 0) write(*,*) 'bb_kap', INU, wl(INU), BB_NUS(INU), BB_NUF(INU)
         KKK  = BB_KK(I)
         JJ  = BB_JJ(I)

         IF (NG(JJ,KKK).GT.-85.) THEN

           ABBX = NG(JJ,KKK) + BB_GF(I) - BB_EX(I)*BKT 

!        Doppler approximation
           ABBD = ABBX - BB_NU(I)*BBDNU(KKK) 
           ABBD = EXP ( ABBD )

!        include damping via the Voigt function
!          ABBV = EXP ( ABBX )
!          ABBV = ABBV * BB_VOIGT (ATEST,SQRT(BB_NU(I)*BBDNU(KKK)))

!        include damping via a tabulated Voigt function
           NP = NINT( (BB_GAM(I) + XXI - BBLDP(KKK) - BB_PZERO) * BB_PDELTA)
           NP = MAX( MIN( NP, BB_MAXNP), 0)
           NQ = NINT( (BB_LN(I) - BBLDP(KKK) - BB_QZERO) * BB_QDELTA)
           NQ = MAX( MIN( NQ, BB_MAXNQ), 0)
           ABBV = ABBX + BB_HH(NP,NQ)

           ABBV = EXP ( ABBV )
!! use 3 x rad
!           ABBV = 3.0*ABBV   

!! here select lines, if ABBV*AM / cont absorp is greater than eps, include

!! for line checking:

!!$              if ( INU .gt. 710 .and. INU .lt. 840) then
!!$                if (ABBV .gt. 1.0e-21) then
!!$                  write(2101,'(E10.3,3X,I3,I3,3X,F10.3,3X,F10.3,3X,F10.3,1X,E10.3)') ABBV, KKK, JJ, wl(INU), BB_GF(I)/TAPM_LN10, BB_EX(I)/CMERG, ABBD  
!!$                end if
!!$              end if   


!!$           if (ABBV*AM/OPA .gt. ep_0) then 
!!!!select maximum damping, use 3Xrad 
              IF (ABBV > ABBD) THEN
                 SUMBB = SUMBB + ABBV
              ELSE
                 SUMBB = SUMBB + ABBD
              ENDIF 
              line_count = line_count + 1
!!$
!!$           else
!!$              write(*,*) 'line rejected'
!!$           end if 

!!$         else
!!$           write(*,*) KKK,JJ

         ENDIF
       ENDDO

       KBB = CSTIM * SUMBB * AM

!  no lines in this interval
      ELSE

       KBB = 0.0
!!$       write(*,*) 'no lines, interval:', INU

      ENDIF

!!$       write(*,*) KBB, JD, INU, BB_NUS(INU),BB_NUF(INU)

  END SUBROUTINE BB_KAP
