      function bb_voigt(A,V)
!+
!
!     Voigt function for given A and V
!
!     Algorithm is based on a Taylor series expansion for H(x,y) 
!     due to Harris (1948) for small v (v<=0.2). For (y>0.2) 
!     asympotic expansions are used. More details are given by
!     Rees (1992). The original QUB version has been replaced by 
!     one used in the LINFOR package - which is ostensibly three times
!     faster without loss of accuracy. 
!     The algorithm is intrinsically
!     no more accurate than given by a single precision version.
!
!-

      implicit none

      REAL :: A, V, BB_VOIGT

      LOGICAL :: Q     
      
      real, PARAMETER :: ROOTPI = 1.77245385090551

      real :: VV, H, HO, H2, H1
      real :: HH1, HH2, HH3, HH4
      real :: AA
      real :: U, UU
!.

      VV=V*V
      Q=A .lt. 0.2

      SELECT CASE (Q)

        CASE (.true.)
          if ( V .gt. 5) then
            H=((2.12/VV+.8463)/VV+.5642)*A/VV
          else
            HO=EXP(-VV)
            H2=(1.-2.*VV)*HO            
            if (V .gt. 2.4) then
              H1=((-.0032783*VV+.0429913*V-.188326)*VV+.278712*V+.55415)/(VV-1.5)
            else if (V .gt. 1.3) then
              H1=(-.220416*VV+1.989196*V-6.61487)*VV+9.39456*V-4.4848
            else
              H1=(.42139*VV-2.34358*V+3.28868)*VV-.15517*V-1.1247
            end if
            H=(H2*A+H1)*A+HO
          end if

        CASE (.false.)
          if (A .gt. 1.4 .or. A+V .gt. 3.2) then
            AA=A*A
            U=(AA+VV)*1.4142
            UU=U*U
            H=((((AA-10.*VV)*AA*3.+15.*VV*VV)/UU+3.*VV-AA)/UU+1.)*A*.79788/U
          else
            HO=EXP(-VV)
            H2=(1.-2.*VV)*HO            
            if (V .gt. 2.4) then
              H1=((-.0032783*VV+.0429913*V-.188326)*VV+.278712*V+.55415)/(VV-1.5)
            else if (V .gt. 1.3) then
              H1=(-.220416*VV+1.989196*V-6.61487)*VV+9.39456*V-4.4848
            else
              H1=(.42139*VV-2.34358*V+3.28868)*VV-.15517*V-1.1247
            end if
            HH1=H1+HO*1.12838
            HH2=H2+HH1*1.12838-HO
            HH3=(1.-H2)*.37613-HH1*.66667*VV+HH2*1.12838
            HH4=(3.*HH3-HH1)*.37613+HO*.66667*VV*VV
            H=((((HH4*A+HH3)*A+HH2)*A+HH1)*A+HO)*(((-.122727278*A+.532770573)* &
              & A-.96284325)*A+.979895032)
          end if
          
        END SELECT

        BB_VOIGT = H / ROOTPI

      end function bb_voigt
