*+  CHR_DCWRD - Returns all the words in a string.
      SUBROUTINE CHR_DCWRD (STRING,MXW,NWRD,START,STOP,
     :  WORDS,LSTAT)
*    Description :
*     All the words in a character variable are detected and returned
*     as individual elements of a character array.
*     In this context, a word is defined as a continuous string of
*     non-blank characters. Hence words must be separated from each 
*     other by one or more blanks.
*    Invocation :
*     CALL CHR_DCWRD (STRING, MXW, NWRD, START, STOP, WORDS, LSTAT)
*    Arguments :
*     STRING  =  CHARACTER*(*) (Given)
*           Character variable to be decomposed into its constituent
*           words.
*     MXW  =  INTEGER (Given)
*           The maximum permitted number of words that can be
*           extracted from the input string.
*     NWRD  =  INTEGER (Returned)
*           The actual number of words located in the input string.
*           If there are more than MXW words in the input string
*           only the first MXW will be found.
*     START(MXW)  =  INTEGER (Returned)
*           The Ith element contains the position of the first
*           element of the Ith word in the input string.
*     STOP(MXW)  =  INTEGER (Returned)
*           The Ith element contains the position of the last
*           element of the Ith word in the input string.
*     WORDS(MXW)  =  CHARACTER*(*) (Returned)
*           The Ith element contains the Ith word in the input string.
*     LSTAT  =  INTEGER (Returned)
*           Local status. This is a return status only and the routine
*           is not affected by the value on input. It has the following
*           values;
*            0  -  Success.
*            1  -  Number of words exceeds MXW.
*    Method :
*     Set the local status = 0.
*     Get the length of the non-blank characters in the input string
*     If the length of the non-blank characters > 0 then
*       set the word counter = 0
*       if the first element not equal to ' ' then
*         increment the word counter.
*         set the start position of the first word = 1
*       end if
*       for all string elements from the second to the last
*         If the previous element is blank and this element is non-blank
*         then
*           If there is still space for more words then
*             increment the word counter
*             set start position of the current word.
*           else
*             set the local status
*           end if
*         else if the previous element was non-blank and this element
*         is blank then
*           if the status is not set then
*             set the current stop position
*           end if
*         end if
*       end for
*       if the status is not set then
*         if the last element non-blank then
*           set the current stop position = the last element
*         end if
*       end if
*       Get the length of each element of the output array.
*       for all the words located
*         copy into appropriate element of the output array
*       end for
*     else
*       set the number of words = 0
*     end if
*    Bugs :
*     None known.
*    Authors :
*     A C Davenhall.      (ROE::ACD)
*    History :
*     2/3/84:   Original version.                           (ROE::ACD)
*     30/6/84:  Modified for inclusion in the CHR_ library. (ROE::ACD)
*     11/16/84: Name changed from CHR_STRDEC to CHR_DCWRD   (ROE::ACD)
*               in order to meet the naming convention.
*     03/10/88: Improved documentation                      (RAL::AJC)
*    Type Definitions :
      IMPLICIT NONE
*    Import :
      CHARACTER
     :  STRING*(*)     ! String to be examined for words.
      INTEGER
     :  MXW         ! Maximum permitted no. of words.
*    Export :
      INTEGER
     :  NWRD,        ! Number of words found.
     :  START(MXW), ! Start position of each word in the input string.
     :  STOP(MXW)   ! Stop     "     "   "    "   "   "    "     "   .
      CHARACTER
     :  WORDS(MXW)*(*)   ! Contents of each word found.
*    Status :
      INTEGER
     :  LSTAT          ! Local status. Return only. = 0 for success.
*    External references :
      EXTERNAL
     :  CHR_LEN
      INTEGER
     :  CHR_LEN
*    Local variables :
      INTEGER
     :  INDEX,     ! Current element of the input string.
     :  STRGLN,    ! length of non-blank characters in the input string.
     :  MXWLN,    ! Max. no. of characters in each element of 'WORDS'.
     :  CURWRD,    ! Current word.
     :  CURLEN     ! length of the current word.
*-

      LSTAT=0

*
*    Get the length of the non-blank characters in the input string,
*    and proceed if the input string is non-blank

      STRGLN=CHR_LEN(STRING)
      IF (STRGLN.GE.0) THEN

        NWRD=0

*
*      Check whether the first element is blank or not. If non-blank
*      increment the number of words and set the starting point.

        IF (STRING(1:1).NE.' ') THEN
          NWRD=NWRD+1
          START(1)=1
        END IF

*
*      Examine all the remaining elements to check for the starting and
*      stopping positions of words.

        DO INDEX=2,STRGLN

*
*        Check for the start of a new word.

          IF (STRING(INDEX:INDEX).NE.' '.AND.
     :        STRING(INDEX-1:INDEX-1).EQ.' ') THEN
            IF (NWRD.LT.MXW) THEN
              NWRD=NWRD+1
              START(NWRD)=INDEX
            ELSE
              LSTAT=1
            END IF

*
*        Check for the end of a word.

          ELSE IF (STRING(INDEX:INDEX).EQ.' '.AND.
     :             STRING(INDEX-1:INDEX-1).NE.' ') THEN
            IF (LSTAT.EQ.0)
     :        STOP(NWRD)=INDEX-1
          END IF
        END DO

*
*      Check whether the last element of 'STRING' is non-blank.
*      If so this terminates the last word.

        IF (LSTAT.EQ.0) THEN
          IF (STRING(STRGLN:STRGLN).NE.' ') 
     :      STOP(NWRD)=STRGLN
        END IF

*
*      Find the maximum length of the output words.

        MXWLN=LEN(WORDS(1))

*
*      Copy the words found into the output array. The words are
*      copied left justified.

        DO CURWRD=1,NWRD
          WORDS(CURWRD)=' '
          CURLEN=STOP(CURWRD)+1-START(CURWRD)
          CURLEN=MIN(CURLEN,MXWLN)
          WORDS(CURWRD)(1:CURLEN)=STRING(START(CURWRD):
     :                              START(CURWRD)+CURLEN-1)
        END DO

      ELSE

*
*      The input string is blank.

        NWRD=0
      END IF

      END
