*+ CHR_DTOC - Encode a double precision value as a string.
      SUBROUTINE CHR_DTOC (DVALUE,CVALUE,NCHAR)
*    Description :
*     Encode a double precision value as a character string, using
*     as concise a format as possible, and return the number of 
*     characters used.
*     In the event of an error, `*'s will be written to the string.
*    Invocation :
*     CALL CHR_DTOC (DVALUE, CVALUE, NCHAR)
*    Arguments :
*     DVALUE  =  DOUBLE PRECISION (Given)
*           Value to be encoded.
*     CVALUE  =  CHARACTER*(*) (Returned)
*           String into which the value is to be encoded.
*     NCHAR  =  INTEGER (Returned)
*           Field width used in encoding the value.
*    Method :
*     Base field width on string size, subject to the maximum
*     required to display any real number.
*     Write to string in G format.   Remove any blanks.
*     (Can then remove any trailing zeros in the decimal part,
*     and also any '+' signs.)
*    Authors :
*     Jack Giddings.      (UCL::JRG)
*     A C Davenhall.      (ROE::ACD)
*     A J Chipperfield    (RAL::AJC)
*    History :
*     03-JAN-1983:   Original.                     (UCL::JRG)
*     04-OCT-1984:   Documentation improved.       (ROE::ACD)
*     01-SEP-1988:   Use LEN instead of CHR_SIZE   (RAL::AJC)
*     30-SEP-1988:   Use precision 14              (RAL::AJC)
*     16-JUN-1989:   Improve speed and comments
*                    Rely on WRITE to clear remainder of
*                    string
*                    Only clean and measure string once
*                    Work on FIELD within string
*                    Remove terminating space      (RAL::AJC)
*     25-OCT-1989:   Write to whole of CVALUE      (RAL::AJC)
*                    
*    Type definitions :
      IMPLICIT NONE     ! Force Total Declarations.
*    Import :
      DOUBLE PRECISION
     :   DVALUE         ! Decoded value.
*    Export :
      CHARACTER
     :  CVALUE*(*)      ! String to be filled.
      INTEGER
     :  NCHAR           ! Field width.
*    External references :
      INTEGER
     :  CHR_LEN,        ! String length.
     :  INDEX,          ! Index of character in string.
     :  MIN,            ! Minimum value.
     :  MAX             ! Maximum value.
*    Local variables :
      INTEGER
     :  FIELD,          ! Field width.
     :  EXTRAS,         ! Extra characters needed.
     :  PREC,           ! Precision.
     :  SIZE,           ! Character count.
     :  CODE,           ! Local status.
     :  I1,             ! Loop start index.
     :  I2,             ! Loop end index.
     :  I               ! Loop index.
      CHARACTER
     :  FORMAT*12       ! Fortran 77 format string.
*-

*   Construct a suitable format string.
      SIZE = LEN(CVALUE)
      PREC = 14
      IF (DVALUE .LT. 0.0) THEN
         EXTRAS = 7
      ELSE
         EXTRAS = 6
      END IF
      FIELD = MIN(SIZE, PREC+EXTRAS)
      PREC = MAX(FIELD - EXTRAS, 0)
      WRITE (FORMAT, '(''(1PG'',I3,''.'',I3'')'')', IOSTAT=CODE) FIELD,
     :  PREC

      IF (CODE .EQ. 0) THEN
*      The format string was written OK. Use it to write the value
*      into the buffer.
         WRITE (CVALUE, FORMAT, IOSTAT=CODE) DVALUE

         IF (CODE .EQ. 0) THEN
*         Value is wriiten into CVALUE(1:FIELD)
*         Assume the remainder of CVALUE is cleared by the WRITE.
!            CALL CHR_TERM(FIELD, CVALUE)

*
*          Handle trailing superfluous characters after decimal point.

            I1 = INDEX(CVALUE(1:FIELD), '.')
            IF (I1 .GT. 0) THEN
*            There is a decimal point.
*            Find index of E or D (assumed upper case from WRITE).
*            and set a pointer to the character before it.
               I2 = INDEX(CVALUE(1:FIELD), 'E')
               IF (I2 .EQ. 0) THEN
                  I2 = INDEX(CVALUE(1:FIELD), 'D')
                  IF (I2 .EQ. 0) THEN
                     I2 = FIELD
                  ELSE
*                  No exponent, set pointer to end of buffer
                     I2 = I2 - 1
                  END IF
               ELSE
                  I2 = I2 - 1
               END IF

*
*            Replace superfluous zeros after decimal point with spaces.
*            Work back from the E,D or end - ignore spaces. 
               DO I = I2, (I1+1), -1
                  IF (CVALUE(I:I) .EQ. '0') THEN
                     CVALUE(I:I) = ' '
                  ELSEIF (CVALUE(I:I) .NE. ' ') THEN
                     GOTO 1
                  END IF
               END DO
 1             CONTINUE
            END IF

*
*          Repace superfluous plus signs in exponent with space.

            DO I = I2, FIELD
               IF (CVALUE(I:I) .EQ. '+') THEN
                  CVALUE(I:I) = ' '
               END IF
            END DO
            CALL CHR_RMBLK(CVALUE(1:FIELD))
            NCHAR = CHR_LEN(CVALUE(1:FIELD))

*         Replace terminal '.' with space
            IF (CVALUE(NCHAR:NCHAR) .EQ. '.') THEN
               CVALUE(NCHAR:NCHAR) = ' '            
               NCHAR = NCHAR-1
            ENDIF

         ELSE
            CALL CHR_FILL('*', CVALUE(1:FIELD))
            NCHAR = FIELD
         END IF
      ELSE
         CALL CHR_FILL('*', CVALUE(1:FIELD))
         NCHAR = FIELD
      END IF

      END
