*+  CHR_RTOAN - Write a real into character string as hr/deg:min:sec.
      SUBROUTINE CHR_RTOAN (RVALUE, UNITS, STRING, LENGTH)
*    Description :
*     Write a real value into a character string, but format it as
*     {hour/degrees:minutes:seconds}. Ie. this routine is for writing
*     angular measures into a character string in a format suitable
*     for presentation to an astronomer.
*
*       If the absolute value of the number to be written exceeds
*     a predefined maximum a conversion is not attempted, but the
*     number is written as a real number in Fortran 'exponential'
*     format and a couple of question marks are appended to its
*     end. This prevents silly results when very large numbers
*     are input. The variable UNITS controls the maximum permitted
*     value for the conversion to be carried out.
*
*       The value is written into the part of the string beginning
*     at position LENGTH+1 and on output LENGTH is updated to 
*     reflect the new length of the string (excluding trailing
*     blanks) after adding the value.
*    Invocation :
*     CALL CHR_RTOAN (RVALUE, UNITS, STRING, LENGTH)
*    Arguments :
*     RVALUE  =  REAL (Given)
*           Value to be encoded into the string.
*           The value probably represents an angular measure.
*     UNITS  =  CHARACTER*(*) (Given)
*           Controls the maximum value which will be formatted 
*           as {hr/deg:min:sec}.
*           If UNITS = 'HOURS', the maximum permitted value is 24.0
*           If UNITS = 'DEGREES', the  maximum permitted is 360.0. 
*           In all other cases the maximum is 1000.0.
*     STRING  =  CHARACTER*(*) (Modified)
*           String to which RVALUE is to be written.
*     LENGTH  =  INTEGER (Modified)
*           Current length of STRING (excluding trailing blanks).
*    Method :
*     If units = 'HOURS' then
*       set the maximum permitted value = 24.0
*     else if units = 'DEGREES' then
*       set the maximum permitted value = 360.0
*     else
*       set the maximum permitted value = 1000.0
*     end if
*     if (the absolute value of the input number .le. the permitted
*     maximum ) then
*       compute the integer hours/degrees.
*       compute the minutes.
*       compute the seconds.
*       start the string with a minus sign if the value is negative.
*       write the values to the output string.
*     else
*       write the value into the string as a real number.
*       append ' ???' to the output string.
*     end if
*
*     Note; negative values are handled the way they are to avoid the
*     minus sign being lost when the value is negative but the number
*     of degrees or hours is zero.
*    Bugs :
*     None known.
*    Authors :
*     A C Davenhall.      (ROE::ACD)
*    History :
*     18/10/84:  Original version.                         (ROE::ACD)
*     16/11/84:  Name changed from CHR_RTOANG to CHR_RTOAN (ROE::ACD)
*                to meet the naming convention.
*     9/10/85:   Changed the way negative numbers are      (ROE::ACD)
*                handled.
*    Type Definitions :
      IMPLICIT NONE
*    Import :
      REAL
     :  RVALUE      ! Value to be encoded into the string.
      CHARACTER
     :  UNITS*(*)   ! Units of RVALUE ('HOURS' or 'DEGREES').
*    Import-Export :
      CHARACTER
     :  STRING*(*)  ! String to be updated.
      INTEGER
     :  LENGTH      ! Current length of STRING (excl. trail. blanks).
*    Local Constants :
      REAL
     :  MAXHR,      ! Maximum number of Hours.
     :  MAXDEG,     !    "      "    "  Degrees.
     :  MAXANY      ! Maximum in all other cases.
      PARAMETER
     : (MAXHR=2.4E1,
     :  MAXDEG=3.60E2,
     :  MAXANY=1.0E3)
      REAL
     :  SIXTY       ! No. of minutes in hour, seconds in minute etc.
      PARAMETER
     : (SIXTY=6.0E1)
*    Local variables :
      INTEGER
     :  IVALUE,     ! Integer part of RVALUE (Hours or degrees).
     :  MIN         ! Minutes.
      REAL
     :  MAXVAL,     ! Max. permitted value.
     :  ABSVAL,     ! Absolute value of RVALUE.
     :  SEC,        ! Seconds.
     :  RMIN        ! Minutes held as a REAL variable.
*-

*
*    Check the value of the units field to determine the appropriate
*    maximum value.

      IF (UNITS .EQ. 'HOURS') THEN
         MAXVAL = MAXHR
      ELSE IF (UNITS .EQ. 'DEGREES') THEN
         MAXVAL = MAXDEG
      ELSE
         MAXVAL = MAXANY
      END IF

*
*    Check if the value lies in the permitted range. If it does then
*    format it as hours/degrees:min:sec, otherwise write it as a
*    floating point number.

      ABSVAL = ABS(RVALUE)
      IF (ABSVAL .LE. MAXVAL) THEN

*
*       Compute the hours/degrees.

         IVALUE = INT(ABSVAL)

*
*       Compute the minutes.

         RMIN = (ABSVAL - FLOAT(IVALUE)) * SIXTY
         MIN = INT(RMIN)

*
*       Compute the seconds.

         SEC = (RMIN - FLOAT(MIN)) * SIXTY

*
*       If the original value was negative then start the output string
*       with a minus sign.

         IF (RVALUE .LT. 0.0E0)  CALL CHR_PUTC ('-', STRING, LENGTH)

*
*       Append the values to the output string.

         CALL CHR_PUTI (IVALUE, STRING, LENGTH)
         CALL CHR_PUTC (':', STRING, LENGTH)
         IF (MIN .LT. 10) CALL CHR_PUTC ('0', STRING, LENGTH)
         CALL CHR_PUTI (MIN, STRING, LENGTH)
         CALL CHR_PUTC (':', STRING, LENGTH)
         IF (SEC .LT. 1.0E1) CALL CHR_PUTC ('0', STRING, LENGTH)
         CALL CHR_PUTR (SEC, STRING, LENGTH)

      ELSE
*
*       The given value is outside the range. Write it to the string as
*       a real number and append a couple of question marks to the
*       string.

         CALL CHR_PUTR (RVALUE, STRING, LENGTH)
         CALL CHR_PUTC (' ???', STRING, LENGTH)

      END IF

      END
