
      SUBROUTINE FTRANK (X,Y,N, DF,KL,KH, FN,FR,FI,FF,WR,WI,WW)

!+
!=    COMPUTES THE FOURIER TRANSFORM, POWER SPECTRUM AND WINDOW TRANSFORM OF Y(X),
!     WHERE Y(X) IS A FUNCTION DEFINED BY UNEQUALLY SPACED DATA.
!
!     REF: DEEMING,T.J., 1975. ASTROPHYS. + SP. SCI. 36, 137.
!          KURTZ,D.W., 1985. MON.NOT.R.ASTR.SOC. 213,773.
!
!     NOTE: KURTZ' PHASE INFORMATION IS NOT COMPUTED, HOWEVER THE
!           POWER WINDOW IS COMPUTED SIMULTANEOUSLY WITH THE P.S.
!
!     INPUT:  X    ARRAY OF X VALUES
!             Y    ARRAY OF Y VALUES
!             N    NUMBER OF DATA POINTS
!             DF   FREQUENCY INTERVAL FOR TRANSFORM
!             KL   LOWER FREQUENCY LIMIT (/DF)
!             KH   UPPER FREQUENCY LIMIT (/DF)
!
!     OUTPUT: FN   ARRAY OF FREQUENCY POINTS
!             FR   REAL PART OF FOURIER TRANSFORM
!             FI   IMAGINARY PART OF FOURIER TRANSFORM
!             FF   NORMALISED AMPLITUDE OF F.T. (POWER) SPECTRUM)
!             WR   REAL PART OF WINDOW TRANSFORM
!             WI   IMAGINARY PART OF WINDOW TRANSFORM
!             WW   NORMALISED AMPLITUDE OF W.T. (POWER WINDOW)
!-

      DIMENSION X(*),Y(*),FN(*),FR(*),FI(*),FF(*),WR(*),WI(*),WW(*)
      DATA PI/3.1415927/, TWOPI/6.2831853/
!.

      DO 10 K = KL,KH
         FN(K) = K * DF
         FR(K) = 0.
         FI(K) = 0.
         WR(K) = 0.
         WI(K) = 0.
10       CONTINUE

      A0 = TWOPI * DF
      B0 = (KL-1) * A0
      DO 30 I = 1,N
         XF0 = B0 * X(I)
         XDF = A0 * X(I)
!        XF0 = AMOD (XF0, TWOPI)
!        XDF = AMOD (XDF, TWOPI)
         S0  = SIN (XF0)
         C0  = COS (XF0)
         SF  = SIN (XDF)
         CF  = COS (XDF)

         DO 20 K = KL,KH
            C     = C0*CF - S0*SF
            S     = S0*CF + C0*SF
            FR(K) = FR(K) + Y(I) * C
            FI(K) = FI(K) + Y(I) * S
            WR(K) = WR(K) + C
            WI(K) = WI(K) + S
            C0    = C
            S0    = S
20          CONTINUE

30       CONTINUE

!.....ACCUMULATE POWER SPECTRUM AND POWER WINDOW
!     NB AMPLITUDE SPECTRUM ** 2 = POWER SPECTRUM * 4

      DO 40 K = KL,KH
         FF(K) = (FR(K)*FR(K) + FI(K)*FI(K)) / (N*N)
         WW(K) = (WR(K)*WR(K) + WI(K)*WI(K)) / (N*N)
40       CONTINUE

      END
