      subroutine l2fit (npts, axpts, aypts, fline, iers)

!+
!= Routine to do a cubic spline fit with knots
!  effectively drawing the best line through the data.
!
!  Version for use with DIPSO function L2FRES, called by USER
!  PWH / StA  6 Mar 90
!
!  Updated:
!     npts   integer*4       No of points in the data
!     axpts  real*4 array    x values
!     aypts  real*4 array    y values
!
!  Given:
!     fline  real*4          = 1. for best line through data 
!                            = 0. for interpolated y values at x points
!  Returned:
!     iers   integer         Error code = 0 if OK
!                                       = 1-7 : error from subroutine L2FRES
!                                       = 100 : array mismatch
!
!  Subroutine called:        l2fres (Origin believed to be DAO)
!
!  This subroutine prepares data for fitting with L2FRES, a cubic spline
!  fitting routine in which the number of knots is set by the routine
!  itself to optimize the fit. The maximum number of knots is KNOTMAX i.e.
!  2/3 the number of data points. AXPTS and AYPTS are the input x,y data 
!  points. NPTS is the number of original data points. NFITS is the number 
!  of points used for the fit and NINTS is the number of points interpolated 
!  from the spline. KNOTS is the number of knots used. CH2S and RMSS are the 
!  chi squared and R.M.S. errors of the fit respectively. IERS is an error 
!  code explained in L2FRES. W is a dummy weighting array. XFIT and YFIT are 
!  the points used for the fit and XDINT and YDINT are the interpolated 
!  points. RESS is the observed minus calculated residue of the fit. WK1 
!  to WK6 are work spaces.
!
!  If LINE is 1.0 a continuous line will be fitted between the minimum and
!  maximum input x data values at equal intervals and returned in the
!  arrays XDINT and YDINT. NINTS is returned as the number of values.
!  If LINE is 0.0 interpolation will be performed at the NINTS x data
!  values in XDINT with the interpolated values returned in YDINT.
!-
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)

      real*4 axpts(*), aypts(*), fline

      parameter (nmax = 2000,mmax = nmax / 4, lmax = 13 * nmax + 6)

      dimension w(nmax),xfit(nmax),yfit(nmax),
     >          ress(nmax),xdint(mmax),ydint(mmax),wk1(nmax),wk2(nmax),
     >          wk3(nmax),wk4(nmax),wk5(nmax),wk6(lmax)

      logical*4 fit,weight,interp
!.
      fit = .true.
      weight = .true.
      interp = .true.
      iers = 0
      nfits = 0
      rmss = 0.0d0

!  CH2S is set to an arbitrarily high number to indicate that this statistic
!  may not be possible to calculate for some fits i.e. small numbers of points.

      ch2s = 1.0d36

!  Check array dimensions

      if (npts.gt.nmax) then 
         ierr = 100
         goto 5
      end if

!  Fill the fitting point arrays and sort them in increasing order of x.

      nfits = npts
      do 1 i = 1,nfits
         xfit(i) = dble(axpts(i))
         yfit(i) = dble(aypts(i))
         w(i) = 1.0d0
1     continue
      call sort2 (xfit,yfit,nfits)

!  Set up the upper limit on knots.

      knotmax = 2 * nfits / 3

!  Test for continuous line or array for interpolation.

      if (fline.eq.1.0) then

!        Set up the number of interpolation points, calculate the range
!        of X and set up the x array of interpolation points.

         nints = mmax
         xstep = (xfit(nfits) - xfit(1)) / (nints - 1)
         do i = 1,nints
           xdint(i) = (i-1) * xstep + xfit(1)
         end do

      else

!        Interpolate at input X for all data

         nints = npts
         do i = 1,nints
            xdint(i) = dble(axpts(i))
         end do

      end if

!  Call L2FRES with FIT set to .TRUE. so as the spline is calculated,
!  WEIGHT to .TRUE. as weights are set to unity. INTERP is set to .TRUE. as
!  interpolations are being made.

      call l2fres (iers,knotmax,knots,wk1,wk2,wk3,wk4,wk5,fit,nfits,
     >             xfit,yfit,ress,weight,w,interp,nints,xdint,ydint,wk6)
      if (iers .gt. 0) goto 5

!  Calculate R.M.S. error and chi squared.

      do 4 i = 1,nfits
4       rmss = rmss + ress(i) * ress(i)
      if ((nfits - 1) .gt. knots) ch2s = rmss / (nfits - knots - 1)
      rmss = dsqrt (rmss / (nfits - 1))
      write(6,'(1x,a,g14.4)') '  r.m.s. error of fit = ', rmss

!     Get the single precision interpolated points

      npts = nints
      do n = 1,nints
         axpts(n) = sngl(xdint(n))
         aypts(n) = sngl(ydint(n))
      end do

5     return
      end
