SUBROUTINE MTH_LININT (XA,YA,N,X,Y,DY)
  !+
  !  Program:
  !     MTH_LININT

  !  Purpose:
  !=    Perform linear interpolation or extrapolation in a 1D table.

  !  Type:
  !     SUBROUTINE subprogram

  !  Description:
  !     Given array XA and YA, each of length N, and given a value X,
  !     this routine returns a values Y, which is the linear interpolant.
  !     the argument list is identical to that for subroutine NR_POLINT.
  !
  !     NB. Expects x values to be monotonically increasing

  !  Arguments given:
  !     XA (double)              input array containing N x-values
  !     YA (double)              input array containing N y-values
  !     N (integer)              number of elements in input arrays
  !     X (double)               value at which Y=P(x) is to be evaluated

  !  Arguments returned:
  !     Y (double)               Y = P(x)
  !     DY (double)              error estimate in Y (dummy value)

  !  Authors:.
  !     C.S.Jeffery (CSJ) (St Andrews/Armagh)

  !  History:
  !     01-JUL-2021: CSJ
  !        allow for x values monotonically decreasing
  !     20-OCT-2004: CSJ
  !        linear interpolation extension added
  !     c. 1980: CSJ
  !        original version, LININT

  !-

  IMPLICIT NONE

  !  Subroutine arguments (given):
  DOUBLE PRECISION, DIMENSION(*) :: XA, YA
  INTEGER                        :: N
  DOUBLE PRECISION               :: X

  !  Subroutine arguments (returned):
  DOUBLE PRECISION               :: Y
  DOUBLE PRECISION               :: DY

  ! local variables.
  INTEGER, SAVE :: IERCNT = 0   ! error handler
  DOUBLE PRECISION               :: SINT
  INTEGER       :: I, IL, IH
  LOGICAL       :: X_INCREASING  ! flag if x is increasing or decreasing

  !.

  !     ...Trap single-value or invalid interpolation
  IF ( N <= 1 ) THEN
     Y = YA(1)
     RETURN
  ENDIF

  !     ... resolve decreasing x-value fault
  X_INCREASING = .TRUE.
  IF ( XA(N)<XA(1) ) THEN
     X_INCREASING = .FALSE.
     XA(1:N) = XA(N:1:-1)
     YA(1:N) = YA(N:1:-1)
  ENDIF
  
  !     ...LOCATE GRID NEIGHBOURS
  IF (X < XA(2)) THEN
    IL = 1
    IH = 2
  ELSE
    DO I = 2,N-1
      IF (X < XA(I)) THEN
        IL = I - 1
        EXIT
      ENDIF
      IL = I
    ENDDO
    IH = IL + 1
  ENDIF

  !     ...INTERPOLATION INTERVAL
  IL = MIN(MAX(1,IL),N-1)
  IH = MIN(MAX(IL+1,IH),N)
  SINT = XA(IH) - XA(IL)

  !     ...TRAP INFINITE GRADIENT
  IF ( ABS(SINT/XA(IH)) < 1.E-10 ) THEN
    IF (IERCNT <= 10) THEN
      WRITE (*,*) 'MTH_LININ2: * Error * Infinite gradient'
      WRITE (*,*) XA(1:N),IL,IH
      IERCNT = IERCNT + 1
    ENDIF
    Y = (YA(IL) + YA(IH)) / 2.

    !     ...EVALUATE INTERPOLANT
  ELSE
    Y = YA(IL) + (X - XA(IL)) * (YA(IH)-YA(IL)) / SINT
  ENDIF

  DY = 0.


  ! Restore x-values
  IF ( .NOT. X_INCREASING ) THEN
     X_INCREASING = .TRUE.
     XA(1:N) = XA(N:1:-1)
     YA(1:N) = YA(N:1:-1)
  ENDIF
     

END SUBROUTINE MTH_LININT
