      SUBROUTINE PNLSQF (X,Y,NPTS,NORDER,COEF,ERR,RMS,CH2)

!+
!     PROGRAM:      PNLSQF
!     PURPOSE:      SEE BELOW
!     REFERENCES:   BEVINGTON, P.140.
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     0.0                           .  .69  P.R.BEVINGTON
!     >>> POLFIT
!     1.0           VAX 11/780      .  .87  S.A.BELL / ST.ANDREWS
!     1.1           microVAX           .87  C.S.JEFFERY / ST.ANDREWS
!     2.0           microVAX      02.03.88  C.S.JEFFERY / ST.ANDREWS
!
!
!= THIS ROUTINE PERFORMS A LEAST SQUARES FIT OF ORDER UP TO KMAX.
!  NORMAL EQUATIONS ARE FORMED AND SOLUTION FOR COEFFICIENTS AND ERRORS
!  ARE MADE USING ROUTINES MXINV AND MXSUBS. VARIABLES ARE AS FOLLOWS.
!
!  X,Y    - X AND Y DATA ARRAYS FORMING PAIRS
!  NPTS   - NUMBER OF POINTS
!  NORDER - ORDER OF FIT E.G. FOR 2, FIT IS OF FORM Y = A0 + A1.X + A2.X**2
!  NTERMS - NUMBER OF COEFFICIENTS
!  COEF   - ARRAY CONTAINING COEFFICIENTS
!  ERR    - ARRAY CONTAINING ERRORS ON THE ABOVE
!  R      - LINEAR CORRELATION COEFFICIENTS FOR EACH TERM (EXCEPT A0)
!  RMS    - ROOT MEAN SQUARE ERROR OF FIT
!  CH2    - CHI**2 FOR FIT
!  FRT    - F TEST STATISTIC ON FIT
!  RML    - MULTIPLE CORRELATION COEFFICIENT
!-

      IMPLICIT REAL    (A-H,O-Z)
      IMPLICIT INTEGER (I-N)

!.....SUBROUTINE ARGUMENTS
      INTEGER NPTS,NORDER
      REAL    X(*),Y(*), COEF(11),ERR(11), RMS,CH2,RML,FRT,R2B

!.....LOCAL WORKING ARRAYS
      INTEGER    KMAX
      PARAMETER (KMAX=12)
      REAL       XP(KMAX),SUMXP(KMAX),XMEAN(KMAX),SIGMAX(KMAX),R(KMAX)
      REAL       A(KMAX,KMAX), U(KMAX,KMAX), C(KMAX)
!.

      XP(1)  = 1.0
      SUMY   = 0.0
      SIGMA  = 0.0
      NTERMS = NORDER + 1

!  ZERO ARRAYS

      DO 2 I = 1,11
        SIGMAX(I) = 0.0
        XMEAN(I)  = 0.0
        SUMXP(I)  = 0.0
        C(I)      = 0.0
        R(I)      = 0.0
        COEF(I)   = 0.0
        ERR(I)    = 0.0
        DO 1 J = 1,11
          A(I,J)  = 0.0
1         CONTINUE
2       CONTINUE

!.....ACCUMULATE SUMS FOR THE FOLLOWING 
!     ...XP      : X**M where M=0 to M=10
!     ...SUMXP   : sum of X**(2*NORDER)
!     ...SUMXPY  : sum of (X**NORDER)*Y,
!     ...SUMY    : sum of Y and SUMYY contains sum of Y**2.

      DO 7 N = 1,NPTS
        DO 3 M = 2,NTERMS
3          XP(M) = XP(M-1) * X(N)
        DO 4 I = 2,NTERMS
4         SUMXP(I) = SUMXP(I) + XP(I)
        SUMY = SUMY + Y(N)

!.....NORMAL EQUATIONS
!     ... e.g.  S(x.y) = S(a0.x) + S(a1.x.x) + S(a2.x.x.x) + ...
!         IN MATRIX FORM C = A.B 
!         WHERE ARRAY C IS LHS, ARRAY A IS RHS, B IS SOLUTION

        DO 6 J = 1,NTERMS
          C(J) = C(J) + XP(J) * Y(N)
          DO 5 K = 1,NTERMS
5           A(J,K) = A(J,K) + XP(J) * XP(K)
6         CONTINUE
7       CONTINUE

      YMEAN = SUMY / NPTS
      DO 8 I = 2,NTERMS
8       XMEAN(I) = SUMXP(I) / NPTS
      DO 11 N = 1,NPTS
        SIGMA = SIGMA + (Y(N) - YMEAN) * (Y(N) - YMEAN)
        DO 9 M = 2,NTERMS
9         XP(M) = XP(M-1) * X(N)
        DO 10 J = 2,NTERMS
          SIGMAX(J) = SIGMAX(J) + (XP(J) - XMEAN(J)) 
     1                * (XP(J) - XMEAN(J))
          R(J) = R(J) + (XP(J) - XMEAN(J)) * (Y(N) - YMEAN)
10      CONTINUE
11    CONTINUE
      SIGMA = SQRT (SIGMA / (NPTS - 1))
      DO 12 J = 2,NTERMS
        SIGMAX(J) = SQRT (SIGMAX(J) / (NPTS - 1))
        R(J) = R(J) / ((NPTS - 1) * SIGMAX(J) * SIGMA)
12    CONTINUE

!.....INVERT COEFFICIENT MATRIX AND BACKSUBSTITUTE FOR SOLUTION

      CALL MXINV  (A,U,KMAX, NTERMS, IFAIL)
      CALL MXSUBS (C,COEF)
      CALL MXDIAG (U, ERR, KMAX)
      DO 121 I = 1,NTERMS
121      ERR(I) = SQRT (ABS (ERR(I)) )

!.....CALCULATE THE R.M.S ERROR AND CHI**2 OF THE FIT 

      RMS = 0.0
      DO 15 I = 1,NPTS
        DO 13 M = 2,NTERMS
13        XP(M) = XP(M-1) * X(I)
        YC = COEF(1)
        DO 14 J = 2,NTERMS
14         YC = YC + COEF(J) * XP(J)
        RMS = RMS + (YC - Y(I)) * (YC - Y(I))
15      CONTINUE
      CH2 = RMS / FLOAT (NPTS - NORDER - 1)
      RMS = SQRT (RMS / FLOAT (NPTS - NORDER - 1))

!.....CALCULATE THE MULTIPLE CORRELATION COEFFICIENT AND THE F TEST STATISTIC

      RML = 0.0
      DO 16 J = 2,NTERMS
16      RML = RML +COEF(J) * R(J) * SIGMAX(J) / SIGMA
      IF(RML.EQ.1.0) THEN
        FRT = 9.99999E36
      ELSE
        FRT = (RML / NORDER) / ((1.0 - RML) / (NPTS - NORDER - 1))
      ENDIF
      R2B = 1.0 - (1.0 - RML) * (NPTS - 1) / (NPTS - NORDER - 1)
      RML = SQRT (RML)

!.....SCALE THE COEFFICIENT ERRORS BY RMS OF FIT

      DO 17 I = 1,NTERMS
17      ERR(I) = ERR(I) * RMS

!.....QUIT
      END
