SUBROUTINE QSMOOTHW(X,YIN,SIGIN,YOUT,SIGOUT,NPOINT,SIGMA)

  !+
  !
  !  Program:
  !      QSMOOTHW
  !
  !  Purpose:
  !=     Apply a Gaussian smoothing function to NPOINT data in function YIN(X), 
  !      where the FWHM of the box is 2.354*SIGMA. Output is YOUT.
  !
  !  Accuracy:
  !      Preserves integrated equivalent widths of lines in smoothed spectra 
  !      to 0.04 % 
  !
  !  Language:
  !     Fortran 77
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !     D.R.Morgan (DRM) (Armagh Observatory)
  !
  !  History:
  !     03-FEB-2000: CSJ
  !        original version based on qsm function in DIPSO
  !     11-MAR-2005: DRM
  !        updated version using weightings based on sigma.
  !     15-MAR-2016: CSJ
  !        corrected serious bugs in weighting functions -- eurghh.
  !
  !-

  !  Subroutine arguments:
  REAL, DIMENSION(*), INTENT(IN)  :: X       ! array of x values
  REAL, DIMENSION(*), INTENT(IN)  :: YIN     ! array y(x)
  REAL, DIMENSION(*), INTENT(IN)  :: SIGIN   ! errors (sigma) on each y
  REAL, DIMENSION(*), INTENT(OUT) :: YOUT    ! smoothed outout array
  REAL, DIMENSION(*), INTENT(OUT) :: SIGOUT  ! propagated errors on output
  INTEGER,            INTENT(IN)  :: NPOINT  ! number of valid points in X & Y
  REAL,               INTENT(IN)  :: SIGMA   ! width of smoothing function

  ! Local variables:
  REAL GAUSS(0:      100)
  DOUBLE PRECISION THRSIG, SWT, SSG, SUM, DX, X0, BINSZE, FRAC
  INTEGER I, J, DINT

  !  Local Data:
  DATA GAUSS/1.00000, 0.99910, 0.99641, 0.99193, 0.98570, 0.97775, &
       0.96812, 0.95686, 0.94403, 0.92969, 0.91393, 0.89682, & 
       0.87845, 0.85890, 0.83828, 0.81669, 0.79422, 0.77097, & 
       0.74707, 0.72260, 0.69768, 0.67240, 0.64688, 0.62120, &
       0.59547, 0.56978, 0.54422, 0.51887, 0.49381, 0.46912, &
       0.44486, 0.42109, 0.39788, 0.37527, 0.35331, 0.33204, &
       0.31149, 0.29168, 0.27264, 0.25439, 0.23693, 0.22027, &
       0.20442, 0.18936, 0.17510, 0.16162, 0.14891, 0.13696, &
       0.12573, 0.11522, 0.10540, 0.09624, 0.08772, 0.07981, &
       0.07248, 0.06571, 0.05946, 0.05371, 0.04843, 0.04359, &
       0.03916, 0.03512, 0.03144, 0.02810, 0.02506, 0.02231, &
       0.01983, 0.01760, 0.01558, 0.01378, 0.01216, 0.01071, &
       0.00941, 0.00826, 0.00724, 0.00633, 0.00553, 0.00481, &
       0.00419, 0.00364, 0.00315, 0.00273, 0.00235, 0.00203, &
       0.00175, 0.00150, 0.00129, 0.00110, 0.00094, 0.00080, &
       0.00068, 0.00058, 0.00049, 0.00042, 0.00035, 0.00030, &
       0.00025, 0.00021, 0.00018, 0.00015, 0.00012/

  !.

  !   Correction factors must be applied because this subroutine was originally
  !   written as a normal distribution, not a gaussian.
  THRSIG = 4.242640687*SIGMA
  BINSZE = 0.01*THRSIG

  DO I = 1, NPOINT
    X0 = X(I)
    SUM = 0.
    SWT = 0.
    SSG = 0.

    DO J=1,NPOINT,+1
      DX = ABS(X(J)-X0)
      IF (DX < THRSIG) THEN
      FRAC = DX/BINSZE
      DINT = INT(FRAC)
      FRAC = FRAC - DINT
      WT = (1.0-FRAC)*GAUSS(DINT) + FRAC*GAUSS(DINT+1)
      IF ( SIGIN(J) > 0. ) WT = WT/SIGIN(J)
      SUM = SUM + WT*YIN(J)
      SSG = SSG + WT*SIGIN(J)*SIGIN(J)
      SWT = SWT + WT
      ENDIF
    ENDDO

    YOUT(I)   =        SUM / SWT
    SIGOUT(I) = SQRT ( SSG / SWT )

  ENDDO

END SUBROUTINE QSMOOTHW
