      SUBROUTINE NR_AMOEBA (P,Y,MP,NP,NDIM,FTOL,FUNK,ITER)

!+
!
!  nr_amoeba
!
!  PURPOSE:
!= Multidimensional minimization of a function of a N-dimensional vector.

!  Multidimensional minimization of the function FUNK(X) where X is
!  an NDIM-dimensional vector, by the downhill simplex method of Nelder
!  and Mead. 

!  Input is a matrix P whose NDIM+1 rows are NDIM-dimensional
!  vectors which are the vertices of the starting simplex. [Logical
!  dimension of P are P(NDIM+1,NDIM); physical dimsnesions are input
!  as P(MP,NP)]. Also input is the vector Y of length NDIM+1 whose 
!  components must be pre-initialized to the values of FUNK evaluated 
!  at the NDIM+1 vertices (rows) of P; and FTOL the fractional convergence 
!  tolerance of to be acheived in the function value (n.b.!).  
!
!  On output, P and Y will have been reset to NDIM+1 new points all within 
!  FTOL of a minimum function value, and ITER gives the number of iterations 
!  taken.
!
!  Press et al. 1989. Numerical Recipes
!
!-
!
!  Local constants.
      PARAMETER (NMAX=20,ALPHA=1.0,BETA=0.5,GAMMA=2.0,ITMAX=500)

!  Local arrays.
      DIMENSION P(MP,NP),Y(MP),PR(NMAX),PRR(NMAX),PBAR(NMAX)

!.

      MPTS=NDIM+1
      ITER=0

!  Determine which point is the highest (worst), next-highest and lowest (best)
1     ILO=1
      IF(Y(1).GT.Y(2))THEN
        IHI=1
        INHI=2
      ELSE
        IHI=2
        INHI=1
      ENDIF

!  ... by looping over the points in the simplex
      DO I=1,MPTS
        IF(Y(I).LT.Y(ILO)) ILO=I
        IF(Y(I).GT.Y(IHI))THEN
          INHI=IHI
          IHI=I
        ELSE IF(Y(I).GT.Y(INHI))THEN
          IF(I.NE.IHI) INHI=I
        ENDIF
      ENDDO

!  Compute rhe fractional range from highest to lowest 
!  and return if satisfactory
      RTOL=2.*ABS(Y(IHI)-Y(ILO))/(ABS(Y(IHI))+ABS(Y(ILO)))
      IF(RTOL.LT.FTOL)RETURN
      IF(ITER.EQ.ITMAX) THEN
         WRITE (*,*) 'Amoeba exceeding maximum iterations.'
         WRITE (*,*) ITER,ITMAX,RTOL,FTOL
      ENDIF
      ITER=ITER+1

!  Begin a new iteration. Compute the vector average of all points except
!  the highest, i.e. the center of the "face" of the simplex across from
!  the high point. We will subsequently explore along the ray from the high
!  point through that center.
      DO J=1,NDIM
        PBAR(J)=0.
      ENDDO

      DO I=1,MPTS
        IF(I.NE.IHI)THEN
          DO J=1,NDIM
            PBAR(J)=PBAR(J)+P(I,J)
          ENDDO
        ENDIF
      ENDDO

!  Extrapolate by a factor ALPHA through the face, 
!  i.e. reflect the simplex from the high point.
      DO J=1,NDIM
        PBAR(J)=PBAR(J)/NDIM
        PR(J)=(1.+ALPHA)*PBAR(J)-ALPHA*P(IHI,J)
      ENDDO

!  Evaluate the function at the reflected point.
      YPR=FUNK(PR)

!  If the result is better than the best point, 
!  try an additional extrapolation by a factor GAMMA,
      IF(YPR.LE.Y(ILO))THEN

        DO J=1,NDIM
          PRR(J)=GAMMA*PR(J)+(1.-GAMMA)*PBAR(J)
        ENDDO

!     and check out the function there.
        YPRR=FUNK(PRR)

!     The additional extrapolation succeeded, and replaces the high point.
        IF(YPRR.LT.Y(ILO))THEN
          DO J=1,NDIM
            P(IHI,J)=PRR(J)
          ENDDO
          Y(IHI)=YPRR

!     The additional extrapolation failed, but we can still use the 
!     reflected point.
        ELSE
          DO J=1,NDIM
            P(IHI,J)=PR(J)
          ENDDO
          Y(IHI)=YPR
        ENDIF

!  The reflected point is worse than the second-highest. 
      ELSE IF(YPR.GE.Y(INHI))THEN

!     If its better than the highest, then replace the highest,
        IF(YPR.LT.Y(IHI))THEN
          DO J=1,NDIM
            P(IHI,J)=PR(J)
          ENDDO
          Y(IHI)=YPR
        ENDIF

!     but look for an immediate lower point. In other words, perform
!     an immediate contraction of the simplex along one dimension.
        DO J=1,NDIM
          PRR(J)=BETA*P(IHI,J)+(1.-BETA)*PBAR(J)
        ENDDO

!     Then evaluate the function.
        YPRR=FUNK(PRR)

!     Contraction gives an improvement, so accept it.
        IF(YPRR.LT.Y(IHI))THEN
          DO J=1,NDIM
            P(IHI,J)=PRR(J)
          ENDDO
          Y(IHI)=YPRR

!     Can't seem to get rid of that high point. Better contract around
!     the lowest (best) point.
        ELSE
          DO I=1,MPTS
            IF(I.NE.ILO)THEN
              DO J=1,NDIM
                PR(J)=0.5*(P(I,J)+P(ILO,J))
                P(I,J)=PR(J)
              ENDDO
              Y(I)=FUNK(PR)
            ENDIF
          ENDDO
        ENDIF

!  We arrive here if the original refelction gives a middling point.
!  Replace the old high point and continue
      ELSE
        DO J=1,NDIM
          P(IHI,J)=PR(J)
        ENDDO
        Y(IHI)=YPR
      ENDIF

!  for the test of doneness and the next iteration
      GO TO 1

      END
