      SUBROUTINE NR_GAULEG (X1,X2,X,W,N)

!+
!  Program:
!     NR_GAULEG

!  Purpose:
!=    Calculate weights and abscissae for Gauss-Legendre quadrature
!

!  Description:
!     Given the lower and upper limits of integration X1 and X2, 
!     and given N, this routine returns arrays X and W of length N, 
!     containing the abscissas and weights of the Gauss-Legendre i
!     N-point quadrature formula.

!  Arguments given:
!     X1 (real)           lower limit of integration
!     X2 (real)           upper limit of integration
!     N (integer)         order of quadrature

!  Arguments returned:
!     X (real array)      abscissae
!     W (real array)      weights

!  Reference:
!     Press et al. 1989, Numerical Recipes, p. 125

!  Authors:
!     Press et al. 
!     Y.C.Unruh (YCU) (Sussex)
!     C.S.Jeffery (CSJ) (St Andrews)

!  History:
!     1-DEC-1994: YCU
!        original version, adapted from Numerical Recipes
!     17-OCT-1997: CSJ
!        adapted for package NR

!-

!  Use double precision internally for this subroutine
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Subroutuine arguments (given):
      DOUBLE PRECISION X1
      DOUBLE PRECISION X2
      INTEGER N

!  Subroutine arguments (returned):
      DOUBLE PRECISION X(N)
      DOUBLE PRECISION W(N)

!Constant and variable declarations:
!  Local constants:
!     EPS should be increased if the floating precision is not this precise
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.D-14)

!.

!  The roots are symmetric in the interval, so only find half of them.
      M=(N+1)/2
      XM=0.5D0*(X2+X1)
      XL=0.5D0*(X2-X1)

!  Loop over required roots.
      DO I=1,M
        Z=COS(3.141592654D0*(I-.25D0)/(N+.5D0))

!    Using the above stating approximation, use Newton's method
!    to obtain the required accuracy
1       CONTINUE
          P1=1.D0
          P2=0.D0

!    Recurrence relation to evaluate Legendre polynomialP1, P1, at Z
          DO J=1,N
            P3=P2
            P2=P1
            P1=((2.D0*J-1.D0)*Z*P2-(J-1.D0)*P3)/J
          ENDDO

!    Now find PP, the derivative of P1, using a stndard relation.
          PP=N*(Z*P1-P2)/(Z*Z-1.D0)

!    Refine Z by Newton's method
          Z1=Z
          Z=Z1-P1/PP
        IF(ABS(Z-Z1).GT.EPS)GO TO 1

!    Scale the root to the desired interval and insert the symmetric
!    counterpart, the weight and its symmetric counterpart.
        X(I)=XM-XL*Z
        X(N+1-I)=XM+XL*Z
        W(I)=2.D0*XL/((1.D0-Z*Z)*PP*PP)
        W(N+1-I)=W(I)
      ENDDO

      END
