SUBROUTINE NR_GAUSSJ (A,N,NP,B,M,MP)

  !+
  !  Program:
  !     NR_GAUSSJ

  !  Purpose:
  !=    Perform Gauss-Jordan elimination with full pivoting.

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Linear equation solution of A x = B by Gauss-Jordan elimination.
  ! 
  !     A is an input matrix of NxN elements, stored in an array of physical
  !     dimensions NPxNP. B is an input matrix of N by M containing the
  !     M right-hand side vectors, stored in an array of physical 
  !     dimensions NP by NP. On output, A is replaced by its matrix inverse,
  !     and B is replaced by the corresponding set of solution vectors.

  !  Arguments given:
  !     A (real)               input matrix containinn NxN coefficients
  !     N (integer)            number of elements in input matrix
  !     NP (integer)           physical dimension of A
  !     B (real)               input matrix containing M rhs vectors
  !     M (integer)            number of rhs vectors
  !     MP (integer)           2nd physical dimension of B

  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 28

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     1-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  !  Constant and variable declarations:

  !  Subroutine arguments (given):
  INTEGER NP
  INTEGER MP
  INTEGER N
  INTEGER M

  !  Subroutine arguments (given and returned):
  REAL A(NP,NP)
  REAL B(NP,MP)

  !  Local constants:
  !     NMAX should be as large as the largest anticipated value of N.
  PARAMETER (NMAX=50)

  !  Local arrays:
  !     Integer arrays used for book keeping on the pivoting.
  INTEGER IPIV(NMAX)
  INTEGER INDXR(NMAX)
  INTEGER INDXC(NMAX)

  !.

  !  Zero pivot arrays.
  IPIV = 0

  !  Main loop over the columns to be reduced.
  IROW = 1
  DO I = 1,N
     BIG = 0.
     ICOL = 1
     DO J = 1,N
        IF (IPIV(J) /= 1) THEN
           DO K = 1,N
              IF (IPIV(K) == 0) THEN
                 IF (ABS(A(J,K)) >= BIG) THEN
                    BIG = ABS(A(J,K))
                    IROW = J
                    ICOL = K
                 ENDIF
              ELSEIF (IPIV(K) > 1) THEN
                 write(*,*) 'nr_gaussj : singular matrix - ipiv(k)>1, k= ',k
                 write(*,*) 'ipiv = ',ipiv(1:n)
                 return
              ENDIF
           ENDDO
        ENDIF
     ENDDO
     IPIV(ICOL) = IPIV(ICOL) + 1


     !  We now have the pivot element, so we interchange rows, if needed, to
     !  put the pivot element on the diagonal. The columns are not physically
     !  interchanged, only relabelled: IBDXC(I), the column of the Ith pivot
     !  element, is the Ith column that is reduced, while INDXR(I) is the row
     !  in which that pivot element was originally located. If INDXR(I)~=
     !  INDXC(I) there is an implied column interchange. With this form of
     !  bookkeeping, the solution B's will end up in the correct order, and
     !  the inverse matrix will be scrambled by columns.
     IF (IROW /= ICOL) THEN
        DO L = 1,N
           DUM = A(IROW,L)
           A(IROW,L) = A(ICOL,L)
           A(ICOL,L) = DUM
        ENDDO
        DO L = 1,M
           DUM = B(IROW,L)
           B(IROW,L) = B(ICOL,L)
           B(ICOL,L) = DUM
        ENDDO
     ENDIF


     !  We are now ready to divide the pivot row by the pivot element,
     !  located at IROW and ICOL
     INDXR(I)=IROW
     INDXC(I)=ICOL
     IF (A(ICOL,ICOL) == 0.) THEN
        write (*,*)  'nr_gaussj: Singular matrix - Aii = 0, i=',ICOL
        write (*,*)  'Aii = ',(a(l,l),l=1,n)
        return
     ENDIF
     PIVINV = 1./A(ICOL,ICOL)
     A(ICOL,ICOL)=1.
     A(ICOL,1:N) = A(ICOL,1:N) * PIVINV
     B(ICOL,1:M) = B(ICOL,1:M) * PIVINV


     !  Next we reduce the rows ...
     DO LL = 1,N
        !     ... except for the pivot row, of course
        IF (LL /= ICOL) THEN
           DUM = A(LL,ICOL)
           A(LL,ICOL) = 0.
           A(LL,1:N) = A(LL,1:N) - A(ICOL,1:N) * DUM
           B(LL,1:M) = B(LL,1:M) - B(ICOL,1:M) * DUM
        ENDIF
     ENDDO
  ENDDO


  !  This is the end of the main loop over columns of the reduction
  !  It only remains to unscramble the solution in view of the column
  !  interchanges. We do this by interchanging pairs of columns in the
  !  reverse order that the permutation was built up. 
  DO L = N,1,-1
     IF (INDXR(L) /= INDXC(L)) THEN
        DO K = 1,N
           DUM=A(K,INDXR(L))
           A(K,INDXR(L))=A(K,INDXC(L))
           A(K,INDXC(L))=DUM
        ENDDO
     ENDIF
  ENDDO


END SUBROUTINE NR_GAUSSJ
