      SUBROUTINE NR_LUBKSB (A,N,NP,INDX,B)

!+
!  Program:
!     NR_LUBKSUB

!  Purpose:
!=    Solves the set of linear equations A.X=B
!

!  Description:
!     A is input, not as the the matrix A, but rather as its LU decomposition.
!     determined by the routine LUDCMP. INDX is input as the permutation
!     vector returned by LUDCMP. B is input ast the right-hand side vector B,
!     and returns with the solution vector X. A,N, NP and INDX are not modified
!     by this routine and can be left in place for successive calls with
!     different righ-hand sodes B. This routine takes into account the
!     possibility that B will begin with many zero elements, so it is efficient 
!     for use in matrix inversion.

!  Arguments given:
!     A (real array)      LU decomposition of matrix A provided by LUDCMP
!     N (integer)         order of matrix
!     NP (integer)        physical dimension of array A
!     B (real vector)     rhs of equation A.X = B

!  Arguments returned:
!     INDX (integer vector) permutation vector
!     B (real vector)     solution X of A.X = B

!  Reference:
!     Press et al. 1989, Numerical Recipes, p. 36

!  Authors:
!     Press et al. 
!     Y.C.Unruh (YCU) (Sussex)
!     C.S.Jeffery (CSJ) (St Andrews)

!  History:
!     1-DEC-1994: YCU
!        original version, adapted from Numerical Recipes
!     17-OCT-1997: CSJ
!        adapted for package NR

!  See also:
!     pack/math/lineqt.f      Solves A.X=B using Gaussian elimination
!                             with total partial pivoting.
!     pack/math/matinv.f      Matrix inversion routine due to Bevington
!     pack/mx/mxfact.f        Upper triangular matrix factorization
!     pack/mx/mxinv.f         Matrix inversion using up.tri. factorization
 
!-

!  Global declarations:
      IMPLICIT NONE

!  Subroutine arguments (given and returned):
      INTEGER          N
      INTEGER          NP
      DOUBLE PRECISION A(NP,NP)
      INTEGER          INDX(N)
      DOUBLE PRECISION B(N)

!  Local variables:
      DOUBLE PRECISION SUM
      INTEGER          I, J, LL, II

!.

!  When II is set to a positive value, it will become the index of the 
!  first non-vanishing element of B. We now do the forward substitution,
!  Press et al. eq. 2.3.6, unbscrambing the permutation on the fly.
      II=0
      DO I=1,N
        LL=INDX(I)
        SUM=B(LL)
        B(LL)=B(I)
        IF (II.NE.0)THEN
          DO J=II,I-1
            SUM=SUM-A(I,J)*B(J)
          ENDDO
        ELSEIF (SUM.NE.0.) THEN
!     Non-zero value encountered, from now on do the sums!
          II=I
        ENDIF
        B(I)=SUM
      ENDDO
     
!  Back substiutution loop (Press et al. eq. 2.3.7)     
      DO I=N,1,-1
        SUM=B(I)
        IF (I.LT.N)THEN
          DO J=I+1,N
            SUM=SUM-A(I,J)*B(J)
          ENDDO
        ENDIF
        B(I)=SUM/A(I,I)
      ENDDO

      END
