      SUBROUTINE NR_LUDCMP (A,N,NP,INDX,D)

!+
!  Program:
!     NR_LUDCMP

!  Purpose:
!=    Find LU decomposition of a NxN matrix.
!

!  Description:
!     Given an N x N matrix A, with physical dimension NP, this routine
!     replaces it by the LU decomposition of a rowwise permutation of itself.
!     A and N are input. A is output, arranged as in Press et al. eqn 2.3.14;
!     INDX is an output vector which recors the row permutation effected
!     by the partial pivoting; D is output as +/-1 depending on whether the
!     number of row interchanges was even or odd, respectively. This routine
!     is used in combination with LUBKSUB to solve linear equations or
!     invert a matrix.

!  Arguments given:
!     A (real array)      matrix to be decomposed
!     N (integer)         order of matrix
!     NP (integer)        physical dimension of array A

!  Arguments returned:
!     A (real array)       reordered matrix A
!     INDX (integer array) record of row order
!     D (real)             flag recording odd or even interchanges

!  Reference:
!     Press et al. 1989, Numerical Recipes, p. 35

!  Authors:
!     Press et al. 
!     Y.C.Unruh (YCU) (Sussex)
!     C.S.Jeffery (CSJ) (St Andrews)

!  History:
!     1-DEC-1994: YCU
!        original version, adapted from Numerical Recipes
!     17-OCT-1997: CSJ
!        adapted for package NR

!-

!  Global declarations:
      IMPLICIT NONE

!  Subroutine arguments (given and returned):
      INTEGER          N
      INTEGER          NP
      DOUBLE PRECISION A(NP,NP)
      INTEGER          INDX(N)
      INTEGER          D

!  Local constants:
      INTEGER          NMAX
      DOUBLE PRECISION TINY
      PARAMETER       (NMAX=100,TINY=1.0D-20)

!  Local variables:
      DOUBLE PRECISION VV, AAMAX, DUM, SUM
      INTEGER          I, J, K, IMAX
      DIMENSION        VV(NMAX)

!.

!  Flag no row interchanges yet.
      D=1.
      
! Loop over rows to get implicit scaling information.
      DO I=1,N
        AAMAX=0.
        DO J=1,N
          IF (ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
        ENDDO
        IF (AAMAX.EQ.0.) WRITE(*,*) 'nr_ludcmp: Singular matrix.'  ! no non-zero element
!     Save the scaleing information
        VV(I)=1./AAMAX
      ENDDO
      DO J=1,N
        IF (J.GT.1) THEN
          DO I=1,J-1
            SUM=A(I,J)
            IF (I.GT.1)THEN
              DO K=1,I-1
                SUM=SUM-A(I,K)*A(K,J)
              ENDDO
              A(I,J)=SUM
            ENDIF
          ENDDO
        ENDIF
        AAMAX=0.
        DO I=J,N
          SUM=A(I,J)
          IF (J.GT.1)THEN
            DO K=1,J-1
              SUM=SUM-A(I,K)*A(K,J)
            ENDDO
            A(I,J)=SUM
          ENDIF
          DUM=VV(I)*ABS(SUM)
          IF (DUM.GE.AAMAX) THEN
            IMAX=I
            AAMAX=DUM
          ENDIF
        ENDDO
        IF (J.NE.IMAX)THEN
          DO K=1,N
            DUM=A(IMAX,K)
            A(IMAX,K)=A(J,K)
            A(J,K)=DUM
          ENDDO
          D=-D
          VV(IMAX)=VV(J)
        ENDIF
        INDX(J)=IMAX
        IF (J.NE.N)THEN
          IF(A(J,J).EQ.0.)A(J,J)=TINY
          DUM=1./A(J,J)
          DO I=J+1,N
            A(I,J)=A(I,J)*DUM
          ENDDO
        ENDIF
      ENDDO
      IF(A(N,N).EQ.0.)A(N,N)=TINY

      END
