SUBROUTINE NR_POLCOF (XA,YA,N,COF)

  !+
  !  Program:
  !     NR_POLCOF

  !  Purpose:
  !=    Evaluate the coefficients of the interpolating polynomial

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Given arrays XA and YA of length N containing tabulated functions
  !     YA_i = f(XA_i), this routine returns an array of coefficients
  !     COF, of length N, such that YA_i = Sum_j COF_j XA_i^j-1
  !
  !     Note that this routine will, in general only work well
  !     when x=0 is within or close to the range of XA_i. The
  !     intepolating polynomial should be shifted in x if necessary.

  !  Arguments given:
  !     XA (real)              input array containing N x-values
  !     YA (real)              input array containing N y-values
  !     N (integer)            number of elements in input arrays

  !  Arguments returned:
  !     COF (real)             Array of polynomial coefficients

  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 94

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     22-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  !  Constant and variable declarations:

  !  Subroutine arguments (given):
  REAL XA(N)
  REAL YA(N)
  INTEGER N

  !  Subroutine arguments (returned):
  REAL COF(N)
  REAL DY

  !  Local constants:
  !     NMAX should be the largest anticipated value of N.
  PARAMETER (NMAX=15)

  !  Local arrays:
  REAL X(NMAX)
  REAL Y(NMAX)

  !.

  DO J = 1,N
     X(J) = XA(J)
     Y(J) = YA(J)
  ENDDO

  DO J = 1,N
     !  using the interpolating polynomial, we extrapolate to 0.
     CALL NR_POLINT (X,Y,0.,COF(J),DY)
     XMIN = 1.0E+38
     K = 0

     !   find the remaining X_i of smallest absolute value,
     DO I = 1,N+1-J
        IF (ABS(X(I)).LT.XMIN) THEN
           XMIN = ABS(X(I))
           K = I
        ENDIF
        !   (meanwhile reducing all the terms)
        IF (X(I).NE.0) Y(I) = (Y(I) - COF(J)) / X(I)
     ENDDO
     !   and eliminate it
     DO I = K+1,N+1-J
        Y(I-1) = Y(I)
        X(I-1) = X(I)
     ENDDO
  ENDDO

END SUBROUTINE NR_POLCOF
