SUBROUTINE NR_POLINT (XA,YA,X,Y,DY)

  !+
  !  Program:
  !     NR_POLINT

  !  Purpose:
  !=    Perform polynomial interpolation or extrapolation.

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Given array XA and YA, each of length N, and given a value X,
  !     this routine returns a values Y, and an error estimate DY. If
  !     P(x) is the polynomial of degree N-1 such that P(XAi) = YAi,
  !     i = 1,...,N, then the returned value Y=P(x).
  !
  !     This subroutine implements Nevilles's algorithm for the
  !     interpolating polynomial, for N>2
  !
  !     It includes checks for N = 1, 2 in order to avoid errors and
  !     save time (added by CSJ)

  !  Arguments given:
  !     XA (real)              input array containing N x-values
  !     YA (real)              input array containing N y-values
  !     X (real)               value at which Y=P(x) is to be evaluated

  !  Arguments returned:
  !     Y (real)               Y = P(x)
  !     DY (real)              error estimate in Y

  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 82, (f90: p.1043)

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     10-NOV-2012: CSJ
  !        converted to full (1999) f90 version (with linear extension)
  !     20-NOV-2000: CSJ
  !        linear interpolation extension added
  !     2-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  !  NR utilities
  USE NRUTIL, ONLY : NRERROR, IMINLOC
 

  !  Constant and variable declarations:

  !  Subroutine arguments (given):
  DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XA
  DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: YA
  DOUBLE PRECISION, INTENT(IN)               :: X

  !  Subroutine arguments (returned):
  DOUBLE PRECISION, INTENT(OUT) :: Y
  DOUBLE PRECISION, INTENT(OUT) :: DY

  !  Local arrays:
  INTEGER N

  DOUBLE PRECISION, DIMENSION(SIZE(XA)) :: C
  DOUBLE PRECISION, DIMENSION(SIZE(XA)) :: D
  DOUBLE PRECISION, DIMENSION(SIZE(XA)) :: HO
  DOUBLE PRECISION, DIMENSION(SIZE(XA)) :: DEN

  !.

  N = SIZE (XA)

  !  Use Neville's algorithm for N > 2, avoiding extrapolation
  IF ( N > 2 .AND.  ( (X-XA(1))*(XA(N)-X) >= 0 ) ) THEN

     C  = YA
     D  = YA
     HO = XA - X
     HO = XA - X
     ! find index ns of closest table entry
     NS = IMINLOC(REAL(ABS(X-XA)))
     !  this is initial approximation to y
     Y  = YA(NS) 
     NS = NS - 1

     DO M = 1,N-1 
        DEN(1:N-M) = HO(1:N-M) - HO(1+M:N)
        IF (ANY(DEN(1:N-M) == 0.0 )) &  
             CALL NRERROR('nr_polint: calculation failure')
        DEN(1:N-M) = (C(2:N-M+1)-D(1:N-M)) / DEN(1:N-M)
        D(1:N-M)   = HO(1+M:N) * DEN(1:N-M)
        C(1:N-M)   = HO(1:N-M) * DEN(1:N-M)

        IF (2*NS < N-M) THEN
           DY = C(NS+1)
        ELSE
           DY = D(NS)
           NS = NS - 1
        END IF
        Y = Y+DY

     END DO

     !  linear extrapolation if N > 2 and X out of range
  ELSEIF ( N > 2 .AND. (X-XA(1))*(XA(N)-XA(1)) < 0 ) THEN
     Y = YA(1) !+ &
         ! (X - XA(1)) * ((YA(2) - YA(1)) / (XA(2) - XA(1)))
     DY = 0.
     !   linear extrapolation if N > 2 and X out of range
  ELSEIF ( N > 2 .AND. (XA(N)-X)*(XA(N)-XA(1)) < 0 ) THEN
     Y = YA(N) !+ &
         ! (X - XA(N)) * ((YA(N-1) - YA(N)) / (XA(N-1) - XA(N)))
     DY = 0.
     !  Use linear interpolation for N = 2 
  ELSEIF ( N == 2 .AND. XA(2) /= XA(1) ) THEN
     Y = YA(1) + &
          (X - XA(1)) * ((YA(2) - YA(1)) / (XA(2) - XA(1)))
     DY = 0.
  ELSEIF ( N == 2  ) THEN
     Y = (YA(1) + YA(2)) / 2.
     DY = 0.
  ELSE
     Y = YA(1)
     DY = 0.

  ENDIF


END SUBROUTINE NR_POLINT
