SUBROUTINE NR_SPLINT (XA,YA,Y2A,N,X,Y)

  !+
  !  Program:
  !     NR_SPLINT

  !  Purpose:
  !=    Evaluate the cubic spline interpolate Y = f(X)
  !

  !  Description:
  !     Given arrays XA and YA of length N, containing a tabulated function,
  !     Yi = f(Xi), with X1<X2<...<XN, and given the array Y2 which is the
  !     output from NR_SPLINE, and given a value of X, this routine returns
  !     a cubic-spline interpolated value Y.
  !
  !     This version is optimal for random values of X.
  !     It should be re-optimized for ordered values of X.

  !  Arguments given:
  !     XA (real vector)    abscissae   Xi, i=1,N
  !     YA (real vector)    ordinate    Yi = f(Xi)
  !     Y2A (real vector)   second derivatives of interpolating function
  !     N  (integer)        number of abscissae
  !     X  (real)           value of X for which f(X) is required

  !  Arguments returned:
  !     Y  (real)           cubic spline interpolant  Y=f(X)

  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 89

  !  Authors:
  !     Press et al. 
  !     Y.C.Unruh (YCU) (Sussex)
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     1-DEC-1994: YCU
  !        original version, adapted from Numerical Recipes
  !     17-OCT-1997: CSJ
  !        adapted for package NR

  !-

  !  Subroutine arguments (given):
  REAL             XA(N)
  REAL             YA(N)
  REAL             Y2A(N)
  INTEGER          N
  REAL             X


  !  Subroutine arguments (returned):
  REAL             Y

  !.

  !  Locate the correct location in the table by means of bisection.
  !  This is optimal if sequential calls to this routine are
  !  at random values of X.
  KLO=1
  KHI=N
1 IF (KHI-KLO.GT.1) THEN
     K=(KHI+KLO)/2
     IF (XA(K).GT.X) THEN
        KHI=K
     ELSE
        KLO=K
     ENDIF
     GOTO 1
  ENDIF

  !  Check the tabulated XA's are distinct.      
  H=XA(KHI)-XA(KLO)
  IF (H.EQ.0.) WRITE(*,*) 'nr_splint: Bad XA input.'

  !  Evaluate the cubic spline.
  A=(XA(KHI)-X)/H
  B=(X-XA(KLO))/H
  Y=A*YA(KLO)+B*YA(KHI)+ &
   ((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.

END SUBROUTINE NR_SPLINT
