      SUBROUTINE NR_D_GAUHER (X,W,N)

!+
!  Program:
!     NR_GAUHER

!  Purpose:
!=    Calculate weights and abscissae for Gauss-Hermite quadrature
!

!  Description:
!     Given N, this routine returns arrays X and W of length N, 
!     containing the abscissas and weights of the Gauss-Hermite 
!     N-point quadrature formula.

!  Arguments given:
!     N (integer)         order of quadrature

!  Arguments returned:
!     X (real array)      abscissae
!     W (real array)      weights

!  Reference:
!     Press et al. 1989, Numerical Recipes, p. 147

!  Authors:
!     Press et al. 
!     Y.C.Unruh (YCU) (Sussex)
!     C.S.Jeffery (CSJ) (St Andrews)
!     P.L.Dufton (PLD) (QUB)

!  History:
!     1-DEC-1994: YCU
!        original version, adapted from Numerical Recipes
!     17-OCT-1997: CSJ
!        adapted for package NR
!     20-NOV-2001: PLD
!	 adapted from GAULEG
!
!-

!  Use double precision internally for this subroutine
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Subroutine arguments (given):
      INTEGER N

!  Subroutine arguments (returned):
      DOUBLE PRECISION X(N)
      DOUBLE PRECISION W(N)

!Constant and variable declarations:
!  Local constants:
!     EPS should be increased if the floating precision is not this precise
      INTEGER MAXIT
      DOUBLE PRECISION EPS, PIM4
      PARAMETER (EPS=3.D-10,PIM4=0.7511255444649425D0,MAXIT=20)

!.

!  The roots are symmetric in the interval, so only find half of them.
      M=(N+1)/2

!  Loop over required roots.
      DO I=1,M
      
        IF(I.EQ.1) THEN
          Z=SQRT(FLOAT(2*N+1))-1.85575*(2*N+1)**(-0.16667)
        ELSEIF(I.EQ.2) THEN
          Z=Z-1.14*n**0.426/Z
        ELSEIF(I.EQ.3) THEN
          Z=1.86*Z-0.86*X(1)
        ELSEIF(I.EQ.3) THEN
          Z=1.91*Z**(-0.91)*X(2)
        ELSE
          Z=2.0*Z-X(I-2)
        ENDIF
        
        
!    Using the above stating approximation, use Newton's method
!    to obtain the required accuracy
        DO ITS=1,MAXIT
          P1=PIM4
          P2=0.D0
          DO J=1,N
            P3=P2
            P2=P1
            P1=Z*SQRT(2.0D0/J)*P2-SQRT(DBLE(J-1)/DBLE(J))*P3
          ENDDO

!    P1 is the Hermite Polynomial
!    Now find PP, the derivative of P1, using a stndard relation.
          PP=SQRT(2.0D0*N)*P2

!    Refine Z by Newton's method
          Z1=Z
          Z=Z1-P1/PP
          IF(ABS(Z-Z1).LE.EPS)GOTO 1
        ENDDO

!
!    Convergence failure
    1   CONTINUE
        

!    Scale the root to the desired interval and insert the symmetric
!    counterpart, the weight and its symmetric counterpart.
        X(I)=Z
        X(N+1-I)=-Z
        W(I)=2.D0/(PP*PP)
        W(N+1-I)=W(I)
      ENDDO
      RETURN

      END
