      SUBROUTINE NR_D_SPLINE (X,Y,N,YP1,YPN,Y2)

!+
!  Program:
!     NR_D_SPLINE
!
!     double precision version

!  Purpose:
!=   Obtain the coefficients of the interpolating cubic spline function
!

!  Description:
!     Given arrays X and Y of length N containing a tabulated function,
!     Yi = f(Xi), with X1<X2<...<XN, and given values YP1 and YPN for the
!     first derivative of the interpolating function at points 1 and N
!     respectively, this routine returns an array Y2 of length N which
!     contains the second derivatives of the interpolating function at the
!     tabulated points Xi. If YP1 and/or YPN are equal to 1.E30 or larger,
!     the routine is signalled to set the corresponding boundary conditions
!     for a natural spline, with zero second derivative on that boundary.

!  Arguments given:
!     X (real vector)     abscissae   Xi, i=1,N
!     Y (real vector)     ordinate    Yi = f(Xi)
!     N (integer)         number of abscissae
!     YP1 (real)          second derivative at lower boundary
!     YPN (real)          second derivative at upper boundary

!  Arguments returned:
!     Y2 (real vector)    second derivatives of interpolating function

!  Reference:
!     Press et al. 1989, Numerical Recipes, p. 88

!  Authors:
!     Press et al. 
!     Y.C.Unruh (YCU) (Sussex)
!     C.S.Jeffery (CSJ) (St Andrews)

!  History:
!     1-DEC-1994: YCU
!        original version, adapted from Numerical Recipes
!     17-OCT-1997: CSJ
!        adapted for package NR

!-

!  Switch off global typing
      IMPLICIT NONE

!  Subroutine arguments (given):
      DOUBLE PRECISION X(*)
      DOUBLE PRECISION Y(*)
      INTEGER N
      DOUBLE PRECISION YP1
      DOUBLE PRECISION YPN
      
!  Subroutine arguments (returned):
      DOUBLE PRECISION Y2(*)
      
!  Local constants.
      INTEGER, PARAMETER :: NMAX=1000

!  Local variables.
      INTEGER :: I, K
      DOUBLE PRECISION, DIMENSION(NMAX) :: U
      DOUBLE PRECISION :: SIG, P, QN, UN

!.


!  The lower boundary condition is set to be either 'natural'
      IF (YP1.GT..99E30) THEN
        Y2(1)=0.
        U(1)=0.
!  or else to have a specified first derivative
      ELSE
        Y2(1)=-0.5
        U(1)=(3./(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      ENDIF

!  Decomposition loop of the tridiagonal algorithm. Y2 and U are used
!  for temporary storage of the decomposed factors.
      DO I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.
        Y2(I)=(SIG-1.)/P
        U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))
     :             /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
      ENDDO
      
!  Upper boundary condition is set to be either 'natural'      
      IF (YPN.GT..99E30) THEN
        QN=0.
        UN=0.
!  or else to have a specified first derivative
      ELSE
        QN=0.5
        UN=(3./(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)

!  Backsubstitution loop of the tridiagonal algorithm
      DO K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
      ENDDO

      END
