MODULE OP_GLOBAL

  !+
  !
  !  Module:
  !     OP_GLOBAL
  !
  !  Purpose:
  !     Global variable holders and constants 
  !     for Opacity Project continuous opacities
  !
  !     ns	number of states
  !     alpimin,alpimax	min,max ion numbers being treated
  !     xwt	weights for states
  !     xchi	ionisation energy
  !     xion	fractional abundance for each ion
  !     xpop      fractional population for each state
  !     xs        cross sections
  !
  !  Authors:
  !     PMH: P.M.Harrison (Armagh)
  !     CSJ: C.S.Jeffery  (Armagh)
  !
  !  History:
  !     11-Feb-1998 (PMH): Created as an include file with common blocks
  !     25-Feb-2003 (CSJ): Converted to module  
  !
  !-

  ! User defined types
  USE OP_PRECISIONS

  ! No implicit typing
  IMPLICIT NONE

  PRIVATE :: QQ, RR

  !  Array index parameters.
  INTEGER, PARAMETER :: OP_MAXNA=40        !  max atoms with x-sections
  INTEGER, PARAMETER :: OP_MAXNZ=90        !  max atoms overall
  INTEGER, PARAMETER :: OP_MAXNI=6         !  max ion species
  INTEGER, PARAMETER :: OP_MAXNS=60        !  max levels
  INTEGER, PARAMETER :: OP_MAXNQ=5000      !  max levels over all ions and all species
  INTEGER, PARAMETER :: OP_MAXND=201       !  max depth points
  INTEGER, PARAMETER :: OP_MAXNW=10000     !  max wavelengths in resampled data
  INTEGER, PARAMETER :: OP_MAXNX=15000     !  max wavelengths in x-section data

  !  Array of selected atoms . 
  INTEGER :: OP_NATOMS                       ! number of atoms selected for inclusion
  INTEGER, DIMENSION(OP_MAXNZ) :: OP_SATOMS  ! array of atomic numbers selected
  INTEGER :: OP_IAS                          ! array index for indirect referencing 

  !  Global variables.
  INTEGER :: OP_NS(OP_MAXNI,OP_MAXNA)            ! number of levels
  !! INTEGER :: OP_NX(OP_MAXNS,OP_MAXNI,OP_MAXNA)   ! number of x-sections  !! not used
  INTEGER :: OP_NQ(OP_MAXNI,OP_MAXNA)            ! cumulative number of x-sections

  INTEGER :: OP_ALPIMIN(OP_MAXNA)
  INTEGER :: OP_ALPIMAX(OP_MAXNA)                ! ion range
  INTEGER :: OP_ALPZ(OP_MAXNA)                   ! atomic numbers
  REAL :: OP_ALPAM(OP_MAXNA)                     ! atomic weights
  REAL :: OP_ALPNN(OP_MAXNA,OP_MAXND)            ! atomic abundances
  REAL :: OP__ALPMU(OP_MAXND)                     ! mean at. weights
  REAL :: OP_ALPMF(OP_MAXNA,OP_MAXND)            ! mass fractions


  REAL :: OP_XCHI(OP_MAXNS,OP_MAXNI,OP_MAXNZ)    ! ionisation energies (eV)
  REAL :: OP_XCHIG(OP_MAXNI,OP_MAXNZ)            ! ground state ionisation energies (Rydberg)
  REAL :: OP_XION(OP_MAXNI,OP_MAXNZ,OP_MAXND)    ! fractional ion abundances
  REAL :: OP_XUU(OP_MAXNI,OP_MAXNZ,OP_MAXND)        ! partition functions
  REAL :: OP_XWT(OP_MAXNS,OP_MAXNI,OP_MAXNA)     ! weights for levels
  REAL :: OP_XSQ(OP_MAXNW+1,OP_MAXNQ)                  ! cross-sections ! condensed data structure
  REAL :: OP_XPOP(OP_MAXNS,OP_MAXNI,OP_MAXNA,OP_MAXND) ! level populations
  REAL :: OP_XSSUM(OP_MAXNW+1,OP_MAXNI,OP_MAXNA) ! summed cross-sections

  REAL :: OP_XWLIM(OP_MAXNS,OP_MAXNI,OP_MAXNA)   ! OP wavelength limit corrections

  INTEGER :: OP_NW = 0                           ! number of wavelength points
  REAL, ALLOCATABLE :: OP_WL(:)         ! wavelength grid assoc. with model
  REAL, ALLOCATABLE :: OP_NU(:)         ! frequency grid assoc. with model

  !  Holders for OP energy level information

  REAL :: OP_ECONT ( OP_MAXNS, OP_MAXNI, OP_MAXNA )     !  Ionisation energies (see XCHIG)
  INTEGER :: OP_ISLP  ( OP_MAXNS, OP_MAXNI, OP_MAXNA )  !  Series
  INTEGER :: OP_ILV   ( OP_MAXNS, OP_MAXNI, OP_MAXNA )  !  Level
  CHARACTER(LEN=15):: OP_ICONF ( OP_MAXNS, OP_MAXNI, OP_MAXNA )  !  Configurations

  !! Holders for free-free continuous opacities

  REAL :: OPFFH1, OPFFHM, OPFFHE1, OPFFHE2, OPFFHEM
  REAL :: OPFFC1, OPFFC2, OPFFC2X, OPFFC3, OPFFCM
  REAL :: OPFFN1, OPFFN2, OPFFN3
  REAL :: OPFFMG1, OPFFAL1, OPFFSI1, OPFFO1
  REAL :: OPFFMG2, OPFFSI2, OPFFCA2

  !! Holders for individual bound-free opacities

  REAL :: KAPPABF(OP_MAXNA,OP_MAXNI)   

  REAL :: OPAC_REF
  REAL :: OPAC_NU

  !! depth-dependent atomic data for ff-opacities
  DOUBLE PRECISION, DIMENSION(22,OP_MAXND) :: AT     ! T dependent 


 ! Ionisation limits for specified levels in Angstrom 
  ! If OP_XWLIM = 0. OR NaN use value from datafile,
  ! All unassigned values should be assumed NaN 
  DATA  OP_XWLIM(1:3,1,2)  / 504.26, 0., 3124.15 /
  DATA  OP_XWLIM(1:3,1,6)  / 1101.09,1239.0,1444.0 /
  DATA  OP_XWLIM(1:15,2,6) / 508.482, 655., 0., 1002., 1218., 1248., 1540., 1877., 1956., 0., 2536., 2929., 0., 3504., 3612. /
  DATA  OP_XWLIM(1:24,3,6) / 258.91, 299., 353., 0., 0., 0., 675., 719., 785., 790., 860., 911., 0., 1302., 0., 1342., 1461., &
       0., 1543., 1557., 0., 0., 0., 1612. /
  DATA  OP_XWLIM(1:3,1,7)  / 853.05, 1020.438, 1131.234 /
  DATA  OP_XWLIM(1:12,2,7) / 418.84, 447.0, 485.0, 0., 0., 0., 0., 112.0, 1115.0, 0., 1346.0, 1384.0 /
  !
  DATA  OP_XWLIM(1:OP_MAXNS,1:3,26) / OP_MAXNS*1575.39, OP_MAXNS*767.22, OP_MAXNS*404.50 /    !! one limit per level in op_fein.f90 ??
  
  ! Ground-state binding energies for selected ions where these are not given as first level in data file

  DATA OP_XCHIG(1,22)    /  5.074830E-01 /
  DATA OP_XCHIG(1,24)    /  4.933080E-01 /
  DATA OP_XCHIG(1:5,26)  /  5.699880E-01, 1.178240E+00,  2.265090E+00, 3.984050E+00, 5.510020E+00 /
  DATA OP_XCHIG(2,28)    /  1.301730E+00 /
  DATA OP_XCHIG(2,28)    /  1.301730E+00 /

  
CONTAINS

  INTEGER FUNCTION OP_Q ( I, J, K )

    ! Function OP_Q computes the position of a given x-section in the array OP_XSQ
    ! (this might be done with pointers, but not really clear what the difference is)
    
    INTEGER :: I ! level
    INTEGER :: J ! ion
    INTEGER :: K ! atomic species

    OP_Q = I + OP_NQ(J,K)

  END FUNCTION OP_Q

  
END MODULE OP_GLOBAL
