      SUBROUTINE rtf_feaut
     :           (nuzero,tauz,temp,pe,opz,nd,  fnu,cap,sig,opl,nn, 
     :            rnu,a,phi,nr,    mu,b,nm,    taunu,ss,jj)

!:::::::::::::::::::::::::::::::::::::::::::::::::!::::::::::::::::::::+::::::::
!
!     Purpose:
!=     To determine the source function (Sv) and mean intentesity (Jv) 
!      by solving the equations of 
!      radiative transfer in a semi-infinite plane parallel atmosphere,
!      assuming local thermodynamic, radiative and hydrostatic equilibrium.
!
!     Method:
!      Feautrier's scheme of difference equations, described by 
!      D.Mihalas, Stellar Atmospheres, 2nd edition, pp.151-158
!      I.Hubeny 1994, Computational Astrphysics, ed. Thieleman et al. Springer Verlag
!      Rybicki G.B., Hummer D.G., 1991, A&A 245, 171
!
!      This version allows for nm angle quadrature points, which
!      may either be prespecified, or obtained within the program.
!
!     History:      started 30 Apr 1991        Author:
!      Version 1.0          13 Feb 1992         C.S.Jeffery (St Andrews)
!              1.1          17 Oct 1997         C.S.Jeffery (Armagh)
!              1.2          27 Jan 1998         C.S.Jeffery (Armagh)
!                   use of diagonal only matrices
!                   forcing of diffusion approximation at depth
!                   other bug corrections
!              1.3          20 Oct 2000         C.S.Jeffery (Armagh)
!                   attempt to introduce Rybicki & Hummer (1991) elimination scheme
!                   code elements for partial redistribution removed 
!                   bug in definition of B corrected
!
!     Known Bugs:
!      SPECTRUM: flux in the cores of strong lines is too weak.
!      STERNE: converegence at large optical depth still slow
!
!.................................................!....................+........
!
!     Input:
!
!     nuzero          Frequency of optical depth scale
!     tauz(1:nd)      Optical depth
!     temp(1:nd)      Temperature in atmosphere
!     pe(1:nd)        Electron pressure in atmosphere
!     opz(1:nd)       Continuous opacity at frequency 'nuzero'
!     nd              Number of depth points in atmosphere
!
!     fnu             Frequencies at which transfer equation is to be solved
!     cap(1:nd)       Continuous absorption coefficient
!     sig(1:nd)       Electron scattering coefficient 
!     opl(1:nd)       Line absorption coefficient  (0 for continuum only)
!     nn              Number of frequencies at which source function reqd.
!
!                     - currently assume nn = 1
!
!     !! current version has eliminated the redistribution function
!
!     rnu(1:nr)       Frequency quadrature points for redistribution function 
!     a(1:nr)         Quadrature weights for frequency integration
!     phi(1:nr)       angle-independent frequency redistribution function
!     nr              Number of frequencies in redistribution function
!
!                     nr = 1 for coherent radiation: R(v,v') = phi(v').d(v-v')
!
!     mu(1:nm)        Angle cosines for the angle quadrature
!     b(1:nm)         Quadrature weights for angle integration
!     nm              Number of angles
!
!                     if mu(1)=0, mu(m) and b(m) are calculated automatically
!
!
!     Output:
!
!     taunu(1:nd)      Optical depth at frequencu fnu
!     ss(1:nn,1:nd)   Source function at specified depth and frequency points
!     jj(1:nn,1:nd)   Mean intensity at  "          "                  "
!.................................................!....................+........
!
!     Scheme:
!
!     1. Check frequency and angle grids set up
!     2. Rescale optical depth scale
!     2. Precalculate scattering coefficients etc. geometric depths, ....
!     3. Set up Feautrier matrix coefficients
!     4. Solve linearized equations
!     5. Evaluate mean intensities and source function

!:::::::::::::::::::::::::::::::::::::::::::::::::!::::::::::::::::::::+::::::::

!     Global constants
      USE TAPM_PAR
      USE TAPP_PAR

!     Global declaration
      IMPLICIT   NONE

!.................................................!....................+........

!     Subroutine arguments

      INTEGER    nd                               ! declare atmosphere structure
      REAL*8     nuzero, tauz, temp, pe, opz
      DIMENSION  tauz(*), temp(*), pe(*), opz(*)

      INTEGER    nn
      REAL*8     fnu, cap, sig, opl               ! declare opacities at freq
      DIMENSION  cap(*), sig(*), opl(*)           ! for which solution reqd.

      INTEGER    nr                               ! declare redistribution
      REAL*8     rnu, a, phi                       ! function frequency structure
      DIMENSION  rnu(*), a(*), phi(*)

      INTEGER    nm                               ! declare angle structure
      REAL*8     mu, b
      DIMENSION  mu(*), b(*)

      REAL*8     ss, jj, taunu                    ! output structures
      DIMENSION  taunu(*)
      DIMENSION  ss(*)
      DIMENSION  jj(*)                            ! Mean intensity 

!.................................................!....................+........

!     Program dimensions
      INTEGER    kd, kn, kr, km, ki, kk           !
      PARAMETER (kd=201,kn=5,kr=5,km=10)           ! maximum dimensions of arrays
      PARAMETER (ki=kr*km,kk=ki*ki)               !

!     Local constants                                ! cgs units
      DOUBLE PRECISION hk
      PARAMETER ( hk = tapp__h / tapp__k )           ! h / k  
      DOUBLE PRECISION n0
      PARAMETER ( n0 = tapp__na )                    ! Avogadro's number
      DOUBLE PRECISION h2
      PARAMETER ( h2 = 2.0 * tapp__h      * 1.D+07 ) ! 2 . h 
      DOUBLE PRECISION c2
      PARAMETER ( c2 = tapp__c * tapp__c  * 1.D+04 ) ! c * c

!     Local variable declarations

      INTEGER    d                                ! depth counter
      INTEGER    n, nx                            ! frequency counter (source f)
      INTEGER    r, rx                            ! frequency counter (redistbn)
      INTEGER    m, mx                            ! angle counter
      INTEGER    i, j                             ! i = m + (n-1) * nm
      INTEGER    ii, ix                           ! ii = nm * nn
!
      REAL*8     opnu(kd)                          ! local total opacity
      REAL*8     alp(kd)                          ! scattering coefficients: (ro)
      REAL*8     bet(kd)                          ! thermal terms: (1-ro).Bnu
      REAL*8     bnu(kd)                          ! Planck function
      REAL*8     dtm(kd), dtp(kd)                 ! difference terms in tau
!
      REAL*8     bnuc, cnd, snd,  jnd,  bnc       ! temporary variables
      REAL*8     opnus, scat, scatt, scal
      REAL*8     dte, dtme, dtpe, dtmp,  dti, lld
      REAL*8     mudt, hnukt, ehnukt, dbnudt, dtdtau, iidup
      REAL*8     mu2(km)                          ! mu*mu
      INTEGER    mj, ndbg, ifail

!.................................................!....................+........

!     Feautrier matrices
      REAL*8     aa, bb, cc, ll, dd, uu, vv       ! General system (6-37)
      DIMENSION  aa(ki,kd)
      DIMENSION  bb(ki,kd)
      DIMENSION  cc(ki,kd)
      DIMENSION  ll(ki,kd)
      DIMENSION  dd(ki,kd)
      DIMENSION  uu(ki,kd)
      DIMENSION  vv(ki,kd)
      REAL*8     hh, ff, zz, dg
      DIMENSION  dg(ki,kd)
      DIMENSION  hh(ki,kd)                        ! for Rybicki-Hummer elimination
      DIMENSION  ff(ki,kd)
      DIMENSION  zz(ki,kd)

      REAL*8     ww, vb 
      DIMENSION  ww(ki,kd)
      DIMENSION  vb(ki)

!.................................................!....................+........

!     Local data initialisation

      DATA ndbg  / 0 /


!     Initialize variables for call statements

      REAL*8 ZERO,ONE
      INTEGER IZERO
      ZERO=0.0
      ONE=1.0
      IZERO=0

!:::::::::::::::::::::::::::::::::::::::::::::::::!::::::::::::::::::::+::::::::

!   ..Check for dimension errors
      IF (nd.GT.kd .OR. nr.GT.kr .OR. nm.GT.km) THEN 
        WRITE (*,*) 'Warning: array dimension error in  -  feautrier'
        nm = min(km,nm)
        nr = min(kr,nr)
        nd = min(kd,nd)
      ENDIF


!   ..Set up angle quadrature 
!     exclude horizontal rays -> singular matrix !
      IF (nm.EQ.0) THEN
         nm = 10
         mu(1) = 0.
         mu2(1) = 0.
      ENDIF
      IF (nm.GT.1 .AND. mu(1).EQ.0.) THEN
        mx = nm
        CALL nr_gauleg(ZERO,ONE,mu,b,mx)            ! Gauss-Legendre quadrature
      ELSEIF (nm.EQ.1 .AND. mu(1).EQ.0.) THEN
        b(1)   = 1.
        mu(1)  = 1.
        mu2(1) = 1.
      ENDIF
      DO m = 1,nm
        IF (abs(mu(m)).LT.1.0E-4) mu(m) = mu(m-1) / 2.
        mu2(m) = mu(m)*mu(m)
      ENDDO      

!   ..Rescale optical depths
      opnu(1)  = cap(1) + sig(1) + opl(1)
      taunu(1) = tauz(1) * 0.5 * ( (opnu(1)/opz(1)) + 1.0 )   ! opacity -> 0
      DO d = 2,nd
        opnu(d)  = cap(d) + sig(d) + opl(d)
	dtm(d)   = (tauz(d)-tauz(d-1)) 
     :           * sqrt( (opnu(d)*opnu(d-1)) / (opz(d)*opz(d-1)) ) ! geometric mean ratio
	taunu(d) = taunu(d-1) + dtm(d)
	dtp(d-1) = dtm(d)
      ENDDO
      

!   ..Evaluate scattering coefficients, thermal terms and tau differences
      bnuc = h2*fnu*fnu*fnu/c2
      DO d = 1,nd
        bnu(d) = bnuc / (exp (min (85.0D00,hk*fnu/temp(d)) ) - 1.0D00)
        snd    = sig(d) 
        cnd    = cap(d) + opl(d)
        alp(d) = snd / (cnd+snd)
        bet(d) = (1.0 - alp(d))
      ENDDO

!................................................!....................+........
!

!   ..Find coefficients of Feautrier matrices, A, B, C, L
      DO d = 1,nd                      ! depth
        DO n = 1,nn                    ! frequency : assume nn = 1
          DO m = 1,nm                  ! angle
            i = m + (n-1) * nm
            IF (d.EQ.1) THEN                       ! (6-33), (6-20)
              mudt    = mu(m) / dtp(d)
              aa(i,d) = 0.
              bb(i,d) = bet(d) + 2.0*mudt + 2.0*mudt*mudt 
              cc(i,d) = 2.0 * mudt*mudt
              ll(i,d) = bet(d) * bnu(d)
            ELSEIF (d.EQ.nd) THEN                  ! (6-35), (6-22)
              mudt    = mu(m) / dtm(d)
              aa(i,d) = 2.0 * mudt*mudt
              bb(i,d) = bet(d) + 2.0*mudt + 2.0*mudt*mudt
              cc(i,d) = 0.
              iidup   = bnu(d) + mu(m) * (bnu(d)-bnu(d-1)) 
     :                                / (taunu(d)-taunu(d-1))
              ll(i,d) = bet(d) * bnu(d) + 2.0*mudt * iidup
            ELSE                                 ! (6-30)
              dte     = 0.5 * (dtm(d) + dtp(d))
              aa(i,d) = mu2(m) / (dtm(d) * dte)
              cc(i,d) = mu2(m) / (dtp(d) * dte)
              bb(i,d) = bet(d) + aa(i,d) + cc(i,d)
              ll(i,d) = bet(d) * bnu(d)
            ENDIF
!           Rybicki and Hummer variant
!           hh(i,d) = -aa(i,d) + bb(i,d) - cc(i,d)
          ENDDO
        ENDDO
      ENDDO

!................................................!....................+........
!
!   ..Solve matrix equations for u
!    .Forward sweep;
      ii = nr * nm


!     Feautrier equations

!   . d = 1
      d = 1
      DO i = 1,ii
        ww(i,d) = 1.0 / bb(i,d)     
        dd(i,d) = ww(i,d) * cc(i,d)
        vv(i,d) = ww(i,d) * ll(i,d)
      ENDDO

!   . 1 < d <= nd
      DO d = 2,nd
        DO i = 1,ii
          ww(i,d) = 1.0 / (bb(i,d) - aa(i,d)*dd(i,d-1))
          dd(i,d) = ww(i,d) * cc(i,d)
          vv(i,d) = ww(i,d) * (ll(i,d) + aa(i,d)*vv(i,d-1))
        ENDDO
      ENDDO

!--
!     Rybicki & Hummer equations
!
!     d = 1
!     DO i = 1,ii
!       ww(i,d) = 1.0 / bb(i,d)     
!       zz(i,d) = ll(i,d) * ww(i,d)
!       dg(i,d) = cc(i,d) * ww(i,d)
!       ff(i,d) = hh(i,d) / cc(i,d)
!     ENDDO
!
!   . 1 < d < nd
!     DO d = 2,nd-1
!       DO i = 1,ii
!          ff(i,d) = (hh(i,d) + aa(i,d) * ff(i,d-1) * dg(i,d-1) ) 
!     :                / cc(i,d)
!          dg(i,d) = 1./(1.+ff(ii,d))
!          zz(i,d) = (ll(i,d) + aa(i,d)*zz(i,d-1)) * dg(i,d) 
!     :                / cc(i,d)
!       ENDDO
!     ENDDO
!
!     d = nd
!     DO i = 1,ii
!       zz(i,d) = (ll(i,d) + aa(i,d)*zz(i,d-1))  
!    :             / (bb(i,d)-aa(i,d)*dg(i,d-1))
!     ENDDO
!--


!    .Pivot
      DO i = 1,ii
        uu(i,nd) = vv(i,nd)
!        uu(i,nd) = zz(i,nd)     !  Rybicki & Hummer variant
!        uu(i,nd) = bnu(d)       !  no change
      ENDDO

!    .Back substitution
      DO d = nd-1,1,-1
        DO i = 1,ii
          uu(i,d) = vv(i,d) + dd(i,d) * uu(i,d+1)
!          uu(i,d) = zz(i,d) + dg(i,d) * uu(i,d+1) 
        ENDDO
      ENDDO

!.................................................!....................+........

!   ..Now calculate the source function
      DO d = 1,nd
        DO r = 1,nr
          jnd = 0.
          DO m = 1,nm
            i = m + (r-1)*nm
            jnd = jnd + b(m) * uu(i,d)
          ENDDO
          jj(d) = jnd
        ENDDO
        ss(d) = alp(d) * jj(d) + bet(d) * bnu(d)
      ENDDO

!! ......................................................................


      END





