      SUBROUTINE rtf_phi ( t, f, n, tau, phi )

!+
!
!  Program.
!     rtf-phi
!
!  Purpose.
!=    monochromatic 'PHI' operator for evaluating the flux integral.
!
!  Bugs:
!     1) Gives discontinuous solutions at high optical depth
!     2) Crashes for some atmosphere model parameters 
!     3) Should treat transfer for tau>5 self-consistently
!
!  Authors:
!     ??-Jan-1998: CSJ & PMH
!     24-Apr-1998: Bug 2 fixed using error catching 
!                  for tmax determination (PMH)
!     28-Apr-1998: Bug 1 fixed by selecting xr more carefully 
!                  and partly by Bug 2 fix (PMH)
!-

!  Global declaration
      IMPLICIT NONE

!  Subroutine Arguments
!    Import
      DOUBLE PRECISION t(*)                 ! t
      DOUBLE PRECISION f(*)                 ! f(t)
      INTEGER n                             ! dimension of f(t)
      DOUBLE PRECISION tau                  ! PHI evaluation 
!    Export
      DOUBLE PRECISION phi                  ! PHI integral

!  Local Constants
      INTEGER    kd                         ! Quadrature dimensions
      PARAMETER (kd=100)

!  Local Variables
!     Depth quadrature
      DOUBLE PRECISION  so(kd)              ! integrand
      DOUBLE PRECISION  e(kd)               ! exponential integral (debug)
      DOUBLE PRECISION  phii                ! inward integral 
      DOUBLE PRECISION  phio                ! outward integral 
      INTEGER  d                            ! Depth quadrature inex
      DOUBLE PRECISION  smax                ! maximum value of integrand
      DOUBLE PRECISION  tmax                ! limit of integral
      INTEGER  dmax                         ! location of smax in integrand
      DOUBLE PRECISION  smin                ! cutoff for S*E2

!    Gauss-Legendre quadrature
      INTEGER nqmax                ! order of quadrature
      PARAMETER (nqmax = 64)
      INTEGER nquad                ! order of quadrature
      PARAMETER (nquad = 12)
      INTEGER nqi                  ! local quadrature order
      INTEGER nlasti               ! previous quadrature order ?
      DOUBLE PRECISION qxi(nqmax)  ! scaled quadrature x values
      DOUBLE PRECISION qwi(nqmax)  ! quadrature weights
      INTEGER nqo                  ! local quadrature order
      INTEGER nlasto               ! previous quadrature order ?
      DOUBLE PRECISION qxo(nqmax)  ! scaled quadrature x values
      DOUBLE PRECISION qwo(nqmax)  ! quadrature weights
      INTEGER iq                   ! qaudratureloop index
      DOUBLE PRECISION xr, xm      ! quadrature range and median
      DOUBLE PRECISION pt, pf      ! interpolated function
      DOUBLE PRECISION et          ! exponential integral E(i,t)
      DOUBLE PRECISION sq          ! quadrature integrand

!  Functions used
      DOUBLE PRECISION rtf_expi

!.

!  Defaults
      smax = 0.
      dmax = 1

!  Compute integrand and store largest value 
!  Store corresponding (a fraction) minimum value to be considered

      DO d = 1,n
         e(d) = rtf_expi(2,abs(t(d)-tau))
         so(d) = f(d) * e(d)

         IF ( so(d).GT.smax ) THEN
           smax = so(d)
           dmax = d
         ENDIF
      ENDDO
      smin = smax/500.


!  Find value of t (tmax) which corresponds to the value of smin 
!  This can be either below or above t(dmax), but will be stored 
!  at the end as a value above t(dmax)

      tmax = t(n)


!  Maximum value at surface
      if (dmax.eq.1) then
        if (so(dmax+1).ne.0.) then

        DO d=dmax+1,n
        IF (so(d-1).GE.smin.AND.so(d).LT.smin.AND.so(d).GT.0) THEN
          tmax = t(d-1) + (t(d)-t(d-1))
     &     * log10(smin/so(d-1)) / log10(so(d)/so(d-1))
        ENDIF
        ENDDO

        else
          tmax = 0.5*(t(2)+t(1))   ! case when dmax=1, so(2)=0
        endif


! Maximum value at deep boundary
      elseif (dmax.eq.n) then
        if (so(dmax-1).ne.0.) then

        DO d=dmax-1,1
        IF (so(d+1).GE.smin.AND.so(d).LT.smin.AND.so(d).GT.0) THEN
          tmax = t(d) + (t(d+1)-t(d))
     &     * log10(smin/so(d)) / log10(so(d+1)/so(d))
        ENDIF
        ENDDO

        else
          tmax = 0.5*(t(n)+t(n-1))  ! case when dmax=n, so(n-1)=0
        endif


!  Maximum value between boundaries
      else
        if (so(dmax+1).ne.0.) then

        DO d=dmax+1,n
        IF (so(d-1).GE.smin.AND.so(d).LT.smin.AND.so(d).GT.0) THEN
          tmax = t(d-1) + (t(d)-t(d-1))
     &     * log10(smin/so(d-1)) / log10(so(d)/so(d-1))
        ENDIF
        ENDDO

        elseif (so(dmax-1).ne.0.) then            

        DO d=dmax-1,1
        IF (so(d+1).GE.smin.AND.so(d).LT.smin.AND.so(d).GT.0) THEN
          tmax = t(d) + (t(d+1)-t(d))
     &     * log10(smin/so(d)) / log10(so(d+1)/so(d))
        ENDIF
        ENDDO

        else
          tmax = t(dmax) + 0.5 * 
     &              max((t(dmax)-t(dmax-1)),(t(dmax+1)-t(dmax))) 
                 ! case when dmax=2,n-1 and so(dmax+1,dmax-1)=0

        endif

      endif
      tmax = tau + abs(tmax-tau)


!  Calculate outward flux integral

!  Get quadrature abscissae and weights
      nqo = nquad
      IF (nqo.NE.nlasto) CALL nr_gauleg ( -1.D0, +1.D0, qxo, qwo, nqo )
      nlasto = nqo
      
!   Fix for bottom layer
      IF (tau.eq.t(n)) THEN
        phio = f(n)
        
      ELSE
        sq = 0.D+0
        xr = 0.5 * (tmax-tau)
        xm = 0.5 * (tmax+tau)
        DO iq = 1,nqo
           pt = xm + qxo(iq) * xr
           CALL rtf_snu (t,f,n,pt,pf)
           et = rtf_expi(2,pt-tau)
           sq = sq + qwo(iq) * pf * et
        ENDDO
        phio = sq * xr
      ENDIF
             

!  Calculate inward flux integral

!  Get quadrature absicssae and weights
      nqi = nquad
      IF (nqi.NE.nlasti) CALL nr_gauleg ( -1.D0, +1.D0, qxi, qwi, nqi )
      nlasti = nqi

!   Fix for top layer
      IF (tau.eq.t(1)) THEN 
        phii=f(1)*tau
        
      ELSE
        sq = 0.D+0
        xr = 0.5 * min(tau,(tmax-tau))
        xm = tau - xr
        DO iq = 1,nqi
           pt = xm + qxi(iq) * xr 
           CALL rtf_snu (t,f,n,pt,pf)
           et = rtf_expi(2,tau-pt)
           sq = sq + qwi(iq) * pf * et
        ENDDO
        phii = sq * xr
      ENDIF
      

!  Result
      phi = 2.0 * (phio - phii)

      END
