      SUBROUTINE DP_ION (K,T,PE,U,CHI, X)

!+
!   dp_ion
!
!
!=  Calculate fractional abundances of ions of a given atomic species
!   from the Saha ionization equilibrium formula. This formulation is
!   not currently suitable for use with -ve ions, eg H and He.
!
!
!       Input:
!       K        Atomic number (Z) of species   !! no longer used 
!       T        Temperature (degrees K)
!       PE       Electron pressure
!       U(*)     Partition functions for first 6 ionisation states (I - VI)
!       CHI(*)   Ionization potentials (in ev) for first 6 ionisation states (I - VI)
!
!       Output:
!       X(6)     Fractional abundance of ions in each state (normalized to 1)
!                NB: the contribution by negative ions is neglected
!
!  History:
!  23.Nov.2001: CSJ: adapted from  opk_ion, using supplied ionisation potentials
!
!-


!  Global Typing.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)


!  Subroutine Arguments (input).
      INTEGER          K         ! Atomic number
      DOUBLE PRECISION T         ! Temperature (K)
      DOUBLE PRECISION PE        ! Electron pressure (dyn cm^-2)
      DOUBLE PRECISION U(*)      ! Partition functions
      DOUBLE PRECISION CHI(*)    ! Ionazation potentials

!  Subroutine Arguments (output).
      DOUBLE PRECISION X(*)      ! Ionization fractions

!  Local variables.
      DIMENSION F(5)             ! Ionization ratios

!  Local constants.
      DOUBLE PRECISION  S0
      PARAMETER (S0=0.666698474)
      DOUBLE PRECISION  BK
      PARAMETER (BKX = 1./8.6167E-05)

!.

      SAHA = S0  * T**2.5
      BKDT = BKX / T

      DO I = 1,5
       F(I) = 0.
      ENDDO

      DO I = 1,MIN(K,5)
       F(I) = SAHA*U(I+1)/U(I)*EXP(-CHI(I)*BKDT)
      ENDDO

      X(2) = 1.0 / (PE/F(1) + 1.0 + F(2)*(F(3)+PE)/(PE*PE))
      X(3) = X(2)*F(2)/PE
      X(4) = X(3)*F(3)/PE
      X(1) = X(2)*PE/F(1)
      X(5) = 0.
      X(6) = 0.

!  find 5th and 6th ionization stages...
      IF (K.GT.5) THEN
        X(5) = X(4)*F(4)/PE
        X(6) = X(5)*F(5)/PE
        SUM  = X(1) + X(2) + X(3) + X(4) + X(5) + X(6)

!    ...by iteration
        DO L = 1,10
          X(1) = X(1)/SUM
          X(2) = X(1)*F(1)/PE
          X(3) = X(2)*F(2)/PE
          X(4) = X(3)*F(3)/PE
          X(5) = X(4)*F(4)/PE
          X(6) = X(5)*F(5)/PE
          SUM  = X(1) + X(2) + X(3) +X(4) + X(5) + X(6)
          IF (ABS(1.- SUM) .LT. 0.0001)  GO TO 20
        ENDDO

        WRITE(*,
     :   '(''dp_ion: Ionization equilibrium not converging ='',F6.4)') 
     :     SUM

 20     CONTINUE
        ENDIF

      END
