SUBROUTINE SPC_READMODEL ( F__MODEL, MODEL, IECHO, IOSTAT )

  !+
  !
  !  Purpose:     
  !=    Read in a Sterne model atmosphere
  !
  !  Description:
  !     This subroutiine provides a framework for reading in a model 
  !     atmosphere structure which has been written in a free format
  !     using a series of labels. Sterne is being adapted to write such formats. 
  ! 
  !     By defining a data structure and reading the model into that
  !     structure, there are a range of ways in which the subroutine can be 
  !     used.
  ! 
  !     1. The data structure can be read in by any program
  !     requiring a model atmosphere as input. Then, individual 
  !     components of the model can be extracted and converted to appropriate 
  !     units as required. Metadata are provided to enable conversion of units
  !
  !     2. The syntax for the input model is designed to be flexible. For 
  !     instance, if we wish to add a field (e.g. population numbers) to the
  !     saved model, then this can be done easily. Model readers will 
  !     continue to work, simply ignoring new fields, or can be modified to 
  !     read the new data as required.
  !
  !     3. Multiple models can be processed by reading different models into
  !     distinct data structures. 
  !
  !  Language:
  !     Fortran 95
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     15-SEP-2006: CSJ
  !         Original version
  !     13-AUG-2020: CSJ
  !         Upgrading to use as default for Spectrum and Sfit
  !
  !  Use:
  !     CALL SPC_READMODEL ( F__MODEL, MODEL )
  !     
  !  Subroutine Arguments (Given):
  !     F__MODEL  : CHARACTER(*)  :  Name of file containing model atmosphere
  !
  !  Subroutine Arguments (Returned):
  !     MODEL  : TYPE(MODEL_ATMOS)  :  Data structure containing model atmosphere
  !
  !     
  !-

  !   public control variables
  USE SPC_CONTROL

  !   collect the model atmosphere data structure
  USE LTE_STRUCTURES

  !  Safe definitions
  IMPLICIT NONE


  !  Explicit Subroutine interfaces
  INTERFACE

     SUBROUTINE KEY_COM (PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_COM

     SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
       CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned)
       CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned)
       CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
       CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
       INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
       INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
     END SUBROUTINE KEY_GET

     LOGICAL FUNCTION KEY_MATCH (STRING,VALUE,OPTIONS)       
       CHARACTER*(*), INTENT(IN)           ::  STRING
       CHARACTER*(*), INTENT(IN)           ::  VALUE
       CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS
     END FUNCTION KEY_MATCH

     LOGICAL FUNCTION KEY_CONTAINS (STRING,VALUE,OPTIONS)       
       CHARACTER*(*), INTENT(IN)           ::  STRING
       CHARACTER*(*), INTENT(IN)           ::  VALUE
       CHARACTER*(*), INTENT(IN), OPTIONAL ::  OPTIONS
     END FUNCTION KEY_CONTAINS


  END INTERFACE


  !  Subroutine Arguments (Given):
  !     F__MODEL  : CHARACTER(*)  :  Name of file containing model atmosphere
  CHARACTER(LEN=*), INTENT(IN) :: F__MODEL

  !
  !  Subroutine Arguments (Returned):
  !     MODEL  : TYPE(MODEL_ATMOS)  :  Data structure containing model atmosphere
  TYPE (MODEL_ATMOS) MODEL


  !  i/o control
  INTEGER :: IECHO     ! echo control
  INTEGER :: IOSTAT    ! error status


  !     Command line control
  CHARACTER(LEN=100) :: COMMAND ! holder for current command line
  CHARACTER(LEN=30)  :: KEYWRD  ! holder for current command or keyword
  CHARACTER(LEN=15)  :: PROG__NAME='stn_readmodel' ! name of this subroutine
  CHARACTER(LEN=6)   :: PARCOM  


  !     Temporary composition variables
  CHARACTER(LEN=2)  :: ATOM    ! atomic identifier
  INTEGER           :: IZ      ! atomic number
  REAL              :: ELABUND ! elemental abundance
  

  !     Other local variables
  INTEGER :: I,J  ! loop index
  INTEGER :: ND   ! number of depth points

  !     External function calls
  INTEGER :: TAP_ZELEM
  DOUBLE PRECISION :: TAP_ABSOL


  !.



  !-------------------------------------------------------------------------

  !     Set up i/o channels
  CALL STR_DEF (NMODEL, NWRITE)

  !     Open input stream
  IF ( LEN_TRIM(F__MODEL) /= 0 ) THEN 
     OPEN(UNIT=NMODEL,FILE=F__MODEL,STATUS='OLD')
  ELSE
     CALL STR_ERR ( PROG__NAME,'no filename' )
     CALL STR_DEF (NREAD, NWRITE)
     RETURN
  ENDIF

  !  Initialize echo
  IECHO = 0



  !-------------------------------------------------------------------------

  !   Loop over keywords in model 
  DO

     !     get command line
     CALL KEY_GET ( KEYWRD, COMMAND,'model>', ',', NMODEL, IECHO, IOSTAT )
     CALL STR_LCASE ( KEYWRD )

     !     trap EOF  ... assume end of model OK
     IF ( IOSTAT /= 0 ) THEN
        EXIT

     ELSEIF ( KEY_MATCH(KEYWRD,'end') ) THEN
        EXIT

        !    skip comments
     ELSEIF ( KEY_MATCH(KEYWRD,'!') ) THEN


        ! echo commands to the terminal
     ELSEIF ( KEY_MATCH(KEYWRD,'echo') ) THEN

        CALL KEY_PC (COMMAND, PARCOM)
        IF (PARCOM(1:1) == "2" )                         IECHO = 2
        IF (PARCOM(1:2) == "on"  .OR. PARCOM(1:1)=="1" ) IECHO = 1
        IF (PARCOM(1:3) == "off" .OR. PARCOM(1:1)=="0" ) IECHO = 0
        
        !    data characterisation
     ELSEIF ( KEY_MATCH(KEYWRD,'model') ) THEN

        !    origin
        IF ( KEY_CONTAINS(KEYWRD,'origin') ) THEN
           CALL KEY_PC (COMMAND, MODEL%ORIGIN)

           !    date
        ELSEIF ( KEY_CONTAINS(KEYWRD,'date') ) THEN
           CALL KEY_PC (COMMAND, MODEL%DATE)

           !    label
        ELSEIF ( KEY_CONTAINS(KEYWRD,'label') ) THEN
           CALL KEY_PC (COMMAND, MODEL%LABEL)

           !    effective temperature
        ELSEIF ( KEY_CONTAINS(KEYWRD,'teff') ) THEN
           CALL KEY_PR (COMMAND, MODEL%TEFF)

           !    surface gravity
        ELSEIF ( KEY_CONTAINS(KEYWRD,'gravity' ) ) THEN
           CALL KEY_PR (COMMAND, MODEL%GRAV)

           !    reference wavelength
        ELSEIF ( KEY_CONTAINS(KEYWRD,'lambda_ref' ) ) THEN
           CALL KEY_PR (COMMAND, MODEL%WAV_REF)

           !    number of layers
        ELSEIF ( KEY_CONTAINS(KEYWRD,'ndepth') ) THEN
           CALL KEY_PI (COMMAND, MODEL%NDEPTH)

           ALLOCATE ( MODEL%TAU  (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%TEMP (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%OPAC (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%PGAS (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%PEL  (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%GRAD (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%MASS (MODEL%NDEPTH) )
           ALLOCATE ( MODEL%Z    (MODEL%NDEPTH) )
           IF (  MODEL%NELEM_STRAT /= 0 ) & 
                ALLOCATE ( MODEL%STRAT (MODEL%NELEM_STRAT,MODEL%NDEPTH) )
           
           !    number of elements for which abundances stored
        ELSEIF ( KEY_CONTAINS(KEYWRD,'nelem') ) THEN
           CALL KEY_PI (COMMAND, MODEL%NELEM)
           ALLOCATE ( MODEL%ABUNDS (MODEL%NELEM) )
           MODEL%ABUNDS = 0.
           
           !    number of stratified elements
        ELSEIF ( KEY_CONTAINS(KEYWRD,'nstrat') ) THEN
           CALL KEY_PI (COMMAND, MODEL%NELEM_STRAT)
           ALLOCATE ( MODEL%ZELEM_STRAT (MODEL%NELEM_STRAT) )
           ALLOCATE ( MODEL%QELEM_STRAT (MODEL%NELEM_STRAT) )

           IF (  MODEL%NDEPTH /= 0 ) & 
                ALLOCATE ( MODEL%STRAT (MODEL%NELEM_STRAT,MODEL%NDEPTH) )
           MODEL%QELEM_STRAT = .FALSE.
           MODEL%ZELEM_STRAT = 0
           
           !    import [Fe/H] and initialise metals if required
        ELSEIF ( KEY_CONTAINS(KEYWRD,'febyh') ) THEN
           CALL KEY_PR (COMMAND, MODEL%FEBYH)

           IF ( MODEL%NELEM > 0 ) THEN
              DO I = 1,MODEL%NELEM
                 MODEL%ABUNDS(I) = 10.** ( TAP_ABSOL( I ) - 12.0 + MODEL%FEBYH )
              END DO
              ! renormalise to ensure these are fractional abundances
              MODEL%ABUNDS = MODEL%ABUNDS / SUM ( MODEL%ABUNDS )
              ! test this.... 
           ENDIF

           !    import [a/Fe] and initialise C,O,Ne,Mg if required
        ELSEIF ( KEY_CONTAINS(KEYWRD,'albyfe') ) THEN
           CALL KEY_PR (COMMAND, MODEL%ALBYFE)

           IF ( MODEL%NELEM > 0 ) THEN
              MODEL%ABUNDS( 6) = MODEL%ABUNDS( 6) * 10.**MODEL%ALBYFE
              MODEL%ABUNDS( 8) = MODEL%ABUNDS( 8) * 10.**MODEL%ALBYFE
              MODEL%ABUNDS(10) = MODEL%ABUNDS(10) * 10.**MODEL%ALBYFE
              MODEL%ABUNDS(12) = MODEL%ABUNDS(12) * 10.**MODEL%ALBYFE
              WRITE (*,'(A,F6.2)') 'alpha abundances set to [a/Fe] = ',MODEL%ALBYFE
           ENDIF

           !    import [He/H] and initialise H,He if required
        ELSEIF ( KEY_CONTAINS(KEYWRD,'frache') ) THEN
           CALL KEY_PR (COMMAND, MODEL%FRACHE)

           IF ( MODEL%NELEM > 0 ) THEN
              MODEL%ABUNDS( 2) =  MODEL%FRACHE 
              MODEL%ABUNDS( 1) = 1. - MODEL%ABUNDS (2) -  SUM ( MODEL%ABUNDS(3:) )
           ENDIF

           !    abundances (individual)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'abundance') ) THEN
           CALL KEY_PI (COMMAND, IZ)
           CALL KEY_PR (COMMAND, ELABUND)
           MODEL%ABUNDS(IZ) = ELABUND

        ENDIF

        !    opacity
     ELSEIF ( KEY_MATCH(KEYWRD,'opacity') ) THEN
        
        !    bf opacity type
        IF ( KEY_CONTAINS(KEYWRD,'bf_source' ) ) THEN
           CALL KEY_PC (COMMAND, MODEL%OP_BF)
           CALL STR_UCASE ( MODEL%OP_BF )

           !    bb opacity type
        ELSEIF ( KEY_CONTAINS(KEYWRD,'bb_source' ) ) THEN
           CALL KEY_PC (COMMAND, MODEL%OP_BB)
           CALL STR_UCASE ( MODEL%OP_BB )

           !    bb int
        ELSEIF ( KEY_CONTAINS(KEYWRD,'bb_int') ) THEN
           CALL KEY_PR (COMMAND, MODEL%OP_ODFINT)

           !    bb linelist
        ELSEIF ( KEY_CONTAINS(KEYWRD,'list') ) THEN
           CALL KEY_PC (COMMAND, MODEL%OP_LINELIST)

           !    bb odf
        ELSEIF ( KEY_CONTAINS(KEYWRD,'odf') ) THEN
           CALL KEY_PC (COMMAND, MODEL%OP_ODFNAME)
           
           !    bb microturbulence
        ELSEIF ( KEY_CONTAINS(KEYWRD,'vturb') ) THEN
           CALL KEY_PR (COMMAND, MODEL%OP_VTURB)

        ENDIF


        !    model atmosphere structure
     ELSEIF ( KEY_MATCH(KEYWRD,'structure') ) THEN

        ! local proxy for model.ndepth
        ND = MODEL%NDEPTH
        
        IF ( MODEL%NDEPTH == 0 ) THEN 
           CALL STR_ERR ( PROG__NAME,'Number of depth points undefined -- aborting' ) 
           STOP

        ELSEIF ( KEY_CONTAINS(KEYWRD,'tau') ) THEN
           READ(NMODEL,*)(MODEL%TAU(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'temp') ) THEN
           READ(NMODEL,*)(MODEL%TEMP(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'opac') ) THEN
           READ(NMODEL,*)(MODEL%OPAC(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'pgas') ) THEN
           READ(NMODEL,*)(MODEL%PGAS(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'pel') ) THEN
           READ(NMODEL,*)(MODEL%PEL(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'grad') ) THEN
           READ(NMODEL,*)(MODEL%GRAD(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'mass') ) THEN
           READ(NMODEL,*)(MODEL%MASS(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'z') ) THEN
           READ(NMODEL,*)(MODEL%Z(J),J=1,ND)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'abund') ) THEN
           CALL KEY_PC (COMMAND, ATOM)
           IZ = TAP_ZELEM ( ATOM )
           READ(NMODEL,*)(MODEL%STRAT(IZ,J),J=1,ND)
           MODEL%QELEM_STRAT(IZ) = .TRUE.
           MODEL%ZELEM_STRAT(IZ) = IZ

        ENDIF


        !  model units
     ELSEIF ( KEY_MATCH(KEYWRD,'units') ) THEN
        
        IF ( KEY_CONTAINS(KEYWRD,'teff') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_TEFF)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'grav') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_GRAV)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'vturb') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_VTURB)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'wav_ref') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_WAV_REF)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'tau') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_TAU)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'temp') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_TEMP)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'pgas') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_PGAS)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'pel' ) ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_PEL)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'grad') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_GRAD)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'opac') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_OPAC)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'mass') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_MASS)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'z'   ) ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_Z)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'wave') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_WAVE)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'flux') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_FLUX)
        ELSEIF ( KEY_CONTAINS(KEYWRD,'abun') ) THEN
           CALL KEY_PC (COMMAND, MODEL%UNITS_ABUND)
        ENDIF

        !    model spectrum :: to be ignored
     ELSEIF ( KEY_MATCH(KEYWRD,'spectrum') ) THEN

        IF ( KEY_CONTAINS(KEYWRD,'length') ) THEN
           CALL KEY_PI (COMMAND, MODEL%NWAVE)

        ELSEIF ( KEY_CONTAINS(KEYWRD,'wave') ) THEN
           ALLOCATE ( MODEL%WAVE (MODEL%NWAVE) )
           READ(NMODEL,*) MODEL%WAVE

        ELSEIF ( KEY_CONTAINS(KEYWRD,'flux') ) THEN
           ALLOCATE ( MODEL%FLUX (MODEL%NWAVE) )
           READ(NMODEL,*) MODEL%FLUX

        ENDIF

     ENDIF


  ENDDO



  !-------------------------------------------------------------------------

  ! sort out stuff to return after all data has been read in

  ! take antilogs ... 

  CALL ANTILOG_S ( MODEL%TEFF, MODEL%UNITS_TEFF )
  CALL ANTILOG_S ( MODEL%GRAV, MODEL%UNITS_GRAV )

  CALL ANTILOG_V ( MODEL%TAU,  MODEL%UNITS_TAU  )
  CALL ANTILOG_V ( MODEL%TEMP, MODEL%UNITS_TEMP )
  CALL ANTILOG_V ( MODEL%PGAS, MODEL%UNITS_PGAS )
  CALL ANTILOG_V ( MODEL%PEL,  MODEL%UNITS_PEL  )
  CALL ANTILOG_V ( MODEL%GRAD, MODEL%UNITS_GRAD )
  CALL ANTILOG_V ( MODEL%OPAC, MODEL%UNITS_OPAC )
  CALL ANTILOG_V ( MODEL%MASS, MODEL%UNITS_MASS )
  CALL ANTILOG_V ( MODEL%Z,    MODEL%UNITS_Z    )

  !-------------------------------------------------------------------------

  ! report model atmosphere read in

  WRITE (*,*)
  WRITE (*,'(2X,A)') 'SPC_READMODEL: '
  WRITE (*,*)
  WRITE (*,'(2X,2A)')        'File         ',F__MODEL
  WRITE (*,'(2X,2A)')        'Origin       ',MODEL%ORIGIN
  WRITE (*,'(2X,2A)')        'Date         ',MODEL%DATE
  WRITE (*,'(2X,2A)')        'Label        ',MODEL%LABEL
  WRITE (*,*)
  WRITE (*,'(2X,A,F10.0,A)') 'Model.Teff   ',MODEL%TEFF,' '//MODEL%UNITS_TEFF
  WRITE (*,'(2X,A,F10.2,A)') 'Model.Grav   ',ALOG10(MODEL%GRAV),' log '//MODEL%UNITS_GRAV
  WRITE (*,'(2X,A,F10.4,A)') 'Model.H      ',MODEL%ABUNDS(1)
  WRITE (*,'(2X,A,F10.4,A)') 'Model.He     ',MODEL%ABUNDS(2)
  WRITE (*,'(2X,A,F10.2,A)') 'Model.logy   ',ALOG10(MODEL%ABUNDS(2)/MAX(MODEL%ABUNDS(1),0.00001))
  WRITE (*,'(2X,A,F10.2,A)') 'Model.[Fe/H] ',MODEL%FEBYH 
  WRITE (*,'(2X,A,F10.2,A)') 'Model.[a/Fe] ',MODEL%ALBYFE
  WRITE (*,*)
  WRITE (*,'(2X,A,I5)') 'N elements    ',MODEL%NELEM
  WRITE (*,'(2X,A,I5)') 'N depths      ',MODEL%NDEPTH
  WRITE (*,'(2X,A,I5)') 'N wavelengths ',MODEL%NWAVE
  WRITE (*,*)


  !-------------------------------------------------------------------------

  !   Reset i/o channels to previous values
  CLOSE (UNIT=NMODEL)
  CALL STR_DEF (NREAD, NWRITE)



CONTAINS

  SUBROUTINE ANTILOG_V ( structure, units )

    REAL, DIMENSION(MODEL%NDEPTH) :: structure
    CHARACTER(LEN=*)               :: units

    IF (  units(1:3) == 'log' ) THEN 
       structure = 10.**structure 
       units = units(5:)
    ENDIF

  END SUBROUTINE ANTILOG_V
  
  SUBROUTINE ANTILOG_S ( structure, units )

    REAL  :: structure
    CHARACTER(LEN=*)  :: units

    IF (  units(1:3) == 'log' ) THEN 
       structure = 10.**structure 
       units = units(5:)
    ENDIF

  END SUBROUTINE ANTILOG_S
  

END SUBROUTINE SPC_READMODEL


