SUBROUTINE SPC_SYNTH (IN_NMODE, IN_NOPAC, IN_NSRC, IN_NTHT, &
     IN_NHED, IN_NHES, SELNO, SELID, SELAB, VTURB, &
     WMIN, WMAX, WL, FL, FC, FN, NWMOD, NEWGRID, &
     IN_MODEL )

  !+
  !
  !  Program:
  !     SPC_SYNTH
  !
  !  Purpose:
  !=    Provides a callable version of the package SPECTRUM
  !     in the mode which generates a synthetic spectrum over a given
  !     wavelength interval. 
  !
  !  Notes:
  !     This subroutine provides an interface to the "SPECTRUM" subroutine
  !     library. The intention is that it should be entirely self-sufficient
  !     and no variables are passed via common or otherwise from or to
  !     the calling program.
  !
  !
  !  Language:
  !     Fortran 77+
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     15-FEB-2000: CSJ
  !        original version adapted from SPECTRUM main program
  !
  !-

  !    Public constants
  USE SPC_CONSTANTS

  !    Global variables
  USE SPC_GLOBAL
  USE SFIT_SYNDATA , ONLY : SYN__LINES
  USE LTE_WLGRID   , ONLY : REFWL

  !  Global declaration.
  IMPLICIT NONE

  !  Subroutine arguments (given).
  INTEGER, INTENT(IN) :: IN_NMODE   ! type of calculation
  INTEGER, INTENT(INOUT) :: IN_NOPAC ! continnuum opacity sources
  INTEGER, INTENT(IN) :: IN_NSRC    ! radiative transfer mode
  INTEGER, INTENT(IN) :: IN_NTHT    ! number of angles at which intensities required
  INTEGER, INTENT(IN) :: IN_NHED    ! source of helium data
  INTEGER, INTENT(IN) :: IN_NHES    ! source of HeI Stark broadening data

  CHARACTER*(*), INTENT(IN) :: IN_MODEL   ! filename containing model atmosphere

  INTEGER, INTENT(INOUT) :: SELNO       ! number of elements treated
  INTEGER, INTENT(IN) :: SELID(*)    ! atomic numbers of elems for which abunds given
  REAL, INTENT(IN) ::   SELAB(*)     ! abundances of above elements
  REAL, INTENT(IN) ::   VTURB        ! microturbulent velocity (km/s)
  REAL, INTENT(IN) ::   WMIN, WMAX   ! limits for wavelength grid (A)

  LOGICAL, INTENT(IN) :: NEWGRID        ! Flag whether to generate a new wavelength grid

  !  Subroutine arguments (given and returned).
  INTEGER, INTENT(INOUT) :: NWMOD              ! Number of wavelength points in WL
  REAL, INTENT(INOUT) ::  WL(*)          ! Wavelengths

  !  Subroutine arguments (returned).
  REAL, INTENT(OUT) ::  FL(*)          ! Emergent flux
  REAL, INTENT(OUT) ::  FC(*)          ! Continuum flux
  REAL, INTENT(OUT) ::  FN(*)          ! Normalized flux (FL/FC)



  !  Local variables.

  INTEGER :: I, J, L

  LOGICAL, SAVE ::  NODATA = .TRUE.  ! Check whether atomic data has been read in
  ! Name of last model atmosphere read in
  CHARACTER*80, SAVE :: LASTMOD = '                                           '  

  DOUBLE PRECISION :: XMIN, XMAX
  DOUBLE PRECISION :: COSTMIN = 0.005
  INTEGER :: MLEN, NMOD, NMSK

  !  Local constants.




  !.

  MLEN = LEN_TRIM(IN_MODEL)
  write (*,*) 
  write (*,'(A)') '   spc_synth '
  write (*,'(A)') '   spc_synth: model: '//IN_MODEL(1:MLEN)
  write (*,'(A)') '   spc_synth: abs,vt: '
  write (*,'(20f8.3)') selab(1:selno),vturb
  write (*,'(a,2f10.3)') '   spc_synth: wmin-wmax: ',wmin,wmax


  !  Default control parameters for computing spectra from single models.
  NMOD = 1        ! number of models to compute spectra for
  NMSK = 1        ! number of models to skip in model grid
  WAVMAX = 0.     ! cutoff wavelength for metal line calculations
  NMODE = IN_NMODE    ! generally : nmode = 11 - spectral synthesis
  NOPTYP = IN_NOPAC   ! 
  NSOURCE = IN_NSRC   ! radiative transfer mode
  NTHETA  = IN_NTHT   ! number of angles at which intensities required
  NHEDAT = IN_NHED    ! 
  NHESTARK = IN_NHES  ! 
  IVTSY = 1       !  microturbulence counter = 1 for LSYNT
  IABSY = 1       !  Abundance counter = 1 for LSYNT passed in common 
  TAU_WL  = 4000.
  REFWL   = TAU_WL


  !  Check if intensities are required and set cosine angles.
  COSTMIN = 0.005
  IF (NSOURCE >= 4) THEN
     NSOURCE = NSOURCE - 4
     NTHETA = MIN(NTHETA,KTH)
     DO I=1,NTHETA
        COSTHETA(I)=(NTHETA-I)/FLOAT(NTHETA-1)
        COSTHETA(I)=MAX(COSTHETA(I),COSTMIN)
     ENDDO
  ELSE
     NTHETA = 0
  ENDIF


  !  Read in atomic data if it is required.
  IF ( NODATA ) THEN
     WRITE (*,'(A,2F8.0)') '   spc_synth: Reading atomic data ' 
     CALL DP_ATDATA
     CALL DP_ATHEI
     CALL DP_ATHEII
     CALL SPC_READWL
     CALL OP_ALP
     CALL OP_BFIN
     ! CALL OP_FEIN
     CALL OP_PEACHIN
  ENDIF

  !  Read in a new model atmosphere if required.
  IF (IN_MODEL(1:MLEN) /= LASTMOD(1:MLEN)) THEN 
     WRITE (*,'(A)') '   spc_synth: Reading model atmosphere '//IN_MODEL(1:MLEN)
     CALL SPC_READQUB ( IN_MODEL(1:MLEN) )
     LASTMOD = IN_MODEL
     IN_NOPAC = NOPTYP
  ENDIF

  !  Read in linelist if it is required.
  !** 2008/03/05
  !** the linelist should be changed to an input argument and
  !   specified in the calling program
  IF ( NODATA ) THEN
     WRITE (*,'(A,2F8.0)') '   spc_synth: Reading linelist in range: ',WMIN,WMAX 
     NLINE = 0
     CALL SPC_LINES ( SYN__LINES, WMIN, WMAX )
     NODATA = .FALSE.
  ENDIF

  !  Establish microturbulence.
  NVTSY   = 1
  VT      = VTURB
  VTSY(1) = VTURB
  QUB__VTRB(1:NDEPTH) = (/ (VT,J=1,NDEPTH) /)


  !  Select atomic data for synthesised spectrum and generate wavelength
  !  grid for new set of data.
  IF ( NEWGRID ) THEN
     WRITE (*,'(A,2F8.0)') '   spc_synth: Selecting lines and generating grid ' 
     XMIN = WMIN
     XMAX = WMAX
     CALL SPC_SCAN (XMIN,XMAX)

     NW=NWSY
     SYMM=.FALSE.
     W(1:NW)=WSY(1:NW)

     !  Otherwise use the given wavelength grid
  ELSE

     NW = NWMOD  
     W(1:NW)=WL(1:NW)

  ENDIF


  !  Establish abundances for this synthesis.
  WRITE (*,'(A,2F8.0)') '   spc_synth: Defining abundances ' 
  CALL SPC_ABUND (SELNO, SELID, SELAB)

  !  Convert abundances to real values
  ABUSY(1:NELSY)=MIN(10.0**(ABUSY(1:NELSY)-ANORM),1.D0)



  !  Calculate line opacities.
  WRITE (*,'(A,2F8.0)') '   spc_synth: Computing line opacities ' 
  CALL LSYNT

  !  Calculate optical depth scale and fluxes.
  WRITE (*,'(A,2F8.0)') '   spc_synth: Computing fluxes ' 
  CALL INTEG

  !  Compute normalized flux and copy results to output arrays.
  WL(1:NW) = W(1:NW)
  FL(1:NW) = FLUX(1:NW)
  FC(1:NW) = FLUXC(1:NW)
  FN(1:NW) = FLUX(1:NW)/FLUXC(1:NW)
  NWMOD = NW


  write (*,*) '  spc_synth ---------------------------'
  write (*,*)

END SUBROUTINE SPC_SYNTH
