SUBROUTINE FLXOS (VMIC, KEY_OPBF, LINE_COUNT)

  !+
  !
  !      Calculates monochromatic and integrated flux and some 
  !      integrals for temperature correction
  !      includes line opacity (opacity sampling)
  !
  !-


  !  User defined data types.
  USE LTE_PRECISIONS
  USE LTE_WLGRID
  USE LTE_MODEL

  !  Public global constants.
  USE TAPM_PAR
  USE TAPP_PAR

  !  Sterne global data.
  USE STN_PARAMETERS
  USE STN_INTGRA
  USE STN_FLUXES

  !  No implicit typing.
  IMPLICIT NONE


  !! subroutine arguments  
  REAL :: VMIC                                ! Microturbulent velocity (cm/s)
  CHARACTER(LEN=12), INTENT(IN) :: KEY_OPBF   ! Switch between KIEL and OP opacities
  INTEGER :: LINE_COUNT


  !  Local constants. 
  REAL, PARAMETER :: BK  = TAPP__K * 1.E07 

  !! local variables
  INTEGER :: I, J, K


  INTEGER, PARAMETER :: NDMAX = 50

  REAL :: EN(NDMAX)                          ! Electron 
  REAL :: CAP(NDMAX)   
  REAL :: SIGM(NDMAX)
  REAL :: OPA(NDMAX)
  REAL :: FREQ
  REAL :: DOP(NDMAX)

  REAL(KIND=QQ) :: TAU_NU(NDMAX)             ! local optical deoth   

  REAL(KIND=QQ) :: CAPLIN(NDMAX)
  REAL(KIND=QQ) :: CAPCON(NDMAX)
  real(KIND=QQ) :: CAPHOP(NDMAX)


  INTEGER :: N, NSTEPS

  REAL(KIND=QQ) :: W
  REAL :: STEPWT, SUMWT

  !! external function declaration
  REAL(KIND=QQ), EXTERNAL :: FREWGT

  REAL :: TEMP1, TEMPK, TEMPKK

  !.


  !! allocate radiative arrays
  ALLOCATE(BNU(ND))                           ! Planck function        B_nu(tau)
  ALLOCATE(SNU(ND))                           ! Source function        S_nu(tau)
  ALLOCATE(JNU(ND))                           ! Mean intensity         J_nu(tau)
  ALLOCATE(FNU(ND))                           ! Total flux             F_nu (tau) 

  ALLOCATE(FNUC(ND))                          ! Total flux in continuum
  ALLOCATE(JNUC(ND))                          ! Mean intensity in continuum
  ALLOCATE(FNUL(ND))                          ! Total flux in lines
  ALLOCATE(JNUL(ND))                          ! Mean intensity in lines   

  ALLOCATE(CAC(ND))


  NU_REFWL = TAPP__C*1.E10 / REFWL 

  !  Calculate frequencies.      
  NU(LA:LL) =  TAPP__C*1.E10 / WL(LA:LL) 

  !  Initialize all variables and integrals      
  SIGM = SIG
  B    = 0.
  FLX  = 0.
  TNS  = 0.
  KAPP = 0.
  KAPJ = 0.
  KAPF = 0.
  GRAD = 0.
  TAU_NU = 0.

  !  Boltzmann factors and electron densities.
  !  Thomson scattering.        
  EN(1:ND) = P_E(1:ND) / (BK * TEMP(1:ND))

  DOP(1:ND) = (1./TAPP__C/100.) * SQRT ( 2 * BK * TEMP(1:ND) * TAPP__NA * 1.E3  + VMIC*VMIC )

  !********************************
  !  Loop over frequencies
  DO I = LA,LL
     W = FREWGT(I,NU,LA,LL)
     FREQ = NU(I)
     KAPPA_NU = 0.0
     SIG = 0.0
     CAC = 0.0
     OPA = 0.0

     ! Planck function.
     CALL BNUE(TEMP,FREQ,BNU,ND)
     !*     to be tested....
     !*       CALL RTF_BNU (temp,ND, nu(i),1, BNU,ND)

     ! Continuous opacities.
     DO K=1,ND
        CALL STN_OPAC (KEY_OPBF,K,I,TEMP(K),P_TOTAL(K),P_E(K),WL(I),OPA(K),CAC(K),SIG(K))
     END DO

     ! Compute hydrogen line opacity (and 
     !     correct hydrogen continuous opacity for dissolution of continuum)
     DO K=1,ND
        CALL OP_HLIN ( K,I,  TEMP(K),P_TOTAL(K),P_E(K),WL(I),DOP(K), &
             CAPLIN(K),CAPCON(K),CAPHOP(K), 3)
        CAC(K) = CAC(K) + CAPCON(K) - CAPHOP(K)
     END DO

     ! Continuum source function (Feautrier)
     CALL FLX_FEAUT (NU(I),JNUC)

     DO K=1,ND-1
        IF (TAU_NU(K) < 5.) THEN
           CALL FLX_PHI (TAU_NU,K,FNUC(K) )  
        END IF
        IF (TAU_NU(K) >= 5. .OR. FNUC(K) <= 0.) THEN
           IF (K > 1) THEN  
              FNUC(K) = 0.5*((BNU(K)-BNU(K-1))/(TAU_NU(K)-TAU_NU(K-1))  &
                   &   +(BNU(K+1)-BNU(K))/(TAU_NU(K+1)-TAU_NU(K)))*(4./3.)
           ELSE
	      FNUC(1) = 0.5*(BNU(K)/TAU_NU(K)  &
                   &   +(BNU(K+1)-BNU(K))/(TAU_NU(K+1)-TAU_NU(K)))*(4./3.)
           END IF
        END IF
     END DO
     FNUC(ND) = (BNU(ND)-BNU(ND-1))/(TAU_NU(ND)-TAU_NU(ND-1))*4./3.
 

     ! Line opacities (Opacity sampling)
     !    Note that it is straightforward to incorporate depth dependence
     !    in the microturbulent velocity.
     !    Add hydrogen line opacity (corrected for dissolution of continuum)
     DO K=1,ND
        CALL BB_KAP (K,I,TEMP(K),VMIC,KAPPA_NU(K),OPA(K), EP_0, LINE_COUNT)
        KAPPA_NU(K) = KAPPA_NU(K) + CAPLIN(K)
     END DO


     ! Line source function (Feautrier).
     CALL FLX_FEAUT (NU(I),JNUL)

     DO K=1,ND-1
        IF (TAU_NU(K) < 5.) THEN
           CALL FLX_PHI (TAU_NU,K,FNUL(K))  
        END IF

        IF (TAU_NU(K) >= 5. .OR. FNUL(k) < 0.) THEN
           IF (K > 1) THEN
              FNUL(K) = 0.5*(((BNU(K)-BNU(K-1)))/(TAU_NU(K)-TAU_NU(K-1)) &
                   &  +((BNU(K+1)-BNU(K)))/(TAU_NU(K+1)-TAU_NU(K)))*(4./3.)
           ELSE
 	      FNUL(1) = 0.5*(BNU(1)/TAU_NU(1) &
                   &  +(BNU(2)-BNU(1))/(TAU_NU(2)-TAU_NU(1)))*(4./3.)
           END IF
        END IF
     END DO
     FNUL(ND) = ABS((BNU(ND)-BNU(ND-1)))/(TAU_NU(ND)-TAU_NU(ND-1))*4./3.



     ! Frequency integration.
     FNU  = FNUL 
     JNU  = JNUL 
     B    = B    + W* BNU 
     FLX  = FLX  + W* FNUL 
     TNS  = TNS  + W* JNUL 
     CAP  = CAC  + KAPPA_NU
     KAPP = KAPP + W* CAP      *BNU 
     KAPJ = KAPJ + W* CAP      *JNUL
     KAPF = KAPF + W* (CAP+SIG)*FNUL

!     DO K=1,ND
!        FNU(K)  = FNUL(K) 
!        JNU(K)  = JNUL(K) 
!        B(K)    = B(K)   + W* BNU(K) 
!        FLX(K)  = FLX(K) + W*FNUL(K) 
!        TNS(K)  = TNS(K) + W*JNUL(K) 
!        CAP(K)  = CAC(K)+KAPPA_NU(K)
!        KAPP(K) = KAPP(K)+ W* CAP(K)        *BNU(K) 
!        KAPJ(K) = KAPJ(K)+ W* CAP(K)        *JNUL(K)
!        KAPF(K) = KAPF(K)+ W*(CAP(K)+SIG(K))*FNUL(K)
!     END DO

     BLK(I) = 1-(FNU(1)/FNUC(1))
     FNUE(I) = FNU(1)


  END DO

  !  end of frequency loop
  !***********************************




!  DO K = 1,ND                                 
     GRAD= (TAPM__PI/TAPP__C/100.) / KAPPA * KAPF
     SIG = SIGM
!  END DO

  !! deallocate arrays

  DEALLOCATE(BNU)
  DEALLOCATE(SNU)    
  DEALLOCATE(JNU)    
  DEALLOCATE(FNU)    
  DEALLOCATE(FNUC)   
  DEALLOCATE(JNUC)   
  DEALLOCATE(FNUL)   
  DEALLOCATE(JNUL)   
  DEALLOCATE(CAC)




CONTAINS


  SUBROUTINE FLX_FEAUT (FLX_NU,JNU)

    ! This subroutine provides an interface to the standard rtf_feaut 
    ! subroutine. It is required because this version of flxos uses a
    ! "real(kind=qq)" to define the variable precision, whilst "rtf" 
    ! assumes double precision throughout. Its a bit of a stopgap solution
    ! and can be improved -- but not by rewriting the rtf and nr libraries !!

    !  sb arguments
    REAL(KIND=QQ)               :: FLX_NU
    REAL(KIND=QQ), DIMENSION(*) :: JNU

    !  local copies for interface to rtf

    !      INTEGER    nd                              ! declare atmosphere structure
    REAL*8                 :: NUZERO
    REAL*8, DIMENSION(100) :: TAUZ, TT, PE, OPZ

    REAL*8                 :: FNU               ! declare opacities at freq
    REAL*8, DIMENSION(100) :: OPC, OPS, OPL     ! for which solution reqd.

    REAL*8, DIMENSION(100) :: RNU, A, PHI       ! function frequency structure

    REAL*8, DIMENSION(100) :: SS, JJ, TAUNU                    ! output structures

    REAL*8, DIMENSION(10)  :: XNU = 0.
    REAL*8, DIMENSION(10)  :: XA = (/1.0,(0.0,i=2,10)/)
    REAL*8, DIMENSION(10)  :: XPHI = (/1.0,(0.0,i=2,10)/)
    REAL*8, DIMENSION(16)  :: XMU = 0.
    REAL*8, DIMENSION(16)  :: XB = 0.

    INTEGER :: NM = 4

    !   copy into local arrays for passing to rtf

    NUZERO      = NU_REFWL
    TAUZ(1:ND)  = TAU(1:ND)
    TT(1:ND)    = TEMP(1:ND)
    PE(1:ND)    = P_E(1:ND)
    OPZ(1:ND)   = KAPPA(1:ND)
    FNU         = FLX_NU
    OPC(1:ND)   = CAC(1:ND)
    OPS(1:ND)   = SIG(1:ND)
    OPL(1:ND)   = KAPPA_NU(1:ND)


    CALL RTF_FEAUT ( NUZERO, TAUZ, TT, PE, OPZ, ND, &
         FNU, OPC, OPS, OPL, 1, &
         XNU, XA, XPHI, 1, &
         XMU, XB, NM, &
         TAUNU, SS, JJ )

    !   copy back local arrays for passing to rtf


    TAU_NU(1:ND) = TAUNU(1:ND)
    SNU(1:ND) = SS(1:ND)
    JNU(1:ND) = JJ(1:ND)

  END SUBROUTINE FLX_FEAUT



  SUBROUTINE FLX_PHI  ( TAU_NU, K, FNU )

    ! This subroutine provides an interface to the standard rtf_phi
    ! subroutine. It is required because this version of flxos uses a
    ! "real(kind=qq)" to define the variable precision, whilst "rtf" 
    ! assumes double precision throughout. 

    !  sb arguments
    REAL(KIND=QQ), DIMENSION(*) :: TAU_NU
    INTEGER        :: K
    REAL(KIND=QQ)  :: FNU

    !  local definitions

    !    Import
    DOUBLE PRECISION T(100)                 ! t
    DOUBLE PRECISION F(100)                 ! f(t)
    INTEGER N                             ! dimension of f(t)
    DOUBLE PRECISION TAU                  ! PHI evaluation 
    !    Export
    DOUBLE PRECISION PHI                  ! PHI integral

    ! Convert variable types

    T(1:ND) = TAU_NU(1:ND)
    F(1:ND) = SNU(1:ND)
    TAU     = TAU_NU(K)

    CALL RTF_PHI ( T, F, ND, TAU, PHI  )

    FNU = PHI

  END SUBROUTINE FLX_PHI


END SUBROUTINE FLXOS
