SUBROUTINE STN_INALPHA

  !+

  !! stn_inalpha sets up the elemental abundances 
  !! 
  !! The main programme currently reads in H, and He as number fractions
  !! and Li, Be, B, C, N, O, Mg, Al, Si, S, Ca, and Fe as scaled abundances.
  !! Conversion to scaled abundnaces must be done by mass fraction, since
  !! number fractions with a standard solar H+He composition do not conserve
  !! numbers when using non-standard H+He mixtures.
  !!
  !! In the future it woul dbe preferable to have a more flexible means of
  !! controlling the input composition

  !! 2020 July 2
  !! If inputs are 0 for AC, AN, ... we get problems
  !! Need to add eror trapping, but preferably overhaul the
  !! entire input system.

  !-

  USE STN_PARAMETERS
  USE STN_COMPOSITION
  USE OP_GLOBAL
  USE TAP_PAR

  IMPLICIT NONE

  !! elem: character identifiers for elements 
  !! atw : mean atomic masses
  !! epssol : solar abundances (log rel. to H=12.00)
  !! alpsol : relative solar abundances by number
  !! betsol : relative solar abundances by mass



  ! other local variables
  REAL :: SUMALP
  REAL :: MUSOL, MUMET  !! mean atomic weights
  REAL :: ANSOL !! normalisation constant for the Sun
  REAL :: ANMET !! normalisation constant for the star
  REAL :: ANORM !! normalisation constant ANSOL - ANMET
  INTEGER :: I,J
  INTEGER :: IZ, STATUS=0


  ! external functions
  DOUBLE PRECISION :: TAP_ABSOL
  DOUBLE PRECISION :: TAP_EMASS

  !.

  ! Fetch solar composition in mumber and mass fractions
  ANSOL  = ALOG10( SUM (ATW*ALPSOL) )


  ! Check that sum of abundances (H+He+C+N) approx = 1.00
  !! SUMALP = ALPMET(1) + ALPMET(2) + ALPMET(6)*ALPSOL(6) + ALPMET(7)*ALPSOL(7)
  SUMALP = ALPMET(1) + ALPMET(2) + ALPMET(6) + ALPMET(7)
  IF (ABS(SUMALP - 1.0) > 0.02) THEN
     WRITE(*,*) 'STN_INALPHA *Error* sum of abundances not equal to 1'
     WRITE(*,*) 'H,He,C,N: ',ALPMET(1:2),ALPMET(6:7)                        !! *ALPSOL(6:7)
     STOP 
  ENDIF


  ! Li, Be, B, O, Mg, Al, Si, S, Ca and Fe are given relative to "solar" on input
  ! Propagate these values to other elements

  !! This should only be implemented if ALPMET has not already been set to a
  !! nodefault value. 
  
  WHERE (ALPMET(3:5)   < 0.0001) ALPMET(3:5)   = ALPMET(12) !! scaled to Mg
  WHERE (ALPMET(9:11)  < 0.0001) ALPMET(9:11)  = ALPMET(12) !! scaled to Mg
  IF    (ALPMET(15)    < 0.0001) ALPMET(15)    = ALPMET(16) !! scaled to S
  WHERE (ALPMET(17:25) < 0.0001) ALPMET(17:25) = ALPMET(20) !! scaled to Ca
  WHERE (ALPMET(27:NZ) < 0.0001) ALPMET(27:NZ) = ALPMET(26) !! scaled to Fe

  ! Set the Z>2 abundances to scaled solar values by mass fraction
  BETMET        = ALPMET * BETSOL
  ! Set the H, He, C and N abundances directly by number fraction
  BETMET(1:2)   = ALPMET(1:2)*ATW(1:2)
  BETMET(6:7)   = ALPMET(6:7)*ATW(6:7)


  ! Normalise 
  CALL NORMALISE (BETMET)

  ! and convert to abundance by number
  ALPMET = BETMET / ATW

  ! and normalize everything again
  CALL NORMALISE (ALPMET)
  ANMET  = ALOG10( SUM (ATW*ALPMET) )

  ! compute epsilons for star  !! possibly not correct at present
  ANORM = 12.0 + ANSOL - ANMET
  EPSMET = ALOG10(ALPMET) + ANORM

  ! check mean atomic weights
  MUSOL = SUM ( ATW*BETSOL ) / SUM ( BETSOL )
  MUMET = SUM ( ATW*BETMET ) / SUM ( BETMET )

  ! report adopted composition clearly to user
  WRITE(*,*) 'ELEMENTAL ABUNDANCES'
  WRITE(*,*)
  WRITE(*,'(3(A8,F7.3))') ' ANSOL ',ANSOL,' ANMET ',ANMET,' ANORM ',ANORM
  WRITE(*,'(3(A8,F7.3))') ' MUSOL ',MUSOL,' MUMET ',MUMET
  WRITE(*,*) 
  WRITE(*,*) '    ELEMENT        Z  alpha     beta      eps      alpha_sun beta_sun  eps_sun   beta/beta_sun'
  WRITE(*,'(A18,I3,1X,1P,2E10.3,0P,F7.3,2x,1P,2E10.3,0P,F7.3,2x,F7.3)') &
       & (ELEM(I),I,ALPMET(I),BETMET(I),EPSMET(I),ALPSOL(I),BETSOL(I),EPSSOL(I),BETMET(I)/BETSOL(I),I=1,40)
  WRITE(*,*)

  !!  export composition to stn and opacity globals...
  ALPHA(1:56,ND) = ALPMET(1:56)
  DO I=1,OP_MAXNA
     OP_ALPNN(I,1:ND) = ALPMET(I)
  ENDDO


CONTAINS

  SUBROUTINE NORMALISE ( ARRAY ) 

    REAL, DIMENSION(:) :: ARRAY
    REAL               :: SUMA

    SUMA = SUM ( ARRAY )
    ARRAY = ARRAY / SUMA

  END SUBROUTINE NORMALISE

END SUBROUTINE STN_INALPHA
