SUBROUTINE STN_PRESS (JD,TT,P,PEL)

  !+
  !
  !  Program:
  !     PRESS
  !
  !  Purpose:
  !     To evaluate electron pressure by iteration of ionisation equations for
  !     H, He, C and N
  !
  !  Input arguments:
  !     JD      Current depth point
  !     TT      Temperature (K)
  !     P       Gas pressure (dyn / cm^2)
  !
  !  Output arguments:
  !     PEL     Electron pressure (dyn / cm^2)
  !
  !-

  USE STN_PARAMETERS, ONLY : TEFF
  USE OP_GLOBAL

  IMPLICIT NONE

  !  Subroutine arguments (Given).
  INTEGER :: JD   ! Current depth point
  REAL ::  TT     ! Temperature (K)
  REAL ::  P      ! Gas pressure (dyn / cm^2)

  !  Subroutine arguments (Returned).
  REAL :: PEL    ! Electron pressure (dyn / cm^2)

  !  External function declaration
  REAL, EXTERNAL :: U

  !  Local variables.
  !! Need to state the purpose of all these variables

  INTEGER :: I,J     

  REAL, DIMENSION (4,3) :: F
  REAL, DIMENSION (7,6) :: XM
  REAL, DIMENSION (6) :: FX 

  ! Atomic numbers for alpmet 
  INTEGER, DIMENSION(7) :: ZZ = (/8,12,13,14,16,20,26/)

  ! Part. functions for alpmet
  REAL, DIMENSION (42) :: ZU = &
       & (/9.,4.,9.,6.,1.,2.,1.,2.,1.,&
       &   6.,9.,4.,6.,1.,2.,1.,6.,9.,&
       &   9.,6.,1.,2.,1.,6.,9.,4.,9.,&
       &   6.,1.,2.,1.,2.,1.,6.,9.,4.,&
       &  25.,30.,25.,6.,7.,6./)

  REAL :: SAHA, BKT 
  REAL :: FH1, FHM, FHE1, FHE2, FHEM
  REAL :: X0, FC1, PEL0
  INTEGER :: KEL
  REAL :: PM, SUM 

  REAL :: ZH1, ZHE1, ZHE2

  REAL :: cpu0, cpu1
  CHARACTER*10 :: date, time

  !  Local constants.
  REAL, PARAMETER :: SAHA0 = 0.666698474 
  REAL, PARAMETER :: BK = 8.6167E-05 


  !.

  !  write(*,*) 'st_press',jd,tt
  !     call date_and_time(date,time)
  !     call cpu_time(cpu0)
  !     write(*,*) 'elapsed cpu',cpu0,' s',time

  !  Evaluate T-dependent quantities
  SAHA = SAHA0*TT**2.5
  BKT  = BK*TT
  FH1  = SAHA*EXP(-13.595/BKT)
  FHM  = SAHA*EXP(-0.75/BKT)
  FHE1 = SAHA*EXP(-24.581/BKT)
  FHE2 = SAHA*EXP(-54.403/BKT)
  FHEM = SAHA*EXP(+0.075/BKT)

  !  Starting estimate for electron pressure
  IF (TEFF >= 12000.)  THEN
     F(1,1) = FHE1*2.
     X0 = SQRT(F(1,1)/(P+F(1,1)))
  ELSE
     FC1 = SAHA*EXP(-11.256/BKT)
     F(3,1) = FC1*6./9.
     X0 = SQRT(F(3,1)/(P+F(3,1)))
  ENDIF
  PEL0 = P*X0/(1.+X0)


  !------------------------
  !
  !   Iteration for electron pressure

  DO KEL=1,30
     PEL = PEL0

     !    Hydrogen
     OP_XUU(1,1,JD) = U (1,1,TT,PEL)
     IF (OP_ALPNN(1,JD) > 0.) THEN
        F(1,1) = FH1/OP_XUU(1,1,JD)
        F(1,2) = FHM*OP_XUU(1,1,JD)

        OP_XION(2,1,JD) = F(1,1)*F(1,2)/(PEL**2+PEL*F(1,2)+F(1,1)*F(1,2))
        OP_XION(3,1,JD) = (1.-OP_XION(2,1,JD))*PEL/(PEL+F(1,2))
        OP_XION(1,1,JD) = 1 - OP_XION(2,1,JD) - OP_XION(3,1,JD)

     ENDIF

     !    Helium
     OP_XUU(1,2,JD) = U (2,1,TT,PEL)
     OP_XUU(2,2,JD) = U (2,2,TT,PEL)
     IF (OP_ALPNN(2,JD) > 0.) THEN
        F(2,1)  = FHE1*OP_XUU(2,2,JD)/OP_XUU(1,2,JD)
        F(2,2)  = FHE2/OP_XUU(2,2,JD)
        F(2,3)  = FHEM*OP_XUU(1,2,JD)/6.

        OP_XION(2,2,JD) = PEL*F(2,1)*F(2,3)/(PEL**3+PEL**2*F(2,3)+PEL*&
             F(2,1)*F(2,3)+F(2,1)*F(2,2)*F(2,3))
        OP_XION(3,2,JD) = OP_XION(2,2,JD)*F(2,2)/PEL
        OP_XION(4,2,JD) = OP_XION(2,2,JD)*PEL**2/(F(2,1)*F(2,3))
        OP_XION(1,2,JD) = OP_XION(2,2,JD)*PEL/F(2,1)

     ENDIF

     !! loop over remaining atoms:

     DO J = 3, OP_MAXNA  !atoms
        IF (OP_ALPNN(J,JD) > 0.0) THEN
           DO I = 1, OP_MAXNI !loop over ions
              OP_XUU(I,J,JD) = U (J,I,TT,PEL)
           END DO
           CALL OP_ION (j,TT,PEL,OP_XUU(1:OP_MAXNI,J,JD),OP_XION(1:OP_MAXNI,J,JD)) 
        ENDIF
     ENDDO

     SUM = 0.0

     !    Total
     SUM = OP_ALPNN(1,JD)*(OP_XION(2,1,JD)-OP_XION(3,1,JD)) + &
          OP_ALPNN(2,JD)*(OP_XION(2,2,JD)+2*OP_XION(3,2,JD)-OP_XION(4,2,JD)) + &
          OP_ALPNN(3,JD)*(OP_XION(2,3,JD)+2*OP_XION(3,3,JD)+3*OP_XION(4,3,JD)) + &
          OP_ALPNN(4,JD)*(OP_XION(2,4,JD)+2*OP_XION(3,4,JD)+3*OP_XION(4,4,JD)) + &
          OP_ALPNN(5,JD)*(OP_XION(2,5,JD)+2*OP_XION(3,5,JD)+3*OP_XION(4,5,JD)) + &
          OP_ALPNN(6,JD)*(OP_XION(2,6,JD)+2*OP_XION(3,6,JD)+3*OP_XION(4,6,JD)+ &
          4*OP_XION(5,6,JD)+5*OP_XION(6,6,JD)) +&
          OP_ALPNN(7,JD)*(OP_XION(2,7,JD)+2*OP_XION(3,7,JD)+3*OP_XION(4,7,JD)+ &
          4*OP_XION(5,7,JD)+5*OP_XION(6,7,JD))

     !    Other elements
     PM = 0.0

     DO I = 8, OP_MAXNA
        PM = PM + OP_ALPNN(I,JD)*(OP_XION(2,I,JD)+2*OP_XION(3,I,JD)+ &
             3*OP_XION(4,I,JD)+4*OP_XION(5,I,JD)+5*OP_XION(6,I,JD))
     ENDDO

     SUM = SUM + PM


     !    Check for convergence
     PEL = P/(1.+1./SUM)
     IF (ABS(1.-PEL0/PEL) >= 0.0002) THEN
        PEL0 = (PEL0+PEL)/2.
        IF (KEL == 30) THEN
           WRITE (*,18)  PEL0,PEL
18         FORMAT(2X,2E14.5,2X,'Electron pressure not converging')
           WRITE(*,*) SUM, PM, P, P*X0/(1.+X0) 
        ENDIF
     ELSE
        !          write(*,*) 'Electron pressure converged'
        EXIT 
     ENDIF

     !     call date_and_time(date,time)
     !     call cpu_time(cpu1)
     !     write(*,*) 'elapsed cpu',cpu0-cpu1,' s',time
     !     write(*,*) kel, pel

  ENDDO
  !
  !-------------------------

  !  Find populations for individual states.
  CALL OP_POP (JD,TT)

  !  write(*,*) 'st_press',jd,tt
  !  write(*,*) op_xuu (1:3,1:12,jd)
  !  write(*,*) op_xion(1:3,1:12,jd)

END SUBROUTINE STN_PRESS
