DOUBLE PRECISION FUNCTION TAP_ABNUC ( ZELEM, AELEM )

!+
!  Name:
!     TAP_ABNUC

!  Purpose:
!=    Return the solar system abundance for a given nuclide.

!  Language:
!     Fortran 90

!  Invocation:
!     RESULT = TAP_ABNUC( ZELEM, AELEM )

!  Description:
!     The recommended values for solar system nuclide abundance,
!     relative to unit element abundance, are returned. In general, the
!     terrestrial representative isotopic compositions recommended
!     by IUPAC (Holden et al. 1984) are given, as cited by Anders and
!     Grevesse (1989).
! 
!     The IUPAC compositions represent "the chemicals and/or materials 
!     most commonly encountered in the laboratory" and NOT NECESSARILY 
!     "the most abundant natural material". For light elements (Li, B, 
!     C, O) and elements with a small radiogenic component (Sr, Nd, Hf, 
!     Os) the differences are small, see Holden et al. (1984), and
!     references cited therein, for details.  For elements that are
!     strongly depleted in the Earth (H, N, noble gases) or are largely
!     radiogenic (Pb), solar system values have been used.

!  Algorithm:
!     The nuclide abundances are stored and returned as percentages for
!     each element.
!
!     The nuclide abundances are stored in a REAL array with an INTEGER
!     pointer array indicating the end of the previous element's data.
!     The data in this array are ordered sequentially by element and
!     nuclide for each element.

!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number (Z) for which the abundance is required.
!     AELEM = INTEGER (Given)
!        The atomic (or nuclide) mass number (A) for which the nuclide
!        abundance is required.

!  Returned Value:
!     TAP_ABNUC = DOUBLE PRECISION
!        The percentage relative nuclide abundance.

!  References:
!     Anders E. & Grevesse N. 1989. Geochim. Cosmochim. Acta 53, 197.
!     Holden N.E., Martin R.L., & Barnes I.L. 1984. Pure Appl. Chem. 56,
!        675.

!  Authors:
!     CSJ: C.S.Jeffery (St.Andrews)
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     7-MAY-1992 (CSJ):
!        Original version.
!     13-JUL-1992 (PCTR):
!        Changed the structure of the data table to economise on space.
!     8-JAN-2002 (CSJ):
!        Convert to Fortran 90.
!     {enter_further_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-
      
!  Global Constants:
      USE  SAE_PAR          ! Standard SAE constants
      USE  TAP_PAR          ! TAP public constants

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      INTEGER :: ZELEM
      INTEGER :: AELEM

!  Local Variables:
      INTEGER :: I                  ! Loop index
      INTEGER :: NPOINT( TAP__NELEM+1 ) ! Nuclide table pointer
      INTEGER :: NUCMAS( TAP__NELEM+1 ) ! Nuclide mass offset
      INTEGER :: ISO                ! Element/nuclide identifier

      REAL :: ABUNDN( TAP__NNUCL ) ! Nuclide abundance ratios
      
!  Local Data:
!     Nuclide abundances:

!  Hydrogen, H
      DATA NPOINT( 1 ), NUCMAS( 1 ) / 0, 0 /
      DATA ( ABUNDN( I ), I = 1, 2 ) / 99.9966, 0.0034 /

!  Helium, He
      DATA NPOINT( 2 ), NUCMAS( 2 ) / 2, 2 /
      DATA ( ABUNDN( I ), I = 3, 4 ) / 0.0142, 99.9858 /

!  Lithium, Li
      DATA NPOINT( 3 ), NUCMAS( 3 ) / 4, 5 /
      DATA ( ABUNDN( I ), I = 5, 6 ) / 7.5, 92.5 /

!  Beryllium, Be
      DATA NPOINT( 4 ), NUCMAS( 4 ) / 6, 8 /
      DATA ( ABUNDN( I ), I=7, 7 ) / 100.0 /

!  Boron, B
      DATA NPOINT( 5 ), NUCMAS( 5 ) / 7, 9 /
      DATA ( ABUNDN( I ), I=8, 9 ) / 19.9, 80.1 /

!  Carbon, C
      DATA NPOINT( 6 ), NUCMAS( 6 ) / 9, 11 /
      DATA ( ABUNDN( I ), I=10, 11 ) / 98.90, 1.10 /
                       
!  Nitrogen, N
      DATA NPOINT( 7 ), NUCMAS( 7 ) / 11, 13 /
      DATA ( ABUNDN( I ), I=12, 13 ) / 99.634, 0.366 /

!  Oxygen, O
      DATA NPOINT( 8 ), NUCMAS( 8 ) / 13, 15 /
      DATA ( ABUNDN( I ), I=14, 16 ) / 99.762, 0.038, 0.200 /

!  Fluorine, F
      DATA NPOINT( 9 ), NUCMAS( 9 ) / 16, 18 /
      DATA ( ABUNDN( I ), I=17, 17 ) / 100.0 /

!  Neon, Ne
      DATA NPOINT( 10 ), NUCMAS( 10 ) / 17, 19 /
      DATA ( ABUNDN( I ), I=18, 20 ) / 92.99, 0.226, 6.79 /

!  Sodium, Na
      DATA NPOINT( 11 ), NUCMAS( 11 ) / 20, 22 /
      DATA ( ABUNDN( I ), I=21, 21 ) / 100.0 /

!  Magnesium, Mg
      DATA NPOINT( 12 ), NUCMAS( 12 ) / 21, 23 /
      DATA ( ABUNDN( I ), I=22, 24 ) / 78.99, 10.00, 11.01  /

!  Aluminium, Al
      DATA NPOINT( 13 ), NUCMAS( 13 ) / 24, 26 /
      DATA ( ABUNDN( I ), I=25, 25 ) / 100.0 / 

!  Silicon, Si
      DATA NPOINT( 14 ), NUCMAS( 14 ) / 25, 27 /
      DATA ( ABUNDN( I ), I=26, 28 ) / 92.23,  4.67,  3.10  /

!  Phosphorus, P
      DATA NPOINT( 15 ), NUCMAS( 15 ) / 28, 30 /
      DATA ( ABUNDN( I ), I=29, 29 ) / 100.0 /

!  Sulphur, S
      DATA NPOINT( 16 ), NUCMAS( 16 ) / 29, 31 /
      DATA ( ABUNDN( I ), I=30, 34 ) / 95.02, 0.75, 4.21, 0.0, 0.02 /

!  Chlorine, Cl
      DATA NPOINT( 17 ), NUCMAS( 17 ) / 34, 34 /
      DATA ( ABUNDN( I ), I=35, 37 ) / 75.77, 0.0, 24.23 /

!  Argon, Ar
      DATA NPOINT( 18 ), NUCMAS( 18 ) / 37, 35 /
      DATA ( ABUNDN( I ), I=38, 42 ) / 84.2, 0.0, 15.8, 0.0, 0.0 /

!  Potassium, K
      DATA NPOINT( 19 ), NUCMAS( 19 ) / 42, 38 /
      DATA ( ABUNDN( I ), I=43, 45 ) / 93.2581, 0.01167, 6.7302 /

!  Calcium, Ca
      DATA NPOINT( 20 ), NUCMAS( 20 ) / 45, 39 /
      DATA ( ABUNDN( I ), I=46, 54 ) / 96.941, 0.0, 0.647, 0.135, &
                                       2.0826, 0.0, 0.004, 0.0,   &
                                       0.187 /

!  Scandium, Sc
      DATA NPOINT( 21 ), NUCMAS( 21 ) / 54, 44 /
      DATA ( ABUNDN( I ), I=55, 55 ) / 100.0 /

!  Titanium, Ti
      DATA NPOINT( 22 ), NUCMAS( 22 ) / 55, 45 /
      DATA ( ABUNDN( I ), I=56, 60 ) / 8.0, 7.3, 73.8, 5.5, 5.4 /

!  Vanadium, V
      DATA NPOINT( 23 ), NUCMAS( 23 ) / 60, 49 /
      DATA ( ABUNDN( I ), I=61, 62 ) / 0.25, 99.75 /

!  Chromium, Cr
      DATA NPOINT( 24 ), NUCMAS( 24 ) / 62, 49 /
      DATA ( ABUNDN( I ), I=63, 67 ) / 4.345, 0.0, 83.789, 9.501, &
                                       2.365 /

!  Manganese, Mn
      DATA NPOINT( 25 ), NUCMAS( 25 ) / 67, 54 /
      DATA ( ABUNDN( I ), I=68, 68 ) / 100.0 /

!  Iron, Fe
      DATA NPOINT( 26 ), NUCMAS( 26 ) / 68, 53 /
      DATA ( ABUNDN( I ), I=69, 73 ) / 5.8, 0.0, 91.72, 2.2, 0.28 /

!  Cobalt, Co
      DATA NPOINT( 27 ), NUCMAS( 27 ) / 73, 58 /
      DATA ( ABUNDN( I ), I=74, 74 ) / 100.0 /

!  Nickel, Ni
      DATA NPOINT( 28 ), NUCMAS( 28 ) / 74, 57 /
      DATA ( ABUNDN( I ), I=75, 81 ) / 68.27, 0.0, 26.10, 1.13, 3.59, &
                                        0.0,  0.91 /

!  Copper, Cu
      DATA NPOINT( 29 ), NUCMAS( 29 ) / 81, 62 /
      DATA ( ABUNDN( I ), I=82, 84 ) / 69.17, 0.0, 30.83 /

!  Zinc, Zn
      DATA NPOINT( 30 ), NUCMAS( 30 ) / 84, 63 /
      DATA ( ABUNDN( I ), I=85, 91 ) / 48.63, 0.0, 27.90, 4.10, 18.75, &
                                        0.0,  0.62 /

!  Gallium, Ga
      DATA NPOINT( 31 ), NUCMAS( 31 ) / 91, 68 /
      DATA ( ABUNDN( I ), I=92, 94 ) / 60.108, 0.0, 39.892 /

!  Germanium, Ge
      DATA NPOINT( 32 ), NUCMAS( 32 ) / 94, 69 /
      DATA ( ABUNDN( I ), I=95, 101 ) / 20.5, 0.0, 27.4, 7.8, 36.5, 0.0, &
                                         7.8 /

!  Arsenic, As
      DATA NPOINT( 33 ), NUCMAS( 33 ) / 101, 74 /
      DATA ( ABUNDN( I ), I=102, 102 ) / 100.0 /

!  Selenium, Se
      DATA NPOINT( 34 ), NUCMAS( 34 ) / 102, 73 /
      DATA ( ABUNDN( I ), I=103, 111 ) / 0.88, 0.0, 9.0, 7.6, 23.6, &
                                         0.0, 49.7, 0.0, 9.2 /

!  Bromine, Br
      DATA NPOINT( 35 ), NUCMAS( 35 ) / 111, 78 /
      DATA ( ABUNDN( I ), I=112, 114 ) / 50.69, 0.0, 49.31 /

!  Krypton, Kr
      DATA NPOINT( 36 ), NUCMAS( 36 ) / 114, 77 /
      DATA ( ABUNDN( I ), I=115, 123 ) / 0.339, 0.0,  2.22, 0.0, 11.45, & 
                                        11.47, 57.11, 0.0, 17.42 /

!  Rubidium, Rb
      DATA NPOINT( 37 ), NUCMAS( 37 ) / 123, 84 /
      DATA ( ABUNDN( I ), I=124, 126 ) / 72.165, 0.0, 27.835 /

!  Strontium, Sr
      DATA NPOINT( 38 ), NUCMAS( 38 ) / 126, 83 /
      DATA ( ABUNDN( I ), I=127, 131 ) / 0.56, 0.0, 9.86, 7.00, 82.58 /

!  Yttrium, Y
      DATA NPOINT( 39 ), NUCMAS( 39 ) / 131, 88 /
      DATA ( ABUNDN( I ), I=132, 132 ) / 100.0 /

!  Zirconium, Zr
      DATA NPOINT( 40 ), NUCMAS( 40 ) / 132, 89 /
      DATA ( ABUNDN( I ), I=133, 139 ) / 51.45, 11.22, 17.15, 0.0, &
                                         17.38,  0.0,   2.80 /

!  Niobium, Nb
      DATA NPOINT( 41 ), NUCMAS( 41 ) / 139, 92 /
      DATA ( ABUNDN( I ), I=140, 140 ) / 100.0 /

!  Molybdenum, Mo
      DATA NPOINT( 42 ), NUCMAS( 42 ) / 140, 91 /
      DATA ( ABUNDN( I ), I=141, 149 ) / 14.84, 0.0,   9.25, 15.92, &
                                         16.68, 9.55, 24.13,  0.0, &
                                          9.63 /

!  Technetium, Tc
!  Element has no stable nuclides.
      DATA NPOINT( 43 ), NUCMAS( 43 ) / 149, 149 /

!  Ruthenium, Ru
      DATA NPOINT( 44 ), NUCMAS( 44 ) / 149, 95 /
      DATA ( ABUNDN( I ), I=150, 158 ) / 5.52, 0.0, 1.88, 12.7, 12.6, &
                                        17.0, 31.6, 0.0,  18.7 /

!  Rhodium, Rh
      DATA NPOINT( 45 ), NUCMAS( 45 ) / 158, 102 /
      DATA ( ABUNDN( I ), I=159, 159 ) / 100.0 /

!  Palladium, Pd
      DATA NPOINT( 46 ), NUCMAS( 46 ) / 159, 101 /
      DATA ( ABUNDN( I ), I=160, 168 ) / 1.020, 0.0, 11.14, 22.33, &
                                        27.33,  0.0, 26.46,  0.0, &
                                        11.72 /

!  Silver, Ag
      DATA NPOINT( 47 ), NUCMAS( 47 ) / 168, 106 /
      DATA ( ABUNDN( I ), I=169, 171 ) / 51.839, 0.0, 48.161 /

!  Cadmium, Cd
      DATA NPOINT( 48 ), NUCMAS( 48 ) / 171, 105 /
      DATA ( ABUNDN( I ), I=172, 182 ) / 1.25,  0.0,   0.89,  0.0, &
                                        12.49, 12.80, 24.13, 12.22, &
                                        28.73,  0.0,   7.49 /

!  Indium, In
      DATA NPOINT( 49 ), NUCMAS( 49 ) / 182, 112 /
      DATA ( ABUNDN( I ), I=183, 185 ) / 4.3, 0.0, 95.7 /

!  Tin, Sn
      DATA NPOINT( 50 ), NUCMAS( 50 ) / 185, 111 /
      DATA ( ABUNDN( I ), I=186, 198 ) / 0.973, 0.0,    0.652, 0.359, &
                                        14.532, 7.675, 24.218, 8.583, &
                                        32.590, 0.0,    4.629, 0.0, &
                                         5.789 /

!  Antimony, Sb
      DATA NPOINT( 51 ), NUCMAS( 51 ) / 198, 120 /
      DATA ( ABUNDN( I ), I=199, 201 ) / 57.3, 0.0, 42.7 /

!  Tellurium, Te
      DATA NPOINT( 52 ), NUCMAS( 52 ) / 201, 119 /
      DATA ( ABUNDN( I ), I=202, 212 ) / 0.09,  0.0,  2.57, 0.89, 4.76, & 
                                         7.10, 18.89, 0.0, 31.73, 0.0, &
                                        33.97 /

!  Iodine, I
      DATA NPOINT( 53 ), NUCMAS( 53 ) / 212, 126 /
      DATA ( ABUNDN( I ), I=213, 213 ) / 100.0 /

!  Xenon, Xe
      DATA NPOINT( 54 ), NUCMAS( 54 ) / 213, 123 /
      DATA ( ABUNDN( I ), I=214, 226 ) / 0.121, 0.0,  0.108,  0.0, &
                                         2.19, 27.34, 4.35,  21.69, &
                                        26.50,  0.0,  9.76,   0.0, &
                                         7.94 /

!  Caesium, Cs
      DATA NPOINT( 55 ), NUCMAS( 55 ) / 226, 132 /
      DATA ( ABUNDN( I ), I=227, 227 ) / 100.0 / 

!  Barium, Ba
      DATA NPOINT( 56 ), NUCMAS( 56 ) / 227, 129 /
      DATA ( ABUNDN( I ), I=228, 236 ) / 0.106, 0.0,    0.101,  0.0, &
                                         2.417, 6.592,  7.854, 11.23, &
                                        71.70 /

!  Lanthanum, La
      DATA NPOINT( 57 ), NUCMAS( 57 ) / 236, 137 /
      DATA ( ABUNDN( I ), I=237, 238 ) / 0.089, 99.911 /

!  Cerium, Ce
      DATA NPOINT( 58 ), NUCMAS( 58 ) / 238, 135 /
      DATA ( ABUNDN( I ), I=239, 245 ) / 0.19, 0.0, 0.25, 0.0, 88.48, &
                                         0.0, 11.08 /

!  Praseodymium, Pr
      DATA NPOINT( 59 ), NUCMAS( 59 ) / 245, 140 /
      DATA ( ABUNDN( I ), I=246, 246 ) / 100.0 /

!  Neodymium, Nd
      DATA NPOINT( 60 ), NUCMAS( 60 ) / 246, 141 /
      DATA ( ABUNDN( I ), I=247, 255 ) / 27.13, 12.18, 23.80, 8.30,  &
                                         17.19,  0.0,   5.76, 0.0, &
                                          5.64 /

!  Promethium, Pm
!  Element has no stable nuclides.
      DATA NPOINT( 61 ), NUCMAS( 61 ) / 255, 255 /

!  Samarium, Sm
      DATA NPOINT( 62 ), NUCMAS( 62 ) / 255, 143 /
      DATA ( ABUNDN( I ), I=256, 266 ) / 3.1, 0.0, 0.0, 15.0, 11.3, &
                                        13.8, 7.4, 0.0, 26.7,  0.0, &
                                        22.7 /

!  Europium, Eu
      DATA NPOINT( 63 ), NUCMAS( 63 ) / 266, 150 /
      DATA ( ABUNDN( I ), I=267, 269 ) / 47.8, 0.0, 52.2 /

!  Gadolinium, Gd
      DATA NPOINT( 64 ), NUCMAS( 64 ) / 269, 151 /
      DATA ( ABUNDN( I ), I=270, 278 ) / 0.20,  0.0,   2.18, 14.80, &
                                        20.47, 15.65, 24.84,  0.0, &
                                        21.86 /

!  Terbium, Tb
      DATA NPOINT( 65 ), NUCMAS( 65 ) / 278, 158 /
      DATA ( ABUNDN( I ), I=279, 279 ) / 100.0 /

!  Dysprosium, Dy
      DATA NPOINT( 66 ), NUCMAS( 66 ) / 279, 155 /
      DATA ( ABUNDN( I ), I=280, 288 ) / 0.056, 0.0,   0.096, 0.0, 2.34, &
                                        18.91, 25.51, 24.90, 28.19 /

!  Holmium, Ho
      DATA NPOINT( 67 ), NUCMAS( 67 ) / 288, 164 /
      DATA ( ABUNDN( I ), I=289, 289 ) / 100.0 /

!  Erbium, Er
      DATA NPOINT( 68 ), NUCMAS( 68 ) / 289, 161 /
      DATA ( ABUNDN( I ), I=290, 298 ) / 0.14,  0.0, 1.61, 0.0, 33.6, &
                                        22.95, 26.8, 0.0, 14.9 /

!  Thulium, Tm
      DATA NPOINT( 69 ), NUCMAS( 69 ) / 298, 168 /
      DATA ( ABUNDN( I ), I=299, 299 ) / 100.0 /

!  Ytterbium, Yb
      DATA NPOINT( 70 ), NUCMAS( 70 ) / 299, 167 /
      DATA ( ABUNDN( I ), I=300, 308 ) / 0.13,  0.0, 3.05, 14.3, 21.9, &
                                        16.12, 31.8, 0.0,  12.7 /

!  Lutetium, Lu
      DATA NPOINT( 71 ), NUCMAS( 71 ) / 308, 174 /
      DATA ( ABUNDN( I ), I=309, 310 ) / 97.41, 2.59 /

!  Hafnium, Hf
      DATA NPOINT( 72 ), NUCMAS( 72 ) / 310, 173 /
      DATA ( ABUNDN( I ), I=311, 317 ) / 0.162,  0.0,    5.206, 18.606, &
                                        27.297, 13.629, 35.100 /

!  Tantalum, Ta
      DATA NPOINT( 73 ), NUCMAS( 73 ) / 317, 179 /
      DATA ( ABUNDN( I ), I=318, 319 ) / 0.012, 99.988 /

!  Tungsten, W
      DATA NPOINT( 74 ), NUCMAS( 74 ) / 319, 179 /
      DATA ( ABUNDN( I ), I=320, 326 ) / 0.13, 0.0, 26.3, 14.3, 30.67, &
                                         0.0, 28.6 /

!  Rhenium, Re
      DATA NPOINT( 75 ), NUCMAS( 75 ) / 326, 184 /
      DATA ( ABUNDN( I ), I=327, 329 ) / 37.4, 0.0, 62.6 /

!  Osmium, Os
      DATA NPOINT( 76 ), NUCMAS( 76 ) / 329, 183 /
      DATA ( ABUNDN( I ), I=330, 338 ) / 0.018, 0.0, 1.58, 1.6, 13.3, &
                                        16.1,  26.4, 0.0, 41.0 /

!  Iridium, Ir
      DATA NPOINT( 77 ), NUCMAS( 77 ) / 338, 190 /
      DATA ( ABUNDN( I ), I=339, 341 ) / 37.3, 0.0, 62.7 /

!  Platinum, Pt
      DATA NPOINT( 78 ), NUCMAS( 78 ) / 341, 189 /
      DATA ( ABUNDN( I ), I=342, 350 ) / 0.0127, 0.0, 0.78, 0.0, 32.9, &
                                        33.8,   25.2, 0.0,  7.19 /

!  Gold, Au
      DATA NPOINT( 79 ), NUCMAS( 79 ) / 350, 196 /
      DATA ( ABUNDN( I ), I=351, 351 ) / 100.0 /

!  Mercury, Hg
      DATA NPOINT( 80 ), NUCMAS( 80 ) / 351, 195 /
      DATA ( ABUNDN( I ), I=352, 360 ) / 0.14,  0.0,  10.02, 16.84, &
                                        23.13, 13.22, 29.80,  0.0, &
                                         6.85 /

!  Thallium, Tl
      DATA NPOINT( 81 ), NUCMAS( 81 ) / 360, 202 /
      DATA ( ABUNDN( I ), I=361, 363 ) / 29.524, 0.0, 70.476 /

!  Lead, Pb
      DATA NPOINT( 82 ), NUCMAS( 82 ) / 363, 203 /
      DATA ( ABUNDN( I ), I=364, 368 ) / 1.94, 0.0, 19.12, 20.62, &
                                        58.31 /

!  Bismuth, Bi
      DATA NPOINT( 83 ), NUCMAS( 83 ) / 368, 208 /
      DATA ( ABUNDN( I ), I=369, 369 ) / 100.0 /

!  Polonium, Po
!  Element has no stable nuclides.
      DATA NPOINT( 84 ), NUCMAS( 84 ) / 369, 369 /

!  Astatine, At
!  Element has no stable nuclides.
      DATA NPOINT( 85 ), NUCMAS( 85 ) / 369, 369 /

!  Radon, Rn
!  Element has no stable nuclides.
      DATA NPOINT( 86 ), NUCMAS( 86 ) / 369, 369 /

!  Francium, Fr
!  Element has no stable nuclides.
      DATA NPOINT( 87 ), NUCMAS( 87 ) / 369, 369 /

!  Radium, Ra
!  Element has no stable nuclides.
      DATA NPOINT( 88 ), NUCMAS( 88 ) / 369, 369 /

!  Actinium, Ac
!  Element has no stable nuclides.
      DATA NPOINT( 89 ), NUCMAS( 89 ) / 369, 369 /

!  Thorium, Th
!  Element has no stable nuclides.
      DATA NPOINT( 90 ), NUCMAS( 90 ) / 369, 231 /
      DATA ( ABUNDN( I ), I=370, 370 ) / 100.0 /

!  Protactinium, Pa
!  Element has no stable nuclides.
      DATA NPOINT( 91 ), NUCMAS( 91 ) / 370, 370 /

!  Uranium, U
!  Element has no stable nuclides.
      DATA NPOINT( 92 ), NUCMAS( 92 ) / 370, 233 /
      DATA ( ABUNDN( I ), I=371, 375 ) / 0.0, 0.7200, 0.0, 0.0, &
                                         99.2745 /

!  End
      DATA NPOINT( 93 ), NUCMAS( 93 ) / 375, 375 /

!.

!  Check for bad atomic and mass numbers.
      IF ( ZELEM.LE.0 .OR. ZELEM.GT.TAP__NELEM  ) THEN

!     The element is not tabulated, return a NULL value.
         TAP_ABNUC = TAP__DNULL
      ELSE IF ( AELEM.LT.ZELEM ) THEN

!     This is an inappropriate mass number.
         TAP_ABNUC = TAP__DNULL
      ELSE

!     The element is tabulated, calculate the isotope pointer.
         ISO = NPOINT( ZELEM ) + AELEM - NUCMAS( ZELEM )

!     Check if the isotope identifier is out of range: i.e. either
!     the isotope is unstable and does not exist naturally, or the
!     isotope does not exist.
         IF ( ISO.GT.NPOINT( ZELEM ) &
              .AND. ISO.LE.NPOINT( ZELEM+1 ) ) THEN

!        The isotope is tabulated, return its abundance.
            TAP_ABNUC = DBLE( ABUNDN( ISO ) )
         ELSE

!        The isotope is not tabulated, return a NULL value.
            TAP_ABNUC = TAP__DNULL
         END IF
      END IF

    END FUNCTION TAP_ABNUC
