DOUBLE PRECISION FUNCTION TAP_ABSOL ( ZELEM )

!+
!  Name:
!     TAP_ABSOL

!  Purpose:
!=    Return relative solar photospheric abundance for a given element.

!  Language:
!     Starlink Fortran 77
!     Migrated to Fortran 90+

!  Invocation:
!     RESULT = TAP_ABSOL( ZELEM )

!  Description:
!     Given the atomic number of the element, the recommended value
!     of the solar photospheric element abundance relative to hydrogen
!     is returned. The abundances are based  on the
!     compilation of solar and meteorite data Asplund et al. (2009).
!
!     In general, the solar and meteorite data are in good agreement.
!     Some exceptions (e.g. Lithium) are obvious, whilst others (e.g.
!     Indium and Tungsten) are less so. The solar data will find
!     applications in stellar theory, whilst it is expected that the
!     meteorite data should be more useful for ISM studies.
!     
!     All solar and meteoritic abundances are given in logarithmic form
!     relative to the solar hydrogen abundance, which is defined to be
!     12.00.  Cases where no data are available for a given element
!     are indicated by a returned value of TAP__DNULL.

!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number (Z) for which the abundance is required.

!  Returned Value:
!     TAP_ABSOL = DOUBLE PRECISION
!        The solar photospheric abundance (logarithm with H = 12.00).

!  Accuracy:
!     See references for full details.
!
!     Solar  abundances: errors lie between +/- 0.03 and +/- 0.10 dex.
!            Exceptions:  F, Cl,  K, Ge, Rb, Rh, Ag, Cd, In, Sn, Sb, Ce,
!                        Tb, Dy, Ho, Tm, Yb, Lu,  W, Pt, Au, Tl.
!                       
!     Meteor abundances: errors lie between +/- 0.01 and +/- 0.05 dex.
!            Exceptions: F, Cl, Br, Kr, Sb, I, Xe, Au.

!  References:
!     Asplund, et al. 2009, Ann.Rev.Astron.Astrophys. 47, 471.
!     !* Indirect measurments
!     !m meteoritic abundance from Lodders et al. 2009 
!
!     Previous versions used: 
!        Anders E. & Grevesse N. 1989. Geochim. Cosmochim. Acta 53, 197.
!        Holden N.E., Martin R.L., & Barnes I.L. 1984. Pure Appl. Chem. 56,
!        675.
!     Solar Fe:
!        Holweger H., Heise C. & Kock M. 1990. Astron.Astrophys. 232,
!           510.
!     Solar C:
!        Stuerenberg S. & Holweger H. 1990. Astron.Astrophys. 237, 125.
!

!  Authors:
!     CSJ: C.S.Jeffery (St Andrews)
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     6-MAY-1992 (CSJ):
!        Original version.
!     8-JUL-1992 (PCTR):
!        Changed to a DOUBLE PRECISION function and removed the status 
!        argument.
!     17-JUL-2020 (CSJ):
!         Added meteoritic abundances for elements with out values
!         from Asplund et al. 2009.
!     13-AUG-2020 (CSJ):  
!         Changed all values to Asplund et al. (2009), using photospheric where possible.   
!     {enter_further_changes_here}

!  Bugs:
!     13-AUG-2020 (CSJ): This function neither includes nor returns errors on the
!         experimental values. They have been temporariliy added as comments, but
!         a future version should include an error array and an optional return argument. 
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE TAP_PAR            ! TAP public constants
      
!  Type Definitions:
      IMPLICIT NONE          ! No implicit typing

!  Arguments Given:
      INTEGER, INTENT(IN) :: ZELEM

!  Local Constants:
      ! A small REAL constant 
      REAL, PARAMETER :: SMALL = TAP__RNULL * TAP__RSMAL 
      
!  Local Variables:
      DOUBLE PRECISION :: ABUNDS( TAP__NELEM ) ! Solar element abundance ratios

!  Local Data:
!     Solar photospheric element abundances:  ! error  ! note

!  H
      DATA ABUNDS( 1 ) / 12.00 /

!  He
      DATA ABUNDS( 2 ) / 10.93D00 / ! 0.01  !*

!  Li
      DATA ABUNDS( 3 ) / 1.05D00 /  ! 0.10

!  Be
      DATA ABUNDS( 4 ) / 1.38D00 /  ! 0.09 

!  B
      DATA ABUNDS( 5 ) / 2.70D00 /  ! 0.20

!  C
      DATA ABUNDS( 6 ) / 8.43D00 /  ! 0.05

!  N
      DATA ABUNDS( 7 ) / 7.83D00 /  ! 0.05

!  O
      DATA ABUNDS( 8 ) / 8.69D00 /  ! 0.05

!  F
      DATA ABUNDS( 9 ) / 4.56D00 /  ! 0.30 

!  Ne
      DATA ABUNDS( 10 ) / 7.93D00 / ! 0.10  !*

!  Na
      DATA ABUNDS( 11 ) / 6.24D00 / ! 0.04

!  Mg
      DATA ABUNDS( 12 ) / 7.60D00 / ! 0.04

!  Al
      DATA ABUNDS( 13 ) / 6.45D00 / ! 0.03

!  Si
      DATA ABUNDS( 14 ) / 7.51D00 / ! 0.03

!  P
      DATA ABUNDS( 15 ) / 5.41D00 / ! 0.03

!  S
      DATA ABUNDS( 16 ) / 7.12D00 / ! 0.03

!  Cl
      DATA ABUNDS( 17 ) / 5.50D00 / ! 0.30 

!  Ar
      DATA ABUNDS( 18 ) / 6.40D00 / ! 0.13  !*

!  K
      DATA ABUNDS( 19 ) / 5.03D00 / ! 0.09

!  Ca
      DATA ABUNDS( 20 ) / 6.34D00 / ! 0.04

!  Sc
      DATA ABUNDS( 21 ) / 3.15D00 / ! 0.04

!  Ti
      DATA ABUNDS( 22 ) / 4.95D00 / ! 0.05

!  V
      DATA ABUNDS( 23 ) / 3.93D00 / ! 0.08

!  Cr
      DATA ABUNDS( 24 ) / 5.64D00 / ! 0.04

!  Mn
      DATA ABUNDS( 25 ) / 5.43D00 / ! 0.04

!  Fe
      DATA ABUNDS( 26 ) / 7.50D00 / ! 0.04

!  Co
      DATA ABUNDS( 27 ) / 4.99D00 / ! 0.07

!  Ni
      DATA ABUNDS( 28 ) / 6.22D00 / ! 0.04

!  Cu
      DATA ABUNDS( 29 ) / 4.19D00 / ! 0.04

!  Zn
      DATA ABUNDS( 30 ) / 4.56D00 / ! 0.05

!  Ga
      DATA ABUNDS( 31 ) / 3.04D00 / ! 0.09

!  Ge
      DATA ABUNDS( 32 ) / 3.65D00 / ! 0.10

!  As
      DATA ABUNDS( 33 ) / 2.33D00 / ! 0.04      !m

!  Se
      DATA ABUNDS( 34 ) / 3.34D00 / ! 0.03      !m

!  Br
      DATA ABUNDS( 35 ) / 2.54D00 / ! 0.06      !m

!  Kr
      DATA ABUNDS( 36 ) / 3.25D00 / ! 0.06      !*

!  Rb
      DATA ABUNDS( 37 ) / 2.52D00 / ! 0.10

!  Sr
      DATA ABUNDS( 38 ) / 2.87D00 / ! 0.07

!  Y
      DATA ABUNDS( 39 ) / 2.21D00 / ! 0.05

!  Zr
      DATA ABUNDS( 40 ) / 2.58D00 / ! 0.04

!  Nb
      DATA ABUNDS( 41 ) / 1.46D00 / ! 0.04

!  Mo
      DATA ABUNDS( 42 ) / 1.88D00 / ! 0.08

!  Tc
      DATA ABUNDS( 43 ) / TAP__RNULL /

!  Ru
      DATA ABUNDS( 44 ) / 1.75D00 / ! 0.08

!  Rh
      DATA ABUNDS( 45 ) / 0.91D00 / ! 0.10

!  Pd
      DATA ABUNDS( 46 ) / 1.57D00 / ! 0.10

!  Ag
      DATA ABUNDS( 47 ) / 0.94D00 / ! 0.10

!  Cd
      DATA ABUNDS( 48 ) / 1.71D00 / ! 0.03      !m

!  In
      DATA ABUNDS( 49 ) / 0.80D00 / ! 0.20

!  Sn
      DATA ABUNDS( 50 ) / 2.04D00 / ! 0.10

!  Sb
      DATA ABUNDS( 51 ) / 1.01D00 / ! 0.06      !m

!  Te
      DATA ABUNDS( 52 ) / 2.18D00 / ! 0.03      !m

!  I
      DATA ABUNDS( 53 ) / 1.55D00 / ! 0.08      !m

!  Xe
      DATA ABUNDS( 54 ) / 2.24D00 / ! 0.06      !*

!  Cs
      DATA ABUNDS( 55 ) / 1.08D00 / ! 0.02      !m

!  Ba
      DATA ABUNDS( 56 ) / 2.18D00 / ! 0.09

!  La
      DATA ABUNDS( 57 ) / 1.10D00 / ! 0.04

!  Ce
      DATA ABUNDS( 58 ) / 1.58D00 / ! 0.04

!  Pr
      DATA ABUNDS( 59 ) / 0.72D00 / ! 0.04

!  Nd
      DATA ABUNDS( 60 ) / 1.42D00 / ! 0.04

!  Pm
      DATA ABUNDS( 61 ) / TAP__RNULL /

!  Sm
      DATA ABUNDS( 62 ) / 0.96D00 / ! 0.04

!  Eu
      DATA ABUNDS( 63 ) / 0.52D00 / ! 0.04

!  Gd
      DATA ABUNDS( 64 ) / 1.07D00 / ! 0.04

!  Tb
      DATA ABUNDS( 65 ) / 0.30D00 / ! 0.10

!  Dy
      DATA ABUNDS( 66 ) / 1.10D00 / ! 0.04

!  Ho
      DATA ABUNDS( 67 ) / 0.48D00 / ! 0.11

!  Er
      DATA ABUNDS( 68 ) / 0.92D00 / ! 0.05

!  Tm
      DATA ABUNDS( 69 ) / 0.10D00 / ! 0.04

!  Yb
      DATA ABUNDS( 70 ) / 0.84D00 / ! 0.11

!  Lu
      DATA ABUNDS( 71 ) / 0.10D00 / ! 0.09

!  Hf
      DATA ABUNDS( 72 ) / 0.85D00 / ! 0.04

!  Ta
      DATA ABUNDS( 73 ) / -0.12D00 / ! 0.04     !m

!  W
      DATA ABUNDS( 74 ) / 0.85D00 / ! 0.12

!  Re
      DATA ABUNDS( 75 ) / 0.26D00 / ! 0.04      !m

!  Os
      DATA ABUNDS( 76 ) / 1.40D00 / ! 0.08

!  Ir
      DATA ABUNDS( 77 ) / 1.38D00 / ! 0.07

!  Pt
      DATA ABUNDS( 78 ) / 1.62D00 / ! 0.03      !m

!  Au
      DATA ABUNDS( 79 ) / 0.92D00 / ! 0.10

!  Hg
      DATA ABUNDS( 80 ) / 1.17D00 / ! 0.08      !m

!  Tl
      DATA ABUNDS( 81 ) / 0.90D00 / ! 0.20

!  Pb
      DATA ABUNDS( 82 ) / 1.75D00 / ! 0.10

!  Bi
      DATA ABUNDS( 83 ) / 0.65D00 / ! 0.04      !m

!  Po
      DATA ABUNDS( 84 ) / TAP__RNULL /

!  At
      DATA ABUNDS( 85 ) / TAP__RNULL /

!  Rn
      DATA ABUNDS( 86 ) / TAP__RNULL /

!  Fr
      DATA ABUNDS( 87 ) / TAP__RNULL /

!  Ra
      DATA ABUNDS( 88 ) / TAP__RNULL /

!  Ac
      DATA ABUNDS( 89 ) / TAP__RNULL /

!  Th
      DATA ABUNDS( 90 ) / 0.02D00 / ! 0.10  

!  Pa
      DATA ABUNDS( 91 ) / TAP__RNULL /

!  U
      DATA ABUNDS( 92 ) / -0.54D00 / ! 0.03       !m
  
!.

!  Check for a bad atomic number and missing data.

!     Bad atomic number.
      IF ( ( ZELEM .LE. 0 ) .OR. ( ZELEM .GT. TAP__NELEM ) ) THEN
         TAP_ABSOL = TAP__DNULL

!     Missing solar abundance data.
      ELSE IF( ( ABUNDS( ZELEM ) .LT. TAP__RNULL+ABS( SMALL ) ) &
               .AND. ( ABUNDS( ZELEM )                          &
                       .GT. TAP__RNULL-ABS( SMALL ) ) ) THEN
         TAP_ABSOL = TAP__DNULL

!     Return the solar photospheric abundance.
      ELSE
         TAP_ABSOL = ABUNDS( ZELEM )

      END IF

      
    END FUNCTION TAP_ABSOL
