DOUBLE PRECISION FUNCTION TAP_EMASS( IZ )

!+
!  Name:
!     TAP_EMASS

!  Purpose:
!=    Return the mean atomic weight of an element.

!  Language:
!     Fortran 90

!  Invocation:
!     RESULT = TAP_EMASS( IZ )

!  Description:
!     The mean atomic weight of the element associated with the given atomic 
!     number is returned. The atomic weight  assumes solar system relative
!     nuclide abundances. If the atomic number is not recognized, or if no
!     stable nuclides exist for the element, a value of TAP__DNULL is returned.

!  Arguments:
!     IZ = INTEGER (Given)
!        The atomic number of the element for which A is required.

!  Returned Value:
!     TAP_AELEM = DOUBLE PRECISION
!        The mean atomic weight, A, of the given element.

!  Authors:
!     CSJ: Simon Jeffery (St Andrews)
!     {enter_new_authors_here}

!  History:
!     24-NOV-1993 (CSJ):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Convertd to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-
      
!  Global Constants:
      USE SAE_PAR           ! Standard SAE constants
      USE TAP_PAR           ! TAP public constants

!  Global Variables:
      USE TAPC_CMN          ! TAP chemical global data
!        TACNAM( TAP__NELEM ) = CHARACTER ( 12 ) (Given)
!           Table of element names.
!        TACSYM( TAP__NELEM ) = CHARACTER * ( 2 ) (Given)
!           Table of element symbols.

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      INTEGER :: IZ                 ! Atomic number

!  External References:
      DOUBLE PRECISION :: TAP_ABNUC ! Relative nuclide abundances

!  Local Variables:
      INTEGER :: IA                 ! Atomic weight loop index

      INTEGER :: NUCMIN             ! Minimum nuclide mass is NUCMIN*IZ
      INTEGER :: NUCMAX             ! Maximum nuclide mass is NUCMAX*IZ

      DOUBLE PRECISION :: AFRAC    ! Relative nuclide abundance (per cent)
      DOUBLE PRECISION :: ANORM    ! Normalisation sum of nuclide abundances
      DOUBLE PRECISION :: AMASS    ! Total mass nuclides

!.

!  Initialize sums
      ANORM = 0.0
      AMASS = 0.0

!  Range of possible nuclides is NUCMIN*IZ to NUCMAX*IZ
!  Generally
      NUCMIN = 2
      NUCMAX = 3

!  Hydrogen and helium are exceptional
      IF (IZ.EQ.1.OR.IZ.EQ.2) THEN
        NUCMIN = 1
        NUCMAX = 2
      ENDIF

!  Loop over possible nuclides
      DO IA = NUCMIN*IZ, NUCMAX*IZ

!  Find the nuclide abundance
         AFRAC = TAP_ABNUC ( IZ, IA )

!     If the nuclide is tabulated, augment the sums
         IF (AFRAC.NE.TAP__DNULL) THEN
            ANORM = ANORM + AFRAC
            AMASS = AMASS + AFRAC * IA
         ENDIF
      ENDDO

!  If ANORM is sensible (it should be 100 per cent), 
!  calculate the mean atomic weight
      IF (ABS(ANORM-100.0).LT.1.0) THEN
         TAP_EMASS = AMASS / ANORM

!  Otherwise return a null value
      ELSE
         TAP_EMASS = TAP__DNULL
      ENDIF

      
      END FUNCTION TAP_EMASS
