DOUBLE PRECISION FUNCTION TAP_FACTRL ( N , STATUS )

!+
!  Name:
!     TAP_FACTRL

!  Purpose:
!=    Return the value of factorial N,  i.e. N!  as floating point number

!  Language:
!     Fortran 90

!  Invocation:
!     DOUBLE PRECISION TAP_FACTRL
!     RESULT = TAP_FACTRL( N, STATUS )

!  Package:
!     TAP: Theoretical Astrophysics Package

!  Module type:
!     double precision function

!  Description:
!     The factorial function is evaluated explicitly the first time it
!     is required for a given N > NTOP. The results of n! for all n <= N 
!     are stored in an array, so that repeat calculations are not performed. 
!     NTOP, initially 0, is reset to N. If N < NTOP, the value of N! is
!     recovered from a look up table.
!
!     The program is only valid for N! < the machine largest number. If
!     N! = nmax! > this value, an error value is flagged. TAP_FACTRL is not
!     defined for -1 < n < nmax.

!  Accuracy:
!     This function is accurate to at least 14 significant figures
!     for N <= 169. It has been tested by comparison with the MAPLE
!     symbolic computation package (Char, B.W., Geddes, K.O., 
!     Gonnet, G.H., Monagan, M.B., and Watt, S.M., 1988, Watcom 
!     Publications, Waterloo, Ontario, Canada).

!  Arguments:
!     N = INTEGER (Given)
!        The integer N for which N! is required.
!     STATUS = INTEGER (Given and Returned)
!        The global status.

!  Returned Value:
!     TAP_FACTRL = DOUBLE PRECISION
!        N! as a floating point number

!  Notes:

!  Authors:
!     CSJ: C.S.Jeffery (St. Andrews)
!     {enter_new_authors_here}

!  History:
!     2-AUG-1993 (CSJ):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Converted to F90.
!        {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE SAE_PAR
      USE MCH_PAR
      USE TAP_ERR

!  Type Definitions
      IMPLICIT NONE

!  Arguments Given:
      INTEGER :: N

!  Status:
      INTEGER :: STATUS

!  Local constants:
      INTEGER, PARAMETER :: NMAX = 500

!  Local variables:
      INTEGER :: I                     ! DO loop index
      INTEGER :: NTOP = 0              ! Current maximum N tabulated for N!


!  Local Arrays:
      DOUBLE PRECISION :: FACTRLS(0:NMAX)
      DATA FACTRLS(0) /1.0D+00/

!.

!  Check inherited global status.
      IF ( STATUS .NE. SAI__OK ) RETURN 

!  Check for negative integer.
      IF ( N.LT.0 ) THEN 
         STATUS = TAP__SINGULAR

!  Check for N already calculated.
      ELSEIF ( N.LE.NTOP ) THEN
         TAP_FACTRL = FACTRLS(N)

!  Fill in table as far as required.
      ELSEIF ( N.LE.NMAX ) THEN
         DO I = NTOP+1,N
            FACTRLS(I) = FLOAT(I) * FACTRLS(I-1)
            IF ( FACTRLS(I) .GT. MCH__LARGE/FLOAT(I+1) ) GOTO 99
         ENDDO
         NTOP = N
         TAP_FACTRL = FACTRLS(N)

!  N too large.
      ELSE
         STATUS = TAP__BADN

      ENDIF
      RETURN

!  If N! greater than machine largest number, or N larger than maximum
!  dimension of array FACTRLS, then return error code.
99    STATUS = TAP__BADN

    END FUNCTION TAP_FACTRL

