SUBROUTINE TAP_SAHA ( ZELEM, TEMP, NEL, ION_MIN, ION_MAX, U, CHI, &
                      IONFRN, STATUS )

!+
!  Program:
!     TAP_SAHA

!  Purpose:
!=    Calculate relative abundances of ions of species K.

!  Language:
!     Fortan 90

!  Invocation:
!     CALL TAP_SAHA ( zelem, temp, nel, ion_min, ion_max, u, chi,
!    :                ion_frn, status )

!  Description:
!     Calculate relative ionisation fractions of ions j = 1,.... for
!     atomic species (Z), given temperature and electron density, 
!     using the Saha ionisation equation. By recursive application of
!     the usual Saha equation relating relative abundances of successive
!     ionisation stages (Njk/Nj+1k), a general equation giving the
!     fraction Njk/Nk is obtained (Mihalas eqn. 5-17).  This is
!     evaluated using scaled logarithmic variables.

!     The programmer must supply the atomic number of the species to be
!     treated, the gas temperature (K) and electron density (m**-3).
!     He is also required to provide the partition functions  and
!     ionisation potentials for each ionisation stage to be considered.
!     These requirements allow for some flexibility in the way the
!     subroutine may be used, and for the inclusion of additional
!     phsyics (e.g. pressure ionisation, electron shielding, etc.)

!     By specifying ION_MIN and ION_MAX, the programmer may treat only
!     selected ionisation stages (saving unnecessary computations).

!     By using an internal OFFSET, the locations 1,2,... in the partition 
!     function,  ionisation potential and returned ionisation fraction 
!     arrays map onto the ionisation stages ION_MIN ... ION_MAX.
!
!     Consider two examples:
!     i) calculate ion fractions  for neutrals through triply ionised
!     ii) calculate ion fractions for -ve ions through doubly ionized
!
!     Arrays U, CHI and ION should have data in the locations 1 onwards
!
!                   example (i)        example (ii)
!
!     U             0 1 2 3            -1 0 1 2
!     CHI           0 1 2              -1 0 1
!     IONFRN        0 1 2 3            -1 0 1 2


!  Arguments:
!     ZELEM = INTEGER (Given)
!        Atomic number (ie hydrogen = 1, helium = 2, etc)
!     TEMP = DOUBLE PRECISION (Given)
!        Temperature (degrees K)
!     NEL = DOUBLE PRECISION (Given)
!        Electron density (m**-3) 
!     ION_MIN = INTEGER (Given)
!        Minimum ionic charge to be treated (ie negative = -1, ...)
!     ION_MAX = INTEGER (Given)
!        Maximum ionic charge to be treated (ie neutral = 0, ...)
!     U = VARIABLE LENGTH DOUBLE PRECISION ARRAY (Given)
!        Partition function for each ionisation stage
!     CHI = VARIABLE LENGTH DOUBLE PRECISION ARRAY (Given)
!        Ionisation potential for each ionisation stage
!     IONFRN = VARIABLE LENGTH DOUBLE PRECISION ARRAY (Returned)
!        Array containing fraction in each ionisation stage
!     STATUS = INTEGER (Returned)
!        The global status.

!  References:
!     Mihalas D., 1978, Stellar Atmospheres (2nd ed.) Freeman

!  External Routines Used:
!     TAP:
!        TAP_PAR, [routine_used]...

!  [optional_subroutine_items]...
 
!  Authors:
!     CSJ:   C.S.Jeffery (St Andrews/CCP7)
!     {enter_new_authors_here}

!  History:
!     06-MAY-1994 (CSJ):
!        Original version.
!     {enter_changes_here}

!  Bugs:
!     Does not treat negative ions.
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE SAE_PAR          ! Standard SAE constants
      USE TAPM_PAR         ! TAP Mathematical constants
      USE TAPP_PAR         ! TAP Physical constants

!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      INTEGER          ::  ZELEM
      DOUBLE PRECISION ::  TEMP
      DOUBLE PRECISION ::  NEL
      INTEGER          ::  ION_MIN 
      INTEGER          ::  ION_MAX
      DOUBLE PRECISION ::  U(*)
      DOUBLE PRECISION ::  CHI(*)
      
!  Arguments Returned:
      DOUBLE PRECISION ::  IONFRN(*)

!  Status:
      INTEGER :: STATUS             ! Global status

!  Local Constants:
    ! Most positive ion to be treated
      INTEGER, PARAMETER :: JJMAX = 10 
  

!  Local Variables:
      INTEGER :: J                  ! Do Loop Index (ion)
      INTEGER :: JMIN               ! Lowest ionisation stage to be treated 
      INTEGER :: JMAX               ! Highest ionisation stage to be treated
      INTEGER :: JJ                 ! Offset pointer for U and IONFRN arrays
      INTEGER :: L                  ! Do Loop Index
      DOUBLE PRECISION :: LNE       ! log (electron density) (m**-3)
      DOUBLE PRECISION :: LT        ! log (temperature) (K)
      DOUBLE PRECISION :: CHIJK     ! chi_jk - ionisation potential species j,k
      DOUBLE PRECISION :: PSC       ! Scaling factor for P.
      DOUBLE PRECISION :: S         ! Sk - Summation over product terms
      DOUBLE PRECISION :: LC_I      ! log C_I = 1/2(h**2/(2.pi.m.k)**3/2
      SAVE LC_I
      LOGICAL :: XCHI = .FALSE.     ! Flag whether ion potentials are loaded
      SAVE XCHI

!  Local Arrays:
      DOUBLE PRECISION :: PHIB(-1:JJMAX)     ! T-dependent part of Saha equation
      DOUBLE PRECISION :: P(-1:JJMAX)        ! Pjk - Product terms


!.

!  Initialise constants (do this as a parameter, using Maple).
      IF (.NOT.XCHI) THEN
         LC_I = LOG ( 0.5 ) + &
                1.5 * LOG (TAPP__H * TAPP__HB2PI / TAPP__ME / TAPP__K)
         XCHI = .TRUE.
      ENDIF


!  Check min and max ion stages to treat
!     Unsure about negative ions
!     JMIN = MAX (-1,ION_MIN)
      JMIN = MAX (0,ION_MIN)
      JMAX = MIN (ZELEM,ION_MAX)


!  Establish log of temperature and electron density.
      LT  = LOG ( TEMP )
      LNE = LOG ( NEL ) 


!  Establish T-dependent terms
      DO J = JMIN,JMAX-1
         JJ = J - ION_MIN + 1
         CHIJK   = CHI(JJ) * TAPP__EV / TAPP__K / TEMP
         PHIB(J) = LOG(U(JJ)/U(JJ+1)) + LC_I - 1.5*LT + CHIJK
      ENDDO

!  Establish Product terms P(J)
      DO J = JMIN,JMAX
         P(J)   = 1.
         DO L = J,JMAX-1
            P(J) = P(J) + LNE + PHIB(L)
         ENDDO
      ENDDO
      P(JMAX) = 1

!  Scale by maximum value of P(J)
      PSC = 0.
      DO J = JMIN,JMAX
         PSC = MAX (PSC,P(J))
      ENDDO
      DO J =  JMIN,JMAX
         P(J) = P(J) - PSC
      ENDDO

!  Establish Summation terms S(J)
      S = 0.
      DO L = JMIN,JMAX
         S = S + EXP(P(L))
      ENDDO


! Establish final ion fractions.
      DO J = JMIN,JMAX
         JJ = J - ION_MIN + 1
         IONFRN(JJ) = EXP(P(J)) / S
      ENDDO

    END SUBROUTINE TAP_SAHA
