MODULE TAPA_PAR

!+
!  Name:
!     TAPA_PAR

!  Purpose:
!=    Theoretical Astrophysics Library public astronomical constants

!  Language:
!     Starlink Fortran 77

!  Type of Module:
!     Global constants Module

!  Description:
!  
!     This file contains the astronomical global constants used by the
!     Theoretical Astrophysics Library. These come from a number of
!     sources, including:
!     
!     a) IAU (1976) System of Astronomical Constants 
!
!     The IAU recognises a set of astronomical units of length, mass
!     and time for use in connection with motions in the Solar System;
!     they are related to each other through the adopted value of the
!     constant of gravitation when expressed in these units. The unit
!     of length is the au (A), the unit of time is 1 day (D) of 86 400
!     SI seconds (s), the unit of mass is equal to the mass of the Sun
!     (S).  For larger quantities, the IAU recommend that the parsec,
!     defined in au, and the Julian year (365.25 d) should be used.
!
!     The defining constants in this system are the Gaussian
!     gravitational constant (k) and the speed of light (c).
!     Primary constants are best values from observations and
!     derived constants are calculated from defining and primary
!     constants.
!
!     The question of combining constants from two systems (CODATA and
!     IAU) in the TAP library raises certain difficulties.  For
!     example, the recommended value for the gravitational constant is
!     specified with greater precision in CODATA than in IAU (1976).
!     The IAU recommendations are given without errors.
!
!     To overcome possible conflicts, TAP adopts CODATA constants
!     where available. Errors for IAU (1976) constants are estimated
!     from the modifications required to prepare and fit ephemerides
!     to observation (Astronomical Almanac 1992). Derived constants are
!     calculated accordingly.
!     
!     b) Astrophysical Quantities (1973).
!
!     Observed values for the solar radius and luminosity are mean
!     values (1973). A variation in luminosity of 0.05 percent around
!     the 22 year solar cycle has been measured. Larger amplitude
!     variations over other timescales are indicated. Historical
!     measurements of the solar radius indicate a reduction by 0.3 per
!     cent since observations by Picard and others during the Maunder
!     minimum (1640 - 1700).

!  References:
!     Transactions of the International Astronomical Union. Vol. XVIB
!        1976. Proceedings of the Sixteenth General Assembly, Grenoble
!        1976, p.58. eds. E.A.Mueller & A.Jappel. Reidel, Dordrecht.
!     Astronomical Almanac, 1992. p K6. US Government Printing Office,
!        Washington, & HM Stationery Office, London
!     Allen C.W., 1973. Astrophysical Quantities (3rd edn), p. 161.

!  Authors:
!     CSJ: C.S.Jeffery (St Andrews)
!     {enter_new_authors_here}

!  History:
!     30-APR-1992 (CSJ):
!        Original version.
!     08-JAN-2001 (CSJ):
!        Conversion to f95 MODULE
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-


!  Pi. (needed locally)
!  TAPA__PI = PI
      DOUBLE PRECISION, PARAMETER :: TAPA__PI = 3.141592653589793238462643383279503D+00

!  Gaussian gravitational constant: k = 0.01720209895 (exact).
      DOUBLE PRECISION, PARAMETER ::TAPA__K = 1.720209895D-02

!  Day: d = 8.64E+04 s (exact).
      DOUBLE PRECISION, PARAMETER ::TAPA__DAY = 8.64D+04

!  Julian Year: Jyear = 365.25 d = 3.15576E+07 s (exact).
     DOUBLE PRECISION, PARAMETER ::TAPA__JYEAR = 3.15576D+07

!  Astronomical Unit: AU = 1.49597870E+11 m
!                  error = 0.00000001E+11.
      DOUBLE PRECISION, PARAMETER ::TAPA__AU = 1.49597870D+11

!  Parsec: pc = 360 * 60 * 60 / 2pi * A m (derived).
      DOUBLE PRECISION, PARAMETER ::TAPA__PC = 6.48D+05 * TAPA__AU / TAPA__PI

!  Solar Mass: M(sol) = 1.98892E+30 kg
!               error = 0.00025E+30.
      DOUBLE PRECISION, PARAMETER ::TAPA__MSOL = 1.98892D+30

!  Solar radius: R(sol) = 6.9599E+08 m
!                 error = 0.0007E+08.
      DOUBLE PRECISION, PARAMETER ::TAPA__RSOL = 6.9599D+08

!  Solar surface gravity: g(sol) = G * M(sol) / R(sol)**2 
!                                = 2.7397E+02 m/s**2
!                          error = 0.0005E+02.
      DOUBLE PRECISION, PARAMETER ::TAPA__GSOL = 2.7397D+02

!  Solar luminosity: L(sol) = 3.826E+26 J/s
!                     error = 0.008E+26.
      DOUBLE PRECISION, PARAMETER ::TAPA__LSOL = 3.826D+26

!  Solar effective temperature:
!               Teff(sol) = ( L / ( 4 * pi * sigma * R**2 ) ) ** (1/4)
!                         = 5.770E+03 K
!                   error = 0.006E+03.
      DOUBLE PRECISION, PARAMETER ::TAPA__TESOL = 5.770D+03

!.
END MODULE
