MODULE TAPC_CMN

!+
!  Name:
!     TAPC_CMN

!  Purpose:
!=    Define the chemical abundance table.

!  Language:
!     Fortran 90

!  Type of module:
!     Global variables module.

!  Description:
!     Define the common blocks holding the chemical abundance data
!     for the TAP_ subroutine library.

!     The abundance of each element is initialised to the special value 
!     TAP__RNULL.  This is
!     a null value indicating the abundance value to be undefined.
!     Nuclide atomic weights are initialised to their published values.

!  Authors:
!     PCTR: P.C.T. Rees (STARLINK)
!     CSJ: C.S. Jeffery (Armagh Observatory)
!     {enter_new_authors_here}

!  History:
!     8-JUL-1992 (PCTR):
!        Original version.
!     23-JUL-1992 (PCTR):
!        Added nuclide common blocks.
!     08-JAN-2002 (CSJ):
!        conversion to F90.
!     {enter_further_changes_here}

!-

!  Global Constants:
      USE TAP_PAR          ! TAP public constants

!  Abundance table.
      REAL :: TABUND( TAP__NELEM ) ! Logarithm relative abundances

!  Nuclide table.
      INTEGER :: NUCPNT( TAP__NELEM+1 ) ! Element pointer
      INTEGER :: NUCMAS( TAP__NELEM+1 ) ! Element atomic mass offset
      REAL    :: TABNUC( TAP__NNUCL ) ! Nuclide relative abundances
      DOUBLE PRECISION :: TAWNUC( TAP__NNUCL ) ! Nuclide atomic weights

!  Name tables.
      CHARACTER(LEN=13) :: TACNAM( TAP__NELEM ) ! Element names
      CHARACTER(LEN=2)  :: TACSYM( TAP__NELEM ) ! Element symbols



!  Local Variables:
      INTEGER I                  ! Loop index

!  Global Data:
!  Element abundance table (initialised null).
      DATA TABUND / TAP__NELEM * TAP__RNULL /

!  Nuclide abundance table (initialised null).
      DATA TABNUC / TAP__NNUCL * TAP__RNULL /

!  Nuclide table element pointers (stored by atomic number).
      DATA NUCPNT /   0,   2,   4,   6,   7,   9,  11,  13,  16,  17, &
                     20,  21,  24,  25,  28,  29,  34,  37,  42,  45, &
                     54,  55,  60,  62,  67,  68,  73,  74,  81,  84, &
                     91,  94, 101, 102, 111, 114, 123, 126, 131, 132, &
                    139, 140, 149, 149, 158, 159, 168, 171, 182, 185, &
                    198, 201, 212, 213, 226, 227, 236, 238, 245, 246, &
                    255, 255, 266, 269, 278, 279, 288, 289, 298, 299, &
                    308, 310, 317, 319, 326, 329, 338, 341, 350, 351, &
                    360, 363, 368, 369, 369, 369, 369, 369, 369, 369, &
                    370, 370, 375 /

!  Nuclide atomic mass offsets (stored by atomic number).
      DATA NUCMAS /   0,   2,   5,   8,   9,  11,  13,  15,  18,  19, &
                     22,  23,  26,  27,  30,  31,  34,  35,  38,  39, &
                     44,  45,  49,  49,  54,  53,  58,  57,  62,  63, &
                     68,  69,  74,  73,  78,  77,  84,  83,  88,  89, &
                     92,  91, 149,  95, 102, 101, 106, 105, 112, 111, &
                    120, 119, 126, 123, 132, 129, 137, 135, 140, 141, &
                    255, 143, 150, 151, 158, 155, 164, 161, 168, 167, &
                    174, 173, 179, 179, 184, 183, 190, 189, 196, 195, &
                    202, 203, 208, 369, 369, 369, 369, 369, 369, 231, &
                    370, 233, 375 /

!  Nuclide atomic weights.
!  Hydrogen: (1)H, (2)H.
      DATA ( TAWNUC( I ), I=1, 2 ) / 1.007825D+00, 2.0140D+00 /

!  Helium: (3)He, (4)He.
      DATA ( TAWNUC( I ), I=3, 4 ) / 3.01603D+00, 4.00260D+00 /

!  Lithium: (6)Li, (7)Li.
      DATA ( TAWNUC( I ), I=5, 6 ) / 6.015121D+00, 7.016003D+00 /

!  Beryllium: (9)Be.
      DATA ( TAWNUC( I ), I=7, 7 ) / 9.012182D+00 /

!  Boron: (10)B, (11)B.
      DATA ( TAWNUC( I ), I=8, 9 ) / 10.012937D+00, 11.009305D+00 /

!  Carbon: (12)C, (13)C.
      DATA ( TAWNUC( I ), I=10, 11 ) / 12.000000D+00, 13.003355D+00 /

!  Nitrogen: (14)N, (15)N.
      DATA ( TAWNUC( I ), I=12, 13 ) / 14.003074D+00, 15.000108D+00 /

!  Oxygen: (16)O, (17)O, (18)O.
      DATA ( TAWNUC( I ), I=14, 16 ) / 15.994915D+00, 16.999131D+00, &
                                       17.999160D+00 /

!  Fluorine: (19)F.
      DATA ( TAWNUC( I ), I=17, 17 ) / 18.998403D+00 /

!  Neon: (20)Ne, (21)Ne, (22)Ne.
      DATA ( TAWNUC( I ), I=18, 20 ) / 19.992435D+00, 20.993843D+00, &
                                       21.991383D+00 /

!  Sodium: (23)Na.
      DATA ( TAWNUC( I ), I=21, 21 ) / 22.989767D+00 /

!  Magnesium: (24)Mg, (25)Mg, (26)Mg.
      DATA ( TAWNUC( I ), I=22, 24 ) / 23.985042D+00, 24.985837D+00, &
                                       25.982593D+00 /

!  Aluminium: (27)Al.
      DATA ( TAWNUC( I ), I=25, 25 ) / 26.981539D+00 /

!  Silicon: (28)Si, (29)Si, (30)Si.
      DATA ( TAWNUC( I ), I=26, 28 ) / 27.976927D+00, 28.976495D+00, &
                                       29.973770D+00 /

!  Phosphorus: (31)P.
      DATA ( TAWNUC( I ), I=29, 29 ) / 30.973762D+00 /

!  Sulphur: (32)S, (33)S, (34)S, (35)S (nuclide is unstable), (36)S.
      DATA ( TAWNUC( I ), I=30, 34 ) / 31.972070D+00, 32.971456D+00, &
                                       33.967866D+00,   TAP__DNULL, &
                                       35.967080D+00 /

!  Chlorine: (35)Cl, (36)Cl (nuclide is unstable), (37)Cl.
      DATA ( TAWNUC( I ), I=35, 37 ) / 34.968852D+00, TAP__DNULL, &
                                       36.965903D+00 /

!  Argon: (36)Ar, (37)Ar (nuclide is unstable), (38)Ar,
!         (39)Ar (nuclide is unstable), (40)Ar.
      DATA ( TAWNUC( I ), I=38, 42 ) / 35.967545D+00, TAP__DNULL, &
                                       37.962732D+00, TAP__DNULL, &
                                       39.962384D+00 /

!  Potassium: (39)K, (40)K (nuclide is unstable), (41)K.
      DATA ( TAWNUC( I ), I=43, 45 ) / 38.963707D+00, 39.963999D+00, &
                                       40.961825D+00 /

!  Calcium: (40)Ca, (41)Ca (nuclide is unstable), (42)Ca, (43)Ca,
!           (44)Ca, (45)Ca (nuclide is unstable), (46)Ca, 
!           (47)Ca (nuclide is unstable), (48)Ca.
      DATA ( TAWNUC( I ), I=46, 54 ) / 39.962591D+00,   TAP__DNULL, &
                                       41.958618D+00, 42.958766D+00, &
                                       43.955480D+00,   TAP__DNULL, &
                                       45.953689D+00,   TAP__DNULL, &
                                       47.952533D+00 /

!  Scandium: (45)Sc.
      DATA ( TAWNUC( I ), I=55, 55 ) / 44.955910D+00 /

!  Titanium: (46)Ti, (47)Ti, (48)Ti, (49)Ti, (50)Ti.
      DATA ( TAWNUC( I ), I=56, 60 ) / 45.952629D+00, 46.951764D+00, &
                                       47.947947D+00, 48.947871D+00, &
                                       49.944792D+00 /

!  Vanadium: (50)V (nuclide is unstable), (51)V.
      DATA ( TAWNUC( I ), I=61, 62 ) / 49.947161D+00, 50.943962D+00 /

!  Chromium: (50)Cr, (51)Cr (nuclide is unstable), (52)Cr, (53)Cr,
!            (54)Cr.
      DATA ( TAWNUC( I ), I=63, 67 ) / 49.946046D+00,   TAP__DNULL, &
                                       51.940509D+00, 52.940651D+00, &
                                       53.938882D+00 /

!  Manganese: (55)Mn.
      DATA ( TAWNUC( I ), I=68, 68 ) / 54.938047D+00 /

!  Iron: (54)Fe, (55)Fe (nuclide is unstable), (56)Fe, (57)Fe,
!        (58)Fe.
      DATA ( TAWNUC( I ), I=69, 73 ) / 53.939612D+00,   TAP__DNULL, &
                                       55.934939D+00, 56.935396D+00, &
                                       57.933277D+00 /

!  Cobalt: (59)Co.
      DATA ( TAWNUC( I ), I=74, 74 ) / 58.933198D+00 /

!  Nickel: (58)Ni, (59)Ni (nuclide is unstable), (60)Ni, (61)Ni,
!          (62)Ni, (63)Ni (nuclide is unstable), (64)Ni.
      DATA ( TAWNUC( I ), I=75, 81 ) / 57.935346D+00,   TAP__DNULL, &
                                       59.930788D+00, 60.931058D+00, &
                                       61.928346D+00,   TAP__DNULL, &
                                       63.927968D+00 /

!  Copper: (65)Cu, (66)Cu (nuclide is unstable), (67)Cu.
      DATA ( TAWNUC( I ), I=82, 84 ) / 62.939598D+00, TAP__DNULL, &
                                       64.927793D+00 /

!  Zinc: (64)Zn, (65)Zn (nuclide is unstable), (66)Zn, (67)Zn, (68)Zn,
!        (69)Zn (nuclide is unstable), (70)Zn.
      DATA ( TAWNUC( I ), I=85, 91 ) / 63.929145D+00,   TAP__DNULL, &
                                       65.926034D+00, 66.927129D+00, &
                                       67.924846D+00,   TAP__DNULL, &
                                       69.925325D+00 /

!  Gallium: (69)Ga, (70)Ga (nuclide is unstable), (71)Ga.
      DATA ( TAWNUC( I ), I=92, 94 ) / 68.925580D+00, TAP__DNULL, &
                                       70.924700D+00 /

!  Germanium: (70)Ge, (71)Ge (nuclide is unstable), (72)Ge, (73)Ge,
!             (74)Ge, (75)Ge, (nuclide is unstable), (76)Ge.
      DATA ( TAWNUC( I ), I=95, 101 ) / 69.924250D+00,   TAP__DNULL, &
                                        71.922079D+00, 72.923463D+00, &
                                        73.921177D+00,   TAP__DNULL, &
                                        75.921401D+00 /

!  Arsenic: (75)As.
      DATA ( TAWNUC( I ), I=102, 102 ) / 74.921594D+00 /

!  Selenium: (74)Se, (75)Se (nuclide is unstable), (76)Se, (77)Se,
!            (78)Se, (79)Se (nuclide is unstable), (80)Se,
!            (81)Se (nuclide is unstable), (82)Se.
      DATA ( TAWNUC( I ), I=103, 111 ) / 73.922475D+00,   TAP__DNULL, &
                                         75.919212D+00, 76.919912D+00, &
                                         TAP__DNULL, TAP__DNULL, &
                                         79.916520D+00,   TAP__DNULL, &
                                         81.916698D+00 /

!  Bromine: (79)Br, (80)Br (nuclide is unstable), (81)Br.
      DATA ( TAWNUC( I ), I=112, 114 ) / 78.918336D+00, TAP__DNULL, &
                                         80.916289D+00 /

!  Krypton: (78)Kr, (79)Kr (nuclide is unstable), (80)Kr,
!           (81)Kr (nuclide is unstable), (82)Kr, (83)Kr, (84)Kr,
!           (85)Kr (nuclide is unstable), (86)Kr.
      DATA ( TAWNUC( I ), I=115, 123 ) / TAP__DNULL, TAP__DNULL, &
                                         79.916380D+00,   TAP__DNULL, &
                                         81.913482D+00, 82.914135D+00, &
                                         83.911507D+00,   TAP__DNULL, &
                                         85.910616D+00 /

!  Rubidium: (85)Rb, (86)Rb (nuclide is unstable), (87)Rb.
      DATA ( TAWNUC( I ), I=124, 126 ) / 84.911794D+00, TAP__DNULL, &
                                         86.909187D+00 /

!  Strontium: (84)Sr, (85)Sr (nuclide is unstable), (86)Sr, (87)Sr,
!             (88)Sr.
      DATA ( TAWNUC( I ), I=127, 131 ) / 83.913430D+00,   TAP__DNULL, &
                                         85.909267D+00, 86.908884D+00, &
                                         87.905619D+00 /

!  Yttrium: (89)Y.
      DATA ( TAWNUC( I ), I=132, 132 ) / 88.905849D+00 /

!  Zirconium: (90)Zr, (91)Zr, (92)Zr, (93)Zr (nuclide is unstable),
!             (94)Zr, (95)Zr (nuclide is unstable), (96)Zr.
      DATA ( TAWNUC( I ), I=133, 139 ) / 89.904703D+00, 90.905644D+00, &
                                         91.905039D+00,   TAP__DNULL, &
                                         93.906314D+00,   TAP__DNULL, &
                                         95.908275D+00 /

!  Niobium: (93)Nb.
      DATA ( TAWNUC( I ), I=140, 140 ) / 92.906377D+00 /

!  Molybdenum: (92)Mo, (93)Mo (nuclide is unstable), (94)Mo, (95)Mo,
!              (96)Mo, (97)Mo, (98)Mo, (99)Mo (nuclide is unstable),
!              (100)Mo.
      DATA ( TAWNUC( I ), I=141, 149 ) / 91.906808D+00,   TAP__DNULL, &
                                         93.905085D+00, 94.905840D+00, &
                                         95.904678D+00, 96.906020D+00, &
                                         97.905406D+00,   TAP__DNULL, &
                                         99.907477D+00 /

!  Technetium: Tc.
!  Element has no stable nuclides.

!  Ruthenium: (96)Ru, (97)Ru (nuclide is unstable), (98)Ru, (99)Ru,
!             (100)Ru, (101)Ru, (102)Ru, (103)Ru (nuclide is unstable),
!             (104)Ru.
      DATA ( TAWNUC( I ), I=150, 158 ) /  95.907599D+00,    TAP__DNULL, &
                                          97.905287D+00,  98.905939D+00, &
                                          99.904219D+00, 100.905582D+00, &
                                         101.904348D+00,    TAP__DNULL, &
                                         103.905424D+00 /

!  Rhodium: (103)Rh.
      DATA ( TAWNUC( I ), I=159, 159 ) / 102.905500D+00 /

!  Palladium: (102)Pd, (103)Pd (nuclide is unstable), (104)Pd,
!             (105)Pd, (106)Pd, (107)Pd (nuclide is unstable), (108)Pd,
!             (109)Pd (nuclide is unstable), (110)Pd.
      DATA ( TAWNUC( I ), I=160, 168 ) / 101.905634D+00,    TAP__DNULL, &
                                         103.904029D+00, 104.905079D+00, &
                                         105.903478D+00,    TAP__DNULL, &
                                         107.903895D+00,    TAP__DNULL, &
                                         109.905167D+00 /

!  Silver: (107)Ag, (108)Ag (nuclide is unstable), (109)Ag.
      DATA ( TAWNUC( I ), I=169, 171 ) / 106.905092D+00, TAP__DNULL, &
                                         108.904757D+00 /

!  Cadmium: (106)Cd, (107)Cd (nuclide is unstable), (108)Cd,
!           (109)Cd (nuclide is unstable), (110)Cd, (111)Cd, (112)Cd,
!           (113)Cd, (114)Cd, (115)Cd, (nuclide is unstable), (116)Cd.
      DATA ( TAWNUC( I ), I=172, 182 ) / 105.906461D+00,    TAP__DNULL, &
                                         107.904176D+00,    TAP__DNULL, &
                                         109.903005D+00, 110.904182D+00, &
                                         111.902758D+00, 112.904400D+00, &
                                         113.903357D+00,    TAP__DNULL, &
                                         115.904754D+00 /

!  Indium: (113)In, (114)In (nuclide is unstable),
!          (115)In (nuclide is unstable).
      DATA ( TAWNUC( I ), I=183, 185 ) / 112.904061D+00, TAP__DNULL, &
                                         114.903880D+00 /

!  Tin: (112)Sn, (113)Sn (nuclide is unstable), (114)Sn, (115)Sn,
!       (116)Sn, (117)Sn, (118)Sn, (119)Sn, (120)Sn,
!       (121)Sn (nuclide is unstable), (122)Sn,
!       (123)Sn (nuclide is unstable), (124)Sn.
      DATA ( TAWNUC( I ), I=186, 198 ) / 111.904826D+00,    TAP__DNULL, &
                                         113.902784D+00, 114.903348D+00, &
                                         115.901747D+00, 116.902956D+00, &
                                         117.901609D+00, 118.903310D+00, &
                                         119.902200D+00,    TAP__DNULL, &
                                         121.903440D+00,    TAP__DNULL, &
                                         123.905274D+00 /

!  Antimony: (121)Sb, (122)Sb (nuclide is unstable), (123)Sb.
      DATA ( TAWNUC( I ), I=199, 201 ) / 120.903821D+00, TAP__DNULL, &
                                         122.904216D+00 /

!  Tellurium: (120)Te, (121)Te (nuclide is unstable), (122)Te,
!             (123)Te (nuclide is unstable), (124)Te, (125)Te, (126)Te,
!             (127)Te (nuclide is unstable), (128)Te,
!             (129)Te (nuclide is unstable),
!             (130)Te (nuclide is unstable).
      DATA ( TAWNUC( I ), I=202, 212 ) / 119.904048D+00,    TAP__DNULL, &
                                         121.903054D+00, 122.904271D+00, &
                                         123.902823D+00, 124.904433D+00, &
                                         125.903314D+00,    TAP__DNULL, &
                                         127.904463D+00,    TAP__DNULL, &
                                         129.906229D+00 /

!  Iodine, (127)I.
      DATA ( TAWNUC( I ), I=213, 213 ) / 126.904473D+00 /

!  Xenon: (124)Xe, (125)Xe (nuclide is unstable), (126)Xe,
!         (127)Xe (nuclide is unstable), (128)Xe, (129)Xe, (130)Xe,
!         (131)Xe, (132)Xe, (133)Xe (nuclide is unstable), (134)Xe,
!         (135)Xe (nuclide is unstable), (136)Xe.
      DATA ( TAWNUC( I ), I=214, 226 ) / 123.905894D+00,    TAP__DNULL, &
                                         125.904281D+00,    TAP__DNULL, &
                                         127.903531D+00, 128.904780D+00, &
                                         129.903509D+00, 130.905072D+00, &
                                         131.904144D+00,    TAP__DNULL, &
                                         133.905395D+00,    TAP__DNULL, &
                                         135.907214D+00 /

!  Caesium: (133)Cs.
      DATA ( TAWNUC( I ), I=227, 227 ) / 132.905429D+00 /

!  Barium: (130)Ba, (131)Ba (nuclide is unstable), (132)Ba,
!          (133)Ba (nuclide is unstable), (134)Ba, (135)Ba, (136)Ba,
!          (137)Ba, (138)Ba.
      DATA ( TAWNUC( I ), I=228, 236 ) / 129.906282D+00,    TAP__DNULL, &
                                         131.905042D+00,    TAP__DNULL, &
                                         133.904486D+00, 134.905665D+00, &
                                         135.904553D+00, 136.905812D+00, &
                                         137.905232D+00 /

!  Lanthanum: (138)La (nuclide is unstable), (139)La.
      DATA ( TAWNUC( I ), I=237, 238 ) / 137.90711D+00, 138.906347D+00 /

!  Cerium: (136)Ce, (137)Ce (nuclide is unstable), (138)Ce,
!          (139)Ce (nuclide is unstable), (140)Ce, (141)Ce (nuclide is
!          unstable), (142)Ce.
      DATA ( TAWNUC( I ), I=239, 245 ) / 135.907140D+00, TAP__DNULL, &
                                         137.905985D+00, TAP__DNULL, &
                                         139.905433D+00, TAP__DNULL, &
                                         141.909241D+00 /

!  Praseodymium: (141)Pr.
      DATA ( TAWNUC( I ), I=246, 246 ) / 140.907647D+00 /

!  Neodymium: (142)Nd, (143)Nd, (144)Nd (nuclide is unstable), (145)Nd,
!             (146)Nd, (147)Nd (nuclide is unstable), (148)Nd,
!             (149)Nd (nuclide is unstable), (150)Nd.
      DATA ( TAWNUC( I ), I=247, 255 ) / 141.907719D+00, 142.909810D+00, &
                                         143.910083D+00, 144.912570D+00, &
                                         145.913113D+00,    TAP__DNULL, &
                                         147.916889D+00,    TAP__DNULL, &
                                         149.920887D+00 /

!  Promethium: Pm.
!  Element has no stable nuclides.

!  Samarium: (144)Sm, (145)Sm (nuclide is unstable),
!            (146)Sm (nuclide is unstable),
!            (147)Sm (nuclide is unstable),
!            (148)Sm (nuclide is unstable),
!            (149)Sm (nuclide is unstable), (150)Sm,
!            (151)Sm (nuclide is unstable), (152)Sm,
!            (153)Sm (nuclide is unstable), (154)Sm.
      DATA ( TAWNUC( I ), I=256, 266 ) / 143.911998D+00,    TAP__DNULL, &
                                            TAP__DNULL, 146.914895D+00, &
                                         147.914820D+00, 148.917181D+00, &
                                         149.917273D+00,    TAP__DNULL, &
                                         151.919729D+00,    TAP__DNULL, &
                                         153.922206D+00 /

!  Europium: (151)Eu, (152)Eu (nuclide is unstable), (153)Eu.
      DATA ( TAWNUC( I ), I=267, 269 ) / 150.919847D+00, TAP__DNULL, &
                                         152.921225D+00 /

!  Gadolinium: (152)Gd, (153)Gd (nuclide is unstable), (154)Gd, (155)Gd,
!              (156)Gd, (157)Gd, (158)Gd, (159)Gd (nuclide is unstable),
!              (160)Gd.
      DATA ( TAWNUC( I ), I=270, 278 ) / 151.919786D+00,    TAP__DNULL, &
                                         153.920861D+00, 154.922618D+00, &
                                         155.922118D+00, 156.923956D+00, &
                                         157.924099D+00,    TAP__DNULL, &
                                         159.927049D+00 /

!  Terbium: (159)Tb.
      DATA ( TAWNUC( I ), I=279, 279 ) / 158.925342D+00 /

!  Dysprosium: (156)Dy, (157)Dy (nuclide is unstable), (158)Dy,
!              (159)Dy (nuclide is unstable), (160)Dy, (161)Dy, (162)Dy,
!              (163)Dy, (164)Dy.
      DATA ( TAWNUC( I ), I=280, 288 ) / 155.925277D+00,    TAP__DNULL, &
                                         157.924403D+00, 158.925735D+00, &
                                         159.925193D+00, 160.926930D+00, &
                                         161.926795D+00, 162.928728D+00, &
                                         163.929171D+00 /

!  Holmium: (165)Ho.
      DATA ( TAWNUC( I ), I=289, 289 ) / 164.930319D+00 /

!  Erbium: (162)Er, (163)Er (nuclide is unstable), (164)Er,
!          (165)Er (nuclide is unstable), (166)Er, (167)Er, (168)Er,
!          (169)Er (nuclide is unstable), (170)Er.
      DATA ( TAWNUC( I ), I=290, 298 ) / 161.928775D+00,    TAP__DNULL, &
                                         163.929198D+00,    TAP__DNULL, &
                                         165.930290D+00, 166.932046D+00, &
                                         167.932368D+00,    TAP__DNULL, &
                                         169.935461D+00 /

!  Thulium: (169)Tm.
      DATA ( TAWNUC( I ), I=299, 299 ) / 168.934212D+00 /

!  Ytterbium: (168)Yb, (169)Yb (nuclide is unstable), (170)Yb, (171)Yb,
!             (172)Yb, (173)Yb, (174)Yb, (175)Yb (nuclide is unstable),
!             (176)Yb.
      DATA ( TAWNUC( I ), I=300, 308 ) / 167.933894D+00,    TAP__DNULL, &
                                         169.934759D+00, 170.936323D+00, &
                                         171.936378D+00, 172.938208D+00, &
                                         173.938859D+00,    TAP__DNULL, &
                                         175.942564D+00 /

!  Lutetium: (175)Lu, (176)Lu (nuclide is unstable).
      DATA ( TAWNUC( I ), I=309, 310 ) / 174.940770D+00, 175.942679D+00 &
                                         /

!  Hafnium: (174)Hf (nuclide is unstable),
!           (175)Hf (nuclide is unstable), (176)Hf, (177)Hf, (178)Hf,
!           (179)Hf, (180)Hf.
      DATA ( TAWNUC( I ), I=311, 317 ) / 173.940044D+00,    TAP__DNULL, &
                                         175.941406D+00, 176.943217D+00, &
                                         177.943696D+00, 178.945812D+00, &
                                         179.946545D+00 /

!  Tantalum: (180)Ta (nuclide is unstable), (181)Ta.
      DATA ( TAWNUC( I ), I=318, 319 ) / 179.947462D+00, 180.947992D+00 &
                                         /

!  Tungsten: (180)W, (181)W (nuclide is unstable), (182)W, (183)W,
!            (184)W, (185)W (nuclide is unstable), (186)W.
      DATA ( TAWNUC( I ), I=320, 326 ) / 179.946701D+00,    TAP__DNULL, &
                                         181.948202D+00, 182.950220D+00, &
                                         183.950928D+00,    TAP__DNULL, &
                                         185.954357D+00 /

!  Rhenium: (185)Re, (186)Re (nuclide is unstable),
!           (187)Re (nuclide is unstable).
      DATA ( TAWNUC( I ), I=327, 329 ) / 184.952951D+00, TAP__DNULL, &
                                         186.955744D+00 /

!  Osmium: (184)Os, (185)Os (nuclide is unstable),
!          (186)Os (nuclide is unstable), (187)Os, (188)Os, (189)Os,
!          (190)Os, (191)Os (nuclide is unstable), (192)Os.
      DATA ( TAWNUC( I ), I=330, 338 ) / 183.952488D+00,    TAP__DNULL, &
                                         185.953830D+00, 186.955741D+00, &
                                         187.955860D+00, 188.958137D+00, &
                                         189.958436D+00,    TAP__DNULL, &
                                         191.961467D+00 /

!  Iridium: (191)Ir, (192)Ir (nuclide is unstable), (193)Ir.
      DATA ( TAWNUC( I ), I=339, 341 ) / 190.960584D+00, TAP__DNULL, &
                                         192.962917D+00 /

!  Platinum: (190)Pt (nuclide is unstable),
!            (191)Pt (nuclide is unstable), (192)Pt,
!            (193)Pt (nuclide is unstable), (194)Pt, (195)Pt, (196)Pt,
!            (197)Pt (nuclide is unstable), (198)Pt.
      DATA ( TAWNUC( I ), I=342, 350 ) / 189.959917D+00,    TAP__DNULL, &
                                         191.961019D+00,    TAP__DNULL, &
                                         193.962655D+00, 194.964766D+00, &
                                         195.964926D+00,    TAP__DNULL, &
                                         197.967869D+00 /

!  Gold: (197)Au.
      DATA ( TAWNUC( I ), I=351, 351 ) / 196.966543D+00 /

!  Mercury: (196)Hg, (197)Hg (nuclide is unstable), (198)Hg, (199)Hg,
!           (200)Hg, (201)Hg, (202)Hg, (203)Hg (nuclide is unstable),
!           (204)Hg.
      DATA ( TAWNUC( I ), I=352, 360 ) / 195.965807D+00,    TAP__DNULL, &
                                         197.966743D+00, 198.968254D+00, &
                                         199.968300D+00, 200.970277D+00, &
                                         201.970617D+00,    TAP__DNULL, &
                                         203.973467D+00 /

!  Thallium: (203)Tl, (204)Tl (nuclide is unstable), (205)Tl.
      DATA ( TAWNUC( I ), I=361, 363 ) / 202.972320D+00, TAP__DNULL, &
                                         204.974401D+00 /

!  Lead: (204)Pb, (205)Pb (nuclide is unstable), (206)Pb, (207)Pb,
!        (208)Pb.
      DATA ( TAWNUC( I ), I=364, 368 ) / 203.973020D+00,    TAP__DNULL, &
                                         205.974440D+00, 206.975872D+00, &
                                         207.976627D+00 /

!  Bismuth: (209)Bi.
      DATA ( TAWNUC( I ), I=369, 369 ) / 208.980374D+00 /

!  Polonium: Po.
!  Element has no stable nuclides.

!  Astatine: At.
!  Element has no stable nuclides.

!  Radon: Rn.
!  Element has no stable nuclides.

!  Francium: Fr.
!  Element has no stable nuclides.

!  Radium: Ra.
!  Element has no stable nuclides.

!  Actinium: Ac.
!  Element has no stable nuclides.

!  Thorium: (232)Th (nuclide is unstable).
!  Element has no stable nuclides.
      DATA ( TAWNUC( I ), I=370, 370 ) / 232.038054D+00 /

!  Protactinium: Pa.
!  Element has no stable nuclides.

!  Uranium: (234)U (nuclide is unstable), (235)U (nuclide is unstable),
!           (236)U (nuclide is unstable), (237)U (nuclide is unstable),
!           (238)U (nuclide is unstable).
!  Element has no stable nuclides.
      DATA ( TAWNUC( I ), I=371, 375 ) / 234.040946D+00, 235.043924D+00, &
                                            TAP__DNULL,    TAP__DNULL, &
                                         238.050784D+00 /

!  Table of element names.
      DATA ( TACNAM( I ), I = 1, 48 ) /  &
                    'Hydrogen',      'Helium',       'Lithium',  &
                    'Beryllium',     'Boron',        'Carbon', &
                    'Nitrogen',      'Oxygen',       'Fluorine', &
                    'Neon',          'Sodium',       'Magnesium', &
                    'Aluminium',     'Silicon',      'Phosphorus', &
                    'Sulphur',       'Chlorine',     'Argon', &
                    'Potassium',     'Calcium',      'Scandium', &
                    'Titanium',      'Vanadium',     'Chromium', &
                    'Manganese',     'Iron',         'Cobalt', &
                    'Nickel',        'Copper',       'Zinc', &
                    'Gallium',       'Germanium',    'Arsenic', &
                    'Selenium',      'Bromine',      'Krypton', &
                    'Rubidium',      'Strontium',    'Yttrium', &
                    'Zirconium',     'Niobium',      'Molybdenum', &
                    'Technetium',    'Ruthenium',    'Rhodium', &
                    'Palladium',     'Silver',       'Cadmium' /
      DATA ( TACNAM( I ), I = 49, TAP__NELEM ) /  &
                    'Indium',        'Tin',          'Antimony', &
                    'Tellurium',     'Iodine',       'Xenon', &
                    'Caesium',       'Barium',       'Lanthanum', &
                    'Cerium',        'Praseodymium', 'Neodymium', &
                    'Promethium',    'Samarium',     'Europium', &
                    'Gadolinium',    'Terbium',      'Dysprosium', &
                    'Holmium',       'Erbium',       'Thulium', &
                    'Ytterbium',     'Lutetium',     'Hafnium', &
                    'Tantalum',      'Tungsten',     'Rhenium', &
                    'Osmium',        'Iridium',      'Platinum', &
                    'Gold',          'Mercury',      'Thallium', &
                    'Lead',          'Bismuth',      'Polonium', &
                    'Astatine',      'Radon',        'Francium', &
                    'Radium',        'Actinium',     'Thorium', &
                    'Protoactinium', 'Uranium' /

!  Table of element symbols.
      DATA TACSYM / 'H',  'He', 'Li', 'Be', 'B',  'C',  'N',  'O',  &
                    'F',  'Ne', 'Na', 'Mg', 'Al', 'Si', 'P',  'S',  &
                    'Cl', 'Ar', 'K',  'Ca', 'Sc', 'Ti', 'V',  'Cr', &
                    'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', &
                    'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr', 'Y',  'Zr', &
                    'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', &
                    'In', 'Sn', 'Sb', 'Te', 'I',  'Xe', 'Cs', 'Ba', &
                    'La', 'Ce', 'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd', &
                    'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu', 'Hf', &
                    'Ta', 'W',  'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', &
                    'Tl', 'Pb', 'Bi', 'Po', 'At', 'Rn', 'Fr', 'Ra', &
                    'Ac', 'Th', 'Pa', 'U' /

!.

    END MODULE TAPC_CMN
