      SUBROUTINE BINF_PRED (W,PARS,F,DERIVS,NP)

!+
!
!  Programme:
!     BINF_PRED
!
!  Purpose:
!=    Returns the predicted flux at a given wavelength
!     and for a given combination of Teff(1,2), theta(1,2), Eb-v.
! 
!     This subroutine works on the basis that the wavelength
!     grid for observations and models is the same 
!  
!  Language:
!     Fortran 90
! 
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     27-May-1999: CSJ
!        original version
!
!-


!  Global variables:
      USE TFIT_MODELS


!  Subroutine arguments:
      REAL               ::  W       ! wavelength (given)
      REAL, DIMENSION(*) ::  PARS    ! parameters (given)
      REAL               ::  F       ! flux       (returned)
      REAL, DIMENSION(*) ::  DERIVS  ! derivatives wrt PARS (returned)
      INTEGER            ::  NP      ! number of parameters in PARS (given)


!  Local variables:
      REAL     ::  TEFF_1       ! Effective temperature (K) - primary
      REAL     ::  THETA_1      ! Angular diameter (radians) 
      REAL     ::  TEFF_2       ! Effective temperature (K) - secondary
      REAL     ::  THETA_2      ! Angular diameter (radians)
      REAL     ::  EBV          ! E(B-V) extinction coefficient


      REAL     ::  FTTAB(NTMAX) ! Single wavelength flux holder
      REAL     ::  F1           ! Flux from atmosphere - primary
      REAL     ::  F2           ! Flux from atmosphere - secondary
      REAL     ::  DF           ! Flux for calculating derivatives
      REAL     ::  DF1DT        ! Temperature derivative of flux
      REAL     ::  DF2DT        ! Temperature derivative of flux
      REAL     ::  DRDE         ! Reddening derivative of extinction


!  Local constants:
      DOUBLE PRECISION, PARAMETER :: TAPM__PI = 3.141592653589793238462643383279503D+00

!.

!   Currently assume NG = NH = 1
      NG = 1
      NH = 1

!   Map local values of parameters
      TEFF_1 = PARS(1); TEFF_2 = PARS(2)
      THETA_1= PARS(3); THETA_2 = PARS(4) 
      EBV = PARS(5) 

!   Identify NW from wavelength
      DO NW = 1,NWMOD
         NWX = NW
         IF (ABS((W-WMOD(NW))/W)<0.000001) EXIT
      END DO

!   Interpolate model fluxes in grid at Teff_1
      FTTAB(1:NTMOD) = LOG ( FMOD(NWX,1:NTMOD,NG,NH) )
      CALL TABINT (NTMOD,TMOD,FTTAB,4,TEFF_1,     F0)
      CALL TABINT (NTMOD,TMOD,FTTAB,4,TEFF_1*1.01,DF)
      F1 = EXP(F0); DF = EXP(DF)
      DF1DT  = (DF-F1)/(0.01*TEFF_1)

!   Interpolate model fluxes in grid at Teff_2
!!      FTTAB = LOG ( FMOD(NWX,:) )
      CALL TABINT (NTMOD,TMOD,FTTAB,4,TEFF_2,     F0)
      CALL TABINT (NTMOD,TMOD,FTTAB,4,TEFF_2*1.01,DF)
      F2 = EXP(F0); DF = EXP(DF)
      DF2DT  = (DF-F2)/(0.01*TEFF_2)

!   Calculate insterstellar reddening
      CALL ISRED (W,EBV,     RED, A, 3.1,0)
      CALL ISRED (W,EBV+0.01,REDX,A, 3.1,0)
      DRDE      = (REDX-RED)/0.01
         
!   Multiply by angular radii squared, add and redden
      F         = ( F1 * THETA_1**2 + F2 * THETA_2**2 ) * RED
      DERIVS(1) = ( DF1DT * THETA_1**2 ) * RED 
      DERIVS(2) = ( DF2DT * THETA_2**2 ) * RED 
      DERIVS(3) = ( F1 * 2 * THETA_1 ) * RED
      DERIVS(4) = ( F2 * 2 * THETA_2 ) * RED
      DERIVS(5) = F * DRDE / RED 

      END SUBROUTINE BINF_PRED




