SUBROUTINE ISLYA ( WL, EBV, NH, LYA )

  !+
  !
  !  Program.
  !     ISLYA
  !
  !  Purpose.
  !=    Calculate the Ly alpha profile at wavelength WL
  !     for reddening EBV or hydrogen atom density NH.
  !
  !  References.
  !     Groenewegen M.A.T., Lamers H.J.G.L.M., 1989: A&A Supp. 79, 359.
  !     Bohlin R.C., Savage B.D., Drake J.F., 1978, ApJ 224, 132.
  !
  !  Authors.
  !     C.S.Jeffery (CSJ) Armagh Observatory
  !     D.R.Morgan (DRM) Armagh Observatory
  !
  !  History.
  !     21-MAR-1996 (CSJ): original version
  !     25-FEB-2005 (DRM): conversion to subroutine for SFIT
  !
  !-

  IMPLICIT NONE

  !  Subroutine Arguments (Given).
  REAL, INTENT(IN)    :: WL       ! Wavelength (Angstroms)
  REAL, INTENT(IN)    :: NH       ! Hydrogen column density (cm^-2)
  REAL, INTENT(IN)    :: EBV      ! Extinction E(B-V)

  !  Subroutine Arguments (Returned).
  REAL, INTENT(OUT)   :: LYA      ! Line profile at WL

  !  Local Variables
  REAL       :: DW, FF, NHX
  REAL, PARAMETER :: DW_CUT = 500. ! Ignore line beyond this limit

  !.


  !  Default
  LYA = 1.0

  !  Wavelength relative to line centre
  DW = ABS(WL - 1215.67)

  !  Check whether near line
  IF (DW < DW_CUT) THEN

    !  Default at line centre
    LYA = 0.0

    !  Convert from E(B-V) to NH given the Bohlin et al. relation.
    NHX = NH
    IF (NH < 10.0) NHX = EBV*(10.**21.68)

    !  Calculate the Ly-alpha exinction profile (Lorentzian)
    FF = -7.0562E-11 * NHX/( 1. + 1.6574E9*DW*DW )
    IF (FF > -80) LYA = EXP(FF)

  ENDIF

END SUBROUTINE ISLYA
