!!!! Name    : mod_tetra_list.f90
!!!! Author  : Chris Winter
!!!!
!!!! Description: 
!!!!   List of tetrahedra generated by tetgen.
!!!!

module mod_tetra_list

  use mod_model_list
  use sfit_mod_tgh
  use sfit_obsdata

  use iso_c_binding

  implicit none

  !!! Interface to C helper routine
  interface
     subroutine c_tetgen(points, npoints, tetra, ntetra, neighbours) bind(C,name='tetgen')
        use iso_c_binding
        implicit none
        ! next two lines changed from 3.1.0: 
        ! type(c_ptr), value, intent(in) :: points
        ! type(c_ptr), value, intent(in) :: npoints
        ! type(c_ptr) :: tetra
        ! type(c_ptr) :: ntetra
        ! type(c_ptr) :: neighbours
        real(kind=c_double), intent(in) :: points(*)
        integer(c_int), value, intent(in) :: npoints
        integer(c_long) :: tetra
        integer(c_int) :: ntetra
        integer(c_long) :: neighbours
     end subroutine c_tetgen
  end interface


  private
  public :: tetobj, tetralist, current_tetrahedron, tetralist_new, &
            tetralist_destroy, tetralist_length, tetralist_nth, tetralist_print, &
            tetralist_load, tetralist_save, tetrahedralise, locate_point, &
            tetralist_interpolate, tetralist_point_in_hull


  !!! Type Definitions
  type tetobj
     integer, dimension(4) :: corners     !!! Indices into model list
     integer, dimension(4) :: neighbours  !!! Indices into tetrahedra list
     double precision, dimension(3) :: centroid
  end type tetobj


  !!! SFIT2 Global variables
  type(tetobj), dimension(:), pointer :: tetralist
  integer :: current_tetrahedron = -1  ! Current tetrahedron. Initial value = -1


  !!! Procedure Definitions
  contains

    !!! Create a new tetralist with data returned from tetgen
    subroutine tetralist_new(mlist, tetra, ntetra, nbrlist, tlist)
      type(modlist), pointer :: mlist
      integer :: tetra(*), nbrlist(*)
      integer :: ntetra, i, j, k
      type(tetobj), dimension(:), pointer :: tlist

      allocate( tlist(ntetra) )

      !!! Copy across the data
      j = 1
      k = 4
      do i = 1, ntetra
         tlist(i)%corners = tetra(j:k)
         tlist(i)%neighbours = nbrlist(j:k)
         call tetrahedron_centroid(tlist(i), mlist)

         j = j + 4
         k = k + 4
      end do
    end subroutine tetralist_new


    subroutine tetralist_destroy(l)
      type(tetobj), dimension(:), pointer :: l

      deallocate(l)
    end subroutine tetralist_destroy


    integer function tetralist_length(l)
      type(tetobj), dimension(:), pointer :: l

      tetralist_length = 0

      if( associated(l) ) then
         tetralist_length = size(l)
      endif
    end function tetralist_length


    !!! Returns a pointer ELEMENT to the NTH element of LIST.
    !!! NTH = 0 refers to the first element in the list.
    subroutine tetralist_nth(l, nth, element)
      type(tetobj), dimension(:), pointer :: l
      type(tetobj) :: element
      integer :: nth

      if( associated(l) ) then
         !!! Access using nth + 1 because the list is actually
         !!! an array.
         element = l(nth + 1)
      end if
    end subroutine tetralist_nth


    !!! Traverse a list and print out each struct
    subroutine tetralist_print(l)
      type(tetobj), dimension(:), pointer :: l
      type(tetobj) :: ith
      integer :: i

      if(.not. associated(l)) then
         print *, "*NULL*"
      else
         do i = 1, tetralist_length(l)
            call tetralist_nth(l, i, ith)

            print *, "Corners: ", ith%corners, " Neighbours: ", ith%neighbours, " Centroid: ", ith%centroid
         end do
      endif
    end subroutine tetralist_print


    !!! All-in-one routine to generate a tetrahedralisation from
    !!! a list of models.
    subroutine tetrahedralise(models, tlist)
      type(modlist), pointer :: models
      type(tetobj), dimension(:), pointer :: tlist

      ! changed type of modlist-ptr  and other pointers to variables matching type in modlist_flatten
      ! 2019/2/13
      ! integer :: ntetrahedra, modlist_ptr
      integer :: ntetrahedra
      real(kind=c_double), allocatable :: modlist_ptr(:)
      integer :: tetrahedra(*), neighbours(*)

      pointer(tetra, tetrahedra)
      pointer(nbrs, neighbours)

      call modlist_flatten(models, modlist_ptr)
      write (*,*) 'tetrahedralise: modlist_flatten complete'

      call c_tetgen(modlist_ptr, modlist_length(models), tetra, ntetrahedra, nbrs)
      write (*,*) 'tetrahedralise: c_tetgen complete'

      call tetralist_new(models, tetrahedra, ntetrahedra, neighbours, tlist)
      write (*,*) 'tetrahedralise: tetralist_new complete'

    end subroutine tetrahedralise


    !!! Load a tetrahedralisation from a file already opened for reading
    subroutine tetralist_load(unit, tlist)
      type(tetobj), dimension(:), pointer :: tlist
      integer :: unit, i, length
      type(tetobj) :: nth

      !!! Read list length
      read (unit) length

      !!! Create list
      allocate( tlist(length) )

      do i = 1, length
         !!! Read corners
         read (unit=unit) tlist(i)%corners
            
         !!! Read neighbours
         read (unit=unit) tlist(i)%neighbours

         !!! Read centroid
         read (unit=unit) tlist(i)%centroid
      end do
    end subroutine tetralist_load

    subroutine tetralist_save(tlist, unit)
      !!! Saves a tetrahedralisation to a file already opened for writing
      type(tetobj), dimension(:), pointer :: tlist
      integer :: unit, i

     print *,"TETRALIST_SAVE: length =", tetralist_length(tlist)
     print *,"TETRALIST_SAVE: tlist =", associated(tlist)
    if( associated(tlist) .and. tetralist_length(tlist) .gt. 0) then
         !!! Write list length
          write (unit) tetralist_length(tlist)

         do i = 1, tetralist_length(tlist)               
            !!! Write corners
            write (unit) tlist(i)%corners
            
            !!! Write neighbours
            write (unit) tlist(i)%neighbours

            !!! Write centroid
            write (unit) tlist(i)%centroid
         end do
      endif
    end subroutine tetralist_save


    !!! Test to see if a point lies within the convex hull of the
    !!! tetrahedralization.
    !!!
    !!! USE WITH CAUTION! - May not work as advertised.
    !!!
    logical function tetralist_point_in_hull(point, tlist, mlist)
      double precision, dimension(:) :: point !!! Optimiser's current point
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist

      double precision, dimension(3) :: p
      integer :: tet, error, start
      double precision, dimension(4) :: weights

      tetralist_point_in_hull = .true.

      start = -1

      !!! Before locating point, scale the params accordingly
      call modlist_scale_point(mlist, point, p)

      if ( .not. locate_point(start, p, tlist, mlist, tet, weights, error) ) then
         if ( error .eq. 1 ) then
            tetralist_point_in_hull = .false.
            return
         end if
      end if
    end function tetralist_point_in_hull


    !!! Locate and interpolate linearly to a point somewhere in the
    !!! tetrahedralisation.
    subroutine tetralist_interpolate(point, tlist, mlist, ftype_r, fluxes)
      double precision, dimension(:) :: point !!! Optimiser's current point
      real :: ftype_r                        !!! Flux type to use: AMOD=1, FMOD=2, CMOD=3
      integer :: ftype
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist
      real, dimension(:) :: fluxes            !!! Interpolated, resampled fluxes

      real, dimension(nwmod) :: interp_tmp
      double precision, dimension(3) :: p
      integer :: tet, i, nw, error
      double precision, dimension(4) :: weights

      type(tetobj) :: tet_nth

      real, dimension(:,:), pointer :: models

      !!! Initialisation
      interp_tmp = 0.0
      allocate( models(nwmod, 4) )

      ftype = 1
      if (ftype_r .eq. 2.0) then
        ftype = 2
      else if (ftype_r .eq. 3.0) then
        ftype = 3
      end if

      !!! Before locating point, scale the params accordingly
      call modlist_scale_point(mlist, point, p)
      
      !!! Locate point in tetrahedralisation. Use weights
      !!! to determine which models to read, and for doing
      !!! the interpolation.
      if ( .not. locate_point(current_tetrahedron, p, tlist, mlist, tet, weights, error) ) then
         if ( error .eq. 1 ) then
            return
         end if
      end if

      current_tetrahedron = tet

      call tetralist_nth(tlist, tet, tet_nth)

      !!! Read in the models
      do i = 1, 4
         call modlist_read_model(mlist, tet_nth%corners(i), ftype, models(1:nwmod, i))
      end do

       !!! Do the interpolation.
      interp_tmp = ( weights(1) * models(1:nwmod, 1) ) + &
                   ( weights(2) * models(1:nwmod, 2) ) + &
                   ( weights(3) * models(1:nwmod, 3) ) + &
                   ( weights(4) * models(1:nwmod, 4) )

      !!! Resample interpolated fluxes onto observation wavelength grid
      call resample(wmod, interp_tmp, nwmod, wobs, fluxes(1:nwobs), nwobs, 1.0)

      !!! Tidy up
      deallocate(models)
    end subroutine tetralist_interpolate



    !!! ***************************************** !!!
    !!! ***                                   *** !!!
    !!! ***  Computational geometry routines  *** !!!
    !!! ***                                   *** !!!
    !!! ***  Refer to Winter (2006) for full  *** !!!
    !!! ***  details                          *** !!!
    !!! ***                                   *** !!!
    !!! ***************************************** !!!


    logical function locate_point(start, point, tlist, mlist, tet, weights, error)
      !!! start   - index of starting tetrahedron
      !!! point   - point we want to find enclosing tetrahedron
      !!! tlist   - list of tetrahedra
      !!! mlist   - list of models
      !!! tet     - index of tetrahedron containing point
      !!! weights - barycentric coordinates of point within tetrahedron

      integer :: start, tet, error, try
      double precision, dimension(3) :: point
      double precision, dimension(4) :: weights
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist

      locate_point = .true.

      !!! if start == -1, we need to find the starting point by calling
      !!! find_starting_point
      if (start .eq. -1) then
         !!! Locate a suitable starting point
         call find_starting_point(point, tlist, mlist, start)
      end if 

      !!! Traverse the tetrahedralisation until enclosing tetrahedron
      !!! is found. Sets global variable current to enclosing tetrahedron
      call walk(start, point, tlist, mlist, tet, weights, error)

      !!! Check error code
      if ( error .gt. 0 ) then
            !!! As it happens, walk can get 'disoriented' and
            !!! erroneously report that a point is not inside the convex hull.
            !!! We work around this by generating a new starting point, then attempting
            !!! the walk from there. Repeated 10 times, just to be sure.
            
            do try = 1, 20
               start = -1
               error = 0
               call find_starting_point(point, tlist, mlist, start)
               call walk(start, point, tlist, mlist, tet, weights, error)

               if (error .eq. 0) exit
            end do

            if (try .gt. 20 .and. error .ne. 0) then
               !print *, "locate_point: Cannot find point"
               !print *, "locate_point: ", point

               !call modlist_unscale_point(mlist, point, point)

                !print *, point

               locate_point = .false.
            end if
         !end if
      end if
    end function locate_point


    subroutine walk(start, point, tlist, mlist, tet, weights, error)
      !!! start   - index of starting tetrahedron
      !!! point   - point we want to find enclosing tetrahedron
      !!! tlist   - list of tetrahedron objects
      !!! mlist   - list of models
      !!! tet     - index of tetrahedron containing point
      !!! weights - barycentric coordinates of point within tetrahedron
      !!! error   - signals if point cannot be located

      integer :: start, current, tet, error
      double precision, dimension(3) :: point
      double precision, dimension(4) :: weights
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist

      double precision, dimension(3) :: orig, dir, dir_norm, v1, v2, v3, v4
      double precision :: u, v
      type(model) :: mod_nth
      type(tetobj) :: tet_nth
      integer :: nbr, i

      logical, dimension(4) :: intersect

      intersect = .false.
      error = 0

      !!! We maintain the origin and direction of the ray established
      !!! from the starting tetrahedron. Consequently, we can't use the
      !!! two-sided triangle part of the MT intersection code.
      call tetralist_nth(tlist, start, tet_nth)

      !!! Set up ray variables:
      !!!   - Origin is starting tetrahedron's centroid
      !!!   - Direction is normalised, translated point vector
      orig = tet_nth%centroid

      dir = (point - orig)
      dir_norm = dir / vec_length(dir)

      !!! Initialise
      current = start

      do
         !!! If point in current tetrahedron, we're done
         if ( point_in_tetrahedron(point, current, tlist, mlist, weights) ) then
            tet = current

            exit
         else
            !!! Find out which neighbour we must traverse to
            nbr = -1

            call tetralist_nth(tlist, current, tet_nth)

            !!! Set up vertices
            call modlist_nth(mlist, tet_nth%corners(1), mod_nth)
            v1 = mod_nth%data
            call modlist_nth(mlist, tet_nth%corners(2), mod_nth)
            v2 = mod_nth%data
            call modlist_nth(mlist, tet_nth%corners(3), mod_nth)
            v3 = mod_nth%data
            call modlist_nth(mlist, tet_nth%corners(4), mod_nth)
            v4 = mod_nth%data

            !!! Check which face is intersected. Several faces could be
            !!! intersected simultaneously, so choose the one which does
            !!! not lie on the convex hull

            !!! Assume error code 0: everything's ok
            error = 0
            if ( ray_triangle_intersect(orig, dir_norm, v2, v4, v3, u, v) ) then
               if ( tet_nth%neighbours(1) .ne. -1 ) then
                  nbr = 1
               else
                  error = 1 ! Point is outside convex hull
                  weights(1) = 0.0
                  weights(2) = (1.0 - u - v)
                  weights(4) = u
                  weights(3) = v

                  tet = current
                  
                  return
               end if
            else if ( ray_triangle_intersect(orig, dir_norm, v1, v3, v4, u, v) ) then
               if ( tet_nth%neighbours(2) .ne. -1 ) then
                  nbr = 2
               else
                  error = 1 ! Point is outside convex hull
                  weights(1) = (1.0 - u - v)
                  weights(2) = 0.0
                  weights(3) = u
                  weights(4) = v

                  tet = current

                  return
               end if
            else if ( ray_triangle_intersect(orig, dir_norm, v1, v4, v2, u, v) ) then
               if ( tet_nth%neighbours(3) .ne. -1 ) then
                  nbr = 3
               else
                  error = 1 ! Point is outside convex hull
                  weights(1) = (1.0 - u - v)
                  weights(2) = v
                  weights(3) = 0.0
                  weights(4) = u

                  tet = current

                  return
               end if
            else if ( ray_triangle_intersect(orig, dir_norm, v1, v2, v3, u, v) ) then
               if ( tet_nth%neighbours(4) .ne. -1 ) then
                  nbr = 4
               else
                  error = 1 ! Point is outside convex hull
                  weights(1) = (1.0 - u - v)
                  weights(2) = u
                  weights(3) = v
                  weights(4) = 0.0

                  tet = current

                  return
               end if
            else
               error = 2  !!! No rays intersect
            end if
            
            !!! Check error value
            if ( error .gt. 0) return
            
            !!! Traverse to chosen neighbouring tetrahedron
            current = tet_nth%neighbours(nbr)
         end if
      end do
    end subroutine walk


    subroutine find_starting_point(point, tlist, mlist, start)
      !!! point - Optimiser's starting point
      !!! tlist - array of tetrahedron objects
      !!! mlist - list of models
      !!! start - index of tetrahedron containing starting point

      !!! The algorithm starts by randomly sampling tetlist, then
      !!! distance tests each tetrahedron to find closest to starting
      !!! point. Enclosing tetrahedron is found by walking tetlist

      double precision, dimension(3) :: point
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist
      integer :: start

      type(tetobj) :: tet_nth
      double precision :: mindist
      integer :: nselect, mindist_idx, i
      integer, allocatable, dimension(:) :: sample
      real, allocatable, dimension(:) :: real_sample
      double precision, allocatable, dimension(:) :: dists
  
      !!! Number of tetrahedra to select (modification of formula from
      !!! sect. 5 of Mucke, Saias, and Zhu (1996) )
      nselect = ceiling(2 * (tetralist_length(tlist) ** 0.3) )

      !!! Allocate arrays
      allocate(real_sample(nselect), sample(nselect), dists(nselect))

      !!! Generate random sample and distance array. Look for mindist
      !!! on the fly.
      call random_seed()
      call random_number(real_sample)

      mindist = -1
      do i = 1, nselect
         sample(i) = int(real_sample(i) * tetralist_length(tlist))

         call tetralist_nth(tlist, sample(i), tet_nth)
         dists(i) = sqrd_distance(tet_nth%centroid, point)

         if ( mindist .eq. -1 .OR. dists(i) .lt. mindist) then
            mindist = dists(i)
            mindist_idx = sample(i)
         end if
      end do

      !!! Closest tetrahedron
      start = mindist_idx
    end subroutine find_starting_point


    logical function point_in_tetrahedron(point, tet, tlist, mlist, weights)
      !!! point - location in 3D space we want to test for
      !!! tet   - index of tetrahedron we want to test
      !!! tlist - array of tetrahedron objects
      !!! mlist - list of models
      !!! weights - barycentric coordinates of point within tetrahedron

      double precision, dimension(3) :: point
      integer :: tet
      type(tetobj), dimension(:), pointer :: tlist
      type(modlist), pointer :: mlist
      double precision, dimension(4) :: weights

      type(model) :: mod_nth
      type(tetobj) :: tet_nth
      integer :: i
      integer, dimension(4) :: perm  !! Not used. For CSJ's nr_invert subroutine
      double precision, dimension(4,4) :: smatrix, sm_inv  ! Simplex matrix
      double precision, dimension(4)   :: p                ! Point
      double precision :: epsilon

      !!! Initialise
      point_in_tetrahedron = .false.

      call tetralist_nth(tlist, tet, tet_nth)

      epsilon = 1.0E-05
      smatrix = 1.0
      p = 1.0

      p(1:3) = point  ! Copy over point

      !!! Build simplex matrix
      do i = 1, 4
         call modlist_nth(mlist, tet_nth%corners(i), mod_nth)
         smatrix(1,i) = mod_nth%data(1)
         smatrix(2,i) = mod_nth%data(2)
         smatrix(3,i) = mod_nth%data(3)
      end do

      !!! Invert and solve
      call nr_invert(smatrix, 4, 4, sm_inv, perm)
      weights = matmul(sm_inv, p)
      !weights = p

      if ( ( (1.0 - sum(weights)) .le. 1.0 ) &
           .AND. ( (1.0 - sum(weights)) .gt. -epsilon) &
           .AND. ( all(weights .le. (1.0 + epsilon)) ) &
           .AND. ( all(weights .gt. -epsilon) ) ) then

         point_in_tetrahedron = .true.

      end if

    end function point_in_tetrahedron


    logical function ray_triangle_intersect(orig, dir, v0, v1, v2, u, v)
      !!! Implements the M\"oller & Trumbore ray/triangle intersection
      !!! algorithm. Culling branch only.
      double precision, dimension(3) :: orig, dir, v0, v1, v2
      double precision, dimension(3) :: edge1, edge2, tvec, pvec, qvec
      double precision :: det, inv_det, epsilon, t, u, v
      
      epsilon = 0.000001

      ray_triangle_intersect = .false.

      !!! find vectors for two edges sharing v0
      edge1 = v1 - v0
      edge2 = v2 - v0

      !!! begin calculating determinant - also used to calculate U parameter
      call cross_product(dir, edge2, pvec)

      !!! if determinant is near zero, ray lies in plane of triangle
      det = dot_product(edge1, pvec)

      !!! The culling branch of MT algorithm
      if (det < epsilon) return

      !!! calculate distance from v0 to ray origin
      tvec = orig - v0

      !!! calculate U parameter and test bounds
      u = dot_product(tvec, pvec)
      if (u < 0.0 .or. u > det) return

      !!! prepare to test V parameter
      call cross_product(tvec, edge1, qvec)

      !!! calculate V parameter and test bounds
      v = dot_product(dir, qvec)
      if (v < 0.0 .or. (u + v) > det) return

      !!! calculate t, scale parameters, ray intersects triangle.
      !!! As we're only interested in whether an intersection occurs,
      !!! and not in actual t,u,v values, we can skip these calculations.
      !!!
      !!! t = dot_product(edge2, qvec)
      inv_det = 1.0 / det
      !!! t = t * inv_det
      u = u * inv_det
      v = v * inv_det

      ray_triangle_intersect = .true.
    end function ray_triangle_intersect


    subroutine tetrahedron_centroid(tet, mlist)
      !!! Compute the centre of gravity for a given tetrahedron
      type(tetobj) :: tet
      type(modlist), pointer :: mlist

      type(model) :: mod_nth
      integer :: i

      tet%centroid = 0.0

      !!! Centroid is average of four corners
      do i = 1, 4
         call modlist_nth(mlist, tet%corners(i), mod_nth)
         tet%centroid = tet%centroid + mod_nth%data
      end do

      tet%centroid = tet%centroid / 4.0
    end subroutine tetrahedron_centroid



    !!! Basic linear algebra routines

    subroutine cross_product(a, b, c)
      double precision :: a(3), b(3), c(3)

      c(1) = (a(2) * b(3)) - (a(3) * b(2))
      c(2) = (a(3) * b(1)) - (a(1) * b(3))
      c(3) = (a(1) * b(2)) - (a(2) * b(1))
    end subroutine cross_product

    double precision function vec_length(v)
      double precision, dimension(3) :: v
      
      vec_length = 0.0
      
      vec_length = sqrt( v(1)**2 + v(2)**2 + v(3)**2 )
    end function vec_length

    double precision function sqrd_distance(v1, v2)
      double precision, dimension(3) :: v1, v2

      sqrd_distance = ((v1(1) - v2(1)) ** 2) + &
                      ((v1(2) - v2(2)) ** 2) + &
                      ((v1(3) - v2(3)) ** 2)
    end function sqrd_distance

end module mod_tetra_list
