      SUBROUTINE REBIN ( X1,Y1,S1,N1, X2,Y2,S2,N2, MODE )

!+
!
!  Program:
!     rebin
!
!  Purpose:
!=    resample data in a function on x-values x1 onto x-values x2.
!     In general, x1 should be better sampled than x2, otherwise
!     the results are unpredictable. 
!
!     Data are assumed to be associated with an error (s1). 
!     Output errors (s2) may either be the average value of s1 (mode 0)
!     or the standard deviation in the y-values in each bin (mode 1) 
!     
!  Language:
!     Fortran 90
! 
!
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     27-May-1999: CSJ
!        original version
!
!
!-

!  Subroutine arguments:
      REAL     :: X1(*)              !  input x-values  (given)
      REAL     :: Y1(*)              !  input y-values  (given)
      REAL     :: S1(*)              !  input error values  (given)
      INTEGER  :: N1                 !  number of input x-values

      REAL     :: X2(*)              !  x-values to bin (given)
      REAL     :: Y2(*)              !  binned y-values  (returned)
      REAL     :: S2(*)              !  binned error values  (returned)
      INTEGER  :: N2                 !  number of bin x-values

      INTEGER  :: MODE               !  type of error values required  

!  Local variables:

!.

!  Preset output data
      Y2(1:N2) = 0.
      SELECT CASE (MODE); CASE (0); S2(1:N2) = 0.
                          CASE (1); S2(1:N2) = 1. ; END SELECT
      

!  Loop over bins in output spectrum
      J1 = 1
      BINS: DO I = 1,N2


!     identify bin limits
        IF (I==1) THEN
          XLO = X2(I) - (X2(I+1)-X2(I)) / 2.
          XHI = (X2(I) + X2(I+1)) / 2.
        ELSE IF (I==N2) THEN 
          XLO = XHI
          XHI = XLO + DX2
        ELSE
          XLO = XHI
          XHI = (X2(I) + X2(I+1)) / 2.
        END IF
        DX2 = XHI - XLO


!     set summations to zero
        SY  = 0.;  SY2 = 0.;  NS = 0
        SS  = 0.;  SS2 = 0.


!     loop through data in spectrum to be binned
        INPUT: DO J = J1,N1


!       identify end of valid data
          IF (X1(J)>XHI) THEN
            IF (NS>0) THEN
              Y2(I) = SY / NS
              SELECT CASE (MODE); CASE (0); S2(I) = SS / NS
                                  CASE (1); S2(I) = SQRT ( ABS (SY2/NS - (SY/NS)*(SY/NS) ) )
              END SELECT 
            ELSE
              Y2(I) = 0.
              SELECT CASE (MODE); CASE (0); S2(I) = 0.
                                  CASE (1); S2(I) = 1.
              END SELECT 
            ENDIF
            J1 = J
            EXIT INPUT

!       identify valid data for this bin
          ELSEIF (X1(J)>XLO) THEN 
            SY  = SY  + Y1(J)
            SY2 = SY2 + Y1(J)*Y1(J)
            SS  = SS  + S1(J)
            SS2 = SS2 + S1(J)*S1(J)
            NS  = NS  + 1
          ENDIF

        END DO INPUT

      END DO BINS


!  Check output spectrum for bad values -- assuming they are isolated
      FORALL ( I = 2:N2-1, ABS(Y2(I)) < 1.E-30 ) 
         Y2(I) = Y2(I-1) + (X2(I)-X2(I-1)) * (Y2(I+1)-Y2(I-1)) / (X2(I+1)-X2(I-1))
         S2(I) = 0.5 * (S2(I-1)+S2(I+1))
      ENDFORALL
      FORALL ( I = 2:N2-1, ABS(S2(I)) < 1.E-30 ) 
         S2(I) = 0.5 * (S2(I-1)+S2(I+1))
      ENDFORALL

      END SUBROUTINE REBIN
