SUBROUTINE resample (x, y, nx, xx, yy, nxx, mode)

  !+
  !
  !  Program:
  !     RESAMPLE
  !
  !  Purpose:
  !=    Resample a function y(x) onto  a new set of x-values xx.
  !     The new function is returned as yy. nx and nxx are the
  !     number of values in arrays x and xx respectively.
  !
  !  Method: 
  !     If the resampling interval is smaller than the data interval, 
  !     the result is linearly interpolated.
  !     If the resampling interval is larger than the data interval,
  !     the result is the average flux in corresponding data bins,
  !     with the boundary data bins appropriately weighted
  !     If the resampled function specifies points outside the given data
  !     the missing points are set = "mode" EXCEPT if "mode == 0" in 
  !     which case the missing data is set = to the given data at the
  !     appropriate upper or lower boundary
  !    
  !  Accuracy:
  !     The program was tested by rebinning a theroretical
  !     spectrum sampled very unevenly (dx = 0.001A in line cores,
  !     increasing to ~1 A in continuum regions) onto a
  !     slightly uneven observed grid with <dx>~0.225 A.
  !     Integrating the equivalent width over several lines
  !     provided accuracy of 0.01-0.1 per cent, with the error
  !     increasing with the number of lines.  (CSJ: 2000-FEB-09)
  !  
  !  Language:
  !     Fortran 90
  !
  !  Author:
  !     CSJ: Simon Jeffery, Armagh Observatory
  !     (adapted from a similar routine written for IDL)
  !
  !  History
  !     20-02-2000: CSJ 
  !        Original IDL code
  !     06-01-2005: CSJ 
  !        Fixed bug in end-value corrections
  !     17-04-2106: CSJ
  !        Added extra comments in the preamble
  !     11-06-2021: CSJ
  !        Identified major error causing 1 % errors in output
  !        Ensure internal arithmetic done in double precision
  !        Still failed -- USE remap from DIPSO
  !
  !
  !-

  !  Switch off global typing
  IMPLICIT NONE

  !  Subroutine arguments (given).
  REAL, DIMENSION(*) :: x     ! input x values 
  REAL, DIMENSION(*) :: y     ! input y values
  INTEGER            :: nx    ! number of x values

  REAL, DIMENSION(*) :: xx    ! output xx values
  INTEGER            :: nxx   ! number of xx values

  REAL               :: mode  ! default value outside valid range

  !  Subroutine arguments (returned).
  REAL, DIMENSION(*) :: yy    ! output yy values

  !  Local variables.
  DOUBLE PRECISION   :: dx, xg, dg, s
  DOUBLE PRECISION   :: b1,b2
  INTEGER            :: ix, jx 
  INTEGER            :: c1,c2

  !.


  write (*,*) ' Warning -- bugs here: use REMAP '
  
  c1 = 1
  DO ix = 1,nxx  ! loop over new grid

     IF (xx(ix) >= x(1) .AND. xx(ix) <= x(nx)) THEN 

        IF (ix == 1) THEN
           dx = 0.5*(xx(2)-xx(1))  
           b1 = xx(1)-dx    ;  b2 = b1+2*dx
        ELSEIF (ix == nxx) THEN
           dx = 0.5*(xx(nxx)-xx(nxx-1))
           b1 = xx(nxx)-dx  ;  b2 = b1+2*dx
        ELSE
           b1 = 0.5*(xx(ix-1)+xx(ix))  ;  b2 = 0.5*(xx(ix)+xx(ix+1))
        ENDIF
        c2 = 1

        DO jx = MAX(2,c1),nx-1        ! locate data
           xg = x(jx) 
           dg = 0.5*(xg-x(jx-1))
           IF (xg <= b1-dg) c1 = jx+1
           dg = 0.5*(x(jx+1)-xg)
           IF (xg <= b2+dg) c2 = jx
           IF (xg > b2+dg) EXIT
        END DO


        IF (c2 > c1) THEN                      ! bin up (trapezoidal integration)

           s = y(c1)*(0.5*(x(c1)+x(c1+1))-b1)
           IF (c2 > c1+1) THEN 
              DO jx = c1+1,c2-1
                 s = s + y(jx)*0.5*(x(jx+1)-x(jx-1))  
              END DO
           ENDIF
           s = s + y(c2)*(b2-0.5*(x(c2-1)+x(c2)))
           s = s / (b2-b1)    

        ELSEIF  ( x(c1+1) /= x(c1) ) THEN         ! interpolate

           c1 = MIN( c1, nx-1 )
           s = y(c1) + (y(c1+1)-y(c1))*(xx(ix)-x(c1))/(x(c1+1)-x(c1))

        ELSE                                   ! average

           s = 0.5*(y(c1)+y(c1+1))

        ENDIF


     ELSE 

        !  assign values outside valid x range
        s = mode
        IF ( abs(mode)<1.e-6 ) THEN
           IF (xx(ix) < x(1) ) s = y(1) 
           IF (xx(ix) > x(nx)) s = y(nx) 
        ENDIF

     ENDIF

     yy(ix) = REAL(s)

  END DO

END SUBROUTINE resample
