SUBROUTINE SFIT_MAKEGRID  ( MODE, GRDFILE, NWMAX, NWMOD, WMOD)

  !+
  !  Program:
  !     sfit_makegrid
  !
  !  Purpose:
  !     Given a list of files on stdin (unit *), create a 3 dimensional 
  !     grid of model spectra. Models are read, resampled, and written
  !     to a direct access file. Model parameters and file record numbers
  !     are stored in a linked list which is then passed to the kdtree
  !     routines to build the partition tree index. This index is then
  !     saved off as "GRIDNAME.idx". This subroutine was adapted from
  !     SFIT_RDGRID
  !
  !     Different types of model grid may be specified:
  !     MODE = 0:   Spectrum.000  (including fluxes, continuum and residual)
  !     MODE = 1:   Spectrum.001  (residual or normalized flux only)
  !     MODE = 2:   Sterne.s      (total flux only)
  !     MODE = 3:   Spectrum.sp2  (total flux, parameters not in header)
  !     MODE = 4:   Synthe        (including fluxes, continuum and residual)
  !     MODE = 5:   Synthe        (residual or normalized flux only)
  !     MODE = 6:   Spectrum.sp4  (header, fluxes, continuum, residual)
  !
  !     Format of data on stdin must be as follows
  !     NTMOD NGMOD NHMOD
  !     filename[1]
  !     filename[2]
  !     ...
  !     filename[NT*NG*NH]
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Reference:
  !
  !  Authors:
  !     Original code from SFIT_RDGRID - C.S.Jeffery (CSJ) (Armagh Observatory)
  !     Code to create on-disk index   - C. Winter (CWR) (Armagh Observatory
  !
  !  History:
  !     09-Dec-2004: CWR:        adapted from SFIT_RDGRID
  !                  CWR:        extensions to create on-disk index
  !     09-Jun-2009: CSJ:        additional input format 
  !                              deleted dead code     
  !
  !-


  USE mod_model_list
  USE mod_tetra_list

  !  Implicit declaration
  IMPLICIT NONE


  CHARACTER(LEN=256), EXTERNAL :: STR_APPEND


  !  Pi.
  !  TAPM__PI = PI
  DOUBLE PRECISION :: TAPM__PI
  PARAMETER ( TAPM__PI = 3.141592653589793238462643383279503D+00 )


  !  Global Variables.
  !     This subroutine should contain no global variables. 


  ! Subroutine Arguments.

  INTEGER,INTENT(IN)   :: MODE              ! The operation of this subroutine
  ! depends on the type of grid to be read
  CHARACTER,INTENT(IN) :: GRDFILE*(*)       ! filename for model grid
  INTEGER,INTENT(IN)   :: NWMAX             ! maximum number of wavelengths 
  INTEGER,INTENT(OUT)  :: NWMOD             ! number of wavelengths
  REAL,INTENT(OUT) :: WMOD(NWMAX)       ! wavelengths in model

  ! These arrays are only used locally now
  REAL,ALLOCATABLE,DIMENSION(:)     :: FMOD  ! emergent fluxes
  REAL,ALLOCATABLE,DIMENSION(:)     :: CMOD  ! continuum fluxes
  REAL,ALLOCATABLE,DIMENSION(:)     :: AMOD  ! normalized fluxes

  ! Local variables
  CHARACTER(LEN=200) :: MODFILE   ! filename for model spectra
  CHARACTER(LEN=200) :: MODHEAD1  ! header from sp2 file
  INTEGER            :: MODEXIST  ! IOstatus for MODFILE 

  INTEGER :: NMOD   ! Number of models to read
  DOUBLE PRECISION, DIMENSION(:), POINTER      :: PARAMS    ! Array to store T_eff, g, He of current model
  !   (used to constructed linked list)
  INTEGER                          :: RECNUM    ! Record number of current model
  INTEGER,DIMENSION(:),POINTER     :: RECORDS

  DOUBLE PRECISION, DIMENSION(6) :: SCALE_FACS  ! Scaling factors for each dimension

  INTEGER   :: BADVAL            ! Status flag for i/o

  REAL      :: TAU               ! optical depth, not saved

  ! Temporary spectrum arrays
  INTEGER                       :: NWLOC  ! size of data arrays
  REAL,ALLOCATABLE,DIMENSION(:) ::  WLOC  ! wavelengths
  REAL,ALLOCATABLE,DIMENSION(:) ::  FLOC  ! emergent fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  CLOC  ! continuum fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  ALOC  ! normalized fluxes

  !  Model grid wavelength definition
  REAL                  :: WSTT  ! Start wavelength of stored model grid
  REAL                  :: WEND  ! End   wavelength of stored model grid
  REAL                  :: WDEL  ! Wavelength increment of stored model grid

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance

  ! Local constants
  CHARACTER :: MODEL_FMT(0:6)*80
  DATA MODEL_FMT(0) /'Spectrum.000  (including fluxes, continuum and residual) '/
  DATA MODEL_FMT(1) /'Spectrum.001  (residual or normalized flux only) ' /
  DATA MODEL_FMT(2) /'Sterne.s      (total flux, poutput from Sterne) ' /
  DATA MODEL_FMT(3) /'sp2           (spectrum format 2: header useless) ' /
  DATA MODEL_FMT(4) /'Synthe        (including fluxes, continuum and residual) '/
  DATA MODEL_FMT(5) /'Synthe        (residual or normalized flux only) '/
  DATA MODEL_FMT(6) /'Spectrum.sp4  (header, fluxes, continuum and residual) '/


  ! 

  ! Initialize the model grid
  WMOD = 0.
  !CMOD = 0.
  !FMOD = 0.


  SCALE_FACS(1:4) = 999999.0
  SCALE_FACS(5:6) = -1.0


  ! Define the model grid
  print *, "Defining the model grid"
  WRITE (*,'(a,i2,2x,a)') ' sfit_makegrid: mode = ',MODE,MODEL_FMT(MODE)
  WRITE (*,*) 'Start and end wavelengths and wavelength increment'
  WRITE (*,*) '0 0 0 to define wavelength grid from first model'
  READ  (*,*) WSTT, WEND, WDEL
  WRITE (*,*) WSTT, WEND, WDEL
  WRITE (*,*)


  ! Initialise models list
  CALL MODLIST_NEW(MODSLIST)


  !  Setup model wavelength grid
  IF ( WSTT > 0 .AND. WEND > 0 .AND. WDEL > 0 ) THEN 
     NWMOD = 1 + (WEND - WSTT) / WDEL
     DO NW = 1,NWMOD
        WMOD(NW) = WSTT + (NW-1) * WDEL
     ENDDO

     ! Otherwise use WMOD as provided by first model
  ELSE
     WRITE (*,*) 'sfit_makegrid: *Warning* : using wavelength grid from first model'
  ENDIF


  ! Allocate local resampled flux arrays
  ALLOCATE(AMOD(NWMOD), FMOD(NWMOD), CMOD(NWMOD))
  AMOD = 0.
  FMOD = 0.
  CMOD = 0.


  ! Allocate Spectral Parameters Array - currently holds 3 params
  ALLOCATE(PARAMS(3))


  ! Open direct access file for saving models
  OPEN(20, FILE=GRDFILE, ACCESS="DIRECT", RECL=4*NWMOD, &
       STATUS="UNKNOWN", FORM="UNFORMATTED")

  RECNUM = 1

  ! First record is always wavelength grid
  WRITE(20, REC=RECNUM) WMOD(1:NWMOD)
  RECNUM = RECNUM + 1

  print *, "Reading models"

  ! MODE = 0:   Spectrum.000  (including fluxes, continuum and residual)
  IF ( MODE==0 .OR. MODE==6 ) THEN

     ALLOCATE(RECORDS(3))

     DO

        CALL STR_NULL ( MODFILE )
        READ  (*,'(A)') MODFILE

        IF ( MODFILE(1:1) == '}' ) EXIT

        MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

        IF ( MODE==0 ) THEN

           OPEN  (1,FILE=MODFILE,STATUS='OLD',IOSTAT=MODEXIST)

           ! Skipe this model if file does not exist 
           IF ( MODEXIST > 0 ) CYCLE 

           ! Read the header lines
           READ  (1,'(7X,F6.0,10x,F5.3,13X,F7.5)') &
                PARAMS(1), PARAMS(2), PARAMS(3)

           !  Convert temperature to kK
           PARAMS(1) = PARAMS(1) / 1000.

           ! Read the dimension of the spectrum
           READ  (1,*) NWLOC

           ! Allocate the array sizes
           ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC),FLOC(NWLOC),CLOC(NWLOC) )

           ! Read the data
           READ  (1,*) 
           READ  (1,*) 
           READ  (1,*)
           READ  (1,*) ( WLOC(NW),ALOC(NW),FLOC(NW),CLOC(NW), TAU, NW = 1,NWLOC )
           CLOSE (1)

        ELSE

           ! should codify the following  
           ! CALL sp4rd  (  )

           OPEN  (1,FILE=MODFILE,STATUS='OLD')

           ! read the header -- assume fixed format for now;
           READ (1,*)                       ! '[header]'
           READ (1,'(10x,f10.0)') PARAMS(1) ! 'teff    ', s_teff
           READ (1,'(10x,f10.0)') PARAMS(2) ! 'logg    ', s_logg
           READ (1,*)                       ! 'vturb   ', sp_vt
           READ (1,*)                       ! 'febyh   ', m_febyh
           READ (1,*)                       ! 'albyhe  ', m_albyhe
           READ (1,*)                       ! 'abund_h ', s_abh/100.
           READ (1,'(10x,f10.0)') PARAMS(3) ! 'abund_he', s_abhe/100.
           READ (1,*)                       ! 'abund_c ', s_abc/100.
           READ (1,*)                       ! 'abund_n ', s_abn/100.
           READ (1,'(10x,i10)') NWLOC       ! 'nwave   ', sp_nw
           READ (1,*)
           READ (1,*)                       ! '[data-fields]'
           READ (1,*)                       ! 'wavelength = 0'
           READ (1,*)                       ! 'flux = 1'
           READ (1,*)                       ! 'continuum = 2'
           READ (1,*)                       ! 'normalized_flux = 3'
           READ (1,*)                       ! 'optical_depth = 4'
           READ (1,*)
           READ (1,*)                       ! '[data-units]'
           READ (1,*)                       ! 'wavelength = Angstroms'
           READ (1,*)                       ! 'flux = ergs cm^-2 s^-1 Hz^-1'
           READ (1,*)                       ! 'continuum = ergs cm^-2 s^-1 Hz^-1'
           READ (1,*)                       ! 'normalized_flux = dimensionless'
           READ (1,*)                       ! 'optical_depth = dimensionless'
           READ (1,*)
           READ (1,*)                       ! '[data]'

           !  Convert temperature to kK
           PARAMS(1) = PARAMS(1) / 1000.

           ! Allocate the array sizes
           ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC),FLOC(NWLOC),CLOC(NWLOC) )

           ! read the data
           READ  (1,*) ( WLOC(NW),FLOC(NW),CLOC(NW),ALOC(NW), TAU, NW = 1,NWLOC )

           ! close off
           CLOSE (1)
        ENDIF


        ! Use initial model to define wavelength grid
        IF ( NWMOD == 0 ) THEN
           NWMOD = NWLOC
           WMOD(1:NWMOD) = WLOC(1:NWMOD)
        ENDIF

        ! Resample arrays onto prespecified grid
        CALL REMAP (WLOC,ALOC(1:NWLOC),NWLOC, &
             WMOD,AMOD(1:NWMOD),NWMOD, 1.)
        CALL REMAP (WLOC,FLOC(1:NWLOC),NWLOC, &
             WMOD,FMOD(1:NWMOD),NWMOD, 0.)
        CALL REMAP (WLOC,CLOC(1:NWLOC),NWLOC, &
             WMOD,CMOD(1:NWMOD),NWMOD, 0.)

        !  convert to astrophysical fluxes (F_nu, C_nu)
        FMOD = FMOD * TAPM__PI
        CMOD = CMOD * TAPM__PI


        IF ( .NOT. MODLIST_IS_DUPE(MODSLIST, PARAMS) ) THEN

!!! 1) Write spectrum to direct access file
           WRITE(20, REC=RECNUM) AMOD(1:NWMOD)
           RECORDS(1) = RECNUM
           RECNUM = RECNUM + 1

           WRITE(20, REC=RECNUM) FMOD(1:NWMOD)
           RECORDS(2) = RECNUM
           RECNUM = RECNUM + 1

           WRITE(20, REC=RECNUM) CMOD(1:NWMOD)
           RECORDS(3) = RECNUM
           RECNUM = RECNUM + 1

!!! 2) Add enrty to list
           CALL MODLIST_ADD(MODSLIST, PARAMS, RECORDS)
        ENDIF

        ! Check Teff scaling factors
        IF ( PARAMS(1) .LT. SCALE_FACS(1) ) SCALE_FACS(1) = PARAMS(1)
        IF ( PARAMS(1) .LT. SCALE_FACS(2) .AND. &
             PARAMS(1) .GT. SCALE_FACS(1) ) SCALE_FACS(2) = PARAMS(1)

        ! Check logg scaling factors
        IF ( PARAMS(2) .LT. SCALE_FACS(3) ) SCALE_FACS(3) = PARAMS(2)
        IF ( PARAMS(2) .LT. SCALE_FACS(4) .AND. &
             PARAMS(2) .GT. SCALE_FACS(3) ) SCALE_FACS(4) = PARAMS(2)

        ! Check nHe scaling factors
        IF ( PARAMS(3) .GT. SCALE_FACS(5) ) SCALE_FACS(5) = PARAMS(3)
        IF ( PARAMS(3) .GT. SCALE_FACS(6) .AND. &
             PARAMS(3) .LT. SCALE_FACS(5) ) SCALE_FACS(6) = PARAMS(3)


        ! Done with model arrays, free up for next use
        DEALLOCATE ( WLOC, ALOC, FLOC, CLOC )


        ! Report the spectrum 
        WRITE (*,'(A,''  Teff='',F6.0,'' log g='',F5.3, '' nHe='',F8.5,'' nw='',I6)') &
             TRIM(MODFILE), PARAMS(1), PARAMS(2), PARAMS(3), NWLOC
        WRITE (11,'(A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F8.5,'' nw='',I6)') &
             TRIM(MODFILE), PARAMS(1), PARAMS(2), PARAMS(3), NWLOC

     END DO


     ! MODE = 1:   Spectrum.001  (residual or normalized flux only)
  ELSEIF ( MODE.EQ.1 ) THEN

     ALLOCATE(RECORDS(1))

     DO

        WRITE (*,*) 'File containing model spectrum?'
        READ  (*,'(A)') MODFILE

        IF( MODFILE(1:1) == '}') EXIT

        MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

        OPEN  (1,FILE=MODFILE,STATUS='OLD')
        READ  (1,'(2X,F5.0,2x,F5.0,3X,F7.5)') &
             PARAMS(1), PARAMS(2), PARAMS(3)
        READ  (1,*) 
        READ  (1,*) NWLOC
        ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC) )
        READ  (1,*) (WLOC(NW),ALOC(NW),NW=1,NWLOC)
        CLOSE (1)

        ! Use initial model to define wavelength grid
        IF ( NWMOD == 0 ) THEN
           NWMOD = NWLOC
           WMOD(1:NWMOD) = WLOC(1:NWMOD)
        ENDIF

        ! Resample the data
        CALL REMAP (WLOC,ALOC(1:NWLOC),NWLOC, &
             WMOD,AMOD(1:NWMOD),NWMOD, 1.)


        ! 1) Write spectrum to direct access file
        WRITE(20, REC=RECNUM) AMOD
        RECORDS(1) = RECNUM
        RECNUM = RECNUM + 1

        ! 2) Push spectrum params and record number to list
        CALL MODLIST_ADD(MODSLIST, PARAMS, RECORDS)

        ! Check Teff scaling factors
        IF ( PARAMS(1) .LT. SCALE_FACS(1) ) SCALE_FACS(1) = PARAMS(1)
        IF ( PARAMS(1) .LT. SCALE_FACS(2) .AND. &
             PARAMS(1) .GT. SCALE_FACS(1) ) SCALE_FACS(2) = PARAMS(1)

        ! Check logg scaling factors
        IF ( PARAMS(2) .LT. SCALE_FACS(3) ) SCALE_FACS(3) = PARAMS(2)
        IF ( PARAMS(2) .LT. SCALE_FACS(4) .AND. &
             PARAMS(2) .GT. SCALE_FACS(3) ) SCALE_FACS(4) = PARAMS(2)

        ! Check nHe scaling factors
        IF ( PARAMS(3) .GT. SCALE_FACS(5) ) SCALE_FACS(5) = PARAMS(3)
        IF ( PARAMS(3) .GT. SCALE_FACS(6) .AND. &
             PARAMS(3) .LT. SCALE_FACS(5) ) SCALE_FACS(6) = PARAMS(3)

        ! Done with arrays, free up for next loop
        DEALLOCATE ( WLOC, ALOC )

        ! Report the spectrum
        WRITE (*,'(A,''  Teff='',F5.1,'' log g='',F4.1, '' nHe='',F8.5)') &
             TRIM(MODFILE), PARAMS(1), PARAMS(2), PARAMS(3)
        WRITE (11,'(''Model '',A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F8.5)') &
             TRIM(MODFILE), PARAMS(1), PARAMS(2), PARAMS(3)

     END DO

  ENDIF


  ! Close output grid file
  CLOSE(22)


!!! List of models has been completed
!!! Now:
!!!     1) Flatten MODLIST PARAMS arrays to 1D vector
!!!     2) Perform tetrahedralisation using tetgen
!!!     3) Write index to file
!!! (?) 4) Keep index in memory for possible use later


  print *, "scale_facs: ", scale_facs

  CALL MODLIST_RESCALE_LIST(MODSLIST, SCALE_FACS)

  print *, "modlist - rescaled"

  CALL MODLIST_PRINT(MODSLIST)

  print *, "modlist - printed"

  CALL TETRAHEDRALISE(MODSLIST, TETRALIST)

  print *, "Saving index to ", STR_APPEND(GRDFILE, ".idx")


  OPEN (1, FILE=STR_APPEND(GRDFILE, ".idx"), STATUS='UNKNOWN', FORM='UNFORMATTED')
!!! Write grid dimensions
  WRITE (1) NWMOD

  WRITE  (1) WMOD(1:NWMOD)

  CALL MODLIST_SAVE(MODSLIST, 1)
  CALL TETRALIST_SAVE(TETRALIST, 1)
  CLOSE (1)

  ! Tidy up
  DEALLOCATE(RECORDS, AMOD, FMOD, CMOD)

  !  Report grid
  WRITE (*,'(/'' Model grid:'')')
  WRITE (*,'(4x,''WMOD '',I5,10F10.2)') NWMOD,WMOD(1),WMOD(NWMOD)
  WRITE (*,*)

  WRITE (11,'(/'' Model grid:'')')
  WRITE (11,'(4x,''WMOD '',I5,10F10.2)') NWMOD,WMOD(1),WMOD(NWMOD)
  WRITE (11,*)

END SUBROUTINE SFIT_MAKEGRID
