SUBROUTINE SFIT_SMGRID  ( FWHM, &
     NWMAX,NTMAX,NGMAX,NHMAX, &
     NWMOD,NTMOD,NGMOD,NHMOD, &
     WMOD, TMOD, GMOD, HMOD, &
     FMOD, CMOD )

  !+
  !  Program:
  !     sfit_smgrid
  !
  !  Purpose:
  !=    Given a grid of model spectra ... convolve all with a gaussian fwhm = sigma * 2.534 
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Reference:
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     18-Apr-2016: CSJ: V1.0 
  !
  !-

  !  Implicit declaration
  IMPLICIT NONE

  !  Pi.
  !  TAPM__PI = PI
  DOUBLE PRECISION :: TAPM__PI
  PARAMETER ( TAPM__PI = 3.141592653589793238462643383279503D+00 )


  !  Global Variables.
  !     This subroutine should contain no global variables. 


  ! Subroutine Arguments.

  REAL,INTENT(IN)      :: FWHM              ! fwhm of smoothing function in wl units
  REAL                 :: SIGMA             ! local value for use in qsmooth
  ! depends on the type of grid to be read

  INTEGER,INTENT(IN)   :: NWMAX             ! maximum number of wavelengths
  INTEGER,INTENT(IN)   :: NTMAX             ! maximum number of temperatures
  INTEGER,INTENT(IN)   :: NGMAX             ! maximum number of gravities
  INTEGER,INTENT(IN)   :: NHMAX             ! maximum number of helium abundances 

  INTEGER,INTENT(OUT)  :: NWMOD             ! number of wavelengths
  INTEGER,INTENT(OUT)  :: NTMOD             ! number of temperatures 
  INTEGER,INTENT(OUT)  :: NGMOD             ! number of gravities
  INTEGER,INTENT(OUT)  :: NHMOD             ! number of helium abundances

  REAL,INTENT(OUT)     :: TMOD(NTMAX)       ! temperatures in grid
  REAL,INTENT(OUT)     :: GMOD(NGMAX)       ! gravities in grid
  REAL,INTENT(OUT)     :: HMOD(NHMAX)       ! helium abundances in grid
  REAL,INTENT(OUT)     :: WMOD(NWMAX)       ! wavelengths in model

  REAL,INTENT(OUT)     :: FMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! emergent fluxes
  REAL,INTENT(OUT)     :: CMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! continuum fluxes
  !  REAL,INTENT(OUT)     :: AMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! normalized fluxes

  ! Local variables
  CHARACTER(LEN=72) :: MODFILE   ! filename for model spectra
  INTEGER           :: LS, IB    ! length of string, pos of blank
  CHARACTER(LEN=72) :: MODHEAD1  ! header from sp2 file
  CHARACTER(LEN=72) :: MODHEAD2  ! header from sp2 file

  INTEGER           :: BADVAL    ! Status flag for i/o

  REAL      :: TAU               ! optical depth, not saved

  ! Temporary spectrum arrays
  INTEGER                       :: NWLOC  ! size of data arrays
  REAL,ALLOCATABLE,DIMENSION(:) ::  WLOC  ! wavelengths
  REAL,ALLOCATABLE,DIMENSION(:) ::  FLOC  ! emergent fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  CLOC  ! continuum fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  ALOC  ! normalized fluxes

  !  Model grid wavelength definition
  REAL                  :: WSTT  ! Start wavelength of stored model grid
  REAL                  :: WEND  ! End   wavelength of stored model grid
  REAL                  :: WDEL  ! Wavelength increment of stored model grid

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance

  INTEGER   :: LF                ! length of filename


  !.

  WRITE (*,*) 'SFIT_SMGRID: Version 1.0: 18-Apr-2016'
  WRITE (*,*) 'fwhm = ',FWHM
  SIGMA = FWHM / 2.534


  ! Allocate the temp arrays 
  ALLOCATE ( WLOC(NWMOD),FLOC(NWMOD),CLOC(NWMOD) )
  WLOC(1:NWMOD) = WMOD(1:NWMOD)

  ! Loop over all arrays in grid
  DO NT = 1,NTMAX
     DO NG = 1,NGMAX
        DO NH = 1,NHMAX
           FLOC(1:NWMOD)=FMOD(1:NWMOD,NT,NG,NH)
           ! apply quick gaussian smoothing
           CALL QSMOOTH(WLOC,FLOC,CLOC,NWMOD,SIGMA)
           FMOD(1:NWMOD,NT,NG,NH)=CLOC(1:NWMOD)
        ENDDO
     ENDDO
  ENDDO

  ! Release the temp arrays
  DEALLOCATE ( WLOC, FLOC, CLOC )

END SUBROUTINE SFIT_SMGRID








