SUBROUTINE SPEC_TGHV_FLX (W,PARS,A,F,C,F1,C1,F2,C2,NP)

  !+
  !
  !  Programme:
  !     spec_tghv_flx
  !
  !  Purpose:
  !=    Returns predicted flux at a given wavelength for a model binary
  !     and for a given combination of Teff, log g, nHe and vsini 
  !     This just does formal soultion, and includes fluxes from both 
  !     components, if present.
  !
  !  Method: 
  !     This subroutine must operate in two modes; the mode is determined
  !     by comparing PARS with the values of PARS last used.
  !
  !     1   On first call with a new set of parameters PARS it must
  !         create a syntetic spectrum containing the predicted flux F at the 
  !         given wavelength W, and derivatives wrt each parameter.
  !
  !     2.  On subsequent calls with the same parameters PARS it must
  !         simply return the predicted flux F at the given wavelength W.
  !         and derivatives, DERIVS, wrt each parameter.
  !
  !  Notes:
  !
  !     1) to be efficient the subroutine should be used when the
  !     external loop is over wavelength... and not over parameters.
  !     2) To avoid nasty effects during extrapolation, 
  !     anu is not allowed to exceed unity -- LTE models asasumed
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     10-MAR-2005: CSJ
  !        original version, adapted from SPEC_TGHV_BIN
  !
  !-

  !  Global variables:
  USE TAPM_PAR               ! mathematical constants
  USE SFIT_MOD_TGH
  USE SFIT_MOD_TGH_2
  USE SFIT_BIN_TGH
  USE SFIT_BIN_TGH_2
  USE SFIT_OBSDATA


  !  No implicit typing
  IMPLICIT NONE


  !  Subroutine arguments:
  INTEGER, INTENT(IN) ::  NP      ! number of parameters in PARS (given)
  REAL                ::  W       ! wavelength (given)
  REAL, DIMENSION(NP) ::  PARS    ! parameters (given)
  REAL                ::  A       ! residual flux *1+*2     (returned)
  REAL                ::  F       ! surface flux *1+*2     (returned)
  REAL                ::  C       ! continuum flux *1+*2   (returned)
  REAL                ::  F1      ! surface flux *1        (returned)
  REAL                ::  C1      ! continuum flux *1      (returned)
  REAL                ::  F2      ! surface flux *2        (returned)
  REAL                ::  C2      ! continuum flux *1      (returned)


  !  Interface to external subroutines
  INTERFACE 

    SUBROUTINE XFT_TGHV_INT ( TEFF, DT, LOGG, DG, ANHY, DH, VSINI, DV, &
         SINST, DVSYS, DELTAV, NWBIN, NTMOD, NGMOD, NHMOD, &
         WBIN, TMOD, GMOD, HMOD, FBIN, FINT )
      IMPLICIT NONE
      INTEGER, INTENT(IN) ::  NWBIN, NTMOD, NGMOD, NHMOD   ! Grid dimensions
      REAL, INTENT(IN), DIMENSION(:)     :: WBIN
      REAL, INTENT(IN), DIMENSION(:)     :: TMOD
      REAL, INTENT(IN), DIMENSION(:)     :: GMOD
      REAL, INTENT(IN), DIMENSION(:)     :: HMOD
      REAL, INTENT(IN), DIMENSION(:,:,:,:) :: FBIN
      REAL, INTENT(IN) ::  TEFF, DT    ! Effective temperature (K) and increments
      REAL, INTENT(IN) ::  LOGG, DG    ! Surface gravity (cgs)
      REAL, INTENT(IN) ::  ANHY, DH    ! Surface helium abundance (fractional)
      REAL, INTENT(IN) ::  VSINI       ! Projected rotation velocity (km/s)
      REAL, INTENT(INOUT) ::  DV       ! Projected rotation velocity increment
      REAL, INTENT(IN) ::  SINST       ! Instrumental profile (Gaussian FWHM)
      REAL, INTENT(IN) ::  DVSYS       ! velocity broadening function (boxcar)
      REAL, INTENT(IN) ::  DELTAV      ! Radial velcoity shift
      REAL, INTENT(OUT), DIMENSION(:,:) :: FINT ! Interpolated fluxes and derivatives
      !      INTEGER, EXTERNAL :: SPECSEG
    END SUBROUTINE XFT_TGHV_INT


    SUBROUTINE XFT_TGHV_IN1 ( TEFF, DT, LOGG, DG, ANHY, DH, VSINI, DV, &
         SINST, DVSYS, DELTAV, NWBIN, NTMOD, NGMOD, NHMOD, &
         WBIN, TMOD, GMOD, HMOD, FBIN, FINT )
      IMPLICIT NONE
      INTEGER, INTENT(IN) ::  NWBIN, NTMOD, NGMOD, NHMOD   ! Grid dimensions
      REAL, INTENT(IN), DIMENSION(:)     :: WBIN
      REAL, INTENT(IN), DIMENSION(:)     :: TMOD
      REAL, INTENT(IN), DIMENSION(:)     :: GMOD
      REAL, INTENT(IN), DIMENSION(:)     :: HMOD
      REAL, INTENT(IN), DIMENSION(:,:,:,:) :: FBIN
      REAL, INTENT(IN) ::  TEFF, DT    ! Effective temperature (K) and increments
      REAL, INTENT(IN) ::  LOGG, DG    ! Surface gravity (cgs)
      REAL, INTENT(IN) ::  ANHY, DH    ! Surface helium abundance (fractional)
      REAL, INTENT(IN) ::  VSINI       ! Projected rotation velocity (km/s)
      REAL, INTENT(INOUT) ::  DV       ! Projected rotation velocity increment
      REAL, INTENT(IN) ::  SINST       ! Instrumental profile (Gaussian FWHM)
      REAL, INTENT(IN) ::  DVSYS       ! velocity broadening function (boxcar)
      REAL, INTENT(IN) ::  DELTAV      ! Radial velcoity shift
      REAL, INTENT(OUT), DIMENSION(:,:) :: FINT ! Interpolated fluxes and derivatives
      !      INTEGER, EXTERNAL :: SPECSEG
    END SUBROUTINE XFT_TGHV_IN1

  END INTERFACE


  !  Local variables:

  !  Loop counters
  INTEGER, SAVE :: NW = 0            ! loop counter for wavelength
  INTEGER, SAVE :: NWBINL = 0          ! remember last value of NWBIN
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance

  !   Stellar system parameters
  REAL, SAVE     ::  TEFF1,  TEFF1_0=0  ! Effective temperature (K)
  REAL, SAVE     ::  LOGG1,  LOGG1_0=0  ! Surface gravity (cgs)
  REAL, SAVE     ::  ANHE1,  ANHE1_0=0  ! Surface helium abundance (fractional)
  REAL, SAVE     ::  VSIN1,  VSIN1_0=0  ! Projected rotation velocity (km/s)
  REAL, SAVE     ::  TEFF2,  TEFF2_0=0  ! Effective temperature (K)
  REAL, SAVE     ::  LOGG2,  LOGG2_0=0  ! Surface gravity (cgs)
  REAL, SAVE     ::  ANHE2,  ANHE2_0=0  ! Surface helium abundance (fractional)
  REAL, SAVE     ::  VSIN2,  VSIN2_0=0  ! Projected rotation velocity (km/s)
  REAL, SAVE     ::  Q1,     Q1_0   =0  ! log Luminosity multiplier (star 1)
  REAL, SAVE     ::  DELTV1, DELTV1_0=0 ! Velocity shift (star 1)
  REAL, SAVE     ::  Q2,     Q2_0   =0  ! log Luminosity multiplier (star 2)
  REAL, SAVE     ::  DELTV2, DELTV2_0=0 ! Velocity shift (star 2)
  REAL, SAVE     ::  EBV,    EBV_0=0    ! Extinction coefficient
  REAL, SAVE     ::  R1SQ, R2SQ         ! Angular radius multipliers 

  !  Interpolated stellar spectra
  REAL, SAVE  ::  FINT1(NWMAX,8)   ! Interpolated fluxes
  REAL, SAVE  ::  CINT1(NWMAX,8)   ! Interpolated fluxes
  REAL, SAVE  ::  FINT2(NWMAX2,8)   ! Interpolated fluxes
  REAL, SAVE  ::  CINT2(NWMAX2,8)   ! Interpolated fluxes

  INTEGER  ::  I                ! Tabulated flux index

  !  Variands for calculating derivatives
  REAL     :: DT                ! delta temperature
  REAL     :: DG                ! delta gravity
  REAL     :: DV                ! delta v sin i
  REAL     :: DH                ! delta abundance

  !  Local flux values
  REAL     :: FNU               ! emergent flux
  REAL     :: CNU               ! continuum flux
  REAL     :: ANU               ! residual flux (Fnu/Cnu)

  !  Reddening coefficients
  REAL     :: RED, REDX         ! Total extinction (fraction)
  REAL     :: AMAG, AMAGX       ! Total extinction (magnitude)
  REAL     :: DRDE              ! Reddening derivative
  REAL     :: RMAG = 3.1        ! Total/Selective extinction Av/E(B-V)
  INTEGER  :: RLAW = 0          ! Reddening law (>=0: galactic)
  REAL     :: NHCOL             ! Hydrogen column density
  REAL     :: LYA, LYAX         ! Lyman alpha absorption (1 = no absorption)

  !.

  !   Map local values of parameters
  TEFF1  = PARS(1) 
  LOGG1  = PARS(2)
  ANHE1  = PARS(3) 
  VSIN1  = PARS(4)
  Q1     = PARS(5) 
  DELTV1 = PARS(6)
  TEFF2  = PARS(7) 
  LOGG2  = PARS(8)
  ANHE2  = PARS(9) 
  VSIN2  = PARS(10) 
  Q2     = PARS(11)
  DELTV2 = PARS(12)
  EBV    = PARS(13)

  R1SQ   = 10.**(2*Q1)
  R2SQ   = 10.**(2*Q2)

  IF (NW.GE.NWBIN .OR. NWBIN.NE.NWBINL) NW = 0

  !   Test whether parameters have changed and compute new spectra
  IF ( NW==0 ) THEN


    !     Principal diagnostic when trouble occurs 
    !        WRITE (*,*) ' 1 ',TEFF1,LOGG1,ANHE1,VSIN1,Q1,DELTV1
    !        WRITE (*,*) ' 2 ',TEFF2,LOGG2,ANHE2,VSIN2,Q2,DELTV2

    NWBINL = NWBIN
    DG = 0.05
    DH = 0.01
    IF ( ANHE1>0.99 ) DH = -0.01
    DV = 1.0

    IF (NWMOD>0) THEN 
      DT = 0.01 * TEFF1
      CALL XFT_TGHV_IN1 ( TEFF1, DT, LOGG1, DG, ANHE1, DH, VSIN1, DV, &
           SINST, DVSYS, DELTV1, NWBIN, NTMOD, NGMOD, NHMOD, &
           WBIN,  TMOD,  GMOD,  HMOD, FBIN,  FINT1 )
      CALL XFT_TGHV_IN1 ( TEFF1, DT, LOGG1, DG, ANHE1, DH, VSIN1, DV, &
           SINST, DVSYS, DELTV1, NWBIN, NTMOD, NGMOD, NHMOD, &
           WBIN,  TMOD,  GMOD,  HMOD, CBIN,  CINT1 )
    ENDIF


    IF (NWMOD2>0) THEN 
      DT = 0.01 * TEFF2
      CALL XFT_TGHV_IN1 ( TEFF2, DT, LOGG2, DG, ANHE2, DH, VSIN2, DV, &
           SINST, DVSYS, DELTV2, NWBIN, NTMOD2, NGMOD2, NHMOD2, &
           WBIN,  TMOD2,  GMOD2,  HMOD2, FBIN2,  FINT2 )
      CALL XFT_TGHV_IN1 ( TEFF2, DT, LOGG2, DG, ANHE2, DH, VSIN2, DV, &
           SINST, DVSYS, DELTV2, NWBIN, NTMOD2, NGMOD2, NHMOD2, &
           WBIN,  TMOD2,  GMOD2,  HMOD2, CBIN2,  CINT2 )
    ENDIF

  ENDIF

  !   Identify NW from wavelength
  NW = NW + 1

  !   total flux and continuum
  FNU = ( R1SQ * FINT1(NW,1) + R2SQ * FINT2(NW,1) ) 
  CNU = ( R1SQ * CINT1(NW,1) + R2SQ * CINT2(NW,1) )

  ! Returns for fitting normalized fluxes
  IF (FOBS_NORM) THEN

    !   check valid continuum data
    IF (CNU<1.E-11) THEN
      F   = 1.0
      RETURN
    ENDIF

    !  Calculate insterstellar reddening
    CALL ISLYA (WBIN(NW),EBV,     0.0, LYA)
    CALL ISLYA (WBIN(NW),EBV+0.01,0.0, LYAX)

    RED  = LYA
    REDX = LYAX
    DRDE = (REDX - RED) / 0.01

    A      = FNU / CNU * RED

    ! Returns for fitting total fluxes
  ELSE

    !  Calculate insterstellar reddening
    CALL ISRED (WBIN(NW),EBV,     RED, AMAG,  RMAG,RLAW)
    CALL ISLYA (WBIN(NW),EBV,     0.0, LYA)
    CALL ISRED (WBIN(NW),EBV+0.01,REDX,AMAGX, RMAG,RLAW)
    CALL ISLYA (WBIN(NW),EBV+0.01,0.0, LYAX)

    RED  = LYA * RED
    REDX = LYAX * REDX
    DRDE = (REDX - RED) / 0.01

    A      = 0.

  ENDIF

    !  Redden fluxes and derivatives
  F  = FNU  * RED
  C  = CNU  * RED
  F1 = R1SQ * FINT1(NW,1) * RED
  C1 = R1SQ * CINT1(NW,1) * RED
  F2 = R2SQ * FINT2(NW,1) * RED
  C2 = R2SQ * CINT2(NW,1) * RED

END SUBROUTINE SPEC_TGHV_FLX





