SUBROUTINE vsini_conv (w,f,nw,vsini,bb)

  !+
  !
  != Rotationally broaden a spectrum by convolution
  !  with the Unsold (1955) function given in Dufton (A+A,16,p301,1972).
  !  Data assumed to be at wavelength intervals.
  !     W(*)      array of wavelength points unchanged on exit
  !     F(*)      array of flux points contains convoluted flux
  !               on exit
  !     NPTS      number of points considered in above arrays
  !     VSINI     vsini in km/s (use 1.5 by default)
  !     B         limb darkening coefficient
  !
  !  Local array CONFLX(MXPTS) contains convoluted flux before
  !  it is written back into F(*).
  !
  !  History:
  !     Original version (ROTATE - written for DIPSO)
  !        C.S.Jeffery, University of St Andrews, 1988
  !
  !     Incorporated to SFIT
  !        C.S.Jeffery, Armagh Observatory, 2000 February
  !
  !     Upgraded to double precision with included subprograms
  !     and corrected to remove bugs at low v sin i
  !        CSJ, AOP, 2020 June 11
  !
  !-

  IMPLICIT NONE
  
  ! Subroutine arguments
  REAL, DIMENSION(*), INTENT(IN)    ::  w       
  REAL, DIMENSION(*), INTENT(INOUT) ::  f
  INTEGER, INTENT(IN)               ::  nw
  REAL, INTENT(IN)                  ::  vsini
  REAL, INTENT(IN)                  ::  bb
  

  ! Local variables
  REAL (8), DIMENSION(:), ALLOCATABLE, SAVE ::conw   ! convolution array
  INTEGER, SAVE ::  mxpts=0                          ! maximum dimension
  REAL (4) :: w0
  INTEGER  :: i 
  REAL (8) :: fac
  REAL (4) :: xlo, xup                               ! same as w for 'locate'
  
  ! Local constants
  REAL (8), PARAMETER :: c = 2.99792458e5            ! light speed (km/s)


  !.

  !  Check limits on number of points and allocate convolution array to suit
  IF ( nw > mxpts ) THEN
     IF ( ALLOCATED ( conw ) ) DEALLOCATE ( conw )
     ALLOCATE ( conw(nw) )
     mxpts = nw
  ENDIF


  !  Check VSINI is non-zero
  IF ( ABS( vsini ) <= 1.e-3 ) RETURN

  !  Call convolution routine and cycle over NPTS points
  DO i = 1,nw,1
     w0  = w(i)
     fac = c/vsini/w0
     xlo = w0 - (3./fac)
     xup = w0 + (3./fac)
     
     conw(i) = vsini_trap2 ( xlo,xup,w0 )

  END DO

  !  Copy convoluted flux back into F
  f(1:nw) = conw(1:nw)

  
  
CONTAINS
  

  FUNCTION vsini_trap2 ( xlo,xup,xc )

    !+
    !
    != Do trapezoidal integration of Y(*) between XLOWER and XUPPER
    !  weighted by rotational broadening function ROT.  ROT is also
    !  integrated between the same limits for correct normalization.
    !  NORM is the normalization factor.
    !
    !-

    IMPLICIT NONE
  
    ! Function type
    REAL (8) :: vsini_trap2

    ! Function arguments
    REAL (4) :: xlo, xup  ! Integral boundaries
    REAL (4) :: xc        ! Value at which integral to be evaluated
    
    ! Local variables
    REAL (8) :: norm    ! normalization integral
    REAL (8) :: trap    ! convolution integral
    REAL (8) :: xa, xb  ! delta lambda  * c / delta v . lambda
    REAL (8) :: va, vb  ! values of rotation profile at xa, xb
    REAL (8) :: rab, yab  ! means
    REAL (8) :: wt
    INTEGER  :: ii
    INTEGER  :: ilo, iup, ic

    !.

    !  Initialize TRAP and NORM
      trap = 0.0d0
      norm = 0.0d0

      !  Locate index in array w corresponding to limits of integration
      CALL LOCATE (w,nw,xlo,ilo)
      CALL LOCATE (w,nw,xup,iup)

      !write (*,*) 'VSIN_TRAP :i,xc,b,fac = ', i,XC,bb,FAC
      !write (*,*) 'VSIN_TRAP :x1,xn,xlo,xup = ', w(1),w(nw),XLO,XUP
      !write (*,*) 'VSIN_TRAP :n,ilo,iup = ', nw,ILO,IUP

      !  Prevent array indices going out of bounds
      ilo = MAX (1,ilo)
      iup = MIN (iup,nw-1)

      
      IF ( iup > ilo ) THEN 

         !  Trapezoidal integration
         
         xa = fac * ( xc-w(ilo) )
         !write (*,*) ilo,xa
         va = vsini_rot2 ( xa, bb )
         !write (*,*) ilo,xa,va
      
         DO ii = ilo+1, iup+1, 1

            xb   = fac * ( xc-w(ii) )
            !write (*,*) ii,xb
            vb   = vsini_rot2 ( xb, bb )
            rab  = 0.5 * ( va+vb )
            yab  = 0.5 * ( f(ii-1)+f(ii) )
         
            wt   = rab * ( xb-xa )
            trap = trap + wt * yab
            norm = norm + wt
         
            xa = xb
            va = vb
         
         END DO

      ENDIF

      !write (*,*) 'VSIN_TRAP :trap,norm = ', trap, norm, trap/norm, f(i)

      IF ( ABS (norm) > 1.d-5 ) THEN
         !  Normalise TRAP
         trap = trap / norm
         
      ELSE
         ! Single-point quadrature for small vsini
         ! pointer inherited from parent routine
         trap = f(i)
      ENDIF
         
      ! Return function value      
      vsini_trap2 = trap

      !write (*,*) 'VSIN_TRAP :vsini_trap2 = ', vsini_trap2


    END FUNCTION vsini_trap2

    
    FUNCTION vsini_rot2 (x,b)

      !+
      !
      != Rotational broadening function.
      !
      !-

      IMPLICIT NONE
      
      ! Function type
      REAL (8) :: vsini_rot2

      ! Function argument
      REAL (8) :: x         ! normalization integral
      REAL (4) :: b         ! limb darkening coefficient

      ! Local constant
      REAL (8), PARAMETER :: PI=3.14159265358979323846
      
      !.

      IF ( ABS(x) < 1.d0 ) THEN
         vsini_rot2 = ( 4.*SQRT(1.-x*x) / pi + b * (1.-x*x) ) * 1.5 / (3.+2.*b)
      ELSE
         vsini_rot2 = 0.
      ENDIF
      
    END FUNCTION vsini_rot2

    
END SUBROUTINE vsini_conv
